/*
 * Copyright 2013 The Closure Compiler Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// This file has been automatically generated.  Please do not edit it manually.
/**
 * @fileoverview Definitions for globals in Chrome.  This file describes the
 * externs API for the chrome.* object when running in a normal browser
 * context.  For APIs available in Chrome Extensions, see chrome_extensions.js
 * in this directory.
 * FROM_EXTERN
 */
@JsName()
library chrome;

import 'package:js/js.dart';
import 'package:func/func.dart';

/** @const */
@JsName()
abstract class AppObject {
  /**
   * @see https://developer.chrome.com/webstore/inline_installation#already-installed
   * @type {boolean}
   */
  bool get isInstalled;
}

/**
 * @const
 * @see https://developer.chrome.com/apps/webstore
 */
@JsName()
abstract class WebstoreObject {
  /**
   * @param {string|function()|function(string)=}
   *     opt_urlOrSuccessCallbackOrFailureCallback Either the URL to install or
   *     the succcess callback taking no arg or the failure callback taking an
   *     error string arg.
   * @param {function()|function(string)=} opt_successCallbackOrFailureCallback
   *     Either the succcess callback taking no arg or the failure callback
   *     taking an error string arg.
   * @param {function(string)=} opt_failureCallback The failure callback.
   */
  install(
      [opt_urlOrSuccessCallbackOrFailureCallback /*String|Function|undefined*/,
      Function opt_successCallbackOrFailureCallback,
      opt_failureCallback(String arg1)]);

  /** @type {!ChromeNumberEvent} */
  ChromeNumberEvent get onDownloadProgress;

  /** @type {!ChromeStringEvent} */
  ChromeStringEvent get onInstallStageChanged;
}

/**
 * @see https://developer.chrome.com/extensions/runtime.html
 * @const
 */
@JsName()
abstract class RuntimeObject {
  /**
   * @param {string|!Object=} opt_extensionIdOrConnectInfo Either the
   *     extensionId to connect to, in which case connectInfo params can be
   *     passed in the next optional argument, or the connectInfo params.
   * @param {!Object=} opt_connectInfo The connectInfo object,
   *     if arg1 was the extensionId to connect to.
   * @return {!Port} New port.
   */
  Port connect([opt_extensionIdOrConnectInfo /*String|Object|undefined*/,
      Object opt_connectInfo]);

  Object get lastError;

  /**
   * @param {string|*} extensionIdOrMessage Either the extensionId to send the
   *     message to, in which case the message is passed as the next arg, or the
   *     message itself.
   * @param {(*|!Object|function(*): void)=} opt_messageOrOptsOrCallback
   *     One of:
   *     The message, if arg1 was the extensionId.
   *     The options for message sending, if arg1 was the message and this
   *     argument is not a function.
   *     The callback, if arg1 was the message and this argument is a function.
   * @param {(!Object|function(*): void)=} opt_optsOrCallback
   *     Either the options for message sending, if arg2 was the message,
   *     or the callback.
   * @param {function(*): void=} opt_callback The callback function which
   *     takes a JSON response object sent by the handler of the request.
   * @return {undefined}
   */
  void sendMessage(extensionIdOrMessage, [opt_messageOrOptsOrCallback,
      Object opt_optsOrCallback, opt_callback(arg1)]);
}

/**
 * namespace
 * @const
 */
@JsName()
abstract class ChromeObject {
  /** @const */
  AppObject get app;

  /**
   * Returns an object containing timing information.
   * @return {!ChromeCsiInfo}
   */
  ChromeCsiInfo csi();

  /**
   * Returns an object representing current load times. Note that the properties
   * on the object do not change and the function must be called again to get
   * up-to-date data.
   *
   * @see http://goto.google.com/chromeloadtimesextension
   *
   * @return {!ChromeLoadTimes}
   */
  ChromeLoadTimes loadTimes();

  /**
   * @see https://developer.chrome.com/extensions/runtime.html
   * @const
   */
  RuntimeObject get runtime;

  /**
   * @const
   * @see https://developer.chrome.com/apps/webstore
   */
  WebstoreObject get webstore;
}

/**
 * namespace
 * @const
 */
@JsName("chrome")
external ChromeObject get chrome;

/**
 * @see http://developer.chrome.com/apps/runtime.html#type-Port
 * @constructor
 */
@JsName("Port")
abstract class Port {
// @Ignore
  external Port.fakeConstructor$();

  /** @type {string} */
  String name;

  /** @type {!ChromeEvent} */
  ChromeEvent onDisconnect;

  /** @type {!ChromeEvent} */
  ChromeEvent onMessage;

  /** @type {!MessageSender|undefined} */
  MessageSender sender;

  /**
   * @param {*} obj Message object.
   * @return {undefined}
   */
  void postMessage(obj);

  /** @return {undefined} */
  void disconnect();
}

/**
 * @see https://developer.chrome.com/extensions/events.html
 * @constructor
 * TODO(tbreisacher): Update *Listener methods to take {function()}
 * instead of {!Function}. See discussion at go/ChromeEvent-TODO
 */
@JsName("ChromeEvent")
abstract class ChromeEvent {
// @Ignore
  external ChromeEvent.fakeConstructor$();

  /**
   * @param {!Function} callback
   * @return {undefined}
   */
  void addListener(Function callback);

  /**
   * @param {!Function} callback
   * @return {undefined}
   */
  void removeListener(Function callback);

  /**
   * @param {!Function} callback
   * @return {boolean}
   */
  bool hasListener(Function callback);

  /** @return {boolean} */
  bool hasListeners();
}

/**
 * Event whose listeners take a string parameter.
 * @constructor
 */
@JsName("ChromeStringEvent")
abstract class ChromeStringEvent {
// @Ignore
  external ChromeStringEvent.fakeConstructor$();

  /** @param {function(string): void} callback */
  addListener(callback(String arg1));

  /** @param {function(string): void} callback */
  removeListener(callback(String arg1));

  /**
   * @param {function(string): void} callback
   * @return {boolean}
   */
  bool hasListener(callback(String arg1));

  /** @return {boolean} */
  bool hasListeners();
}

/**
 * Event whose listeners take a boolean parameter.
 * @constructor
 */
@JsName("ChromeBooleanEvent")
abstract class ChromeBooleanEvent {
// @Ignore
  external ChromeBooleanEvent.fakeConstructor$();

  /**
   * @param {function(boolean): void} callback
   */
  addListener(callback(bool arg1));

  /**
   * @param {function(boolean): void} callback
   */
  removeListener(callback(bool arg1));

  /**
   * @param {function(boolean): void} callback
   * @return {boolean}
   */
  bool hasListener(callback(bool arg1));

  /**
   * @return {boolean}
   */
  bool hasListeners();
}

/**
 * Event whose listeners take a number parameter.
 * @constructor
 */
@JsName("ChromeNumberEvent")
abstract class ChromeNumberEvent {
// @Ignore
  external ChromeNumberEvent.fakeConstructor$();

  /**
   * @param {function(number): void} callback
   */
  addListener(callback(num arg1));

  /**
   * @param {function(number): void} callback
   */
  removeListener(callback(num arg1));

  /**
   * @param {function(number): void} callback
   * @return {boolean}
   */
  bool hasListener(callback(num arg1));

  /**
   * @return {boolean}
   */
  bool hasListeners();
}

/**
 * Event whose listeners take an Object parameter.
 * @constructor
 */
@JsName("ChromeObjectEvent")
abstract class ChromeObjectEvent {
// @Ignore
  external ChromeObjectEvent.fakeConstructor$();

  /**
   * @param {function(!Object): void} callback Callback.
   */
  addListener(callback(Object arg1));

  /**
   * @param {function(!Object): void} callback Callback.
   */
  removeListener(callback(Object arg1));

  /**
   * @param {function(!Object): void} callback Callback.
   * @return {boolean}
   */
  bool hasListener(callback(Object arg1));

  /**
   * @return {boolean}
   */
  bool hasListeners();
}

/**
 * Event whose listeners take a string array parameter.
 * @constructor
 */
@JsName("ChromeStringArrayEvent")
abstract class ChromeStringArrayEvent {
// @Ignore
  external ChromeStringArrayEvent.fakeConstructor$();

  /** @param {function(!Array.<string>): void} callback */
  addListener(callback(List<String> arg1));

  /** @param {function(!Array.<string>): void} callback */
  removeListener(callback(List<String> arg1));

  /**
   * @param {function(!Array.<string>): void} callback
   * @return {boolean}
   */
  bool hasListener(callback(List<String> arg1));

  /** @return {boolean} */
  bool hasListeners();
}

/**
 * Event whose listeners take two strings as parameters.
 * @constructor
 */
@JsName("ChromeStringStringEvent")
abstract class ChromeStringStringEvent {
// @Ignore
  external ChromeStringStringEvent.fakeConstructor$();

  /** @param {function(string, string): void} callback */
  addListener(callback(String arg1, String arg2));

  /** @param {function(string, string): void} callback */
  removeListener(callback(String arg1, String arg2));

  /**
   * @param {function(string, string): void} callback
   * @return {boolean}
   */
  bool hasListener(callback(String arg1, String arg2));

  /** @return {boolean} */
  bool hasListeners();
}

/**
 * @see http://developer.chrome.com/extensions/runtime.html#type-MessageSender
 * @constructor
 */
@JsName("MessageSender")
abstract class MessageSender {
// @Ignore
  external MessageSender.fakeConstructor$();

  /** @type {!Tab|undefined} */
  Tab tab;

  /** @type {number|undefined} */
  num frameId;

  /** @type {string|undefined} */
  String id;

  /** @type {string|undefined} */
  String url;

  /** @type {string|undefined} */
  String tlsChannelId;
}

/**
 * @see https://developer.chrome.com/extensions/tabs
 * @constructor
 */
@JsName("Tab")
abstract class Tab {
// @Ignore
  external Tab.fakeConstructor$();

  /** @type {number} */
  num id;

  /** @type {number} */
  num index;

  /** @type {number} */
  num windowId;

  /** @type {number} */
  num openerTabId;

  /** @type {boolean} */
  bool highlighted;

  /** @type {boolean} */
  bool active;

  /** @type {boolean} */
  bool pinned;

  /** @type {string} */
  String url;

  /** @type {string} */
  String title;

  /** @type {string} */
  String favIconUrl;

  /** @type {string} */
  String status;

  /** @type {boolean} */
  bool incognito;

  /** @type {number|undefined} */
  num width;

  /** @type {number|undefined} */
  num height;

  /** @type {string|undefined} */
  String sessionId;
}

/**
 * The data object given by chrome.loadTimes().
 * @constructor
 */
@JsName("ChromeLoadTimes")
abstract class ChromeLoadTimes {
// @Ignore
  external ChromeLoadTimes.fakeConstructor$();

  /** @type {number} */
  num requestTime;

  /** @type {number} */
  num startLoadTime;

  /** @type {number} */
  num commitLoadTime;

  /** @type {number} */
  num finishDocumentLoadTime;

  /** @type {number} */
  num finishLoadTime;

  /** @type {number} */
  num firstPaintTime;

  /** @type {number} */
  num firstPaintAfterLoadTime;

  /** @type {number} */
  num navigationType;

  /**
   * True iff the resource was fetched over SPDY.
   * @type {boolean}
   */
  bool wasFetchedViaSpdy;

  /** @type {boolean} */
  bool wasNpnNegotiated;

  /** @type {string} */
  String npnNegotiatedProtocol;

  /** @type {boolean} */
  bool wasAlternateProtocolAvailable;

  /** @type {string} */
  String connectionInfo;
}

/**
 * The data object given by chrome.csi().
 * @constructor
 */
@JsName("ChromeCsiInfo")
abstract class ChromeCsiInfo {
// @Ignore
  external ChromeCsiInfo.fakeConstructor$();

  /**
   * Same as chrome.loadTimes().requestTime, if defined.
   * Otherwise, gives the same value as chrome.loadTimes().startLoadTime.
   * In milliseconds, truncated.
   * @type {number}
   */
  num startE;

  /**
   * Same as chrome.loadTimes().finishDocumentLoadTime but in milliseconds and
   * truncated.
   * @type {number}
   */
  num onloadT;

  /**
   * The time since startE in milliseconds.
   * @type {number}
   */
  num pageT;

  /** @type {number} */
  num tran;
}
