/*
 * Copyright 2008 The Closure Compiler Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// This file has been automatically generated.  Please do not edit it manually.
/**
 * @fileoverview ECMAScript 3 Built-Ins. This include common extensions so this
 * is actually ES3+Reality.
 * FROM_EXTERN
 * @author stevey@google.com (Steve Yegge)
 * @author nicksantos@google.com (Nick Santos)
 * @author arv@google.com (Erik Arvidsson)
 * @author johnlenz@google.com (John Lenz)
 */
@JsName()
library es3;

import 'package:js/js.dart';
import 'package:func/func.dart';

/**
 * @const
 * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math
 */
@JsName()
abstract class MathObject {
  /**
   * @type {number}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math/E
   */
  num get E;

  /**
   * @type {number}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math/LN10
   */
  num get LN10;

  /**
   * @type {number}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math/LN2
   */
  num get LN2;

  /**
   * @type {number}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math/LOG10E
   */
  num get LOG10E;

  /**
   * @type {number}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math/LOG2E
   */
  num get LOG2E;

  /**
   * @type {number}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math/PI
   */
  num get PI;

  /**
   * @type {number}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math/SQRT1_2
   */
  num get SQRT1_2;

  /**
   * @type {number}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math/SQRT2
   */
  num get SQRT2;

  /**
   * @param {*} x
   * @return {number}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math/abs
   */
  num abs(x);

  /**
   * @param {*} x
   * @return {number}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math/acos
   */
  num acos(x);

  /**
   * @param {number} value
   * @return {number}
   * @nosideeffects
   */
  num acosh(num value);

  /**
   * @param {*} x
   * @return {number}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math/asin
   */
  num asin(x);

  /**
   * @param {number} value
   * @return {number}
   * @nosideeffects
   */
  num asinh(num value);

  /**
   * @param {*} x
   * @return {number}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math/atan
   */
  num atan(x);

  /**
   * @param {*} y
   * @param {*} x
   * @return {number}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math/atan2
   */
  num atan2(y, x);

  /**
   * @param {number} value
   * @return {number}
   * @nosideeffects
   */
  num atanh(num value);

  /**
   * @param {number} value
   * @return {number}
   * @nosideeffects
   */
  num cbrt(num value);

  /**
   * @param {*} x
   * @return {number}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math/ceil
   */
  num ceil(x);

  /**
   * @param {*} x
   * @return {number}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math/cos
   */
  num cos(x);

  /**
   * @param {number} value
   * @return {number}
   * @nosideeffects
   */
  num cosh(num value);

  /**
   * @param {*} x
   * @return {number}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math/exp
   */
  num exp(x);

  /**
   * @param {number} value
   * @return {number}
   * @nosideeffects
   */
  num expm1(num value);

  /**
   * @param {*} x
   * @return {number}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math/floor
   */
  num floor(x);

  /**
   * @param {number} value1
   * @param {number} var_args
   * @return {number}
   * @nosideeffects
   * @see http://people.mozilla.org/~jorendorff/es6-draft.html#sec-math.hypot
   */
  num hypot(num value1, num var_args);

  /**
   * @param {*} x
   * @return {number}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math/log
   */
  num log(x);

  /**
   * @param {number} value
   * @return {number}
   * @nosideeffects
   */
  num log10(num value);

  /**
   * @param {number} value
   * @return {number}
   * @nosideeffects
   */
  num log1p(num value);

  /**
   * @param {number} value
   * @return {number}
   * @nosideeffects
   */
  num log2(num value);

  /**
   * @param {...*} var_args
   * @return {number}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math/max
   */
  num max([var_args1, var_args2, var_args3, var_args4]);

  /**
   * @param {...*} var_args
   * @return {number}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math/min
   */
  num min([var_args1, var_args2, var_args3, var_args4]);

  /**
   * @param {*} x
   * @param {*} y
   * @return {number}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math/pow
   */
  num pow(x, y);

  /**
   * @return {number}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math/random
   */
  num random();

  /**
   * @param {*} x
   * @return {number}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math/round
   */
  num round(x);

  /**
   * @param {number} value
   * @return {number}
   * @nosideeffects
   */
  num sign(num value);

  /**
   * @param {*} x
   * @return {number}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math/sin
   */
  num sin(x);

  /**
   * @param {number} value
   * @return {number}
   * @nosideeffects
   */
  num sinh(num value);

  /**
   * @param {*} x
   * @return {number}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math/sqrt
   */
  num sqrt(x);

  /**
   * @param {*} x
   * @return {number}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math/tan
   */
  num tan(x);

  /**
   * @param {number} value
   * @return {number}
   * @nosideeffects
   */
  num tanh(num value);

  /**
   * @return {string}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math/toSource
   */
  String toSource();

  /**
   * @param {number} value
   * @return {number}
   * @nosideeffects
   */
  num trunc(num value);
}

/**
 * @constructor
 * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Functions_and_function_scope/arguments
 */
@JsName("Arguments")
abstract class Arguments {
// @Ignore
  external Arguments.fakeConstructor$();

  /**
   * @type {Function}
   * @see http://developer.mozilla.org/En/Core_JavaScript_1.5_Reference/Functions_and_function_scope/arguments/callee
   */
  Function callee;

  /**
   * Use the non-standard {@see Function.prototype.caller} property of a function
   * object instead.
   * @type {Function}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Functions/arguments/caller
   * @deprecated
   */
  Function caller;

  /**
   * @type {number}
   * @see http://developer.mozilla.org/En/Core_JavaScript_1.5_Reference/Functions_and_function_scope/arguments/length
   */
  num length;
}
/**
 * @type {!Arguments}
 * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Functions_and_function_scope/arguments
 */
@JsName("arguments")
external Arguments get arguments;
@JsName("arguments")
external set arguments(Arguments value);
/**
 * @type {number}
 * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Properties/Infinity
 * @const
 */
@JsName("Infinity")
external num get Infinity;
/**
 * @type {number}
 * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Properties/NaN
 * @const
 */
@JsName("NaN")
external num get NaN;
/**
 * @type {undefined}
 * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Properties/undefined
 * @const
 */
@JsName("undefined")
external dynamic get undefined;
/**
 * @param {string} uri
 * @return {string}
 * @nosideeffects
 * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Functions/decodeURI
 */
external String decodeURI(String uri);
/**
 * @param {string} uri
 * @return {string}
 * @nosideeffects
 * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Functions/decodeURIComponent
 */
external String decodeURIComponent(String uri);
/**
 * @param {string} uri
 * @return {string}
 * @nosideeffects
 * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Functions/encodeURI
 */
external String encodeURI(String uri);
/**
 * @param {string} uri
 * @return {string}
 * @nosideeffects
 * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Functions/encodeURIComponent
 */
external String encodeURIComponent(String uri);
/**
 * Should only be used in browsers where encode/decodeURIComponent
 * are not present, as the latter handle fancy Unicode characters.
 * @param {string} str
 * @return {string}
 * @nosideeffects
 * @see https://developer.mozilla.org/en/Core_JavaScript_1.5_Guide/Predefined_Functions/escape_and_unescape_Functions
 */
external String escape(String str);
/**
 * Should only be used in browsers where encode/decodeURIComponent
 * are not present, as the latter handle fancy Unicode characters.
 * @param {string} str
 * @return {string}
 * @nosideeffects
 * @see https://developer.mozilla.org/en/Core_JavaScript_1.5_Guide/Predefined_Functions/escape_and_unescape_Functions
 */
external String unescape(String str);
/**
 * @param {*} num
 * @return {boolean}
 * @nosideeffects
 * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Functions/isFinite
 */
external bool isFinite(num);
/**
 * @param {*} num
 * @return {boolean}
 * @nosideeffects
 * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Functions/isNaN
 */
external bool isNaN(num);
/**
 * @param {*} num
 * @return {number}
 * @nosideeffects
 * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Functions/parseFloat
 */
external num parseFloat(num);
/**
 * Parse an integer. Use of {@code parseInt} without {@code base} is strictly
 * banned in Google. If you really want to parse octal or hex based on the
 * leader, then pass {@code undefined} as the base.
 *
 * @param {*} num
 * @param {number|undefined} base
 * @return {number}
 * @nosideeffects
 * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Functions/parseInt
 */
external num parseInt(num, num base);
/**
 * @param {string} code
 * @return {*}
 * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Functions/eval
 */
external eval(String code);
// Skipping Object as it is mapped to a built in Dart type
// Skipping Function as it is mapped to a built in Dart type

/**
 * @constructor
 * @param {...*} var_args
 * @return {!Array.<?>}
 * @nosideeffects
 * @template T
 * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array
 */
@JsName("Array")
abstract class List<T> {
// @Ignore
  external List.fakeConstructor$();

  /**
   * @param {{length: number}|Array.<T>} arr
   * @param {?function(this:S, T, number, ?) : ?} callback
   * @param {S=} opt_context
   * @template T,S
   */
  @JsName(r"forEach")
  external static forEach$static(arr /*ArrRecord|List<dynamicT>|null*/,
      callback(dynamic /*T*/ arg1, num arg2, arg3),
      [dynamic /*S*/ opt_context]);

  /**
   * @param {{length: number}|Array.<T>} arr
   * @param {?function(this:S, T, number, !Array.<T>): R} callback
   * @param {S=} opt_context
   * @return {!Array.<R>}
   * @template T,S,R
   */
  @JsName(r"map")
  external static List<dynamic /*R*/ > map$static(
      arr /*ArrRecord|List<dynamicT>|null*/, dynamic /*R*/ callback(
          dynamic /*T*/ arg1, num arg2, List<dynamic /*T*/ > arg3),
      [dynamic /*S*/ opt_context]);

  /**
   * Mozilla 1.6+ only.
   * @param {{length: number}|Array.<T>} arr
   * @param {T} obj
   * @param {number=} opt_fromIndex
   * @return {number}
   * @template T
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/lastIndexOf
   */
  @JsName(r"lastIndexOf")
  external static num lastIndexOf$static(
      arr /*ArrRecord|List<dynamicT>|null*/, dynamic /*T*/ obj,
      [num opt_fromIndex]);

  /**
   * Mozilla 1.6+ only.
   * @param {{length: number}|Array.<T>} arr
   * @param {T} obj
   * @param {number=} opt_fromIndex
   * @return {number}
   * @template T
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/indexOf
   */
  @JsName(r"indexOf")
  external static num indexOf$static(
      arr /*ArrRecord|List<dynamicT>|null*/, dynamic /*T*/ obj,
      [num opt_fromIndex]);

  /**
   * @param {{length: number}|Array.<T>} arr
   * @param {?function(this:S, T, number, ?) : ?} callback
   * @param {S=} opt_context
   * @return {boolean}
   * @template T,S
   */
  @JsName(r"every")
  external static bool every$static(arr /*ArrRecord|List<dynamicT>|null*/,
      callback(dynamic /*T*/ arg1, num arg2, arg3),
      [dynamic /*S*/ opt_context]);

  /**
   * @param {{length: number}|Array.<T>} arr
   * @param {?function(this:S, T, number, ?) : ?} callback
   * @param {S=} opt_context
   * @return {boolean}
   * @template T,S
   */
  @JsName(r"some")
  external static bool some$static(arr /*ArrRecord|List<dynamicT>|null*/,
      callback(dynamic /*T*/ arg1, num arg2, arg3),
      [dynamic /*S*/ opt_context]);

  /**
   * Introduced in 1.8.5.
   * @param {*} arr
   * @return {boolean}
   * @see http://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/Array/isArray
   */
  external static bool isArray(arr);

  /**
   * @param {{length: number}|Array.<T>} arr
   * @param {?function(this:S, T, number, ?) : ?} callback
   * @param {S=} opt_context
   * @return {!Array.<T>}
   * @template T,S
   */
  @JsName(r"filter")
  external static List<dynamic /*T*/ > filter$static(
      arr /*ArrRecord|List<dynamicT>|null*/,
      callback(dynamic /*T*/ arg1, num arg2, arg3),
      [dynamic /*S*/ opt_context]);

  /**
   * @constructor
   * @param {...*} var_args
   * @return {!Array.<?>}
   * @nosideeffects
   * @template T
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array
   */
  external factory List([var_args1, var_args2, var_args3, var_args4]);

  /**
   * Returns a new array comprised of this array joined with other array(s)
   * and/or value(s).
   *
   * @param {...*} var_args
   * @return {!Array.<?>}
   * @this {*}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/concat
   */
  List<dynamic> concat([var_args1, var_args2, var_args3, var_args4]);

  /**
   * Joins all elements of an array into a string.
   *
   * @param {*=} opt_separator Specifies a string to separate each element of the
   *     array. The separator is converted to a string if necessary. If omitted,
   *     the array elements are separated with a comma.
   * @return {string}
   * @this {{length: number}|string}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/join
   */
  String join([opt_separator]);

  /**
   * Removes the last element from an array and returns that element.
   *
   * @return {T}
   * @this {{length: number}|Array.<T>}
   * @modifies {this}
   * @template T
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/pop
   */
  dynamic /*T*/ pop();

  /**
   * Mutates an array by appending the given elements and returning the new
   * length of the array.
   *
   * @param {...T} var_args
   * @return {number} The new length of the array.
   * @this {{length: number}|Array.<T>}
   * @template T
   * @modifies {this}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/push
   */
  num push([dynamic /*T*/ var_args1, dynamic /*T*/ var_args2,
      dynamic /*T*/ var_args3, dynamic /*T*/ var_args4]);

  /**
   * Transposes the elements of an array in place: the first array element becomes the
   * last and the last becomes the first.
   *
   * @this {{length: number}}
   * @modifies {this}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/reverse
   */
  reverse();

  /**
   * Removes the first element from an array and returns that element. This
   * method changes the length of the array.
   *
   * @this {{length: number}|Array.<T>}
   * @modifies {this}
   * @return {T}
   * @template T
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/shift
   */
  dynamic /*T*/ shift();

  /**
   * Extracts a section of an array and returns a new array.
   *
   * @param {*=} opt_begin Zero-based index at which to begin extraction.  A
   *     non-number type will be auto-cast by the browser to a number.
   * @param {*=} opt_end Zero-based index at which to end extraction.  slice
   *     extracts up to but not including end.
   * @return {!Array.<T>}
   * @this {{length: number}|Array.<T>|string}
   * @template T
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/slice
   */
  List<dynamic /*T*/ > slice([opt_begin, opt_end]);

  /**
   * Sorts the elements of an array in place.
   *
   * @param {function(T,T):number=} opt_compareFunction Specifies a function that
   *     defines the sort order.
   * @this {{length: number}|Array.<T>}
   * @template T
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/sort
   */
  sort([num opt_compareFunction(dynamic /*T*/ arg1, dynamic /*T*/ arg2)]);

  /**
   * Changes the content of an array, adding new elements while removing old
   * elements.
   *
   * @param {*=} opt_index Index at which to start changing the array. If negative,
   *     will begin that many elements from the end.  A non-number type will be
   *     auto-cast by the browser to a number.
   * @param {*=} opt_howMany An integer indicating the number of old array elements
   *     to remove.
   * @param {...T} var_args
   * @return {!Array.<T>}
   * @this {{length: number}|Array.<T>}
   * @modifies {this}
   * @template T
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/splice
   */
  List<dynamic /*T*/ > splice([opt_index, opt_howMany, dynamic /*T*/ var_args1,
      dynamic /*T*/ var_args2, dynamic /*T*/ var_args3,
      dynamic /*T*/ var_args4]);

  /**
   * @return {string}
   * @this {Object}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/toSource
   */
  external String toSource();
  /**
   * @this {Array.<?>}
   * @return {string}
   * @nosideeffects
   * @override
   */
  String toString();

  /**
   * Adds one or more elements to the beginning of an array and returns the new
   * length of the array.
   *
   * @param {...*} var_args
   * @return {number} The new length of the array
   * @this {{length: number}}
   * @modifies {this}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/unshift
   */
  num unshift([var_args1, var_args2, var_args3, var_args4]);

  /**
   * Apply a function simultaneously against two values of the array (from
   * left-to-right) as to reduce it to a single value.
   *
   * @param {?function(?, T, number, !Array.<T>) : R} callback
   * @param {*=} opt_initialValue
   * @return {R}
   * @this {{length: number}|Array.<T>|string}
   * @template T,R
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/reduce
   */
  dynamic /*R*/ reduce(dynamic /*R*/ callback(
          arg1, dynamic /*T*/ arg2, num arg3, List<dynamic /*T*/ > arg4),
      [opt_initialValue]);

  /**
   * Apply a function simultaneously against two values of the array (from
   * right-to-left) as to reduce it to a single value.
   *
   * @param {?function(?, T, number, !Array.<T>) : R} callback
   * @param {*=} opt_initialValue
   * @return {R}
   * @this {{length: number}|Array.<T>|string}
   * @template T,R
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/reduceRight
   */
  dynamic /*R*/ reduceRight(dynamic /*R*/ callback(
          arg1, dynamic /*T*/ arg2, num arg3, List<dynamic /*T*/ > arg4),
      [opt_initialValue]);

  /**
   * Available in ECMAScript 5, Mozilla 1.6+.
   * @param {?function(this:S, T, number, !Array.<T>): ?} callback
   * @param {S=} opt_thisobj
   * @return {boolean}
   * @this {{length: number}|Array.<T>|string}
   * @template T,S
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/every
   */
  bool every(callback(dynamic /*T*/ arg1, num arg2, List<dynamic /*T*/ > arg3),
      [dynamic /*S*/ opt_thisobj]);

  /**
   * Available in ECMAScript 5, Mozilla 1.6+.
   * @param {?function(this:S, T, number, !Array.<T>): ?} callback
   * @param {S=} opt_thisobj
   * @return {!Array.<T>}
   * @this {{length: number}|Array.<T>|string}
   * @template T,S
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/filter
   */
  List<dynamic /*T*/ > filter(
      callback(dynamic /*T*/ arg1, num arg2, List<dynamic /*T*/ > arg3),
      [dynamic /*S*/ opt_thisobj]);

  /**
   * Available in ECMAScript 5, Mozilla 1.6+.
   * @param {?function(this:S, T, number, !Array.<T>): ?} callback
   * @param {S=} opt_thisobj
   * @this {{length: number}|Array.<T>|string}
   * @template T,S
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/forEach
   */
  forEach(callback(dynamic /*T*/ arg1, num arg2, List<dynamic /*T*/ > arg3),
      [dynamic /*S*/ opt_thisobj]);

  /**
   * Available in ECMAScript 5, Mozilla 1.6+.
   * @param {T} obj
   * @param {number=} opt_fromIndex
   * @return {number}
   * @this {{length: number}|Array.<T>|string}
   * @nosideeffects
   * @template T
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/indexOf
   */
  num indexOf(dynamic /*T*/ obj, [num opt_fromIndex]);

  /**
   * Available in ECMAScript 5, Mozilla 1.6+.
   * @param {T} obj
   * @param {number=} opt_fromIndex
   * @return {number}
   * @this {{length: number}|Array.<T>|string}
   * @nosideeffects
   * @template T
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/lastIndexOf
   */
  num lastIndexOf(dynamic /*T*/ obj, [num opt_fromIndex]);

  /**
   * Available in ECMAScript 5, Mozilla 1.6+.
   * @param {?function(this:S, T, number, !Array.<T>): R} callback
   * @param {S=} opt_thisobj
   * @return {!Array.<R>}
   * @this {{length: number}|Array.<T>|string}
   * @template T,S,R
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/map
   */
  List<dynamic /*R*/ > map(dynamic /*R*/ callback(
          dynamic /*T*/ arg1, num arg2, List<dynamic /*T*/ > arg3),
      [dynamic /*S*/ opt_thisobj]);

  /**
   * Available in ECMAScript 5, Mozilla 1.6+.
   * @param {?function(this:S, T, number, !Array.<T>): ?} callback
   * @param {S=} opt_thisobj
   * @return {boolean}
   * @this {{length: number}|Array.<T>|string}
   * @template T,S
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/some
   */
  bool some(callback(dynamic /*T*/ arg1, num arg2, List<dynamic /*T*/ > arg3),
      [dynamic /*S*/ opt_thisobj]);

  /**
   * @type {number}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/index
   */
  num index;

  /**
   * @type {?string}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/input
   */
  String input;

  /**
   * @type {number}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Array/length
   */
  num length;
}
// Skipping bool as it is mapped to a built in Dart type
// Skipping num as it is mapped to a built in Dart type
/**
 * @const
 * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Math
 */
@JsName("Math")
external MathObject get Math;
// Skipping DateTime as it is mapped to a built in Dart type
// Skipping String as it is mapped to a built in Dart type

/**
 * @constructor
 * @param {*=} opt_pattern
 * @param {*=} opt_flags
 * @return {!RegExp}
 * @nosideeffects
 * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/RegExp
 */
@JsName("RegExp")
abstract class RegExp {
// @Ignore
  external RegExp.fakeConstructor$();

  /**
   * The substring of the input up to the characters most recently matched.
   * @type {string}
   * @see http://www.devguru.com/Technologies/Ecmascript/Quickref/regexp_leftContext.html
   */
  external static String get leftContext;

  external static set leftContext(String value);

  /**
   * @type {string}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/RegExp
   */
  external static String get $1;

  external static set $1(String value);

  /**
   * @type {string}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/RegExp
   */
  external static String get $3;

  external static set $3(String value);

  /**
   * @type {string}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/RegExp
   */
  external static String get $2;

  external static set $2(String value);

  /**
   * @type {string}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/RegExp
   */
  external static String get $4;

  external static set $4(String value);

  /**
   * The last matched characters.
   * @type {string}
   * @see http://www.devguru.com/Technologies/Ecmascript/Quickref/regexp_lastMatch.html
   */
  external static String get lastMatch;

  external static set lastMatch(String value);

  /**
   * @type {string}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/RegExp
   */
  external static String get $5;

  external static set $5(String value);

  /**
   * @type {string}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/RegExp
   */
  external static String get $6;

  external static set $6(String value);

  /**
   * The string against which the last regexp was matched.
   * @type {string}
   * @see http://www.devguru.com/Technologies/Ecmascript/Quickref/regexp_input.html
   */
  external static String get input;

  external static set input(String value);

  /**
   * @type {string}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/RegExp
   */
  external static String get $7;

  external static set $7(String value);

  /**
   * @type {string}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/RegExp
   */
  external static String get $8;

  external static set $8(String value);

  /**
   * @type {string}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/RegExp
   */
  external static String get $9;

  external static set $9(String value);

  /**
   * The substring of the input after the characters most recently matched.
   * @type {string}
   * @see http://www.devguru.com/Technologies/Ecmascript/Quickref/regexp_rightContext.html
   */
  external static String get rightContext;

  external static set rightContext(String value);

  /**
   * The last matched parenthesized substring, if any.
   * @type {string}
   * @see http://www.devguru.com/Technologies/Ecmascript/Quickref/regexp_lastParen.html
   */
  external static String get lastParen;

  external static set lastParen(String value);

  /**
   * @constructor
   * @param {*=} opt_pattern
   * @param {*=} opt_flags
   * @return {!RegExp}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/RegExp
   */
  external factory RegExp([opt_pattern, opt_flags]);

  /**
   * @param {*} pattern
   * @param {*=} opt_flags
   * @return {void}
   * @modifies {this}
   * @deprecated
   * @see http://msdn.microsoft.com/en-us/library/x9cswe0z(v=VS.85).aspx
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/RegExp/compile
   */
  void compile(pattern, [opt_flags]);

  /**
   * @param {*} str The string to search.
   * @return {Array.<string>} This should really return an Array with a few
   *     special properties, but we do not have a good way to model this in
   *     our type system. Also see String.prototype.match.
   * @see http://msdn.microsoft.com/en-us/library/z908hy33(VS.85).aspx
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/RegExp/exec
   */
  List<String> exec(str);

  /**
   * @param {*} str The string to search.
   * @return {boolean} Whether the string was matched.
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/RegExp/test
   */
  bool test(str);

  /**
   * @this {RegExp}
   * @return {string}
   * @nosideeffects
   * @override
   */
  String toString();

  /**
   * Whether to test the regular expression against all possible matches
   * in a string, or only against the first.
   * @type {boolean}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/RegExp/global
   */
  bool global;

  /**
   * Whether to ignore case while attempting a match in a string.
   * @type {boolean}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/RegExp/ignoreCase
   */
  bool ignoreCase;

  /**
   * The index at which to start the next match.
   * @type {number}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/RegExp/lastIndex
   */
  num lastIndex;

  /**
   * Whether or not to search in strings across multiple lines.
   * @type {boolean}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/RegExp/multiline
   */
  bool multiline;

  /**
   * The text of the pattern.
   * @type {string}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/RegExp/source
   */
  String source;
}

/**
 * @constructor
 * @param {*=} opt_message
 * @param {*=} opt_file
 * @param {*=} opt_line
 * @return {!Error}
 * @nosideeffects
 * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Error
 */
@JsName("Error")
abstract class Error {
// @Ignore
  external Error.fakeConstructor$();

  /**
   * Chrome/v8 specific, adds a stack trace to the error object. The optional
   * constructorOpt parameter allows you to pass in a function value. When
   * collecting the stack trace all frames above the topmost call to this
   * function, including that call, will be left out of the stack trace.
   * @param {Object} error The object to add the stack trace to.
   * @param {Function=} opt_constructor A function in the stack trace
   * @see http://code.google.com/p/v8/wiki/JavaScriptStackTraceApi
   */
  external static captureStackTrace(Object error, [Function opt_constructor]);

  /**
   * Chrome/v8 specific, altering the maximum depth of the stack trace
   * (10 by default).
   * @type {number}
   * @see http://code.google.com/p/v8/wiki/JavaScriptStackTraceApi
   */
  external static num get stackTraceLimit;

  external static set stackTraceLimit(num value);

  /**
   * @constructor
   * @param {*=} opt_message
   * @param {*=} opt_file
   * @param {*=} opt_line
   * @return {!Error}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Error
   */
  external factory Error([opt_message, opt_file, opt_line]);

  /**
   * IE-only.
   * @type {string}
   * @see http://msdn.microsoft.com/en-us/library/2w6a45b5.aspx
   */
  String description;

  /**
   * Mozilla-only.
   * @type {number}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Error/lineNumber
   */
  num lineNumber;

  /**
   * Mozilla-only
   * @type {string}
   * @see https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Error/fileName
   */
  String fileName;

  /**
   * @type {string}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Error/name
   */
  String name;

  /**
   * @type {string}
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/Error/message
   */
  String message;

  /**
   * Doesn't seem to exist, but closure/debug.js references it.
   */
  dynamic sourceURL;

  /** @type {string} */
  String stack;
}

/**
 * @constructor
 * @extends {Error}
 * @param {*=} opt_message
 * @param {*=} opt_file
 * @param {*=} opt_line
 * @return {!EvalError}
 * @nosideeffects
 * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/EvalError
 */
@JsName("EvalError")
abstract class EvalError extends Error {
// @Ignore
  external EvalError.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @constructor
   * @extends {Error}
   * @param {*=} opt_message
   * @param {*=} opt_file
   * @param {*=} opt_line
   * @return {!EvalError}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/EvalError
   */
  external factory EvalError([opt_message, opt_file, opt_line]);
}

/**
 * @constructor
 * @extends {Error}
 * @param {*=} opt_message
 * @param {*=} opt_file
 * @param {*=} opt_line
 * @return {!RangeError}
 * @nosideeffects
 * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/RangeError
 */
@JsName("RangeError")
abstract class RangeError extends Error {
// @Ignore
  external RangeError.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @constructor
   * @extends {Error}
   * @param {*=} opt_message
   * @param {*=} opt_file
   * @param {*=} opt_line
   * @return {!RangeError}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/RangeError
   */
  external factory RangeError([opt_message, opt_file, opt_line]);
}

/**
 * @constructor
 * @extends {Error}
 * @param {*=} opt_message
 * @param {*=} opt_file
 * @param {*=} opt_line
 * @return {!ReferenceError}
 * @nosideeffects
 * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/ReferenceError
 */
@JsName("ReferenceError")
abstract class ReferenceError extends Error {
// @Ignore
  external ReferenceError.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @constructor
   * @extends {Error}
   * @param {*=} opt_message
   * @param {*=} opt_file
   * @param {*=} opt_line
   * @return {!ReferenceError}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/ReferenceError
   */
  external factory ReferenceError([opt_message, opt_file, opt_line]);
}

/**
 * @constructor
 * @extends {Error}
 * @param {*=} opt_message
 * @param {*=} opt_file
 * @param {*=} opt_line
 * @return {!SyntaxError}
 * @nosideeffects
 * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/SyntaxError
 */
@JsName("SyntaxError")
abstract class SyntaxError extends Error {
// @Ignore
  external SyntaxError.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @constructor
   * @extends {Error}
   * @param {*=} opt_message
   * @param {*=} opt_file
   * @param {*=} opt_line
   * @return {!SyntaxError}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/SyntaxError
   */
  external factory SyntaxError([opt_message, opt_file, opt_line]);
}

/**
 * @constructor
 * @extends {Error}
 * @param {*=} opt_message
 * @param {*=} opt_file
 * @param {*=} opt_line
 * @return {!TypeError}
 * @nosideeffects
 * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/TypeError
 */
@JsName("TypeError")
abstract class TypeError extends Error {
// @Ignore
  external TypeError.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @constructor
   * @extends {Error}
   * @param {*=} opt_message
   * @param {*=} opt_file
   * @param {*=} opt_line
   * @return {!TypeError}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/TypeError
   */
  external factory TypeError([opt_message, opt_file, opt_line]);
}

/**
 * @constructor
 * @extends {Error}
 * @param {*=} opt_message
 * @param {*=} opt_file
 * @param {*=} opt_line
 * @return {!URIError}
 * @nosideeffects
 * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/URIError
 */
@JsName("URIError")
abstract class URIError extends Error {
// @Ignore
  external URIError.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @constructor
   * @extends {Error}
   * @param {*=} opt_message
   * @param {*=} opt_file
   * @param {*=} opt_line
   * @return {!URIError}
   * @nosideeffects
   * @see http://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/URIError
   */
  external factory URIError([opt_message, opt_file, opt_line]);
}
/**
 * @see http://msdn.microsoft.com/en-us/library/7sw4ddf8.aspx
 * @type {function(new:?, string, string=)}
 */
external ActiveXObject(String progId, [String opt_location]);
/**
 * @return {string}
 * @nosideeffects
 * @see http://msdn.microsoft.com/en-us/library/9k34bww2(VS.80).aspx
 */
external String ScriptEngine();
/**
 * @return {number}
 * @nosideeffects
 * @see http://msdn.microsoft.com/en-us/library/yf25ky07(VS.80).aspx
 */
external num ScriptEngineMajorVersion();
/**
 * @return {number}
 * @nosideeffects
 * @see http://msdn.microsoft.com/en-us/library/wx3812cz(VS.80).aspx
 */
external num ScriptEngineMinorVersion();
/**
 * @return {number}
 * @nosideeffects
 * @see http://msdn.microsoft.com/en-us/library/e98hsk2f(VS.80).aspx
 */
external num ScriptEngineBuildVersion();
/// This record class is not given a name in the Closure JavaScript code base.
/// The name we have chosen may not make sense.
@JsName()
abstract class ArrRecord {
  num length;
}
