/*
 * Copyright 2014 The Closure Compiler Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// This file has been automatically generated.  Please do not edit it manually.
/**
 * @fileoverview Definitions for ECMAScript 6.
 * @see http://wiki.ecmascript.org/doku.php?id=harmony:specification_drafts
 * @see https://www.khronos.org/registry/typedarray/specs/latest/
 */
@JsName()
library es6;

import 'package:js/js.dart';
import 'package:func/func.dart';

/** @typedef {?} */
// Dart does not support aliases for primitive types. Skipping: symbol
/**f
 * @param {string} description
 * @return {symbol}
 */
external Symbol(String description);

/**
 * @interface
 * @template VALUE
 */
@JsName("Iterable")
abstract class Iterable<VALUE> {
// @Ignore
  external Iterable.fakeConstructor$();
}

/**
 * @interface
 * @template VALUE
 * @see https://developer.mozilla.org/en-US/docs/Web/JavaScript/Guide/The_Iterator_protocol
 */
@JsName("Iterator")
abstract class Iterator<VALUE> {
// @Ignore
  external Iterator.fakeConstructor$();

  /**
   * @param {VALUE=} value
   * @return {{value:VALUE, done:boolean}}
   */
  external NextRecord next([VALUE value]);
}

@JsName("Generator")
abstract class Generator<VALUE> implements Iterator<VALUE> {
// @Ignore
  external Generator.fakeConstructor$();

  /**
   * @param {?=} opt_value
   * @return {{value:VALUE, done:boolean}}
   * @override
   */
  NextRecord next([opt_value]);

  /**
   * @param {VALUE} value
   * @return {{value:VALUE, done:boolean}}
   */
  NextRecord return$(VALUE value);

  /**
   * @param {?} exception
   * @return {{value:VALUE, done:boolean}}
   */
  NextRecord throw$(exception);
}

/**
 * @see http://dev.w3.org/html5/postmsg/
 * @interface
 */
@JsName("Transferable")
abstract class Transferable {
// @Ignore
  external Transferable.fakeConstructor$();
}

/**
 * @param {number} length The length in bytes
 * @constructor
 * @noalias
 * @throws {Error}
 * @nosideeffects6
 * @implements {Transferable}
 */
@JsName("ArrayBuffer")
abstract class ArrayBuffer implements Transferable {
// @Ignore
  external ArrayBuffer.fakeConstructor$();

  /**
   * @param {number} length The length in bytes
   * @constructor
   * @noalias
   * @throws {Error}
   * @nosideeffects6
   * @implements {Transferable}
   */
  external factory ArrayBuffer(num length);

  /** @type {number} */
  num byteLength;

  /**
   * @param {number} begin
   * @param {number=} opt_end
   * @return {!ArrayBuffer}
   * @nosideeffects
   */
  ArrayBuffer slice(num begin, [num opt_end]);
}

/**
 * @constructor
 * @noalias
 */
@JsName("ArrayBufferView")
abstract class ArrayBufferView {
// @Ignore
  external ArrayBufferView.fakeConstructor$();

  /** @type {!ArrayBuffer} */
  ArrayBuffer buffer;

  /** @type {number} */
  num byteOffset;

  /** @type {number} */
  num byteLength;
}
/**
 * @typedef {!ArrayBuffer|!ArrayBufferView}
 */
// Dart does not support union types. Skipping: BufferSource

/**
 * @param {number|ArrayBufferView|Array.<number>|ArrayBuffer} length or array
 *     or buffer
 * @param {number=} opt_byteOffset
 * @param {number=} opt_length
 * @extends {ArrayBufferView}
 * @constructor
 * @noalias
 * @throws {Error}
 * @modifies {arguments} If the user passes a backing array, then indexed
 *     accesses will modify the backing array. JSCompiler does not model
 *     this well. In other words, if you have:
 *     <code>
 *     var x = new ArrayBuffer(1);
 *     var y = new Int8Array(x);
 *     y[0] = 2;
 *     </code>
 *     JSCompiler will not recognize that the last assignment modifies x.
 *     We workaround this by marking all these arrays as @modifies {arguments},
 *     to introduce the possibility that x aliases y.
 */
@JsName("Int8Array")
abstract class Int8Array extends ArrayBufferView {
// @Ignore
  external Int8Array.fakeConstructor$() : super.fakeConstructor$();

  /** @type {number} */
  @JsName(r"BYTES_PER_ELEMENT")
  external static num get BYTES_PER_ELEMENT$static;

  @JsName(r"BYTES_PER_ELEMENT")
  external static set BYTES_PER_ELEMENT$static(num value);

  /**
   * @param {number|ArrayBufferView|Array.<number>|ArrayBuffer} length or array
   *     or buffer
   * @param {number=} opt_byteOffset
   * @param {number=} opt_length
   * @extends {ArrayBufferView}
   * @constructor
   * @noalias
   * @throws {Error}
   * @modifies {arguments} If the user passes a backing array, then indexed
   *     accesses will modify the backing array. JSCompiler does not model
   *     this well. In other words, if you have:
   *     <code>
   *     var x = new ArrayBuffer(1);
   *     var y = new Int8Array(x);
   *     y[0] = 2;
   *     </code>
   *     JSCompiler will not recognize that the last assignment modifies x.
   *     We workaround this by marking all these arrays as @modifies {arguments},
   *     to introduce the possibility that x aliases y.
   */
  external factory Int8Array(
      length /*num|ArrayBufferView|null|List<num>|ArrayBuffer*/,
      [num opt_byteOffset, num opt_length]);

  /** @type {number} */
  num BYTES_PER_ELEMENT;

  /** @type {number} */
  num length;

  /**
   * @param {ArrayBufferView|Array.<number>} array
   * @param {number=} opt_offset
   */
  set(array /*ArrayBufferView|null|List<num>*/, [num opt_offset]);

  /**
   * @param {number} begin
   * @param {number=} opt_end
   * @return {!Int8Array}
   * @nosideeffects
   */
  Int8Array subarray(num begin, [num opt_end]);
}

/**
 * @param {number|ArrayBufferView|Array.<number>|ArrayBuffer} length or array
 *     or buffer
 * @param {number=} opt_byteOffset
 * @param {number=} opt_length
 * @extends {ArrayBufferView}
 * @constructor
 * @noalias
 * @throws {Error}
 * @modifies {arguments}
 */
@JsName("Uint8Array")
abstract class Uint8Array extends ArrayBufferView {
// @Ignore
  external Uint8Array.fakeConstructor$() : super.fakeConstructor$();

  /** @type {number} */
  @JsName(r"BYTES_PER_ELEMENT")
  external static num get BYTES_PER_ELEMENT$static;

  @JsName(r"BYTES_PER_ELEMENT")
  external static set BYTES_PER_ELEMENT$static(num value);

  /**
   * @param {number|ArrayBufferView|Array.<number>|ArrayBuffer} length or array
   *     or buffer
   * @param {number=} opt_byteOffset
   * @param {number=} opt_length
   * @extends {ArrayBufferView}
   * @constructor
   * @noalias
   * @throws {Error}
   * @modifies {arguments}
   */
  external factory Uint8Array(
      length /*num|ArrayBufferView|null|List<num>|ArrayBuffer*/,
      [num opt_byteOffset, num opt_length]);

  /** @type {number} */
  num BYTES_PER_ELEMENT;

  /** @type {number} */
  num length;

  /**
   * @param {ArrayBufferView|Array.<number>} array
   * @param {number=} opt_offset
   */
  set(array /*ArrayBufferView|null|List<num>*/, [num opt_offset]);

  /**
   * @param {number} begin
   * @param {number=} opt_end
   * @return {!Uint8Array}
   * @nosideeffects
   */
  Uint8Array subarray(num begin, [num opt_end]);
}

/**
 * @param {number|ArrayBufferView|Array.<number>|ArrayBuffer} length or array
 *     or buffer
 * @param {number=} opt_byteOffset
 * @param {number=} opt_length
 * @extends {ArrayBufferView}
 * @constructor
 * @noalias
 * @throws {Error}
 * @modifies {arguments}
 */
@JsName("Uint8ClampedArray")
abstract class Uint8ClampedArray extends ArrayBufferView {
// @Ignore
  external Uint8ClampedArray.fakeConstructor$() : super.fakeConstructor$();

  /** @type {number} */
  @JsName(r"BYTES_PER_ELEMENT")
  external static num get BYTES_PER_ELEMENT$static;

  @JsName(r"BYTES_PER_ELEMENT")
  external static set BYTES_PER_ELEMENT$static(num value);

  /**
   * @param {number|ArrayBufferView|Array.<number>|ArrayBuffer} length or array
   *     or buffer
   * @param {number=} opt_byteOffset
   * @param {number=} opt_length
   * @extends {ArrayBufferView}
   * @constructor
   * @noalias
   * @throws {Error}
   * @modifies {arguments}
   */
  external factory Uint8ClampedArray(
      length /*num|ArrayBufferView|null|List<num>|ArrayBuffer*/,
      [num opt_byteOffset, num opt_length]);

  /** @type {number} */
  num BYTES_PER_ELEMENT;

  /** @type {number} */
  num length;

  /**
   * @param {ArrayBufferView|Array.<number>} array
   * @param {number=} opt_offset
   */
  set(array /*ArrayBufferView|null|List<num>*/, [num opt_offset]);

  /**
   * @param {number} begin
   * @param {number=} opt_end
   * @return {!Uint8ClampedArray}
   * @nosideeffects
   */
  Uint8ClampedArray subarray(num begin, [num opt_end]);
}
/**
 * @typedef {Uint8ClampedArray}
 * @deprecated CanvasPixelArray has been replaced by Uint8ClampedArray
 *     in the latest spec.
 * @see http://www.w3.org/TR/2dcontext/#imagedata
 */
// Dart does not support union types. Skipping: CanvasPixelArray

/**
 * @param {number|ArrayBufferView|Array.<number>|ArrayBuffer} length or array
 *     or buffer
 * @param {number=} opt_byteOffset
 * @param {number=} opt_length
 * @extends {ArrayBufferView}
 * @constructor
 * @noalias
 * @throws {Error}
 * @modifies {arguments}
 */
@JsName("Int16Array")
abstract class Int16Array extends ArrayBufferView {
// @Ignore
  external Int16Array.fakeConstructor$() : super.fakeConstructor$();

  /** @type {number} */
  @JsName(r"BYTES_PER_ELEMENT")
  external static num get BYTES_PER_ELEMENT$static;

  @JsName(r"BYTES_PER_ELEMENT")
  external static set BYTES_PER_ELEMENT$static(num value);

  /**
   * @param {number|ArrayBufferView|Array.<number>|ArrayBuffer} length or array
   *     or buffer
   * @param {number=} opt_byteOffset
   * @param {number=} opt_length
   * @extends {ArrayBufferView}
   * @constructor
   * @noalias
   * @throws {Error}
   * @modifies {arguments}
   */
  external factory Int16Array(
      length /*num|ArrayBufferView|null|List<num>|ArrayBuffer*/,
      [num opt_byteOffset, num opt_length]);

  /** @type {number} */
  num BYTES_PER_ELEMENT;

  /** @type {number} */
  num length;

  /**
   * @param {ArrayBufferView|Array.<number>} array
   * @param {number=} opt_offset
   */
  set(array /*ArrayBufferView|null|List<num>*/, [num opt_offset]);

  /**
   * @param {number} begin
   * @param {number=} opt_end
   * @return {!Int16Array}
   * @nosideeffects
   */
  Int16Array subarray(num begin, [num opt_end]);
}

/**
 * @param {number|ArrayBufferView|Array.<number>|ArrayBuffer} length or array
 *     or buffer
 * @param {number=} opt_byteOffset
 * @param {number=} opt_length
 * @extends {ArrayBufferView}
 * @constructor
 * @noalias
 * @throws {Error}
 * @modifies {arguments}
 */
@JsName("Uint16Array")
abstract class Uint16Array extends ArrayBufferView {
// @Ignore
  external Uint16Array.fakeConstructor$() : super.fakeConstructor$();

  /** @type {number} */
  @JsName(r"BYTES_PER_ELEMENT")
  external static num get BYTES_PER_ELEMENT$static;

  @JsName(r"BYTES_PER_ELEMENT")
  external static set BYTES_PER_ELEMENT$static(num value);

  /**
   * @param {number|ArrayBufferView|Array.<number>|ArrayBuffer} length or array
   *     or buffer
   * @param {number=} opt_byteOffset
   * @param {number=} opt_length
   * @extends {ArrayBufferView}
   * @constructor
   * @noalias
   * @throws {Error}
   * @modifies {arguments}
   */
  external factory Uint16Array(
      length /*num|ArrayBufferView|null|List<num>|ArrayBuffer*/,
      [num opt_byteOffset, num opt_length]);

  /** @type {number} */
  num BYTES_PER_ELEMENT;

  /** @type {number} */
  num length;

  /**
   * @param {ArrayBufferView|Array.<number>} array
   * @param {number=} opt_offset
   */
  set(array /*ArrayBufferView|null|List<num>*/, [num opt_offset]);

  /**
   * @param {number} begin
   * @param {number=} opt_end
   * @return {!Uint16Array}
   * @nosideeffects
   */
  Uint16Array subarray(num begin, [num opt_end]);
}

/**
 * @param {number|ArrayBufferView|Array.<number>|ArrayBuffer} length or array
 *     or buffer
 * @param {number=} opt_byteOffset
 * @param {number=} opt_length
 * @extends {ArrayBufferView}
 * @constructor
 * @noalias
 * @throws {Error}
 * @modifies {arguments}
 */
@JsName("Int32Array")
abstract class Int32Array extends ArrayBufferView {
// @Ignore
  external Int32Array.fakeConstructor$() : super.fakeConstructor$();

  /** @type {number} */
  @JsName(r"BYTES_PER_ELEMENT")
  external static num get BYTES_PER_ELEMENT$static;

  @JsName(r"BYTES_PER_ELEMENT")
  external static set BYTES_PER_ELEMENT$static(num value);

  /**
   * @param {number|ArrayBufferView|Array.<number>|ArrayBuffer} length or array
   *     or buffer
   * @param {number=} opt_byteOffset
   * @param {number=} opt_length
   * @extends {ArrayBufferView}
   * @constructor
   * @noalias
   * @throws {Error}
   * @modifies {arguments}
   */
  external factory Int32Array(
      length /*num|ArrayBufferView|null|List<num>|ArrayBuffer*/,
      [num opt_byteOffset, num opt_length]);

  /** @type {number} */
  num BYTES_PER_ELEMENT;

  /** @type {number} */
  num length;

  /**
   * @param {ArrayBufferView|Array.<number>} array
   * @param {number=} opt_offset
   */
  set(array /*ArrayBufferView|null|List<num>*/, [num opt_offset]);

  /**
   * @param {number} begin
   * @param {number=} opt_end
   * @return {!Int32Array}
   * @nosideeffects
   */
  Int32Array subarray(num begin, [num opt_end]);
}

/**
 * @param {number|ArrayBufferView|Array.<number>|ArrayBuffer} length or array
 *     or buffer
 * @param {number=} opt_byteOffset
 * @param {number=} opt_length
 * @extends {ArrayBufferView}
 * @constructor
 * @noalias
 * @throws {Error}
 * @modifies {arguments}
 */
@JsName("Uint32Array")
abstract class Uint32Array extends ArrayBufferView {
// @Ignore
  external Uint32Array.fakeConstructor$() : super.fakeConstructor$();

  /** @type {number} */
  @JsName(r"BYTES_PER_ELEMENT")
  external static num get BYTES_PER_ELEMENT$static;

  @JsName(r"BYTES_PER_ELEMENT")
  external static set BYTES_PER_ELEMENT$static(num value);

  /**
   * @param {number|ArrayBufferView|Array.<number>|ArrayBuffer} length or array
   *     or buffer
   * @param {number=} opt_byteOffset
   * @param {number=} opt_length
   * @extends {ArrayBufferView}
   * @constructor
   * @noalias
   * @throws {Error}
   * @modifies {arguments}
   */
  external factory Uint32Array(
      length /*num|ArrayBufferView|null|List<num>|ArrayBuffer*/,
      [num opt_byteOffset, num opt_length]);

  /** @type {number} */
  num BYTES_PER_ELEMENT;

  /** @type {number} */
  num length;

  /**
   * @param {ArrayBufferView|Array.<number>} array
   * @param {number=} opt_offset
   */
  set(array /*ArrayBufferView|null|List<num>*/, [num opt_offset]);

  /**
   * @param {number} begin
   * @param {number=} opt_end
   * @return {!Uint32Array}
   * @nosideeffects
   */
  Uint32Array subarray(num begin, [num opt_end]);
}

/**
 * @param {number|ArrayBufferView|Array.<number>|ArrayBuffer} length or array
 *     or buffer
 * @param {number=} opt_byteOffset
 * @param {number=} opt_length
 * @extends {ArrayBufferView}
 * @constructor
 * @noalias
 * @throws {Error}
 * @modifies {arguments}
 */
@JsName("Float32Array")
abstract class Float32Array extends ArrayBufferView {
// @Ignore
  external Float32Array.fakeConstructor$() : super.fakeConstructor$();

  /** @type {number} */
  @JsName(r"BYTES_PER_ELEMENT")
  external static num get BYTES_PER_ELEMENT$static;

  @JsName(r"BYTES_PER_ELEMENT")
  external static set BYTES_PER_ELEMENT$static(num value);

  /**
   * @param {number|ArrayBufferView|Array.<number>|ArrayBuffer} length or array
   *     or buffer
   * @param {number=} opt_byteOffset
   * @param {number=} opt_length
   * @extends {ArrayBufferView}
   * @constructor
   * @noalias
   * @throws {Error}
   * @modifies {arguments}
   */
  external factory Float32Array(
      length /*num|ArrayBufferView|null|List<num>|ArrayBuffer*/,
      [num opt_byteOffset, num opt_length]);

  /** @type {number} */
  num BYTES_PER_ELEMENT;

  /** @type {number} */
  num length;

  /**
   * @param {ArrayBufferView|Array.<number>} array
   * @param {number=} opt_offset
   */
  set(array /*ArrayBufferView|null|List<num>*/, [num opt_offset]);

  /**
   * @param {number} begin
   * @param {number=} opt_end
   * @return {!Float32Array}
   * @nosideeffects
   */
  Float32Array subarray(num begin, [num opt_end]);
}

/**
 * @param {number|ArrayBufferView|Array.<number>|ArrayBuffer} length or array
 *     or buffer
 * @param {number=} opt_byteOffset
 * @param {number=} opt_length
 * @extends {ArrayBufferView}
 * @constructor
 * @noalias
 * @throws {Error}
 * @modifies {arguments}
 */
@JsName("Float64Array")
abstract class Float64Array extends ArrayBufferView {
// @Ignore
  external Float64Array.fakeConstructor$() : super.fakeConstructor$();

  /** @type {number} */
  @JsName(r"BYTES_PER_ELEMENT")
  external static num get BYTES_PER_ELEMENT$static;

  @JsName(r"BYTES_PER_ELEMENT")
  external static set BYTES_PER_ELEMENT$static(num value);

  /**
   * @param {number|ArrayBufferView|Array.<number>|ArrayBuffer} length or array
   *     or buffer
   * @param {number=} opt_byteOffset
   * @param {number=} opt_length
   * @extends {ArrayBufferView}
   * @constructor
   * @noalias
   * @throws {Error}
   * @modifies {arguments}
   */
  external factory Float64Array(
      length /*num|ArrayBufferView|null|List<num>|ArrayBuffer*/,
      [num opt_byteOffset, num opt_length]);

  /** @type {number} */
  num BYTES_PER_ELEMENT;

  /** @type {number} */
  num length;

  /**
   * @param {ArrayBufferView|Array.<number>} array
   * @param {number=} opt_offset
   */
  set(array /*ArrayBufferView|null|List<num>*/, [num opt_offset]);

  /**
   * @param {number} begin
   * @param {number=} opt_end
   * @return {!Float64Array}
   * @nosideeffects
   */
  Float64Array subarray(num begin, [num opt_end]);
}

/**
 * @param {ArrayBuffer} buffer
 * @param {number=} opt_byteOffset
 * @param {number=} opt_byteLength
 * @extends {ArrayBufferView}
 * @constructor
 * @noalias
 * @throws {Error}
 * @nosideeffects
 * @see https://developer.mozilla.org/en-US/docs/Web/JavaScript/Typed_arrays/DataView
 */
@JsName("DataView")
abstract class DataView extends ArrayBufferView {
// @Ignore
  external DataView.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @param {ArrayBuffer} buffer
   * @param {number=} opt_byteOffset
   * @param {number=} opt_byteLength
   * @extends {ArrayBufferView}
   * @constructor
   * @noalias
   * @throws {Error}
   * @nosideeffects
   * @see https://developer.mozilla.org/en-US/docs/Web/JavaScript/Typed_arrays/DataView
   */
  external factory DataView(ArrayBuffer buffer,
      [num opt_byteOffset, num opt_byteLength]);

  /**
   * @param {number} byteOffset
   * @return {number}
   * @throws {Error}
   * @nosideeffects
   */
  num getInt8(num byteOffset);

  /**
   * @param {number} byteOffset
   * @return {number}
   * @throws {Error}
   * @nosideeffects
   */
  num getUint8(num byteOffset);

  /**
   * @param {number} byteOffset
   * @param {boolean=} opt_littleEndian
   * @return {number}
   * @throws {Error}
   * @nosideeffects
   */
  num getInt16(num byteOffset, [bool opt_littleEndian]);

  /**
   * @param {number} byteOffset
   * @param {boolean=} opt_littleEndian
   * @return {number}
   * @throws {Error}
   * @nosideeffects
   */
  num getUint16(num byteOffset, [bool opt_littleEndian]);

  /**
   * @param {number} byteOffset
   * @param {boolean=} opt_littleEndian
   * @return {number}
   * @throws {Error}
   * @nosideeffects
   */
  num getInt32(num byteOffset, [bool opt_littleEndian]);

  /**
   * @param {number} byteOffset
   * @param {boolean=} opt_littleEndian
   * @return {number}
   * @throws {Error}
   * @nosideeffects
   */
  num getUint32(num byteOffset, [bool opt_littleEndian]);

  /**
   * @param {number} byteOffset
   * @param {boolean=} opt_littleEndian
   * @return {number}
   * @throws {Error}
   * @nosideeffects
   */
  num getFloat32(num byteOffset, [bool opt_littleEndian]);

  /**
   * @param {number} byteOffset
   * @param {boolean=} opt_littleEndian
   * @return {number}
   * @throws {Error}
   * @nosideeffects
   */
  num getFloat64(num byteOffset, [bool opt_littleEndian]);

  /**
   * @param {number} byteOffset
   * @param {number} value
   * @throws {Error}
   */
  setInt8(num byteOffset, num value);

  /**
   * @param {number} byteOffset
   * @param {number} value
   * @throws {Error}
   */
  setUint8(num byteOffset, num value);

  /**
   * @param {number} byteOffset
   * @param {number} value
   * @param {boolean=} opt_littleEndian
   * @throws {Error}
   */
  setInt16(num byteOffset, num value, [bool opt_littleEndian]);

  /**
   * @param {number} byteOffset
   * @param {number} value
   * @param {boolean=} opt_littleEndian
   * @throws {Error}
   */
  setUint16(num byteOffset, num value, [bool opt_littleEndian]);

  /**
   * @param {number} byteOffset
   * @param {number} value
   * @param {boolean=} opt_littleEndian
   * @throws {Error}
   */
  setInt32(num byteOffset, num value, [bool opt_littleEndian]);

  /**
   * @param {number} byteOffset
   * @param {number} value
   * @param {boolean=} opt_littleEndian
   * @throws {Error}
   */
  setUint32(num byteOffset, num value, [bool opt_littleEndian]);

  /**
   * @param {number} byteOffset
   * @param {number} value
   * @param {boolean=} opt_littleEndian
   * @throws {Error}
   */
  setFloat32(num byteOffset, num value, [bool opt_littleEndian]);

  /**
   * @param {number} byteOffset
   * @param {number} value
   * @param {boolean=} opt_littleEndian
   * @throws {Error}
   */
  setFloat64(num byteOffset, num value, [bool opt_littleEndian]);
}
/**
 * @see https://github.com/promises-aplus/promises-spec
 * @typedef {{then: ?}}
 */
@JsName()
abstract class Thenable {
  dynamic then;
}

@JsName("IThenable")
abstract class IThenable<TYPE> {
// @Ignore
  external IThenable.fakeConstructor$();

  /**
   * @param {?(function(TYPE):VALUE)=} opt_onFulfilled
   * @param {?(function(*): *)=} opt_onRejected
   * @return {RESULT}
   * @template VALUE
   *
   * When a Promise (or thenable) is returned from the fulfilled callback,
   * the result is the payload of that promise, not the promise itself.
   *
   * @template RESULT := type('IThenable',
   *     cond(isUnknown(VALUE), unknown(),
   *       mapunion(VALUE, (V) =>
   *         cond(isTemplatized(V) && sub(rawTypeOf(V), 'IThenable'),
   *           templateTypeOf(V, 0),
   *           cond(sub(V, 'Thenable'),
   *              unknown(),
   *              V)))))
   * =:
   */
  dynamic /*RESULT*/ then(
      [dynamic /*VALUE*/ opt_onFulfilled(TYPE arg1), opt_onRejected(arg1)]);
}

@JsName("Promise")
abstract class Promise<TYPE> implements IThenable<TYPE> {
// @Ignore
  external Promise.fakeConstructor$();

  /**
   * @param {(TYPE|IThenable.<TYPE>)=} opt_value
   * @return {!Promise.<TYPE>}
   * @template TYPE
   */
  external static Promise<dynamic /*TYPE*/ > resolve(
      [opt_value /*dynamicTYPE|IThenable<dynamicTYPE>|null|undefined*/]);

  /**
   * @param {*=} opt_error
   * @return {!Promise.<?>}
   */
  external static Promise<dynamic> reject([opt_error]);

  /**
   * @template T
   * @see https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Promise
   * @param {!Array.<T>} iterable
   * @return {!Promise.<T>}
   */
  external static Promise<dynamic /*T*/ > race(List<dynamic /*T*/ > iterable);

  /**
   * @template T
   * @see https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Promise
   * @param {!Array<T|!Promise<T>>} iterable
   * @return {!Promise<!Array<T>>}
   */
  external static Promise<List<dynamic /*T*/ >> all(
      List<dynamic /*dynamicT|Promise<dynamicT>*/ > iterable);

  /**
   * @see https://people.mozilla.org/~jorendorff/es6-draft.html#sec-promise-objects
   * @param {function(
   *             function((TYPE|IThenable.<TYPE>|Thenable|null)=),
   *             function(*=))} resolver
   * @constructor
   * @implements {IThenable.<TYPE>}
   * @template TYPE
   */
  external factory Promise(resolver(arg1(
          [arg1 /*dynamicTYPE|IThenable<dynamicTYPE>|null|Thenable|undefined*/]),
      arg2([arg1])));

  /**
   * @param {?(function(TYPE):VALUE)=} opt_onFulfilled
   * @param {?(function(*): *)=} opt_onRejected
   * @return {RESULT}
   * @template VALUE
   *
   * When a Promise (or thenable) is returned from the fulfilled callback,
   * the result is the payload of that promise, not the promise itself.
   *
   * @template RESULT := type('Promise',
   *     cond(isUnknown(VALUE), unknown(),
   *       mapunion(VALUE, (V) =>
   *         cond(isTemplatized(V) && sub(rawTypeOf(V), 'IThenable'),
   *           templateTypeOf(V, 0),
   *           cond(sub(V, 'Thenable'),
   *              unknown(),
   *              V)))))
   * =:
   * @override
   */
  dynamic /*RESULT*/ then(
      [dynamic /*VALUE*/ opt_onFulfilled(TYPE arg1), opt_onRejected(arg1)]);

  /**
   * @param {function(*): RESULT} onRejected
   * @return {!Promise.<RESULT>}
   * @template RESULT
   */
  Promise<dynamic /*RESULT*/ > catch$(dynamic /*RESULT*/ onRejected(arg1));
}
/// This record class is not given a name in the Closure JavaScript code base.
/// The name we have chosen may not make sense.
@JsName()
abstract class NextRecord<VALUE> {
  bool done;
  VALUE value;
}
