/*
 * Copyright 2014 The Closure Compiler Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
// This file has been automatically generated.  Please do not edit it manually.
/**
 * @fileoverview Definitions of the fetch api.
 *
 * This api is still in development and not yet stable. Use at your
 * own risk.
 *
 * @see https://fetch.spec.whatwg.org/
 * FROM_EXTERN
 */
@JsName()
library fetchapi;

import 'package:js/js.dart';
import 'package:func/func.dart';

import "es6.dart" as es6;

import "fileapi.dart" as fileapi;

import "w3c_xml.dart" as w3c_xml;

/**
 * @typedef {!Headers|!Array.<!Array.<string>>}
 */
// Dart does not support union types. Skipping: HeadersInit

/**
 * @see https://fetch.spec.whatwg.org/#headers
 * @param {HeadersInit=} opt_headersInit
 * @constructor
 */
@JsName("Headers")
abstract class Headers {
// @Ignore
  external Headers.fakeConstructor$();

  /**
   * @see https://fetch.spec.whatwg.org/#headers
   * @param {HeadersInit=} opt_headersInit
   * @constructor
   */
  external factory Headers(
      [opt_headersInit /*Headers|List<List<String>>|undefined*/]);

  /**
   * @param {string} name
   * @param {string} value
   */
  append(String name, String value);

  /**
   * @param {string} name
   */
  delete(String name);

  /**
   * @param {string} name
   * @return {?string}
   */
  String get(String name);

  /**
   * @param {string} name
   * @return {!Array.<string>}
   */
  List<String> getAll(String name);

  /**
   * @param {string} name
   * @return {boolean}
   */
  bool has(String name);

  /**
   * @param {string} name
   * @param {string} value
   */
  set(String name, String value);
}
/**
 * @typedef {!Blob|!FormData|string}
 */
// Dart does not support union types. Skipping: BodyInit

/**
 * @see https://fetch.spec.whatwg.org/#request
 * @param {!RequestInfo} input
 * @param {RequestInit=} opt_init
 * @constructor
 */
@JsName("Request")
abstract class Request {
// @Ignore
  external Request.fakeConstructor$();

  /**
   * @see https://fetch.spec.whatwg.org/#request
   * @param {!RequestInfo} input
   * @param {RequestInit=} opt_init
   * @constructor
   */
  external factory Request(input /*Request|String*/, [RequestInit opt_init]);

  /** @type {boolean} */
  bool bodyUsed;

  /** @return {!Promise.<!ArrayBuffer>} */
  es6.Promise<es6.ArrayBuffer> arrayBuffer();

  /** @return {!Promise.<!Blob>} */
  es6.Promise<fileapi.Blob> blob();

  /** @return {!Promise.<!FormData>} */
  es6.Promise<w3c_xml.FormData> formData();

  /** @return {!Promise.<!Object>} */
  es6.Promise<Object> json();

  /** @return {!Promise.<string>} */
  es6.Promise<String> text();

  /** @type {string} */
  String method;

  /** @type {string} */
  String url;

  /** @type {!Headers} */
  Headers headers;

  /** @type {RequestContext} */
  String context;

  /** @type {string} */
  String referrer;

  /** @type {RequestMode} */
  String mode;

  /** @type {RequestCredentials} */
  String credentials;

  /** @type {RequestCache} */
  String cache;

  /** @return {!Request} */
  Request clone();
}
/** @typedef {!Request|string} */
// Dart does not support union types. Skipping: RequestInfo
/**
 * @typedef {{
 *   method: (string|undefined),
 *   headers: (!HeadersInit|undefined),
 *   body: (!BodyInit|undefined),
 *   mode: (RequestMode|undefined),
 *   credentials: (RequestCredentials|undefined),
 *   cache: (RequestCache|undefined)
 * }}
 */
@JsName()
abstract class RequestInit {
  dynamic /*fileapi.Blob|w3c_xml.FormData|String|undefined*/ body;
  String cache;
  String credentials;
  dynamic /*Headers|List<List<String>>|undefined*/ headers;
  String method;
  String mode;
}
/**
 * @enum {string}
 */
@JsName("RequestContext")
abstract class RequestContext {
  external static String get TRACK;
  external static String get INTERNAL;
  external static String get SCRIPT;
  external static String get AUDIO;
  external static String get BEACON;
  external static String get XMLHTTPREQUEST;
  external static String get EMBED;
  external static String get VIDEO;
  external static String get HYPERLINK;
  external static String get FAVICON;
  external static String get EVENTSOURCE;
  external static String get CSPREPORT;
  external static String get IFRAME;
  external static String get PREFETCH;
  external static String get IMPORT;
  external static String get PING;
  external static String get SERVICEWORKER;
  external static String get XSLT;
  external static String get FETCH;
  external static String get OBJECT;
  external static String get IMAGE;
  external static String get DOWNLOAD;
  external static String get STYLE;
  external static String get LOCATION;
  external static String get PLUGIN;
  external static String get FRAME;
  external static String get FONT;
  external static String get MANIFEST;
  external static String get FORM;
  external static String get SHAREDWORKER;
  external static String get SUBRESOURCE;
  external static String get WORKER;
  external static String get IMAGESET;
}

/**
 * @enum {string}
 */
@JsName("RequestMode")
abstract class RequestMode {
  external static String get SAME_ORIGIN;
  external static String get NO_CORS;
  external static String get CORS;
}

/**
 * @enum {string}
 */
@JsName("RequestCredentials")
abstract class RequestCredentials {
  external static String get SAME_ORIGIN;
  external static String get OMIT;
  external static String get INCLUDE;
}

/**
 * @enum {string}
 */
@JsName("RequestCache")
abstract class RequestCache {
  external static String get ONLY_IF_CACHED;
  external static String get NO_CACHE;
  external static String get RELOAD;
  external static String get NO_STORE;
  external static String get DEFAULT;
  external static String get FORCE_CACHE;
}

/**
 * @see https://fetch.spec.whatwg.org/#response
 * @param {BodyInit=} opt_body
 * @param {ResponseInit=} opt_init
 * @constructor
 */
@JsName("Response")
abstract class Response {
// @Ignore
  external Response.fakeConstructor$();

  /** @return {Response} */
  external static Response error();

  /**
   * @param {string} url
   * @param {number=} opt_status
   * @return {Response}
   */
  external static Response redirect(String url, [num opt_status]);

  /**
   * @see https://fetch.spec.whatwg.org/#response
   * @param {BodyInit=} opt_body
   * @param {ResponseInit=} opt_init
   * @constructor
   */
  external factory Response(
      [opt_body /*fileapi.Blob|w3c_xml.FormData|String|undefined*/,
      ResponseInit opt_init]);

  /** @type {boolean} */
  bool bodyUsed;

  /** @return {!Promise.<!ArrayBuffer>} */
  es6.Promise<es6.ArrayBuffer> arrayBuffer();

  /** @return {!Promise.<!Blob>} */
  es6.Promise<fileapi.Blob> blob();

  /** @return {!Promise.<!FormData>} */
  es6.Promise<w3c_xml.FormData> formData();

  /** @return {!Promise.<!Object>} */
  es6.Promise<Object> json();

  /** @return {!Promise.<string>} */
  es6.Promise<String> text();

  /** @type {ResponseType} */
  String type;

  /** @type {string} */
  String url;

  /** @type {number} */
  num status;

  /** @type {boolean} */
  bool ok;

  /** @type {string} */
  String statusText;

  /** @type {!Headers} */
  Headers headers;

  /** @return {!Response} */
  Response clone();
}
/**
 * @typedef {{
 *   status : number,
 *   statusText: string,
 *   headers: !HeadersInit
 * }}
 */
@JsName()
abstract class ResponseInit {
  Object /*Headers|List<List<String>>*/ headers;
  num status;
  String statusText;
}
/**
 * @enum {string}
 */
@JsName("ResponseType")
abstract class ResponseType {
  external static String get ERROR;
  external static String get BASIC;
  external static String get OPAQUE;
  external static String get DEFAULT;
  external static String get CORS;
}

/**
 * @see https://fetch.spec.whatwg.org/#dom-global-fetch
 * @param {!RequestInfo} input
 * @param {RequestInit=} opt_init
 * @return {!Promise.<!Response>}
 */
external es6.Promise<Response> fetch(input /*Request|String*/,
    [RequestInit opt_init]);
