/*
 * Copyright 2010 The Closure Compiler Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
// This file has been automatically generated.  Please do not edit it manually.
/**
 * @fileoverview Definitions for objects in the File API, File Writer API, and
 * File System API. Details of the API are at:
 * http://www.w3.org/TR/FileAPI/
 * http://www.w3.org/TR/file-writer-api/
 * http://www.w3.org/TR/file-system-api/
 *
 * FROM_EXTERN
 * @author dbk@google.com (David Barrett-Kahn)
 */
@JsName()
library fileapi;

import 'package:js/js.dart';
import 'package:func/func.dart';

import "es6.dart" as es6;

import "html5.dart" as html5;

import "mediasource.dart" as mediasource;

import "w3c_dom1.dart" as w3c_dom1;

import "w3c_dom3.dart" as w3c_dom3;

import "w3c_event.dart" as w3c_event;

import "w3c_rtc.dart" as w3c_rtc;

/**
 * @see http://dev.w3.org/2006/webapi/FileAPI/#dfn-Blob
 * @param {Array.<ArrayBufferView|Blob|string>=} opt_blobParts
 * @param {Object=} opt_options
 * @constructor
 * @nosideeffects
 */
@JsName("Blob")
abstract class Blob {
// @Ignore
  external Blob.fakeConstructor$();

  /**
   * @see http://dev.w3.org/2006/webapi/FileAPI/#dfn-Blob
   * @param {Array.<ArrayBufferView|Blob|string>=} opt_blobParts
   * @param {Object=} opt_options
   * @constructor
   * @nosideeffects
   */
  external factory Blob(
      [List<dynamic /*es6.ArrayBufferView|null|Blob|String*/ > opt_blobParts,
      Object opt_options]);

  /**
   * @see http://www.w3.org/TR/FileAPI/#dfn-size
   * @type {number}
   */
  num size;

  /**
   * @see http://www.w3.org/TR/FileAPI/#dfn-type
   * @type {string}
   */
  String type;

  /**
   * @see http://www.w3.org/TR/FileAPI/#dfn-slice
   * @param {number} start
   * @param {number} length
   * @return {!Blob}
   * @nosideeffects
   */
  Blob slice(num start, num length);

  /**
   * This replaces Blob.slice in Chrome since WebKit revision 84005.
   * @see http://lists.w3.org/Archives/Public/public-webapps/2011AprJun/0222.html
   * @param {number} start
   * @param {number} end
   * @return {!Blob}
   * @nosideeffects
   */
  Blob webkitSlice(num start, num end);

  /**
   * This replaces Blob.slice in Firefox.
   * @see http://lists.w3.org/Archives/Public/public-webapps/2011AprJun/0222.html
   * @param {number} start
   * @param {number} end
   * @return {!Blob}
   * @nosideeffects
   */
  Blob mozSlice(num start, num end);
}

/**
 * @see http://www.w3.org/TR/file-writer-api/#the-blobbuilder-interface
 * @constructor
 */
@JsName("BlobBuilder")
abstract class BlobBuilder {
// @Ignore
  external BlobBuilder.fakeConstructor$();

  /**
   * @see http://www.w3.org/TR/file-writer-api/#widl-BlobBuilder-append0
   * @see http://www.w3.org/TR/file-writer-api/#widl-BlobBuilder-append1
   * @see http://www.w3.org/TR/file-writer-api/#widl-BlobBuilder-append2
   * @param {string|Blob|ArrayBuffer} data
   * @param {string=} endings
   */
  append(data /*String|Blob|null|es6.ArrayBuffer*/, [String endings]);

  /**
   * @see http://www.w3.org/TR/file-writer-api/#widl-BlobBuilder-getBlob
   * @param {string=} contentType
   * @return {!Blob}
   */
  Blob getBlob([String contentType]);
}

/**
 * This has replaced BlobBuilder in Chrome since WebKit revision 84008.
 * @see http://lists.w3.org/Archives/Public/public-webapps/2011AprJun/0222.html
 * @constructor
 */
@JsName("WebKitBlobBuilder")
abstract class WebKitBlobBuilder {
// @Ignore
  external WebKitBlobBuilder.fakeConstructor$();

  /**
   * @see http://www.w3.org/TR/file-writer-api/#widl-BlobBuilder-append0
   * @see http://www.w3.org/TR/file-writer-api/#widl-BlobBuilder-append1
   * @see http://www.w3.org/TR/file-writer-api/#widl-BlobBuilder-append2
   * @param {string|Blob|ArrayBuffer} data
   * @param {string=} endings
   */
  append(data /*String|Blob|null|es6.ArrayBuffer*/, [String endings]);

  /**
   * @see http://www.w3.org/TR/file-writer-api/#widl-BlobBuilder-getBlob
   * @param {string=} contentType
   * @return {!Blob}
   */
  Blob getBlob([String contentType]);
}

/**
 * @see http://www.w3.org/TR/file-system-api/#the-directoryentry-interface
 * @constructor
 * @extends {Entry}
 */
@JsName("DirectoryEntry")
abstract class DirectoryEntry extends Entry {
// @Ignore
  external DirectoryEntry.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @see http://www.w3.org/TR/file-system-api/#widl-DirectoryEntry-createReader
   * @return {!DirectoryReader}
   */
  DirectoryReader createReader();

  /**
   * @see http://www.w3.org/TR/file-system-api/#widl-DirectoryEntry-getFile
   * @param {string} path
   * @param {Object=} options
   * @param {function(!FileEntry)=} successCallback
   * @param {function(!FileError)=} errorCallback
   */
  getFile(String path, [Object options, successCallback(FileEntry arg1),
      errorCallback(FileError arg1)]);

  /**
   * @see http://www.w3.org/TR/file-system-api/#widl-DirectoryEntry-getDirectory
   * @param {string} path
   * @param {Object=} options
   * @param {function(!DirectoryEntry)=} successCallback
   * @param {function(!FileError)=} errorCallback
   */
  getDirectory(String path, [Object options,
      successCallback(DirectoryEntry arg1), errorCallback(FileError arg1)]);

  /**
   * @see http://www.w3.org/TR/file-system-api/#widl-DirectoryEntry-removeRecursively
   * @param {function()} successCallback
   * @param {function(!FileError)=} errorCallback
   */
  removeRecursively(successCallback(), [errorCallback(FileError arg1)]);
}

/**
 * @see http://www.w3.org/TR/file-system-api/#the-directoryreader-interface
 * @constructor
 */
@JsName("DirectoryReader")
abstract class DirectoryReader {
// @Ignore
  external DirectoryReader.fakeConstructor$();

  /**
   * @see http://www.w3.org/TR/file-system-api/#widl-DirectoryReader-readEntries
   * @param {function(!Array.<!Entry>)} successCallback
   * @param {function(!FileError)=} errorCallback
   */
  readEntries(successCallback(List<Entry> arg1),
      [errorCallback(FileError arg1)]);
}

/**
 * @see http://www.w3.org/TR/file-system-api/#the-entry-interface
 * @constructor
 */
@JsName("Entry")
abstract class Entry {
// @Ignore
  external Entry.fakeConstructor$();

  /**
   * @see http://www.w3.org/TR/file-system-api/#widl-Entry-isFile
   * @type {boolean}
   */
  bool isFile;

  /**
   * @see http://www.w3.org/TR/file-system-api/#widl-Entry-isDirectory
   * @type {boolean}
   */
  bool isDirectory;

  /**
   * @see http://www.w3.org/TR/file-system-api/#widl-Entry-name
   * @type {string}
   */
  String name;

  /**
   * @see http://www.w3.org/TR/file-system-api/#widl-Entry-fullPath
   * @type {string}
   */
  String fullPath;

  /**
   * @see http://www.w3.org/TR/file-system-api/#widl-Entry-filesystem
   * @type {!FileSystem}
   */
  FileSystem filesystem;

  /**
   * @see http://www.w3.org/TR/file-system-api/#widl-Entry-moveTo
   * @param {!DirectoryEntry} parent
   * @param {string=} newName
   * @param {function(!Entry)=} successCallback
   * @param {function(!FileError)=} errorCallback
   */
  moveTo(DirectoryEntry parent, [String newName, successCallback(Entry arg1),
      errorCallback(FileError arg1)]);

  /**
   * @see http://www.w3.org/TR/file-system-api/#widl-Entry-copyTo
   * @param {!DirectoryEntry} parent
   * @param {string=} newName
   * @param {function(!Entry)=} successCallback
   * @param {function(!FileError)=} errorCallback
   */
  copyTo(DirectoryEntry parent, [String newName, successCallback(Entry arg1),
      errorCallback(FileError arg1)]);

  /**
   * @see http://www.w3.org/TR/file-system-api/#widl-Entry-toURL
   * @param {string=} mimeType
   * @return {string}
   */
  String toURL([String mimeType]);

  /**
   * @see http://www.w3.org/TR/file-system-api/#widl-Entry-remove
   * @param {function()} successCallback
   * @param {function(!FileError)=} errorCallback
   */
  remove(successCallback(), [errorCallback(FileError arg1)]);

  /**
   * @see http://www.w3.org/TR/file-system-api/#widl-Entry-getMetadata
   * @param {function(!Metadata)} successCallback
   * @param {function(!FileError)=} errorCallback
   */
  getMetadata(successCallback(Metadata arg1), [errorCallback(FileError arg1)]);

  /**
   * @see http://www.w3.org/TR/file-system-api/#widl-Entry-getParent
   * @param {function(!Entry)} successCallback
   * @param {function(!FileError)=} errorCallback
   */
  getParent(successCallback(Entry arg1), [errorCallback(FileError arg1)]);
}

/**
 * @see http://www.w3.org/TR/FileAPI/#dfn-file
 * @param {!Array<string|!Blob|!ArrayBuffer>=} opt_contents
 * @param {string=} opt_name
 * @param {{type: (string|undefined), lastModified: (number|undefined)}=}
 *     opt_properties
 * @constructor
 * @extends {Blob}
 */
@JsName("File")
abstract class File extends Blob {
// @Ignore
  external File.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @see http://www.w3.org/TR/FileAPI/#dfn-file
   * @param {!Array<string|!Blob|!ArrayBuffer>=} opt_contents
   * @param {string=} opt_name
   * @param {{type: (string|undefined), lastModified: (number|undefined)}=}
   *     opt_properties
   * @constructor
   * @extends {Blob}
   */
  external factory File(
      [List<dynamic /*String|Blob|es6.ArrayBuffer*/ > opt_contents,
      String opt_name, PropertiesRecord opt_properties]);

  /**
   * Chrome uses this instead of name.
   * @deprecated Use name instead.
   * @type {string}
   */
  String fileName;

  /**
   * Chrome uses this instead of size.
   * @deprecated Use size instead.
   * @type {string}
   */
  String fileSize;

  /**
   * @see http://www.w3.org/TR/FileAPI/#dfn-name
   * @type {string}
   */
  String name;

  /**
   * @see http://www.w3.org/TR/FileAPI/#dfn-lastModifiedDate
   * @type {Date}
   */
  DateTime lastModifiedDate;
}

/**
 * @see http://www.w3.org/TR/file-system-api/#the-fileentry-interface
 * @constructor
 * @extends {Entry}
 */
@JsName("FileEntry")
abstract class FileEntry extends Entry {
// @Ignore
  external FileEntry.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @see http://www.w3.org/TR/file-system-api/#widl-FileEntry-createWriter
   * @param {function(!FileWriter)} successCallback
   * @param {function(!FileError)=} errorCallback
   */
  createWriter(successCallback(FileWriter arg1),
      [errorCallback(FileError arg1)]);

  /**
   * @see http://www.w3.org/TR/file-system-api/#widl-FileEntry-file
   * @param {function(!File)} successCallback
   * @param {function(!FileError)=} errorCallback
   */
  file(successCallback(File arg1), [errorCallback(FileError arg1)]);
}

/**
 * @see http://www.w3.org/TR/FileAPI/#FileErrorInterface
 * @constructor
 * @extends {DOMError}
 */
@JsName("FileError")
abstract class FileError extends w3c_dom3.DOMError {
// @Ignore
  external FileError.fakeConstructor$() : super.fakeConstructor$();

  @JsName(r"INVALID_MODIFICATION_ERR")
  external static num get INVALID_MODIFICATION_ERR$static;

  @JsName(r"INVALID_MODIFICATION_ERR")
  external static set INVALID_MODIFICATION_ERR$static(num value);

  @JsName(r"INVALID_STATE_ERR")
  external static num get INVALID_STATE_ERR$static;

  @JsName(r"INVALID_STATE_ERR")
  external static set INVALID_STATE_ERR$static(num value);

  @JsName(r"SYNTAX_ERR")
  external static num get SYNTAX_ERR$static;

  @JsName(r"SYNTAX_ERR")
  external static set SYNTAX_ERR$static(num value);

  @JsName(r"TYPE_MISMATCH_ERR")
  external static num get TYPE_MISMATCH_ERR$static;

  @JsName(r"TYPE_MISMATCH_ERR")
  external static set TYPE_MISMATCH_ERR$static(num value);

  @JsName(r"QUOTA_EXCEEDED_ERR")
  external static num get QUOTA_EXCEEDED_ERR$static;

  @JsName(r"QUOTA_EXCEEDED_ERR")
  external static set QUOTA_EXCEEDED_ERR$static(num value);

  @JsName(r"NOT_FOUND_ERR")
  external static num get NOT_FOUND_ERR$static;

  @JsName(r"NOT_FOUND_ERR")
  external static set NOT_FOUND_ERR$static(num value);

  @JsName(r"NOT_READABLE_ERR")
  external static num get NOT_READABLE_ERR$static;

  @JsName(r"NOT_READABLE_ERR")
  external static set NOT_READABLE_ERR$static(num value);

  @JsName(r"SECURITY_ERR")
  external static num get SECURITY_ERR$static;

  @JsName(r"SECURITY_ERR")
  external static set SECURITY_ERR$static(num value);

  @JsName(r"ENCODING_ERR")
  external static num get ENCODING_ERR$static;

  @JsName(r"ENCODING_ERR")
  external static set ENCODING_ERR$static(num value);

  @JsName(r"PATH_EXISTS_ERR")
  external static num get PATH_EXISTS_ERR$static;

  @JsName(r"PATH_EXISTS_ERR")
  external static set PATH_EXISTS_ERR$static(num value);

  @JsName(r"ABORT_ERR")
  external static num get ABORT_ERR$static;

  @JsName(r"ABORT_ERR")
  external static set ABORT_ERR$static(num value);

  @JsName(r"NO_MODIFICATION_ALLOWED_ERR")
  external static num get NO_MODIFICATION_ALLOWED_ERR$static;

  @JsName(r"NO_MODIFICATION_ALLOWED_ERR")
  external static set NO_MODIFICATION_ALLOWED_ERR$static(num value);

  /**
   * @see http://www.w3.org/TR/FileAPI/#dfn-code-exception
   * @type {number}
   * @deprecated Use the 'name' or 'message' attributes of DOMError rather than
   * 'code'
   */
  num code;
}

/**
 * @see http://www.w3.org/TR/FileAPI/#dfn-filereader
 * @constructor
 * @implements {EventTarget}
 */
@JsName("FileReader")
abstract class FileReader implements w3c_event.EventTarget {
// @Ignore
  external FileReader.fakeConstructor$();

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  addEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  removeEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /** @override */
  bool dispatchEvent(w3c_event.Event evt);

  /**
   * @see http://www.w3.org/TR/FileAPI/#dfn-readAsArrayBuffer
   * @param {!Blob} blob
   */
  readAsArrayBuffer(Blob blob);

  /**
   * @see http://www.w3.org/TR/FileAPI/#dfn-readAsBinaryStringAsync
   * @param {!Blob} blob
   */
  readAsBinaryString(Blob blob);

  /**
   * @see http://www.w3.org/TR/FileAPI/#dfn-readAsText
   * @param {!Blob} blob
   * @param {string=} encoding
   */
  readAsText(Blob blob, [String encoding]);

  /**
   * @see http://www.w3.org/TR/FileAPI/#dfn-readAsDataURL
   * @param {!Blob} blob
   */
  readAsDataURL(Blob blob);

  /**
   * @see http://www.w3.org/TR/FileAPI/#dfn-abort
   */
  abort();

  /**
   * @see http://www.w3.org/TR/FileAPI/#dfn-readystate
   * @type {number}
   */
  num readyState;

  /**
   * @see http://www.w3.org/TR/FileAPI/#dfn-result
   * @type {string|Blob|ArrayBuffer}
   */
  dynamic /*String|Blob|null|es6.ArrayBuffer*/ result;

  /**
   * @see http://www.w3.org/TR/FileAPI/#dfn-error
   * @type {FileError}
   */
  FileError error;

  /**
   * @see http://www.w3.org/TR/FileAPI/#dfn-onloadstart
   * @type {?function(!ProgressEvent)}
   */
  Func1<html5.ProgressEvent, dynamic> onloadstart;

  /**
   * @see http://www.w3.org/TR/FileAPI/#dfn-onprogress
   * @type {?function(!ProgressEvent)}
   */
  Func1<html5.ProgressEvent, dynamic> onprogress;

  /**
   * @see http://www.w3.org/TR/FileAPI/#dfn-onload
   * @type {?function(!ProgressEvent)}
   */
  Func1<html5.ProgressEvent, dynamic> onload;

  /**
   * @see http://www.w3.org/TR/FileAPI/#dfn-onabort
   * @type {?function(!ProgressEvent)}
   */
  Func1<html5.ProgressEvent, dynamic> onabort;

  /**
   * @see http://www.w3.org/TR/FileAPI/#dfn-onerror
   * @type {?function(!ProgressEvent)}
   */
  Func1<html5.ProgressEvent, dynamic> onerror;

  /**
   * @see http://www.w3.org/TR/FileAPI/#dfn-onloadend
   * @type {?function(!ProgressEvent)}
   */
  Func1<html5.ProgressEvent, dynamic> onloadend;
}

/**
 * @see http://www.w3.org/TR/file-writer-api/#idl-def-FileSaver
 * @constructor
 */
@JsName("FileSaver")
abstract class FileSaver {
// @Ignore
  external FileSaver.fakeConstructor$();

  /** @see http://www.w3.org/TR/file-writer-api/#widl-FileSaver-abort */
  abort();

  /**
   * @see http://www.w3.org/TR/file-writer-api/#widl-FileSaver-readyState
   * @type {number}
   */
  num readyState;

  /**
   * @see http://www.w3.org/TR/file-writer-api/#widl-FileSaver-error
   * @type {FileError}
   */
  FileError error;

  /**
   * @see http://www.w3.org/TR/file-writer-api/#widl-FileSaver-onwritestart
   * @type {?function(!ProgressEvent)}
   */
  Func1<html5.ProgressEvent, dynamic> onwritestart;

  /**
   * @see http://www.w3.org/TR/file-writer-api/#widl-FileSaver-onprogress
   * @type {?function(!ProgressEvent)}
   */
  Func1<html5.ProgressEvent, dynamic> onprogress;

  /**
   * @see http://www.w3.org/TR/file-writer-api/#widl-FileSaver-onwrite
   * @type {?function(!ProgressEvent)}
   */
  Func1<html5.ProgressEvent, dynamic> onwrite;

  /**
   * @see http://www.w3.org/TR/file-writer-api/#widl-FileSaver-onabort
   * @type {?function(!ProgressEvent)}
   */
  Func1<html5.ProgressEvent, dynamic> onabort;

  /**
   * @see http://www.w3.org/TR/file-writer-api/#widl-FileSaver-onerror
   * @type {?function(!ProgressEvent)}
   */
  Func1<html5.ProgressEvent, dynamic> onerror;

  /**
   * @see http://www.w3.org/TR/file-writer-api/#widl-FileSaver-onwriteend
   * @type {?function(!ProgressEvent)}
   */
  Func1<html5.ProgressEvent, dynamic> onwriteend;
}

/**
 * @see http://www.w3.org/TR/file-system-api/#the-filesystem-interface
 * @constructor
 */
@JsName("FileSystem")
abstract class FileSystem {
// @Ignore
  external FileSystem.fakeConstructor$();

  /**
   * @see http://www.w3.org/TR/file-system-api/#widl-FileSystem-name
   * @type {string}
   */
  String name;

  /**
   * @see http://www.w3.org/TR/file-system-api/#widl-FileSystem-root
   * @type {!DirectoryEntry}
   */
  DirectoryEntry root;
}

/**
 * @see http://www.w3.org/TR/file-writer-api/#idl-def-FileWriter
 * @constructor
 * @extends {FileSaver}
 */
@JsName("FileWriter")
abstract class FileWriter extends FileSaver {
// @Ignore
  external FileWriter.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @see http://www.w3.org/TR/file-writer-api/#widl-FileWriter-position
   * @type {number}
   */
  num position;

  /**
   * @see http://www.w3.org/TR/file-writer-api/#widl-FileWriter-length
   * @type {number}
   */
  num length;

  /**
   * @see http://www.w3.org/TR/file-writer-api/#widl-FileWriter-write
   * @param {!Blob} blob
   */
  write(Blob blob);

  /**
   * @see http://www.w3.org/TR/file-writer-api/#widl-FileWriter-seek
   * @param {number} offset
   */
  seek(num offset);

  /**
   * @see http://www.w3.org/TR/file-writer-api/#widl-FileWriter-truncate
   * @param {number} size
   */
  truncate(num size);
}

/**
 * LocalFileSystem interface, implemented by Window and WorkerGlobalScope.
 * @see http://www.w3.org/TR/file-system-api/#idl-def-LocalFileSystem
 * @constructor
 */
@JsName("LocalFileSystem")
abstract class LocalFileSystem {
// @Ignore
  external LocalFileSystem.fakeConstructor$();
}

/**
 * Metadata interface.
 * @see http://www.w3.org/TR/file-system-api/#idl-def-Metadata
 * @constructor
 */
@JsName("Metadata")
abstract class Metadata {
// @Ignore
  external Metadata.fakeConstructor$();

  /**
   * @see http://www.w3.org/TR/file-system-api/#widl-Metadata-modificationTime
   * @type {!Date}
   */
  DateTime modificationTime;

  /**
   * @see http://www.w3.org/TR/file-system-api/#widl-Metadata-size
   * @type {number}
   */
  num size;
}
/**
 * @see http://www.w3.org/TR/file-system-api/#widl-LocalFileSystem-requestFileSystem
 * @param {number} type
 * @param {number} size
 * @param {function(!FileSystem)} successCallback
 * @param {function(!FileError)=} errorCallback
 */
external requestFileSystem(num type, num size, successCallback(FileSystem arg1),
    [errorCallback(FileError arg1)]);
/**
 * @see http://www.w3.org/TR/file-system-api/#widl-LocalFileSystem-resolveLocalFileSystemURI
 * @param {string} uri
 * @param {function(!Entry)} successCallback
 * @param {function(!FileError)=} errorCallback
 */
external resolveLocalFileSystemURI(String uri, successCallback(Entry arg1),
    [errorCallback(FileError arg1)]);
/**
 * This has replaced requestFileSystem in Chrome since WebKit revision 84224.
 * @see http://www.w3.org/TR/file-system-api/#widl-LocalFileSystem-requestFileSystem
 * @param {number} type
 * @param {number} size
 * @param {function(!FileSystem)} successCallback
 * @param {function(!FileError)=} errorCallback
 */
external webkitRequestFileSystem(
    num type, num size, successCallback(FileSystem arg1),
    [errorCallback(FileError arg1)]);
/**
 * This has replaced resolveLocalFileSystemURI in Chrome since WebKit revision
 * 84224.
 * @see http://www.w3.org/TR/file-system-api/#widl-LocalFileSystem-resolveLocalFileSystemURI
 * @param {string} uri
 * @param {function(!Entry)} successCallback
 * @param {function(!FileError)=} errorCallback
 */
external webkitResolveLocalFileSystemURI(
    String uri, successCallback(Entry arg1), [errorCallback(FileError arg1)]);
/**
 * @see http://www.w3.org/TR/FileAPI/#dfn-createObjectURL
 * @param {!Object} obj
 * @return {string}
 */
external String createObjectURL(Object obj);
/**
 * @see http://www.w3.org/TR/FileAPI/#dfn-revokeObjectURL
 * @param {string} url
 */
external revokeObjectURL(String url);

/**
 * @see http://www.w3.org/TR/FileAPI/#URL-object
 * @constructor
 */
@JsName("DOMURL")
abstract class DOMURL {
// @Ignore
  external DOMURL.fakeConstructor$();
}

/**
 * @see http://www.w3.org/TR/FileAPI/#
 * @constructor
 * @param {string} urlString
 * @param {string=} opt_base
 * @extends {DOMURL}
 */
@JsName("URL")
abstract class URL extends DOMURL {
// @Ignore
  external URL.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @see http://www.w3.org/TR/FileAPI/#dfn-revokeObjectURL
   * @param {string} url
   */
  external static revokeObjectURL(String url);

  /**
   * @see http://www.w3.org/TR/FileAPI/#dfn-createObjectURL
   * @param {!File|!Blob|!MediaSource|!MediaStream} obj
   * @return {string}
   */
  external static String createObjectURL(
      Object obj /*Blob|mediasource.MediaSource|w3c_rtc.MediaStream*/);

  /**
   * @see http://www.w3.org/TR/FileAPI/#
   * @constructor
   * @param {string} urlString
   * @param {string=} opt_base
   * @extends {DOMURL}
   */
  external factory URL(String urlString, [String opt_base]);

  /** @type {string} */
  String protocol;
}

/**
 * This has been replaced by URL in Chrome since WebKit revision 75739.
 * @constructor
 * @param {string} urlString
 * @param {string=} opt_base
 * @extends {DOMURL}
 */
@JsName("webkitURL")
abstract class webkitURL extends DOMURL {
// @Ignore
  external webkitURL.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @see http://www.w3.org/TR/FileAPI/#dfn-revokeObjectURL
   * @param {string} url
   */
  external static revokeObjectURL(String url);

  /**
   * @see http://www.w3.org/TR/FileAPI/#dfn-createObjectURL
   * @param {!Object} obj
   * @return {string}
   */
  external static String createObjectURL(Object obj);

  /**
   * This has been replaced by URL in Chrome since WebKit revision 75739.
   * @constructor
   * @param {string} urlString
   * @param {string=} opt_base
   * @extends {DOMURL}
   */
  external factory webkitURL(String urlString, [String opt_base]);
}

/**
 * @see https://developers.google.com/chrome/whitepapers/storage
 * @constructor
 */
@JsName("StorageInfo")
abstract class StorageInfo {
// @Ignore
  external StorageInfo.fakeConstructor$();

  /**
   * @see https://developers.google.com/chrome/whitepapers/storage#requestQuota
   * @param {number} type
   * @param {number} size
   * @param {function(number)} successCallback
   * @param {function(!DOMException)=} errorCallback
   */
  requestQuota(num type, num size, successCallback(num arg1),
      [errorCallback(w3c_dom1.DOMException arg1)]);

  /**
   * @see https://developers.google.com/chrome/whitepapers/storage#queryUsageAndQuota
   * @param {number} type
   * @param {function(number, number)} successCallback
   * @param {function(!DOMException)=} errorCallback
   */
  queryUsageAndQuota(num type, successCallback(num arg1, num arg2),
      [errorCallback(w3c_dom1.DOMException arg1)]);
}

/**
 * @see https://dvcs.w3.org/hg/quota/raw-file/tip/Overview.html#storagequota-interface.
 * @constructor
 */
@JsName("StorageQuota")
abstract class StorageQuota {
// @Ignore
  external StorageQuota.fakeConstructor$();

  /**
   * @param {number} size
   * @param {function(number)=} opt_successCallback
   * @param {function(!DOMException)=} opt_errorCallback
   */
  requestQuota(num size, [opt_successCallback(num arg1),
      opt_errorCallback(w3c_dom1.DOMException arg1)]);

  /**
   * @param {function(number, number)} successCallback
   * @param {function(!DOMException)=} opt_errorCallback
   */
  queryUsageAndQuota(successCallback(num arg1, num arg2),
      [opt_errorCallback(w3c_dom1.DOMException arg1)]);
}
/// This record class is not given a name in the Closure JavaScript code base.
/// The name we have chosen may not make sense.
@JsName()
abstract class PropertiesRecord {
  num lastModified;
  String type;
}
