/*
 * Copyright 2008 The Closure Compiler Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
// This file has been automatically generated.  Please do not edit it manually.
/**
 * @fileoverview Definitions for all the extensions over the
 *  W3C's DOM3 specification in HTML5. This file depends on
 *  w3c_dom3.js. The whole file has been fully type annotated.
 *
 *  @see http://www.whatwg.org/specs/web-apps/current-work/multipage/index.html
 *  @see http://dev.w3.org/html5/spec/Overview.html
 *
 *  This also includes Typed Array definitions from
 *  http://www.khronos.org/registry/typedarray/specs/latest/
 *
 *  This relies on w3c_event.js being included first.
 *
 * FROM_EXTERN
 */
@JsName()
library html5;

import 'package:js/js.dart';
import 'package:func/func.dart';

import "es6.dart" as es6;

import "fetchapi.dart" as fetchapi;

import "fileapi.dart" as fileapi;

import "gecko_dom.dart" as gecko_dom;

import "ie_dom.dart" as ie_dom;

import "w3c_css.dart" as w3c_css;

import "w3c_dom1.dart" as w3c_dom1;

import "w3c_dom2.dart" as w3c_dom2;

import "w3c_event.dart" as w3c_event;

/**
 * @constructor
 * @see http://www.whatwg.org/specs/web-apps/current-work/multipage/the-canvas-element.html#the-canvas-element
 * @extends {HTMLElement}
 */
@JsName("HTMLCanvasElement")
abstract class HTMLCanvasElement extends w3c_dom2.HTMLElement {
// @Ignore
  external HTMLCanvasElement.fakeConstructor$() : super.fakeConstructor$();

  /** @type {number} */
  num width;

  /** @type {number} */
  num height;

  /**
   * @param {string=} opt_type
   * @param {...*} var_args
   * @return {string}
   * @throws {Error}
   * @nosideeffects
   */
  String toDataURL(
      [String opt_type, var_args1, var_args2, var_args3, var_args4]);

  /**
   * @param {string} contextId
   * @param {Object=} opt_args
   * @return {Object}
   */
  Object getContext(String contextId, [Object opt_args]);
}

/**
 * @constructor
 * @see http://www.w3.org/TR/2dcontext/#canvasrenderingcontext2d
 */
@JsName("CanvasRenderingContext2D")
abstract class CanvasRenderingContext2D {
// @Ignore
  external CanvasRenderingContext2D.fakeConstructor$();

  /** @type {HTMLCanvasElement} */
  HTMLCanvasElement canvas;

  /**
   * @return {undefined}
   */
  void save();

  /**
   * @return {undefined}
   */
  void restore();

  /**
   * @param {number} x
   * @param {number} y
   * @return {undefined}
   */
  void scale(num x, num y);

  /**
   * @param {number} angle
   * @return {undefined}
   */
  void rotate(num angle);

  /**
   * @param {number} x
   * @param {number} y
   * @return {undefined}
   */
  void translate(num x, num y);

  /**
   * @param {number} m11
   * @param {number} m12
   * @param {number} m21
   * @param {number} m22
   * @param {number} dx
   * @param {number} dy
   * @return {undefined}
   */
  void transform(num m11, num m12, num m21, num m22, num dx, num dy);

  /**
   * @param {number} m11
   * @param {number} m12
   * @param {number} m21
   * @param {number} m22
   * @param {number} dx
   * @param {number} dy
   * @return {undefined}
   */
  void setTransform(num m11, num m12, num m21, num m22, num dx, num dy);

  /**
   * @param {number} x0
   * @param {number} y0
   * @param {number} x1
   * @param {number} y1
   * @return {CanvasGradient}
   * @throws {Error}
   * @nosideeffects
   */
  CanvasGradient createLinearGradient(num x0, num y0, num x1, num y1);

  /**
   * @param {number} x0
   * @param {number} y0
   * @param {number} r0
   * @param {number} x1
   * @param {number} y1
   * @param {number} r1
   * @return {CanvasGradient}
   * @throws {Error}
   * @nosideeffects
   */
  CanvasGradient createRadialGradient(
      num x0, num y0, num r0, num x1, num y1, num r1);

  /**
   * @param {HTMLImageElement|HTMLCanvasElement} image
   * @param {string} repetition
   * @return {CanvasPattern}
   * @throws {Error}
   * @nosideeffects
   */
  CanvasPattern createPattern(
      image /*w3c_dom2.HTMLImageElement|null|HTMLCanvasElement*/,
      String repetition);

  /**
   * @param {number} x
   * @param {number} y
   * @param {number} w
   * @param {number} h
   * @return {undefined}
   */
  void clearRect(num x, num y, num w, num h);

  /**
   * @param {number} x
   * @param {number} y
   * @param {number} w
   * @param {number} h
   * @return {undefined}
   */
  void fillRect(num x, num y, num w, num h);

  /**
   * @param {number} x
   * @param {number} y
   * @param {number} w
   * @param {number} h
   * @return {undefined}
   */
  void strokeRect(num x, num y, num w, num h);

  /**
   * @return {undefined}
   */
  void beginPath();

  /**
   * @return {undefined}
   */
  void closePath();

  /**
   * @param {number} x
   * @param {number} y
   * @return {undefined}
   */
  void moveTo(num x, num y);

  /**
   * @param {number} x
   * @param {number} y
   * @return {undefined}
   */
  void lineTo(num x, num y);

  /**
   * @param {number} cpx
   * @param {number} cpy
   * @param {number} x
   * @param {number} y
   * @return {undefined}
   */
  void quadraticCurveTo(num cpx, num cpy, num x, num y);

  /**
   * @param {number} cp1x
   * @param {number} cp1y
   * @param {number} cp2x
   * @param {number} cp2y
   * @param {number} x
   * @param {number} y
   * @return {undefined}
   */
  void bezierCurveTo(num cp1x, num cp1y, num cp2x, num cp2y, num x, num y);

  /**
   * @param {number} x1
   * @param {number} y1
   * @param {number} x2
   * @param {number} y2
   * @param {number} radius
   * @return {undefined}
   */
  void arcTo(num x1, num y1, num x2, num y2, num radius);

  /**
   * @param {number} x
   * @param {number} y
   * @param {number} w
   * @param {number} h
   * @return {undefined}
   */
  void rect(num x, num y, num w, num h);

  /**
   * @param {number} x
   * @param {number} y
   * @param {number} radius
   * @param {number} startAngle
   * @param {number} endAngle
   * @param {boolean=} opt_anticlockwise
   * @return {undefined}
   */
  void arc(num x, num y, num radius, num startAngle, num endAngle,
      [bool opt_anticlockwise]);

  /**
   * @param {string=} opt_fillRule
   * @return {undefined}
   */
  void fill([String opt_fillRule]);

  /**
   * @return {undefined}
   */
  void stroke();

  /**
   * @param {string=} opt_fillRule
   * @return {undefined}
   */
  void clip([String opt_fillRule]);

  /**
   * @param {number} x
   * @param {number} y
   * @param {string=} opt_fillRule
   * @return {boolean}
   * @nosideeffects
   */
  bool isPointInPath(num x, num y, [String opt_fillRule]);

  /**
   * @param {string} text
   * @param {number} x
   * @param {number} y
   * @param {number=} opt_maxWidth
   * @return {undefined}
   */
  void fillText(String text, num x, num y, [num opt_maxWidth]);

  /**
   * @param {string} text
   * @param {number} x
   * @param {number} y
   * @param {number=} opt_maxWidth
   * @return {undefined}
   */
  void strokeText(String text, num x, num y, [num opt_maxWidth]);

  /**
   * @param {string} text
   * @return {TextMetrics}
   * @nosideeffects
   */
  TextMetrics measureText(String text);

  /**
   * @param {HTMLImageElement|HTMLCanvasElement|Image|HTMLVideoElement} image
   * @param {number} dx Destination x coordinate.
   * @param {number} dy Destination y coordinate.
   * @param {number=} opt_dw Destination box width.  Defaults to the image width.
   * @param {number=} opt_dh Destination box height.
   *     Defaults to the image height.
   * @param {number=} opt_sx Source box x coordinate.  Used to select a portion of
   *     the source image to draw.  Defaults to 0.
   * @param {number=} opt_sy Source box y coordinate.  Used to select a portion of
   *     the source image to draw.  Defaults to 0.
   * @param {number=} opt_sw Source box width.  Used to select a portion of
   *     the source image to draw.  Defaults to the full image width.
   * @param {number=} opt_sh Source box height.  Used to select a portion of
   *     the source image to draw.  Defaults to the full image height.
   * @return {undefined}
   */
  void drawImage(
      image /*w3c_dom2.HTMLImageElement|null|HTMLCanvasElement|HTMLVideoElement*/,
      num dx, num dy,
      [num opt_dw, num opt_dh, num opt_sx, num opt_sy, num opt_sw, num opt_sh]);

  /**
   * @param {number} sw
   * @param {number} sh
   * @return {ImageData}
   * @nosideeffects
   */
  ImageData createImageData(num sw, num sh);

  /**
   * @param {number} sx
   * @param {number} sy
   * @param {number} sw
   * @param {number} sh
   * @return {ImageData}
   * @throws {Error}
   * @nosideeffects
   */
  ImageData getImageData(num sx, num sy, num sw, num sh);

  /**
   * @param {ImageData} imagedata
   * @param {number} dx
   * @param {number} dy
   * @param {number=} opt_dirtyX
   * @param {number=} opt_dirtyY
   * @param {number=} opt_dirtyWidth
   * @param {number=} opt_dirtyHeight
   * @return {undefined}
   */
  void putImageData(ImageData imagedata, num dx, num dy, [num opt_dirtyX,
      num opt_dirtyY, num opt_dirtyWidth, num opt_dirtyHeight]);

  /**
   * Note: WebKit only
   * @param {number|string=} opt_a
   * @param {number=} opt_b
   * @param {number=} opt_c
   * @param {number=} opt_d
   * @param {number=} opt_e
   * @see http://developer.apple.com/library/safari/#documentation/appleapplications/reference/WebKitDOMRef/CanvasRenderingContext2D_idl/Classes/CanvasRenderingContext2D/index.html
   * @return {undefined}
   */
  external void setFillColor([opt_a /*num|String|undefined*/, num opt_b,
      num opt_c, num opt_d, num opt_e]);
  /**
   * Note: WebKit only
   * @param {number|string=} opt_a
   * @param {number=} opt_b
   * @param {number=} opt_c
   * @param {number=} opt_d
   * @param {number=} opt_e
   * @see http://developer.apple.com/library/safari/#documentation/appleapplications/reference/WebKitDOMRef/CanvasRenderingContext2D_idl/Classes/CanvasRenderingContext2D/index.html
   * @return {undefined}
   */
  external void setStrokeColor([opt_a /*num|String|undefined*/, num opt_b,
      num opt_c, num opt_d, num opt_e]);
  /**
   * @return {Array.<number>}
   */
  external List<num> getLineDash();
  /**
   * @param {Array.<number>} segments
   * @return {undefined}
   */
  external void setLineDash(List<num> segments);
  /** @type {string} */
  String fillColor;

  /**
   * @type {string}
   * @implicitCast
   */
  String fillStyle;

  /** @type {string} */
  String font;

  /** @type {number} */
  num globalAlpha;

  /** @type {string} */
  String globalCompositeOperation;

  /** @type {number} */
  num lineWidth;

  /** @type {string} */
  String lineCap;

  /** @type {string} */
  String lineJoin;

  /** @type {number} */
  num miterLimit;

  /** @type {number} */
  num shadowBlur;

  /** @type {string} */
  String shadowColor;

  /** @type {number} */
  num shadowOffsetX;

  /** @type {number} */
  num shadowOffsetY;

  /**
   * @type {string}
   * @implicitCast
   */
  String strokeStyle;

  /** @type {string} */
  String strokeColor;

  /** @type {string} */
  String textAlign;

  /** @type {string} */
  String textBaseline;
}

/**
 * @constructor
 */
@JsName("CanvasGradient")
abstract class CanvasGradient {
// @Ignore
  external CanvasGradient.fakeConstructor$();

  /**
   * @param {number} offset
   * @param {string} color
   * @return {undefined}
   */
  void addColorStop(num offset, String color);
}

/**
 * @constructor
 */
@JsName("CanvasPattern")
abstract class CanvasPattern {
// @Ignore
  external CanvasPattern.fakeConstructor$();
}

/**
 * @constructor
 */
@JsName("TextMetrics")
abstract class TextMetrics {
// @Ignore
  external TextMetrics.fakeConstructor$();

  /** @type {number} */
  num width;
}

/**
 * @constructor
 */
@JsName("ImageData")
abstract class ImageData {
// @Ignore
  external ImageData.fakeConstructor$();

  /** @type {Uint8ClampedArray} */
  es6.Uint8ClampedArray data;

  /** @type {number} */
  num width;

  /** @type {number} */
  num height;
}

/**
 * @constructor
 */
@JsName("ClientInformation")
abstract class ClientInformation {
// @Ignore
  external ClientInformation.fakeConstructor$();

  /** @type {boolean} */
  bool onLine;

  /**
   * @param {string} protocol
   * @param {string} uri
   * @param {string} title
   * @return {undefined}
   */
  void registerProtocolHandler(String protocol, String uri, String title);

  /**
   * @param {string} mimeType
   * @param {string} uri
   * @param {string} title
   * @return {undefined}
   */
  void registerContentHandler(String mimeType, String uri, String title);
}

/**
 * @constructor
 */
@JsName("Database")
abstract class Database {
// @Ignore
  external Database.fakeConstructor$();

  /**
   * @type {string}
   */
  String version;

  /**
   * @param {function(!SQLTransaction) : void} callback
   * @param {(function(!SQLError) : void)=} opt_errorCallback
   * @param {Function=} opt_Callback
   */
  transaction(callback(SQLTransaction arg1),
      [opt_errorCallback(SQLError arg1), Function opt_Callback]);

  /**
   * @param {function(!SQLTransaction) : void} callback
   * @param {(function(!SQLError) : void)=} opt_errorCallback
   * @param {Function=} opt_Callback
   */
  readTransaction(callback(SQLTransaction arg1),
      [opt_errorCallback(SQLError arg1), Function opt_Callback]);

  /**
   * @param {string} oldVersion
   * @param {string} newVersion
   * @param {function(!SQLTransaction) : void} callback
   * @param {function(!SQLError) : void} errorCallback
   * @param {Function} successCallback
   */
  changeVersion(String oldVersion, String newVersion,
      callback(SQLTransaction arg1), errorCallback(SQLError arg1),
      Function successCallback);
}

/**
 * @interface
 */
@JsName("DatabaseCallback")
abstract class DatabaseCallback {
// @Ignore
  external DatabaseCallback.fakeConstructor$();

  /**
   * @param {!Database} db
   * @return {undefined}
   */
  void handleEvent(Database db);
}

/**
 * @constructor
 */
@JsName("SQLError")
abstract class SQLError {
// @Ignore
  external SQLError.fakeConstructor$();

  /**
   * @type {number}
   */
  num code;

  /**
   * @type {string}
   */
  String message;
}

/**
 * @constructor
 */
@JsName("SQLTransaction")
abstract class SQLTransaction {
// @Ignore
  external SQLTransaction.fakeConstructor$();

  /**
   * @param {string} sqlStatement
   * @param {Array.<*>=} opt_queryArgs
   * @param {SQLStatementCallback=} opt_callback
   * @param {(function(!SQLTransaction, !SQLError) : (boolean|void))=}
   *     opt_errorCallback
   */
  executeSql(String sqlStatement, [List<dynamic> opt_queryArgs,
      opt_callback(SQLTransaction arg1, SQLResultSet arg2),
      bool opt_errorCallback(SQLTransaction arg1, SQLError arg2)]);
}
/**
 * @typedef {(function(!SQLTransaction, !SQLResultSet) : void)}
 */
typedef SQLStatementCallback(SQLTransaction arg1, SQLResultSet arg2);

/**
 * @constructor
 */
@JsName("SQLResultSet")
abstract class SQLResultSet {
// @Ignore
  external SQLResultSet.fakeConstructor$();

  /**
   * @type {number}
   */
  num insertId;

  /**
   * @type {number}
   */
  num rowsAffected;

  /**
   * @type {SQLResultSetRowList}
   */
  SQLResultSetRowList rows;
}

/**
 * @constructor
 */
@JsName("SQLResultSetRowList")
abstract class SQLResultSetRowList {
// @Ignore
  external SQLResultSetRowList.fakeConstructor$();

  /**
   * @type {number}
   */
  num length;

  /**
   * @param {number} index
   * @return {Object}
   * @nosideeffects
   */
  Object item(num index);
}
/**
 * @param {string} name
 * @param {string} version
 * @param {string} description
 * @param {number} size
 * @param {(DatabaseCallback|function(Database))=} opt_callback
 * @return {Database}
 */
external Database openDatabase(
    String name, String version, String description, num size,
    [opt_callback /*DatabaseCallback|null|Func1<Database,dynamic>|undefined*/]);
/**
 * This is a superposition of the Window and Worker postMessage methods.
 * @param {*} message
 * @param {(string|!Array.<!Transferable>)=} opt_targetOriginOrTransfer
 * @param {(string|!Array.<!MessagePort>|!Array.<!Transferable>)=}
 *     opt_targetOriginOrPortsOrTransfer
 * @return {void}
 */
external void postMessage(message,
    [opt_targetOriginOrTransfer /*String|List<es6.Transferable>|undefined*/,
    opt_targetOriginOrPortsOrTransfer /*String|List|undefined*/]);

/**
 * @see https://developer.apple.com/webapps/docs/documentation/AppleApplications/Reference/SafariJSRef/DOMApplicationCache/DOMApplicationCache.html
 * @constructor
 * @implements {EventTarget}
 */
@JsName("DOMApplicationCache")
abstract class DOMApplicationCache implements w3c_event.EventTarget {
// @Ignore
  external DOMApplicationCache.fakeConstructor$();

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  addEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  removeEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /** @override */
  bool dispatchEvent(w3c_event.Event evt);

  /**
   * The current status of the application cache.
   * @type {number}
   */
  num status;

  /**
   * Sent when the update process finishes for the first time; that is, the first
   * time an application cache is saved.
   * @type {?function(!Event)}
   */
  Func1<w3c_event.Event, dynamic> oncached;

  /**
   * Sent when the cache update process begins.
   * @type {?function(!Event)}
   */
  Func1<w3c_event.Event, dynamic> onchecking;

  /**
   * Sent when the update process begins downloading resources in the manifest
   * file.
   * @type {?function(!Event)}
   */
  Func1<w3c_event.Event, dynamic> ondownloading;

  /**
   * Sent when an error occurs.
   * @type {?function(!Event)}
   */
  Func1<w3c_event.Event, dynamic> onerror;

  /**
   * Sent when the update process finishes but the manifest file does not change.
   * @type {?function(!Event)}
   */
  Func1<w3c_event.Event, dynamic> onnoupdate;

  /**
   * Sent when each resource in the manifest file begins to download.
   * @type {?function(!Event)}
   */
  Func1<w3c_event.Event, dynamic> onprogress;

  /**
   * Sent when there is an existing application cache, the update process
   * finishes, and there is a new application cache ready for use.
   * @type {?function(!Event)}
   */
  Func1<w3c_event.Event, dynamic> onupdateready;

  /**
   * Replaces the active cache with the latest version.
   * @throws {DOMException}
   */
  swapCache();

  /**
   * Manually triggers the update process.
   * @throws {DOMException}
   */
  update();
}
/** @type {DOMApplicationCache} */
@JsName("applicationCache")
external DOMApplicationCache get applicationCache;
@JsName("applicationCache")
external set applicationCache(DOMApplicationCache value);
/**
 * @see https://developer.mozilla.org/En/DOM/Worker/Functions_available_to_workers
 * @param {...string} var_args
 */
external importScripts(
    [String var_args1, String var_args2, String var_args3, String var_args4]);

/**
 * @see http://dev.w3.org/html5/workers/
 * @constructor
 * @implements {EventTarget}
 */
@JsName("WebWorker")
abstract class WebWorker implements w3c_event.EventTarget {
// @Ignore
  external WebWorker.fakeConstructor$();

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  addEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  removeEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /** @override */
  bool dispatchEvent(w3c_event.Event evt);

  /**
   * Stops the worker process
   */
  terminate();

  /**
   * Posts a message to the worker thread.
   * @param {string} message
   */
  postMessage(String message);

  /**
   * Sent when the worker thread posts a message to its creator.
   * @type {?function(!MessageEvent.<*>)}
   */
  Func1<MessageEvent<dynamic>, dynamic> onmessage;

  /**
   * Sent when the worker thread encounters an error.
   * TODO(tbreisacher): Should this change to function(!ErrorEvent)?
   * @type {?function(!Event)}
   */
  Func1<w3c_event.Event, dynamic> onerror;
}

/**
 * @see http://dev.w3.org/html5/workers/
 * @constructor
 * @implements {EventTarget}
 */
@JsName("Worker")
abstract class Worker implements w3c_event.EventTarget {
// @Ignore
  external Worker.fakeConstructor$();

  /**
   * @see http://dev.w3.org/html5/workers/
   * @constructor
   * @implements {EventTarget}
   */
  external factory Worker(opt_arg0);

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  addEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  removeEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /** @override */
  bool dispatchEvent(w3c_event.Event evt);

  /**
   * Stops the worker process
   */
  terminate();

  /**
   * Posts a message to the worker thread.
   * @param {*} message
   * @param {Array.<!Transferable>=} opt_transfer
   */
  postMessage(message, [List<es6.Transferable> opt_transfer]);

  /**
   * Posts a message to the worker thread.
   * @param {*} message
   * @param {Array.<!Transferable>=} opt_transfer
   */
  webkitPostMessage(message, [List<es6.Transferable> opt_transfer]);

  /**
   * Sent when the worker thread posts a message to its creator.
   * @type {?function(!MessageEvent.<*>)}
   */
  Func1<MessageEvent<dynamic>, dynamic> onmessage;

  /**
   * Sent when the worker thread encounters an error.
   * TODO(tbreisacher): Should this change to function(!ErrorEvent)?
   * @type {?function(!Event)}
   */
  Func1<w3c_event.Event, dynamic> onerror;
}

/**
 * @see http://dev.w3.org/html5/workers/
 * @param {string} scriptURL The URL of the script to run in the SharedWorker.
 * @param {string=} opt_name A name that can later be used to obtain a
 *     reference to the same SharedWorker.
 * @constructor
 * @implements {EventTarget}
 */
@JsName("SharedWorker")
abstract class SharedWorker implements w3c_event.EventTarget {
// @Ignore
  external SharedWorker.fakeConstructor$();

  /**
   * @see http://dev.w3.org/html5/workers/
   * @param {string} scriptURL The URL of the script to run in the SharedWorker.
   * @param {string=} opt_name A name that can later be used to obtain a
   *     reference to the same SharedWorker.
   * @constructor
   * @implements {EventTarget}
   */
  external factory SharedWorker(String scriptURL, [String opt_name]);

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  addEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  removeEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /** @override */
  bool dispatchEvent(w3c_event.Event evt);

  /**
   * @type {!MessagePort}
   */
  MessagePort port;

  /**
   * Called on network errors for loading the initial script.
   * TODO(tbreisacher): Should this change to function(!ErrorEvent)?
   * @type {?function(!Event)}
   */
  Func1<w3c_event.Event, dynamic> onerror;
}

/**
 * @see http://dev.w3.org/html5/workers/
 * @see http://www.w3.org/TR/url-1/#dom-urlutilsreadonly
 * @interface
 */
@JsName("WorkerLocation")
abstract class WorkerLocation {
// @Ignore
  external WorkerLocation.fakeConstructor$();

  /** @type {string} */
  String href;

  /** @type {string} */
  String origin;

  /** @type {string} */
  String protocol;

  /** @type {string} */
  String host;

  /** @type {string} */
  String hostname;

  /** @type {string} */
  String port;

  /** @type {string} */
  String pathname;

  /** @type {string} */
  String search;

  /** @type {string} */
  String hash;
}

/**
 * @see http://dev.w3.org/html5/workers/
 * @interface
 * @extends {EventTarget}
 */
@JsName("WorkerGlobalScope")
abstract class WorkerGlobalScope {
// @Ignore
  external WorkerGlobalScope.fakeConstructor$();

  /** @type {WorkerGlobalScope} */
  WorkerGlobalScope self;

  /** @type {WorkerLocation} */
  WorkerLocation location;

  /**
   * Closes the worker represented by this WorkerGlobalScope.
   */
  close();

  /**
   * Sent when the worker encounters an error.
   * @type {?function(!Event)}
   */
  Func1<w3c_event.Event, dynamic> onerror;

  /**
   * Sent when the worker goes offline.
   * @type {?function(!Event)}
   */
  Func1<w3c_event.Event, dynamic> onoffline;

  /**
   * Sent when the worker goes online.
   * @type {?function(!Event)}
   */
  Func1<w3c_event.Event, dynamic> ononline;

  /**
   * @see https://fetch.spec.whatwg.org/#dom-global-fetch
   * @param {!RequestInfo} input
   * @param {RequestInit=} opt_init
   * @return {!Promise.<!Response>}
   */
  es6.Promise<fetchapi.Response> fetch(input /*fetchapi.Request|String*/,
      [fetchapi.RequestInit opt_init]);
}

/**
 * @see http://dev.w3.org/html5/workers/
 * @interface
 * @extends {WorkerGlobalScope}
 */
@JsName("DedicatedWorkerGlobalScope")
abstract class DedicatedWorkerGlobalScope {
// @Ignore
  external DedicatedWorkerGlobalScope.fakeConstructor$();

  /**
   * Posts a message to creator of this worker.
   * @param {*} message
   * @param {Array.<!Transferable>=} opt_transfer
   */
  postMessage(message, [List<es6.Transferable> opt_transfer]);

  /**
   * Posts a message to creator of this worker.
   * @param {*} message
   * @param {Array.<!Transferable>=} opt_transfer
   */
  webkitPostMessage(message, [List<es6.Transferable> opt_transfer]);

  /**
   * Sent when the creator posts a message to this worker.
   * @type {?function(!MessageEvent.<*>)}
   */
  Func1<MessageEvent<dynamic>, dynamic> onmessage;
}

/**
 * @see http://dev.w3.org/html5/workers/
 * @interface
 * @extends {WorkerGlobalScope}
 */
@JsName("SharedWorkerGlobalScope")
abstract class SharedWorkerGlobalScope {
// @Ignore
  external SharedWorkerGlobalScope.fakeConstructor$();

  /** @type {string} */
  String name;

  /**
   * Sent when a connection to this worker is opened.
   * @type {?function(!Event)}
   */
  Func1<w3c_event.Event, dynamic> onconnect;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see https://developer.mozilla.org/en-US/docs/Web/API/HTMLMediaElement
 */
@JsName("HTMLMediaElement")
abstract class HTMLMediaElement extends w3c_dom2.HTMLElement {
// @Ignore
  external HTMLMediaElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {number}
   * @const
   */
  external static num get HAVE_NOTHING;

  external static set HAVE_NOTHING(num value);

  /**
   * @type {number}
   * @const
   */
  external static num get HAVE_CURRENT_DATA;

  external static set HAVE_CURRENT_DATA(num value);

  /**
   * @type {number}
   * @const
   */
  external static num get HAVE_METADATA;

  external static set HAVE_METADATA(num value);

  /**
   * @type {number}
   * @const
   */
  external static num get HAVE_ENOUGH_DATA;

  external static set HAVE_ENOUGH_DATA(num value);

  /**
   * @type {number}
   * @const
   */
  external static num get HAVE_FUTURE_DATA;

  external static set HAVE_FUTURE_DATA(num value);

  /** @type {MediaError} */
  MediaError error;

  /** @type {string} */
  String src;

  /** @type {string} */
  String currentSrc;

  /** @type {number} */
  num networkState;

  /** @type {boolean} */
  bool autobuffer;

  /** @type {TimeRanges} */
  TimeRanges buffered;

  /**
   * Loads the media element.
   */
  load();

  /**
   * @param {string} type Type of the element in question in question.
   * @return {string} Whether it can play the type.
   * @nosideeffects
   */
  String canPlayType(String type);

  /**
   * Callback when the media is buffered and ready to play through.
   * @type {function(!Event)}
   */
  external oncanplaythrough(w3c_event.Event arg1);
  /** @type {number} */
  num readyState;

  /** @type {boolean} */
  bool seeking;

  /**
   * The current time, in seconds.
   * @type {number}
   */
  num currentTime;

  /**
   * The absolute timeline offset.
   * @return {!Date}
   */
  DateTime getStartDate();

  /**
   * The length of the media in seconds.
   * @type {number}
   */
  num duration;

  /** @type {boolean} */
  bool paused;

  /** @type {number} */
  num defaultPlaybackRate;

  /** @type {number} */
  num playbackRate;

  /** @type {TimeRanges} */
  TimeRanges played;

  /** @type {TimeRanges} */
  TimeRanges seekable;

  /** @type {boolean} */
  bool ended;

  /** @type {boolean} */
  bool autoplay;

  /** @type {boolean} */
  bool loop;

  /**
   * Starts playing the media.
   */
  play();

  /**
   * Pauses the media.
   */
  pause();

  /** @type {boolean} */
  bool controls;

  /**
   * The audio volume, from 0.0 (silent) to 1.0 (loudest).
   * @type {number}
   */
  num volume;

  /** @type {boolean} */
  bool muted;

  /**
   * @see http://www.whatwg.org/specs/web-apps/current-work/multipage/the-video-element.html#dom-media-addtexttrack
   * @param {string} kind Kind of the text track.
   * @param {string=} opt_label Label of the text track.
   * @param {string=} opt_language Language of the text track.
   * @return {TextTrack} TextTrack object added to the media element.
   */
  TextTrack addTextTrack(String kind, [String opt_label, String opt_language]);

  /** @type {TextTrackList} */
  TextTrackList textTracks;
}

/**
 * @see http://www.whatwg.org/specs/web-apps/current-work/multipage/the-video-element.html#texttracklist
 * @constructor
 */
@JsName("TextTrackList")
abstract class TextTrackList {
// @Ignore
  external TextTrackList.fakeConstructor$();

  /** @type {number} */
  num length;

  /**
   * @param {string} id
   * @return {TextTrack}
   */
  TextTrack getTrackById(String id);
}

/**
 * @see http://www.whatwg.org/specs/web-apps/current-work/multipage/the-video-element.html#texttrack
 * @constructor
 * @implements {EventTarget}
 */
@JsName("TextTrack")
abstract class TextTrack implements w3c_event.EventTarget {
// @Ignore
  external TextTrack.fakeConstructor$();

  /**
   * @param {TextTrackCue} cue
   */
  addCue(TextTrackCue cue);

  /**
   * @param {TextTrackCue} cue
   */
  removeCue(TextTrackCue cue);

  /**
   * @const {TextTrackCueList}
   */
  TextTrackCueList activeCues;

  /**
   * @const {TextTrackCueList}
   */
  TextTrackCueList cues;

  /** @override */
  addEventListener(
      String type, bool listener(w3c_event.Event arg1), bool useCapture);

  /** @override */
  bool dispatchEvent(w3c_event.Event evt);

  /** @override */
  removeEventListener(
      String type, bool listener(w3c_event.Event arg1), bool useCapture);
}

/**
 * @see http://www.whatwg.org/specs/web-apps/current-work/multipage/the-video-element.html#texttrackcuelist
 * @constructor
 */
@JsName("TextTrackCueList")
abstract class TextTrackCueList {
// @Ignore
  external TextTrackCueList.fakeConstructor$();

  /** @const {number} */
  num length;

  /**
   * @param {string} id
   * @return {TextTrackCue}
   */
  TextTrackCue getCueById(String id);
}

/**
 * @see http://www.whatwg.org/specs/web-apps/current-work/multipage/the-video-element.html#texttrackcue
 * @constructor
 * @param {number} startTime
 * @param {number} endTime
 * @param {string} text
 */
@JsName("TextTrackCue")
abstract class TextTrackCue {
// @Ignore
  external TextTrackCue.fakeConstructor$();

  /**
   * @see http://www.whatwg.org/specs/web-apps/current-work/multipage/the-video-element.html#texttrackcue
   * @constructor
   * @param {number} startTime
   * @param {number} endTime
   * @param {string} text
   */
  external factory TextTrackCue(num startTime, num endTime, String text);

  /** @type {string} */
  String id;

  /** @type {number} */
  num startTime;

  /** @type {number} */
  num endTime;

  /** @type {string} */
  String text;
}

/**
 * @see http://dev.w3.org/html5/webvtt/#the-vttcue-interface
 * @constructor
 * @extends {TextTrackCue}
 */
@JsName("VTTCue")
abstract class VTTCue extends TextTrackCue {
// @Ignore
  external VTTCue.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @see http://dev.w3.org/html5/webvtt/#the-vttcue-interface
   * @constructor
   * @extends {TextTrackCue}
   */
  external factory VTTCue(startTime, endTime, text);
}

/**
 * @constructor
 * @extends {HTMLMediaElement}
 */
@JsName("HTMLAudioElement")
abstract class HTMLAudioElement extends HTMLMediaElement {
// @Ignore
  external HTMLAudioElement.fakeConstructor$() : super.fakeConstructor$();
}

/**
 * @constructor
 * @extends {HTMLMediaElement}
 * The webkit-prefixed attributes are defined in
 * https://code.google.com/p/chromium/codesearch#chromium/src/third_party/WebKit/Source/core/html/HTMLVideoElement.idl
 */
@JsName("HTMLVideoElement")
abstract class HTMLVideoElement extends HTMLMediaElement {
// @Ignore
  external HTMLVideoElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * Starts displaying the video in full screen mode.
   */
  webkitEnterFullscreen();

  /**
   * Starts displaying the video in full screen mode.
   */
  webkitEnterFullScreen();

  /**
   * Stops displaying the video in full screen mode.
   */
  webkitExitFullscreen();

  /**
   * Stops displaying the video in full screen mode.
   */
  webkitExitFullScreen();

  /** @type {string} */
  String width;

  /** @type {string} */
  String height;

  /** @type {number} */
  num videoWidth;

  /** @type {number} */
  num videoHeight;

  /** @type {string} */
  String poster;

  /** @type {boolean} */
  bool webkitSupportsFullscreen;

  /** @type {boolean} */
  bool webkitDisplayingFullscreen;

  /** @type {number} */
  num webkitDecodedFrameCount;

  /** @type {number} */
  num webkitDroppedFrameCount;
}

/**
 * @constructor
 */
@JsName("MediaError")
abstract class MediaError {
// @Ignore
  external MediaError.fakeConstructor$();

  /** @type {number} */
  num code;
}

/**
 * @see http://dev.w3.org/html5/spec/comms.html#messagechannel
 * @constructor
 */
@JsName("MessageChannel")
abstract class MessageChannel {
// @Ignore
  external MessageChannel.fakeConstructor$();

  /**
   * Returns the first port.
   * @type {!MessagePort}
   */
  MessagePort port1;

  /**
   * Returns the second port.
   * @type {!MessagePort}
   */
  MessagePort port2;
}

/**
 * @see http://dev.w3.org/html5/spec/comms.html#messageport
 * @constructor
 * @implements {EventTarget}
 * @implements {Transferable}
 */
@JsName("MessagePort")
abstract class MessagePort implements w3c_event.EventTarget, es6.Transferable {
// @Ignore
  external MessagePort.fakeConstructor$();

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  addEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  removeEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /** @override */
  bool dispatchEvent(w3c_event.Event evt);

  /**
   * Posts a message through the channel, optionally with the given
   * Array of Transferables.
   * @param {*} message
   * @param {Array.<!Transferable>=} opt_transfer
   */
  postMessage(message, [List<es6.Transferable> opt_transfer]);

  /**
   * Begins dispatching messages received on the port.
   */
  start();

  /**
   * Disconnects the port, so that it is no longer active.
   */
  close();

  /**
   * TODO(blickly): Change this to MessageEvent.<*> and add casts as needed
   * @type {?function(!MessageEvent.<?>)}
   */
  Func1<MessageEvent<dynamic>, dynamic> onmessage;
}

/**
 * @see http://dev.w3.org/html5/spec/comms.html#messageevent
 * @constructor
 * @extends {Event}
 * @template T
 */
@JsName("MessageEvent")
abstract class MessageEvent<T> extends w3c_event.Event {
// @Ignore
  external MessageEvent.fakeConstructor$() : super.fakeConstructor$();

  /**
   * The data payload of the message.
   * @type {T}
   */
  T data;

  /**
   * The origin of the message, for server-sent events and cross-document
   * messaging.
   * @type {string}
   */
  String origin;

  /**
   * The last event ID, for server-sent events.
   * @type {string}
   */
  String lastEventId;

  /**
   * The window that dispatched the event.
   * @type {Window}
   */
  w3c_dom1.Window source;

  /**
   * The Array of MessagePorts sent with the message, for cross-document
   * messaging and channel messaging.
   * @type {Array.<MessagePort>}
   */
  List<MessagePort> ports;

  /**
   * Initializes the event in a manner analogous to the similarly-named methods in
   * the DOM Events interfaces.
   * @param {string} typeArg
   * @param {boolean} canBubbleArg
   * @param {boolean} cancelableArg
   * @param {T} dataArg
   * @param {string} originArg
   * @param {string} lastEventIdArg
   * @param {Window} sourceArg
   * @param {Array.<MessagePort>} portsArg
   */
  initMessageEvent(String typeArg, bool canBubbleArg, bool cancelableArg,
      T dataArg, String originArg, String lastEventIdArg,
      w3c_dom1.Window sourceArg, List<MessagePort> portsArg);

  /**
   * Initializes the event in a manner analogous to the similarly-named methods in
   * the DOM Events interfaces.
   * @param {string} namespaceURI
   * @param {string} typeArg
   * @param {boolean} canBubbleArg
   * @param {boolean} cancelableArg
   * @param {T} dataArg
   * @param {string} originArg
   * @param {string} lastEventIdArg
   * @param {Window} sourceArg
   * @param {Array.<MessagePort>} portsArg
   */
  initMessageEventNS(String namespaceURI, String typeArg, bool canBubbleArg,
      bool cancelableArg, T dataArg, String originArg, String lastEventIdArg,
      w3c_dom1.Window sourceArg, List<MessagePort> portsArg);
}

/**
 * HTML5 DataTransfer class.
 *
 * We say that this extends ClipboardData, because Event.prototype.clipboardData
 * is a DataTransfer on WebKit but a ClipboardData on IE. The interfaces are so
 * similar that it's easier to merge them.
 *
 * @see http://www.w3.org/TR/2011/WD-html5-20110113/dnd.html
 * @see http://www.whatwg.org/specs/web-apps/current-work/multipage/dnd.html
 * @see http://developers.whatwg.org/dnd.html#datatransferitem
 * @constructor
 * @extends {ClipboardData}
 */
@JsName("DataTransfer")
abstract class DataTransfer extends ie_dom.ClipboardData {
// @Ignore
  external DataTransfer.fakeConstructor$() : super.fakeConstructor$();

  /** @type {string} */
  String dropEffect;

  /** @type {string} */
  String effectAllowed;

  /** @type {Array.<string>} */
  List<String> types;

  /** @type {FileList} */
  FileList files;

  /**
   * @param {string=} opt_format Format for which to remove data.
   * @override
   */
  clearData([String opt_format]);

  /**
   * @param {string} format Format for which to set data.
   * @param {string} data Data to add.
   * @override
   */
  bool setData(String format, String data);

  /**
   * @param {string} format Format for which to set data.
   * @return {string} Data for the given format.
   * @override
   */
  String getData(String format);

  /**
   * @param {HTMLElement} img The image to use when dragging.
   * @param {number} x Horizontal position of the cursor.
   * @param {number} y Vertical position of the cursor.
   */
  setDragImage(w3c_dom2.HTMLElement img, num x, num y);

  /**
   * @param {HTMLElement} elem Element to receive drag result events.
   */
  addElement(w3c_dom2.HTMLElement elem);

  /** @type {!DataTransferItemList} */
  DataTransferItemList items;
}
/**
 * @typedef {{
 *   bubbles: (boolean|undefined),
 *   cancelable: (boolean|undefined),
 *   view: (Window|undefined),
 *   detail: (number|undefined),
 *   screenX: (number|undefined),
 *   screenY: (number|undefined),
 *   clientX: (number|undefined),
 *   clientY: (number|undefined),
 *   ctrlKey: (boolean|undefined),
 *   shiftKey: (boolean|undefined),
 *   altKey: (boolean|undefined),
 *   metaKey: (boolean|undefined),
 *   button: (number|undefined),
 *   buttons: (number|undefined),
 *   relatedTarget: (EventTarget|undefined),
 *   deltaX: (number|undefined),
 *   deltaY: (number|undefined),
 *   deltaZ: (number|undefined),
 *   deltaMode: (number|undefined)
 * }}
 */
@JsName()
abstract class WheelEventInit {
  bool altKey;
  bool bubbles;
  num button;
  num buttons;
  bool cancelable;
  num clientX;
  num clientY;
  bool ctrlKey;
  num deltaMode;
  num deltaX;
  num deltaY;
  num deltaZ;
  num detail;
  bool metaKey;
  w3c_event.EventTarget relatedTarget;
  num screenX;
  num screenY;
  bool shiftKey;
  w3c_dom1.Window view;
}

@JsName("WheelEvent")
abstract class WheelEvent extends w3c_event.MouseEvent {
// @Ignore
  external WheelEvent.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @param {string} type
   * @param {WheelEventInit=} opt_eventInitDict
   * @see http://www.w3.org/TR/DOM-Level-3-Events/#interface-WheelEvent
   * @constructor
   * @extends {MouseEvent}
   */
  external factory WheelEvent(String type, [WheelEventInit opt_eventInitDict]);

  /** @const {number} */
  num deltaX;

  /** @const {number} */
  num deltaY;

  /** @const {number} */
  num deltaZ;

  /** @const {number} */
  num deltaMode;
}

@JsName("DataTransferItem")
abstract class DataTransferItem {
// @Ignore
  external DataTransferItem.fakeConstructor$();

  /** @type {string} */
  String kind;

  /** @type {string} */
  String type;

  /**
   * @param {function(string)} callback
   * @nosideeffects
   */
  getAsString(callback(String arg1));

  /**
   * @return {?File} The file corresponding to this item, or null.
   * @nosideeffects
   */
  fileapi.File getAsFile();

  /**
   * @return {?Entry} The Entry corresponding to this item, or null. Note that
   * despite its name,this method only works in Chrome, and will eventually
   * be renamed to {@code getAsEntry}.
   * @nosideeffects
   */
  fileapi.Entry webkitGetAsEntry();
}

@JsName("DataTransferItemList")
abstract class DataTransferItemList {
// @Ignore
  external DataTransferItemList.fakeConstructor$();

  /** @type {number} */
  num length;

  /**
   * @param {number} i File to return from the list.
   * @return {DataTransferItem} The ith DataTransferItem in the list, or null.
   * @nosideeffects
   */
  DataTransferItem item(num i);

  /**
   * Adds an item to the list.
   * @param {string|!File} data Data for the item being added.
   * @param {string=} opt_type Mime type of the item being added. MUST be present
   *     if the {@code data} parameter is a string.
   */
  add(data /*String|fileapi.File*/, [String opt_type]);

  /**
   * Removes an item from the list.
   * @param {number} i File to remove from the list.
   */
  remove(num i);

  /**
   * Removes all items from the list.
   */
  clear();
}

/**
 * @see http://www.whatwg.org/specs/web-apps/current-work/multipage/dnd.html#the-dragevent-interface
 * @constructor
 * @extends {MouseEvent}
 */
@JsName("DragEvent")
abstract class DragEvent extends w3c_event.MouseEvent {
// @Ignore
  external DragEvent.fakeConstructor$() : super.fakeConstructor$();

  /** @type {DataTransfer} */
  DataTransfer dataTransfer;
}
/**
 * @typedef {{
 *   lengthComputable: (boolean|undefined),
 *   loaded: (number|undefined),
 *   total: (number|undefined)
 * }}
 */
@JsName()
abstract class ProgressEventInit {
  bool lengthComputable;
  num loaded;
  num total;
}

/**
 * @constructor
 * @param {string} type
 * @param {ProgressEventInit=} opt_progressEventInitDict
 * @extends {Event}
 * @see https://developer.mozilla.org/en-US/docs/Web/API/ProgressEvent
 */
@JsName("ProgressEvent")
abstract class ProgressEvent extends w3c_event.Event {
// @Ignore
  external ProgressEvent.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @constructor
   * @param {string} type
   * @param {ProgressEventInit=} opt_progressEventInitDict
   * @extends {Event}
   * @see https://developer.mozilla.org/en-US/docs/Web/API/ProgressEvent
   */
  external factory ProgressEvent(String type,
      [ProgressEventInit opt_progressEventInitDict]);

  /** @type {number} */
  num total;

  /** @type {number} */
  num loaded;

  /** @type {boolean} */
  bool lengthComputable;
}

/**
 * @constructor
 */
@JsName("TimeRanges")
abstract class TimeRanges {
// @Ignore
  external TimeRanges.fakeConstructor$();

  /** @type {number} */
  num length;

  /**
   * @param {number} index The index.
   * @return {number} The start time of the range at index.
   * @throws {DOMException}
   */
  num start(num index);

  /**
   * @param {number} index The index.
   * @return {number} The end time of the range at index.
   * @throws {DOMException}
   */
  num end(num index);
}

/**
 * @see http://dev.w3.org/html5/websockets/
 * @constructor
 * @param {string} url
 * @param {string=} opt_protocol
 * @implements {EventTarget}
 */
@JsName("WebSocket")
abstract class WebSocket implements w3c_event.EventTarget {
// @Ignore
  external WebSocket.fakeConstructor$();

  /**
   * @see http://dev.w3.org/html5/websockets/
   * @constructor
   * @param {string} url
   * @param {string=} opt_protocol
   * @implements {EventTarget}
   */
  external factory WebSocket(String url, [String opt_protocol]);

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  addEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  removeEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /** @override */
  bool dispatchEvent(w3c_event.Event evt);

  /**
   * Returns the URL value that was passed to the constructor.
   * @type {string}
   */
  String URL;

  /**
   * Represents the state of the connection.
   * @type {number}
   */
  num readyState;

  /**
   * Returns the number of bytes that have been queued but not yet sent.
   * @type {number}
   */
  num bufferedAmount;

  /**
   * An event handler called on open event.
   * @type {?function(!Event)}
   */
  Func1<w3c_event.Event, dynamic> onopen;

  /**
   * An event handler called on message event.
   * TODO(blickly): Change this to MessageEvent.<*> and add casts as needed
   * @type {?function(!MessageEvent.<?>)}
   */
  Func1<MessageEvent<dynamic>, dynamic> onmessage;

  /**
   * An event handler called on close event.
   * @type {?function(!Event)}
   */
  Func1<w3c_event.Event, dynamic> onclose;

  /**
   * Transmits data using the connection.
   * @param {string|ArrayBuffer|ArrayBufferView} data
   * @return {boolean}
   */
  bool send(data /*String|es6.ArrayBuffer|null|es6.ArrayBufferView*/);

  /**
   * Closes the Web Socket connection or connection attempt, if any.
   */
  close();

  /**
   * @type {string} Sets the type of data (blob or arraybuffer) for binary data.
   */
  String binaryType;
}

/**
 * @see http://www.whatwg.org/specs/web-apps/current-work/#popstateevent
 * @constructor
 * @extends {Event}
 *
 * @param {string} type
 * @param {{state: *}=} opt_eventInitDict
 */
@JsName("PopStateEvent")
abstract class PopStateEvent extends w3c_event.Event {
// @Ignore
  external PopStateEvent.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @see http://www.whatwg.org/specs/web-apps/current-work/#popstateevent
   * @constructor
   * @extends {Event}
   *
   * @param {string} type
   * @param {{state: *}=} opt_eventInitDict
   */
  external factory PopStateEvent(String type,
      [EventInitDictRecord opt_eventInitDict]);

  /**
   * @type {*}
   */
  dynamic state;

  /**
   * Initializes the event after it has been created with document.createEvent
   * @param {string} typeArg
   * @param {boolean} canBubbleArg
   * @param {boolean} cancelableArg
   * @param {*} stateArg
   */
  initPopStateEvent(
      String typeArg, bool canBubbleArg, bool cancelableArg, stateArg);
}

/**
 * @see http://www.whatwg.org/specs/web-apps/current-work/#hashchangeevent
 * @constructor
 * @extends {Event}
 *
 * @param {string} type
 * @param {{oldURL: string, newURL: string}=} opt_eventInitDict
 */
@JsName("HashChangeEvent")
abstract class HashChangeEvent extends w3c_event.Event {
// @Ignore
  external HashChangeEvent.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @see http://www.whatwg.org/specs/web-apps/current-work/#hashchangeevent
   * @constructor
   * @extends {Event}
   *
   * @param {string} type
   * @param {{oldURL: string, newURL: string}=} opt_eventInitDict
   */
  external factory HashChangeEvent(String type,
      [EventInitDictRecord2 opt_eventInitDict]);

  /** @type {string} */
  String oldURL;

  /** @type {string} */
  String newURL;

  /**
   * Initializes the event after it has been created with document.createEvent
   * @param {string} typeArg
   * @param {boolean} canBubbleArg
   * @param {boolean} cancelableArg
   * @param {string} oldURLArg
   * @param {string} newURLArg
   */
  initHashChangeEvent(String typeArg, bool canBubbleArg, bool cancelableArg,
      String oldURLArg, String newURLArg);
}

/**
 * @see http://www.whatwg.org/specs/web-apps/current-work/#pagetransitionevent
 * @constructor
 * @extends {Event}
 *
 * @param {string} type
 * @param {{persisted: boolean}=} opt_eventInitDict
 */
@JsName("PageTransitionEvent")
abstract class PageTransitionEvent extends w3c_event.Event {
// @Ignore
  external PageTransitionEvent.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @see http://www.whatwg.org/specs/web-apps/current-work/#pagetransitionevent
   * @constructor
   * @extends {Event}
   *
   * @param {string} type
   * @param {{persisted: boolean}=} opt_eventInitDict
   */
  external factory PageTransitionEvent(String type,
      [EventInitDictRecord3 opt_eventInitDict]);

  /** @type {boolean} */
  bool persisted;

  /**
   * Initializes the event after it has been created with document.createEvent
   * @param {string} typeArg
   * @param {boolean} canBubbleArg
   * @param {boolean} cancelableArg
   * @param {*} persistedArg
   */
  initPageTransitionEvent(
      String typeArg, bool canBubbleArg, bool cancelableArg, persistedArg);
}

/**
 * @constructor
 */
@JsName("FileList")
abstract class FileList {
// @Ignore
  external FileList.fakeConstructor$();

  /** @type {number} */
  num length;

  /**
   * @param {number} i File to return from the list.
   * @return {File} The ith file in the list.
   * @nosideeffects
   */
  fileapi.File item(num i);
}

/**
 * XMLHttpRequestEventTarget defines events for checking the status of a data
 * transfer between a client and a server. This should be a common base class
 * for XMLHttpRequest and XMLHttpRequestUpload.
 *
 * @constructor
 * @implements {EventTarget}
 */
@JsName("XMLHttpRequestEventTarget")
abstract class XMLHttpRequestEventTarget implements w3c_event.EventTarget {
// @Ignore
  external XMLHttpRequestEventTarget.fakeConstructor$();

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  addEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  removeEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /** @override */
  bool dispatchEvent(w3c_event.Event evt);
}

/**
 * An event target to track the status of an upload.
 *
 * @constructor
 * @extends {XMLHttpRequestEventTarget}
 */
@JsName("XMLHttpRequestUpload")
abstract class XMLHttpRequestUpload extends XMLHttpRequestEventTarget {
// @Ignore
  external XMLHttpRequestUpload.fakeConstructor$() : super.fakeConstructor$();
}

/**
 * @param {number=} opt_width
 * @param {number=} opt_height
 * @constructor
 * @extends {HTMLImageElement}
 */
@JsName("Image")
abstract class Image extends w3c_dom2.HTMLImageElement {
// @Ignore
  external Image.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @param {number=} opt_width
   * @param {number=} opt_height
   * @constructor
   * @extends {HTMLImageElement}
   */
  external factory Image([num opt_width, num opt_height]);
}

/**
 * @constructor
 * @see https://dom.spec.whatwg.org/#interface-domtokenlist
 */
@JsName("DOMTokenList")
abstract class DOMTokenList {
// @Ignore
  external DOMTokenList.fakeConstructor$();

  /**
   * Returns the number of CSS classes applied to this Element.
   * @type {number}
   */
  num length;

  /**
   * @param {number} index The index of the item to return.
   * @return {string} The CSS class at the specified index.
   * @nosideeffects
   */
  String item(num index);

  /**
   * @param {string} token The CSS class to check for.
   * @return {boolean} Whether the CSS class has been applied to the Element.
   * @nosideeffects
   */
  bool contains(String token);

  /**
   * @param {...string} var_args The CSS class(es) to add to this element.
   */
  add([String var_args1, String var_args2, String var_args3, String var_args4]);

  /**
   * @param {...string} var_args The CSS class(es) to remove from this element.
   */
  remove(
      [String var_args1, String var_args2, String var_args3, String var_args4]);

  /**
   * @param {string} token The CSS class to toggle from this element.
   * @param {boolean=} opt_force True to add the class whether it exists
   *     or not. False to remove the class whether it exists or not.
   *     This argument is not supported on IE 10 and below, according to
   *     the MDN page linked below.
   * @return {boolean} False if the token was removed; True otherwise.
   * @see https://developer.mozilla.org/en-US/docs/Web/API/Element.classList
   */
  bool toggle(String token, [bool opt_force]);

  /**
   * @return {string} A stringified representation of CSS classes.
   * @nosideeffects
   * @override
   */
  String toString();
}

/** @constructor */
@JsName("ValidityState")
abstract class ValidityState {
// @Ignore
  external ValidityState.fakeConstructor$();

  /** @type {boolean} */
  bool customError;

  /** @type {boolean} */
  bool patternMismatch;

  /** @type {boolean} */
  bool rangeOverflow;

  /** @type {boolean} */
  bool rangeUnderflow;

  /** @type {boolean} */
  bool stepMismatch;

  /** @type {boolean} */
  bool typeMismatch;

  /** @type {boolean} */
  bool tooLong;

  /** @type {boolean} */
  bool valid;

  /** @type {boolean} */
  bool valueMissing;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/html5/the-embed-element.html#htmlembedelement
 */
@JsName("HTMLEmbedElement")
abstract class HTMLEmbedElement extends w3c_dom2.HTMLElement {
// @Ignore
  external HTMLEmbedElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/html5/dimension-attributes.html#dom-dim-width
   */
  String width;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/html5/dimension-attributes.html#dom-dim-height
   */
  String height;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/html5/the-embed-element.html#dom-embed-src
   */
  String src;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/html5/the-embed-element.html#dom-embed-type
   */
  String type;

  /**
   * @param {string|!ArrayBuffer|!Object} message
   * @see https://developers.google.com/native-client/devguide/tutorial
   */
  postMessage(message /*String|Object*/);
}
/**
 * @typedef {{
 *   childList: (boolean|undefined),
 *   attributes: (boolean|undefined),
 *   characterData: (boolean|undefined),
 *   subtree: (boolean|undefined),
 *   attributeOldValue: (boolean|undefined),
 *   characterDataOldValue: (boolean|undefined),
 *   attributeFilter: (!Array<string>|undefined)
 * }}
 */
@JsName()
abstract class MutationObserverInit {
  List<String> attributeFilter;
  bool attributeOldValue;
  bool attributes;
  bool characterData;
  bool characterDataOldValue;
  bool childList;
  bool subtree;
}

/** @constructor */
@JsName("MutationRecord")
abstract class MutationRecord {
// @Ignore
  external MutationRecord.fakeConstructor$();

  /** @type {string} */
  String type;

  /** @type {Node} */
  w3c_dom1.Node target;

  /** @type {NodeList} */
  w3c_dom1.NodeList addedNodes;

  /** @type {NodeList} */
  w3c_dom1.NodeList removedNodes;

  /** @type {Node} */
  w3c_dom1.Node previouSibling;

  /** @type {Node} */
  w3c_dom1.Node nextSibling;

  /** @type {?string} */
  String attributeName;

  /** @type {?string} */
  String attributeNamespace;

  /** @type {?string} */
  String oldValue;
}

/**
 * @see http://www.w3.org/TR/domcore/#mutation-observers
 * @param {function(Array.<MutationRecord>, MutationObserver)} callback
 * @constructor
 */
@JsName("MutationObserver")
abstract class MutationObserver {
// @Ignore
  external MutationObserver.fakeConstructor$();

  /**
   * @see http://www.w3.org/TR/domcore/#mutation-observers
   * @param {function(Array.<MutationRecord>, MutationObserver)} callback
   * @constructor
   */
  external factory MutationObserver(
      callback(List<MutationRecord> arg1, MutationObserver arg2));

  /**
   * @param {Node} target
   * @param {MutationObserverInit=} options
   */
  observe(w3c_dom1.Node target, [MutationObserverInit options]);

  disconnect();
}

/**
 * Definition of ShadowRoot interface,
 * @see http://www.w3.org/TR/shadow-dom/#api-shadow-root
 * @constructor
 * @extends {DocumentFragment}
 */
@JsName("ShadowRoot")
abstract class ShadowRoot extends w3c_dom1.DocumentFragment {
// @Ignore
  external ShadowRoot.fakeConstructor$() : super.fakeConstructor$();

  /**
   * The host element that a ShadowRoot is attached to.
   * Note: this is not yet W3C standard but is undergoing development.
   * W3C feature tracking bug:
   * https://www.w3.org/Bugs/Public/show_bug.cgi?id=22399
   * Draft specification:
   * https://dvcs.w3.org/hg/webcomponents/raw-file/6743f1ace623/spec/shadow/index.html#shadow-root-object
   * @type {!Element}
   */
  w3c_dom1.Element host;

  /**
   * @param {string} id id.
   * @return {HTMLElement}
   * @nosideeffects
   */
  w3c_dom2.HTMLElement getElementById(String id);

  /**
   * @param {string} className
   * @return {!NodeList}
   * @nosideeffects
   */
  w3c_dom1.NodeList getElementsByClassName(String className);

  /**
   * @param {string} tagName
   * @return {!NodeList}
   * @nosideeffects
   */
  w3c_dom1.NodeList getElementsByTagName(String tagName);

  /**
   * @param {string} namespace
   * @param {string} localName
   * @return {!NodeList}
   * @nosideeffects
   */
  w3c_dom1.NodeList getElementsByTagNameNS(String namespace, String localName);

  /**
   * @return {Selection}
   * @nosideeffects
   */
  gecko_dom.Selection getSelection();

  /**
   * @param {number} x
   * @param {number} y
   * @return {Element}
   * @nosideeffects
   */
  w3c_dom1.Element elementFromPoint(num x, num y);

  /**
   * @type {boolean}
   */
  bool applyAuthorStyles;

  /**
   * @type {boolean}
   */
  bool resetStyleInheritance;

  /**
   * @type {Element}
   */
  w3c_dom1.Element activeElement;

  /**
   * @type {?ShadowRoot}
   */
  ShadowRoot olderShadowRoot;

  /**
   * @type {string}
   */
  String innerHTML;

  /**
   * @type {!StyleSheetList}
   */
  w3c_css.StyleSheetList styleSheets;
}

/**
 * @see http://www.w3.org/TR/shadow-dom/#the-content-element
 * @constructor
 * @extends {HTMLElement}
 */
@JsName("HTMLContentElement")
abstract class HTMLContentElement extends w3c_dom2.HTMLElement {
// @Ignore
  external HTMLContentElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {!string}
   */
  String select;

  /**
   * @return {!NodeList}
   */
  w3c_dom1.NodeList getDistributedNodes();
}

/**
 * @see http://www.w3.org/TR/shadow-dom/#the-shadow-element
 * @constructor
 * @extends {HTMLElement}
 */
@JsName("HTMLShadowElement")
abstract class HTMLShadowElement extends w3c_dom2.HTMLElement {
// @Ignore
  external HTMLShadowElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @return {!NodeList}
   */
  w3c_dom1.NodeList getDistributedNodes();
}

/**
 * @see http://www.w3.org/TR/html5/webappapis.html#the-errorevent-interface
 *
 * @constructor
 * @extends {Event}
 *
 * @param {string} type
 * @param {ErrorEventInit=} opt_eventInitDict
 */
@JsName("ErrorEvent")
abstract class ErrorEvent extends w3c_event.Event {
// @Ignore
  external ErrorEvent.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @see http://www.w3.org/TR/html5/webappapis.html#the-errorevent-interface
   *
   * @constructor
   * @extends {Event}
   *
   * @param {string} type
   * @param {ErrorEventInit=} opt_eventInitDict
   */
  external factory ErrorEvent(String type, [ErrorEventInit opt_eventInitDict]);

  /** @const {string} */
  String message;

  /** @const {string} */
  String filename;

  /** @const {number} */
  num lineno;

  /** @const {number} */
  num colno;

  /** @const {*} */
  dynamic error;
}
/**
 * @see http://www.w3.org/TR/html5/webappapis.html#the-errorevent-interface
 *
 * @typedef {{
 *   bubbles: (boolean|undefined),
 *   cancelable: (boolean|undefined),
 *   message: string,
 *   filename: string,
 *   lineno: number,
 *   colno: number,
 *   error: *
 * }}
 */
@JsName()
abstract class ErrorEventInit {
  bool bubbles;
  bool cancelable;
  num colno;
  dynamic error;
  String filename;
  num lineno;
  String message;
}

/**
 * @constructor
 * @see https://html.spec.whatwg.org/multipage/embedded-content.html#the-picture-element
 * @extends {HTMLElement}
 */
@JsName("HTMLPictureElement")
abstract class HTMLPictureElement extends w3c_dom2.HTMLElement {
// @Ignore
  external HTMLPictureElement.fakeConstructor$() : super.fakeConstructor$();
}

/**
 * @constructor
 * @see https://html.spec.whatwg.org/multipage/embedded-content.html#the-picture-element
 * @extends {HTMLElement}
 */
@JsName("HTMLSourceElement")
abstract class HTMLSourceElement extends w3c_dom2.HTMLElement {
// @Ignore
  external HTMLSourceElement.fakeConstructor$() : super.fakeConstructor$();

  /** @type {string} */
  String media;

  /** @type {string} */
  String sizes;

  /** @type {string} */
  String src;

  /** @type {string} */
  String srcset;

  /** @type {string} */
  String type;
}

/**
 * @see http://www.w3.org/html/wg/drafts/html/master/interactive-elements.html#the-details-element
 * @constructor
 * @extends {HTMLElement}
 */
@JsName("HTMLDetailsElement")
abstract class HTMLDetailsElement extends w3c_dom2.HTMLElement {
// @Ignore
  external HTMLDetailsElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @see http://www.w3.org/html/wg/drafts/html/master/interactive-elements.html#dom-details-open
   * @type {boolean}
   */
  bool open;
}

/**
 * @see http://www.w3.org/html/wg/drafts/html/master/interactive-elements.html#the-menuitem-element
 * @constructor
 * @extends {HTMLElement}
 */
@JsName("HTMLMenuItemElement")
abstract class HTMLMenuItemElement extends w3c_dom2.HTMLElement {
// @Ignore
  external HTMLMenuItemElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @see http://www.w3.org/html/wg/drafts/html/master/interactive-elements.html#dom-menuitem-type
   * @type {string}
   */
  String type;

  /**
   * @see http://www.w3.org/html/wg/drafts/html/master/interactive-elements.html#dom-menuitem-label
   * @type {string}
   */
  String label;

  /**
   * @see http://www.w3.org/html/wg/drafts/html/master/interactive-elements.html#dom-menuitem-icon
   * @type {string}
   */
  String icon;

  /**
   * @see http://www.w3.org/html/wg/drafts/html/master/interactive-elements.html#dom-menuitem-disabled
   * @type {boolean}
   */
  bool disabled;

  /**
   * @see http://www.w3.org/html/wg/drafts/html/master/interactive-elements.html#dom-menuitem-checked
   * @type {boolean}
   */
  bool checked;

  /**
   * @see http://www.w3.org/html/wg/drafts/html/master/interactive-elements.html#dom-menuitem-radiogroup
   * @type {string}
   */
  String radiogroup;

  /**
   * @see http://www.w3.org/html/wg/drafts/html/master/interactive-elements.html#dom-menuitem-default
   * @type {boolean}
   */
  bool default$;
}

/**
 * @see http://www.w3.org/html/wg/drafts/html/master/interactive-elements.html#relatedevent
 * @param {string} type
 * @param {{relatedTarget: (EventTarget|undefined)}=} opt_eventInitDict
 * @constructor
 * @extends {Event}
 */
@JsName("RelatedEvent")
abstract class RelatedEvent extends w3c_event.Event {
// @Ignore
  external RelatedEvent.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @see http://www.w3.org/html/wg/drafts/html/master/interactive-elements.html#relatedevent
   * @param {string} type
   * @param {{relatedTarget: (EventTarget|undefined)}=} opt_eventInitDict
   * @constructor
   * @extends {Event}
   */
  external factory RelatedEvent(String type,
      [EventInitDictRecord4 opt_eventInitDict]);

  /**
   * @see http://www.w3.org/html/wg/drafts/html/master/interactive-elements.html#dom-relatedevent-relatedtarget
   * @type {EventTarget|undefined}
   */
  w3c_event.EventTarget relatedTarget;
}

/**
 * @see http://www.w3.org/html/wg/drafts/html/master/interactive-elements.html#the-dialog-element
 * @constructor
 * @extends {HTMLElement}
 */
@JsName("HTMLDialogElement")
abstract class HTMLDialogElement extends w3c_dom2.HTMLElement {
// @Ignore
  external HTMLDialogElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @see http://www.w3.org/html/wg/drafts/html/master/interactive-elements.html#dom-dialog-open
   * @type {boolean}
   */
  bool open;

  /**
   * @see http://www.w3.org/html/wg/drafts/html/master/interactive-elements.html#dom-dialog-returnvalue
   * @type {string}
   */
  String returnValue;

  /**
   * @see http://www.w3.org/html/wg/drafts/html/master/interactive-elements.html#dom-dialog-show
   * @param {(MouseEvent|Element)=} opt_anchor
   */
  show([opt_anchor /*w3c_event.MouseEvent|null|w3c_dom1.Element|undefined*/]);

  /**
   * @see http://www.w3.org/html/wg/drafts/html/master/interactive-elements.html#dom-dialog-showmodal
   * @param {(MouseEvent|Element)=} opt_anchor
   */
  showModal(
      [opt_anchor /*w3c_event.MouseEvent|null|w3c_dom1.Element|undefined*/]);

  /**
   * @see http://www.w3.org/html/wg/drafts/html/master/interactive-elements.html#dom-dialog-close
   * @param {string=} opt_returnValue
   */
  close([String opt_returnValue]);
}

/**
 * @see https://html.spec.whatwg.org/multipage/scripting.html#the-template-element
 * @constructor
 * @extends {HTMLElement}
 */
@JsName("HTMLTemplateElement")
abstract class HTMLTemplateElement extends w3c_dom2.HTMLElement {
// @Ignore
  external HTMLTemplateElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @see https://html.spec.whatwg.org/multipage/scripting.html#the-template-element
   * @type {!DocumentFragment}
   */
  w3c_dom1.DocumentFragment content;
}
/// This record class is not given a name in the Closure JavaScript code base.
/// The name we have chosen may not make sense.
@JsName()
abstract class EventInitDictRecord {
  dynamic state;
}
/// This record class is not given a name in the Closure JavaScript code base.
/// The name we have chosen may not make sense.
@JsName()
abstract class EventInitDictRecord2 {
  String newURL;
  String oldURL;
}
/// This record class is not given a name in the Closure JavaScript code base.
/// The name we have chosen may not make sense.
@JsName()
abstract class EventInitDictRecord3 {
  bool persisted;
}
/// This record class is not given a name in the Closure JavaScript code base.
/// The name we have chosen may not make sense.
@JsName()
abstract class EventInitDictRecord4 {
  w3c_event.EventTarget relatedTarget;
}
