/*
 * Copyright 2009 The Closure Compiler Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
// This file has been automatically generated.  Please do not edit it manually.
/**
 * @fileoverview Definitions for all iPhone extensions. Created from:
 * http://developer.apple.com/library/safari/navigation/
 *
 * FROM_EXTERN
 * @author agrieve@google.com (Andrew Grieve)
 */
@JsName()
library iphone;

import 'package:js/js.dart';
import 'package:func/func.dart';

import "w3c_dom1.dart" as w3c_dom1;

import "w3c_event.dart" as w3c_event;

/**
 * The Touch class represents a single touch on the surface. A touch is the
 * presence or movement of a finger that is part of a unique multi-touch
 * sequence.
 * @see http://developer.apple.com/library/safari/#documentation/UserExperience/Reference/TouchClassReference/Touch/Touch.html
 * @constructor
 */
@JsName("Touch")
abstract class Touch {
// @Ignore
  external Touch.fakeConstructor$();

  /**
   * The x-coordinate of the touch's location relative to the window's viewport.
   * @type {number}
   */
  num clientX;

  /**
   * The y-coordinate of the touch's location relative to the window's viewport.
   * @type {number}
   */
  num clientY;

  /**
   * The unique identifier for this touch object.
   * @type {number}
   */
  num identifier;

  /**
   * The x-coordinate of the touch's location in page coordinates.
   * @type {number}
   */
  num pageX;

  /**
   * The y-coordinate of the touch's location in page coordinates.
   * @type {number}
   */
  num pageY;

  /**
   * The x-coordinate of the touch's location in screen coordinates.
   * @type {number}
   */
  num screenX;

  /**
   * The y-coordinate of the touch's location in screen coordinates.
   * @type {number}
   */
  num screenY;

  /**
   * The target of this touch.
   * @type {EventTarget}
   */
  w3c_event.EventTarget target;
}

/**
 * The TouchList class is used to represent a collection of Touch objects.
 * @see http://developer.apple.com/library/safari/#documentation/UserExperience/Reference/TouchListClassReference/TouchList/TouchList.html
 * @constructor
 */
@JsName("TouchList")
abstract class TouchList {
// @Ignore
  external TouchList.fakeConstructor$();

  /**
   * The number of Touch objects in this TouchList object.
   * @type {number}
   */
  num length;

  /**
   * Returns the Touch object at the given index.
   * @param {number} index
   * @return {!Touch}
   */
  Touch item(num index);
}

/**
 * The TouchEvent class encapsulates information about a touch event.
 *
 * <p>The system continually sends TouchEvent objects to an application as
 * fingers touch and move across a surface. A touch event provides a snapshot of
 * all touches during a multi-touch sequence, most importantly the touches that
 * are new or have changed for a particular target. A multi-touch sequence
 * begins when a finger first touches the surface. Other fingers may
 * subsequently touch the surface, and all fingers may move across the surface.
 * The sequence ends when the last of these fingers is lifted from the surface.
 * An application receives touch event objects during each phase of any touch.
 * </p>
 *
 * <p>The different types of TouchEvent objects that can occur are:
 * <ul>
 *   <li>touchstart - Sent when a finger for a given event touches the surface.
 *   <li>touchmove - Sent when a given event moves on the surface.
 *   <li>touchend - Sent when a given event lifts from the surface.
 *   <li>touchcancel - Sent when the system cancels tracking for the touch.
 * </ul>
 * TouchEvent objects are combined together to form high-level GestureEvent
 * objects that are also sent during a multi-touch sequence.</p>
 *
 * @see http://developer.apple.com/library/safari/#documentation/UserExperience/Reference/TouchEventClassReference/TouchEvent/TouchEvent.html
 * @extends {UIEvent}
 * @constructor
 */
@JsName("TouchEvent")
abstract class TouchEvent extends w3c_event.UIEvent {
// @Ignore
  external TouchEvent.fakeConstructor$() : super.fakeConstructor$();

  /**
   * A collection of Touch objects representing all touches associated with this
   * target.
   * @type {TouchList}
   */
  TouchList touches;

  /**
   * A collection of Touch objects representing all touches associated with this
   * target.
   * @type {TouchList}
   */
  TouchList targetTouches;

  /**
   * A collection of Touch objects representing all touches that changed in this event.
   * @type {TouchList}
   */
  TouchList changedTouches;

  /**
   * The distance between two fingers since the start of an event as a multiplier
   * of the initial distance. The initial value is 1.0. If less than 1.0, the
   * gesture is pinch close (to zoom out). If greater than 1.0, the gesture is
   * pinch open (to zoom in).
   * @type {number}
   */
  num scale;

  /**
   * The delta rotation since the start of an event, in degrees, where clockwise
   * is positive and counter-clockwise is negative. The initial value is 0.0.
   * @type {number}
   */
  num rotation;

  /**
   * Initializes a newly created TouchEvent object.
   * @param {string} type
   * @param {boolean} canBubble
   * @param {boolean} cancelable
   * @param {Window} view
   * @param {number} detail
   * @param {number} screenX
   * @param {number} screenY
   * @param {number} clientX
   * @param {number} clientY
   * @param {boolean} ctrlKey
   * @param {boolean} altKey
   * @param {boolean} shiftKey
   * @param {boolean} metaKey
   * @param {TouchList} touches
   * @param {TouchList} targetTouches
   * @param {TouchList} changedTouches
   * @param {number} scale
   * @param {number} rotation
   */
  initTouchEvent(String type, bool canBubble, bool cancelable,
      w3c_dom1.Window view, num detail, num screenX, num screenY, num clientX,
      num clientY, bool ctrlKey, bool altKey, bool shiftKey, bool metaKey,
      TouchList touches, TouchList targetTouches, TouchList changedTouches,
      num scale, num rotation);
}

/**
 * The GestureEvent class encapsulates information about a multi-touch gesture.
 *
 * GestureEvent objects are high-level events that encapsulate the low-level
 * TouchEvent objects. Both GestureEvent and TouchEvent events are sent during
 * a multi-touch sequence. Gesture events contain scaling and rotation
 * information allowing gestures to be combined, if supported by the platform.
 * If not supported, one gesture ends before another starts. Listen for
 * GestureEvent events if you want to respond to gestures only, not process
 * the low-level TouchEvent objects.
 *
 * @see http://developer.apple.com/library/safari/#documentation/UserExperience/Reference/GestureEventClassReference/GestureEvent/GestureEvent.html
 * @extends {UIEvent}
 * @constructor
 */
@JsName("GestureEvent")
abstract class GestureEvent extends w3c_event.UIEvent {
// @Ignore
  external GestureEvent.fakeConstructor$() : super.fakeConstructor$();

  /**
   * The distance between two fingers since the start of an event as a multiplier
   * of the initial distance. The initial value is 1.0. If less than 1.0, the
   * gesture is pinch close (to zoom out). If greater than 1.0, the gesture is
   * pinch open (to zoom in).
   * @type {number}
   */
  num scale;

  /**
   * The delta rotation since the start of an event, in degrees, where clockwise
   * is positive and counter-clockwise is negative. The initial value is 0.0.
   * @type {number}
   */
  num rotation;

  /**
   * The target of this gesture.
   * @type {EventTarget}
   */
  w3c_event.EventTarget target;

  /**
   * Initializes a newly created GestureEvent object.
   * @param {string} type
   * @param {boolean} canBubble
   * @param {boolean} cancelable
   * @param {Window} view
   * @param {number} detail
   * @param {number} screenX
   * @param {number} screenY
   * @param {number} clientX
   * @param {number} clientY
   * @param {boolean} ctrlKey
   * @param {boolean} altKey
   * @param {boolean} shiftKey
   * @param {boolean} metaKey
   * @param {EventTarget} target
   * @param {number} scale
   * @param {number} rotation
   */
  initGestureEvent(String type, bool canBubble, bool cancelable,
      w3c_dom1.Window view, num detail, num screenX, num screenY, num clientX,
      num clientY, bool ctrlKey, bool altKey, bool shiftKey, bool metaKey,
      w3c_event.EventTarget target, num scale, num rotation);
}
