/*
 * Copyright 2013 The Closure Compiler Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// This file has been automatically generated.  Please do not edit it manually.
/**
 * @fileoverview This file describes the externs API for V8-specific objects.
 * FROM_EXTERN
 */
@JsName()
library v8;

import 'package:js/js.dart';
import 'package:func/func.dart';

/**
 * Stack frame elements in V8.
 * @constructor
 */
@JsName("CallSite")
abstract class CallSite {
// @Ignore
  external CallSite.fakeConstructor$();

  /**
   * Returns the value of this.
   * @return {Object|undefined}
   */
  Object getThis();

  /**
   * Returns the type of this as a string. This is the name of the function stored
   * in the constructor field of this, if available, otherwise the object's
   * [[Class]] internal property.
   * @return {string|undefined}
   */
  String getTypeName();

  /**
   * Returns the current function.
   * @return {!Function|undefined}
   */
  Function getFunction();

  /**
   * Returns the name of the current function, typically its name property. If a
   * name property is not available an attempt will be made to try to infer a name
   * from the function's context.
   * @return {string|undefined}
   */
  String getFunctionName();

  /**
   * Returns the name of the property of this or one of its prototypes that holds
   * the current function.
   * @return {string|undefined}
   */
  String getMethodName();

  /**
   * If this function was defined in a script returns the name of the script
   * @return {string|undefined}
   */
  String getFileName();

  /**
   * If this function was defined in a script returns the current line number.
   * @return {number|undefined}
   */
  num getLineNumber();

  /**
   * If this function was defined in a script returns the current column number.
   * @return {number|undefined}
   */
  num getColumnNumber();

  /**
   * If this function was created using a call to eval, returns a CallSite object
   * representing the location where eval was called
   * @return {CallSite|undefined}
   */
  CallSite getEvalOrigin();

  /**
   * Is this a toplevel invocation, that is, is this the global object?
   * @return {boolean}
   */
  bool isToplevel();

  /**
   * Does this call take place in code defined by a call to eval?
   * @return {boolean}
   */
  bool isEval();

  /**
   * Is this call in native V8 code?
   * @return {boolean}
   */
  bool isNative();

  /**
   * Is this a constructor call?
   * @return {boolean}
   */
  bool isConstructor();
}
