/*
 * Copyright 2012 The Closure Compiler Authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// This file has been automatically generated.  Please do not edit it manually.
/**
 * @fileoverview Definitions for the API related to audio.
 * Definitions for the Web Audio API.
 * This file is based on the W3C Working Draft 15 March 2012.
 * @see http://www.w3.org/TR/webaudio/
 *
 * FROM_EXTERN
 */
@JsName()
library w3c_audio;

import 'package:js/js.dart';
import 'package:func/func.dart';

import "es6.dart" as es6;

import "html5.dart" as html5;

import "w3c_event.dart" as w3c_event;

import "w3c_rtc.dart" as w3c_rtc;

@JsName("AudioContext")
abstract class AudioContext {
// @Ignore
  external AudioContext.fakeConstructor$();

  /** @type {!AudioDestinationNode} */
  AudioDestinationNode destination;

  /** @type {number} */
  num sampleRate;

  /** @type {number} */
  num currentTime;

  /** @type {!AudioListener} */
  AudioListener listener;

  /**
   * @param {number} numberOfChannels
   * @param {number} length
   * @param {number} sampleRate
   * @return {!AudioBuffer}
   */
  AudioBuffer createBuffer(num numberOfChannels, num length, num sampleRate);

  /**
   * @param {ArrayBuffer} audioData
   * @param {function(AudioBuffer)} successCallback
   * @param {function(?)=} errorCallback
   */
  decodeAudioData(es6.ArrayBuffer audioData, successCallback(AudioBuffer arg1),
      [errorCallback(arg1)]);

  /**
   * @return {!AudioBufferSourceNode}
   */
  AudioBufferSourceNode createBufferSource();

  /**
   * @param {HTMLMediaElement} mediaElement
   * @return {!MediaElementAudioSourceNode}
   */
  MediaElementAudioSourceNode createMediaElementSource(
      html5.HTMLMediaElement mediaElement);

  /**
   * @param {MediaStream} mediaStream
   * @return {!MediaStreamAudioSourceNode}
   */
  MediaStreamAudioSourceNode createMediaStreamSource(
      w3c_rtc.MediaStream mediaStream);

  /**
   * @return {!MediaStreamAudioDestinationNode}
   */
  MediaStreamAudioDestinationNode createMediaStreamDestination();

  /**
   * To be deprecated. Use createScriptProcessor instead.
   * @param {number} bufferSize
   * @param {number} numberOfInputs
   * @param {number} numberOfOuputs
   * @return {!ScriptProcessorNode}
   */
  ScriptProcessorNode createJavaScriptNode(
      num bufferSize, num numberOfInputs, num numberOfOuputs);

  /**
   * @param {number} bufferSize
   * @param {number=} numberOfInputChannels_opt
   * @param {number=} numberOfOutputChannels_opt
   * @return {!ScriptProcessorNode}
   */
  ScriptProcessorNode createScriptProcessor(num bufferSize,
      [num numberOfInputChannels_opt, num numberOfOutputChannels_opt]);

  /**
   * @return {!RealtimeAnalyserNode}
   */
  RealtimeAnalyserNode createAnalyser();

  /**
   * @deprecated Use createGain instead.
   * @return {!GainNode}
   */
  GainNode createGainNode();

  /**
   * @return {!GainNode}
   */
  GainNode createGain();

  /**
   * To be deprecated. Use createDelay instead.
   * @param {number=} maxDelayTime
   * @return {!DelayNode}
   */
  DelayNode createDelayNode([num maxDelayTime]);

  /**
   * @param {number=} maxDelayTime
   * @return {!DelayNode}
   */
  DelayNode createDelay([num maxDelayTime]);

  /**
   * @return {!BiquadFilterNode}
   */
  BiquadFilterNode createBiquadFilter();

  /**
   * @return {!WaveShaperNode}
   */
  WaveShaperNode createWaveShaper();

  /**
   * @return {!AudioPannerNode}
   */
  AudioPannerNode createPanner();

  /**
   * @return {!StereoPannerNode}
   */
  StereoPannerNode createStereoPanner();

  /**
   * @return {!ConvolverNode}
   */
  ConvolverNode createConvolver();

  /**
   * @param {number=} numberOfOutputs
   * @return {!AudioChannelSplitter}
   */
  AudioChannelSplitter createChannelSplitter([num numberOfOutputs]);

  /**
   * @param {number=} numberOfInputs
   * @return {!AudioChannelMerger}
   */
  AudioChannelMerger createChannelMerger([num numberOfInputs]);

  /**
   * @return {!DynamicsCompressorNode}
   */
  DynamicsCompressorNode createDynamicsCompressor();

  /**
   * @return {!OscillatorNode}
   */
  OscillatorNode createOscillator();

  /**
   * @param {Float32Array} real
   * @param {Float32Array} imag
   * @return {!PeriodicWave}
   */
  PeriodicWave createPeriodicWave(es6.Float32Array real, es6.Float32Array imag);
}

@JsName("OfflineAudioContext")
abstract class OfflineAudioContext extends AudioContext {
// @Ignore
  external OfflineAudioContext.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @param {number} numberOfChannels
   * @param {number} length
   * @param {number} sampleRate
   * @constructor
   * @extends {AudioContext}
   */
  external factory OfflineAudioContext(
      num numberOfChannels, num length, num sampleRate);

  startRendering();

  /** @type {function(OfflineAudioCompletionEvent)} */
  external oncomplete(OfflineAudioCompletionEvent arg1);
}

@JsName("OfflineAudioCompletionEvent")
abstract class OfflineAudioCompletionEvent extends w3c_event.Event {
// @Ignore
  external OfflineAudioCompletionEvent.fakeConstructor$()
      : super.fakeConstructor$();

  /** @type {AudioBuffer} */
  AudioBuffer renderedBuffer;
}

@JsName("AudioNode")
abstract class AudioNode {
// @Ignore
  external AudioNode.fakeConstructor$();

  /**
   * @param {AudioNode} destination
   * @param {number=} output
   * @param {number=} input
   */
  connect(AudioNode destination, [num output, num input]);

  /**
   * @param {number=} output
   */
  disconnect([num output]);

  /** @type {!AudioContext} */
  AudioContext context;

  /** @type {number} */
  num numberOfInputs;

  /** @type {number} */
  num numberOfOutputs;

  /** @type {number} */
  num channelCount;

  /** @type {string} */
  String channelCountMode;

  /** @type {string} */
  String channelInterpretation;
}

@JsName("AudioSourceNode")
abstract class AudioSourceNode extends AudioNode {
// @Ignore
  external AudioSourceNode.fakeConstructor$() : super.fakeConstructor$();
}

@JsName("AudioDestinationNode")
abstract class AudioDestinationNode extends AudioNode {
// @Ignore
  external AudioDestinationNode.fakeConstructor$() : super.fakeConstructor$();

  /**
   * To be deprecated. Use maxChannelCount instead.
   * @type {number}
   */
  num numberOfChannels;

  /** @type {number} */
  num maxChannelCount;
}

@JsName("AudioParam")
abstract class AudioParam {
// @Ignore
  external AudioParam.fakeConstructor$();

  /** @type {number} */
  num value;

  /**
   * To be deprecated.
   * @type {number}
   */
  num maxValue;

  /**
   * To be deprecated.
   * @type {number}
   */
  num minValue;

  /** @type {number} */
  num defaultValue;

  /**
   * To be deprecated.
   * @type {number}
   */
  num units;

  /**
   * @param {number} value
   * @param {number} startTime
   */
  setValueAtTime(num value, num startTime);

  /**
   * @param {number} value
   * @param {number} endTime
   */
  linearRampToValueAtTime(num value, num endTime);

  /**
   * @param {number} value
   * @param {number} endTime
   */
  exponentialRampToValueAtTime(num value, num endTime);

  /**
   * @param {number} target
   * @param {number} startTime
   * @param {number} timeConstant
   */
  setTargetAtTime(num target, num startTime, num timeConstant);

  /**
   * @deprecated Use setTargetAtTime instead.
   * @param {number} target
   * @param {number} startTime
   * @param {number} timeConstant
   */
  setTargetValueAtTime(num target, num startTime, num timeConstant);

  /**
   * @param {Float32Array} values
   * @param {number} startTime
   * @param {number} duration
   */
  setValueCurveAtTime(es6.Float32Array values, num startTime, num duration);

  /**
   * @param {number} startTime
   */
  cancelScheduledValues(num startTime);
}

@JsName("AudioGain")
abstract class AudioGain extends AudioParam {
// @Ignore
  external AudioGain.fakeConstructor$() : super.fakeConstructor$();
}

@JsName("GainNode")
abstract class GainNode extends AudioNode {
// @Ignore
  external GainNode.fakeConstructor$() : super.fakeConstructor$();

  /** @type {AudioGain} */
  AudioGain gain;
}

@JsName("DelayNode")
abstract class DelayNode extends AudioNode {
// @Ignore
  external DelayNode.fakeConstructor$() : super.fakeConstructor$();

  /** @type {AudioParam} */
  AudioParam delayTime;
}

@JsName("AudioBuffer")
abstract class AudioBuffer {
// @Ignore
  external AudioBuffer.fakeConstructor$();

  /**
   * To be deprecated.
   * @type {AudioGain}
   */
  AudioGain gain;

  /** @type {number} */
  num sampleRate;

  /** @type {number} */
  num length;

  /** @type {number} */
  num duration;

  /** @type {number} */
  num numberOfChannels;

  /**
   * @param {number} channel
   * @return {Float32Array}
   */
  es6.Float32Array getChannelData(num channel);
}

@JsName("AudioBufferSourceNode")
abstract class AudioBufferSourceNode extends AudioSourceNode {
// @Ignore
  external AudioBufferSourceNode.fakeConstructor$() : super.fakeConstructor$();

  /**
   * To be deprecated.
   * @const
   * @type {number}
   */
  num UNSCHEDULED_STATE;

  /**
   * To be deprecated.
   * @const
   * @type {number}
   */
  num SCHEDULED_STATE;

  /**
   * To be deprecated.
   * @const
   * @type {number}
   */
  num PLAYING_STATE;

  /**
   * To be deprecated.
   * @const
   * @type {number}
   */
  num FINISHED_STATE;

  /**
   * To be deprecated.
   * @type {number}
   */
  num playbackState;

  /** @type {AudioBuffer} */
  AudioBuffer buffer;

  /**
   * To be deprecated.
   * @type {number}
   */
  num gain;

  /** @type {!AudioParam} */
  AudioParam playbackRate;

  /** @type {boolean} */
  bool loop;

  /** @type {number} */
  num loopStart;

  /** @type {number} */
  num loopEnd;

  /**
   * @param {number} when
   * @param {number=} opt_offset
   * @param {number=} opt_duration
   */
  start(num when, [num opt_offset, num opt_duration]);

  /**
   * @param {number} when
   */
  stop(num when);

  /**
   * To be deprecated.
   * @param {number} when
   */
  noteOn(num when);

  /**
   * To be deprecated.
   * @param {number} when
   * @param {number} grainOffset
   * @param {number} grainDuration
   */
  noteGrainOn(num when, num grainOffset, num grainDuration);

  /**
   * To be deprecated.
   * @param {number} when
   */
  noteOff(num when);
}

@JsName("MediaElementAudioSourceNode")
abstract class MediaElementAudioSourceNode extends AudioSourceNode {
// @Ignore
  external MediaElementAudioSourceNode.fakeConstructor$()
      : super.fakeConstructor$();
}

@JsName("JavaScriptAudioNode")
abstract class JavaScriptAudioNode extends AudioNode {
// @Ignore
  external JavaScriptAudioNode.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {EventListener|(function(!AudioProcessingEvent):(boolean|undefined))}
   */
  Func1<AudioProcessingEvent, bool> onaudioprocess;

  /**
   * @const
   * @type {number}
   */
  num bufferSize;
}

@JsName("ScriptProcessorNode")
abstract class ScriptProcessorNode extends AudioNode {
// @Ignore
  external ScriptProcessorNode.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {EventListener|(function(!AudioProcessingEvent):(boolean|undefined))}
   */
  Func1<AudioProcessingEvent, bool> onaudioprocess;

  /**
   * @const
   * @type {number}
   */
  num bufferSize;
}

@JsName("AudioProcessingEvent")
abstract class AudioProcessingEvent extends w3c_event.Event {
// @Ignore
  external AudioProcessingEvent.fakeConstructor$() : super.fakeConstructor$();

  /** @type {ScriptProcessorNode} */
  ScriptProcessorNode node;

  /** @type {number} */
  num playbackTime;

  /** @type {AudioBuffer} */
  AudioBuffer inputBuffer;

  /** @type {AudioBuffer} */
  AudioBuffer outputBuffer;
}

@JsName("AudioPannerNode")
abstract class AudioPannerNode extends AudioNode {
// @Ignore
  external AudioPannerNode.fakeConstructor$() : super.fakeConstructor$();

  /** @type {number|string} */
  dynamic /*num|String*/ panningModel;

  /**
   * @param {number} x
   * @param {number} y
   * @param {number} z
   */
  setPosition(num x, num y, num z);

  /**
   * @param {number} x
   * @param {number} y
   * @param {number} z
   */
  setOrientation(num x, num y, num z);

  /**
   * @param {number} x
   * @param {number} y
   * @param {number} z
   */
  setVelocity(num x, num y, num z);

  /** @type {number|string} */
  dynamic /*num|String*/ distanceModel;

  /** @type {number} */
  num refDistance;

  /** @type {number} */
  num maxDistance;

  /** @type {number} */
  num rolloffFactor;

  /** @type {number} */
  num coneInnerAngle;

  /** @type {number} */
  num coneOuterAngle;

  /** @type {number} */
  num coneOuterGain;

  /**
   * To be deprecated.
   * @type {AudioGain}
   */
  AudioGain coneGain;

  /**
   * To be deprecated.
   * @type {AudioGain}
   */
  AudioGain distanceGain;
}

@JsName("StereoPannerNode")
abstract class StereoPannerNode extends AudioNode {
// @Ignore
  external StereoPannerNode.fakeConstructor$() : super.fakeConstructor$();

  /** @type {!AudioParam} */
  AudioParam pan;
}

@JsName("AudioListener")
abstract class AudioListener {
// @Ignore
  external AudioListener.fakeConstructor$();

  /**
   * To be deprecated.
   * @type {number}
   */
  num gain;

  /** @type {number} */
  num dopplerFactor;

  /** @type {number} */
  num speedOfSound;

  /**
   * @param {number} x
   * @param {number} y
   * @param {number} z
   */
  setPosition(num x, num y, num z);

  /**
   * @param {number} x
   * @param {number} y
   * @param {number} z
   * @param {number} xUp
   * @param {number} yUp
   * @param {number} zUp
   */
  setOrientation(num x, num y, num z, num xUp, num yUp, num zUp);

  /**
   * @param {number} x
   * @param {number} y
   * @param {number} z
   */
  setVelocity(num x, num y, num z);
}

@JsName("ConvolverNode")
abstract class ConvolverNode extends AudioNode {
// @Ignore
  external ConvolverNode.fakeConstructor$() : super.fakeConstructor$();

  /** @type {AudioBuffer} */
  AudioBuffer buffer;

  /** @type {boolean} */
  bool normalize;
}

@JsName("RealtimeAnalyserNode")
abstract class RealtimeAnalyserNode extends AudioNode {
// @Ignore
  external RealtimeAnalyserNode.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @param {Float32Array} array
   */
  getFloatFrequencyData(es6.Float32Array array);

  /**
   * @param {Uint8Array} array
   */
  getByteFrequencyData(es6.Uint8Array array);

  /**
   * @param {Uint8Array} array
   */
  getByteTimeDomainData(es6.Uint8Array array);

  /** @type {number} */
  num fftSize;

  /** @type {number} */
  num frequencyBinCount;

  /** @type {number} */
  num minDecibels;

  /** @type {number} */
  num maxDecibels;

  /** @type {number} */
  num smoothingTimeConstant;
}

@JsName("AudioChannelSplitter")
abstract class AudioChannelSplitter extends AudioNode {
// @Ignore
  external AudioChannelSplitter.fakeConstructor$() : super.fakeConstructor$();
}

@JsName("AudioChannelMerger")
abstract class AudioChannelMerger extends AudioNode {
// @Ignore
  external AudioChannelMerger.fakeConstructor$() : super.fakeConstructor$();
}

@JsName("DynamicsCompressorNode")
abstract class DynamicsCompressorNode extends AudioNode {
// @Ignore
  external DynamicsCompressorNode.fakeConstructor$() : super.fakeConstructor$();

  /** @type {!AudioParam} */
  AudioParam threshold;

  /** @type {!AudioParam} */
  AudioParam knee;

  /** @type {!AudioParam} */
  AudioParam ratio;

  /** @type {!AudioParam} */
  AudioParam reduction;

  /** @type {!AudioParam} */
  AudioParam attack;

  /** @type {!AudioParam} */
  AudioParam release;
}

@JsName("BiquadFilterNode")
abstract class BiquadFilterNode extends AudioNode {
// @Ignore
  external BiquadFilterNode.fakeConstructor$() : super.fakeConstructor$();

  /**
   * A read-able and write-able string that specifies the type of the filter.
   * See http://webaudio.github.io/web-audio-api/#the-biquadfilternode-interface
   * for valid values.
   * @type {string}
   */
  String type;

  /** @type {!AudioParam} */
  AudioParam frequency;

  /** @type {!AudioParam} */
  AudioParam detune;

  /** @type {!AudioParam} */
  AudioParam Q;

  /** @type {!AudioParam} */
  AudioParam gain;

  /**
   * @param {Float32Array} frequencyHz
   * @param {Float32Array} magResponse
   * @param {Float32Array} phaseResponse
   */
  getFrequencyResponse(es6.Float32Array frequencyHz,
      es6.Float32Array magResponse, es6.Float32Array phaseResponse);
}

@JsName("WaveShaperNode")
abstract class WaveShaperNode extends AudioNode {
// @Ignore
  external WaveShaperNode.fakeConstructor$() : super.fakeConstructor$();

  /** @type {Float32Array} */
  es6.Float32Array curve;

  /** @type {string} */
  String oversample;
}

@JsName("WaveTable")
abstract class WaveTable {
// @Ignore
  external WaveTable.fakeConstructor$();
}

@JsName("PeriodicWave")
abstract class PeriodicWave {
// @Ignore
  external PeriodicWave.fakeConstructor$();
}

@JsName("OscillatorNode")
abstract class OscillatorNode extends AudioNode {
// @Ignore
  external OscillatorNode.fakeConstructor$() : super.fakeConstructor$();

  /** @type {string} */
  String type;

  /**
   * To be deprecated.
   * @type {number}
   */
  num playbackState;

  /** @type {!AudioParam} */
  AudioParam frequency;

  /** @type {!AudioParam} */
  AudioParam detune;

  /** @type {function(number)} */
  external start(num arg1);
  /** @type {function(number)} */
  external stop(num arg1);
  /**
   * To be deprecated.
   * @type {function(WaveTable)}
   */
  external setWaveTable(WaveTable arg1);
  /** @type {function(PeriodicWave)} */
  external setPeriodicWave(PeriodicWave arg1);
  /** @type {EventListener} */
  dynamic /*w3c_event.EventListener|null*/ onended;
}

@JsName("MediaStreamAudioSourceNode")
abstract class MediaStreamAudioSourceNode extends AudioSourceNode {
// @Ignore
  external MediaStreamAudioSourceNode.fakeConstructor$()
      : super.fakeConstructor$();
}

@JsName("MediaStreamAudioDestinationNode")
abstract class MediaStreamAudioDestinationNode extends AudioDestinationNode {
// @Ignore
  external MediaStreamAudioDestinationNode.fakeConstructor$()
      : super.fakeConstructor$();

  /**
   * @type {!MediaStream}
   * @const
   */
  w3c_rtc.MediaStream stream;
}

@JsName("webkitAudioContext")
abstract class webkitAudioContext extends AudioContext {
// @Ignore
  external webkitAudioContext.fakeConstructor$() : super.fakeConstructor$();
}

@JsName("webkitOfflineAudioContext")
abstract class webkitOfflineAudioContext extends OfflineAudioContext {
// @Ignore
  external webkitOfflineAudioContext.fakeConstructor$()
      : super.fakeConstructor$();

  /**
   * @param {number} numberOfChannels
   * @param {number} length
   * @param {number} sampleRate
   * @constructor
   * @extends {OfflineAudioContext}
   */
  external factory webkitOfflineAudioContext(
      num numberOfChannels, num length, num sampleRate);
}

@JsName("webkitAudioPannerNode")
abstract class webkitAudioPannerNode extends AudioPannerNode {
// @Ignore
  external webkitAudioPannerNode.fakeConstructor$() : super.fakeConstructor$();
}

@JsName("Audio")
abstract class Audio extends html5.HTMLAudioElement {
// @Ignore
  external Audio.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @param {string=} src
   * @constructor
   * @extends {HTMLAudioElement}
   */
  external factory Audio([String src]);

  /**
   * @param {number} channels
   * @param {number} rate
   */
  mozSetup(num channels, num rate);

  /**
   * @param {Array|Float32Array} buffer
   */
  mozWriteAudio(buffer /*List|null|es6.Float32Array*/);

  /**
   * @return {number}
   */
  num mozCurrentSampleOffset();
}
