/*
 * Copyright 2008 The Closure Compiler Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// This file has been automatically generated.  Please do not edit it manually.
/**
 * @fileoverview Definitions for W3C's CSS specification
 *  The whole file has been fully type annotated.
 *  http://www.w3.org/TR/DOM-Level-2-Style/css.html
 * FROM_EXTERN
 * @author stevey@google.com (Steve Yegge)
 *
 * TODO(nicksantos): When there are no more occurrences of w3c_range.js and
 * gecko_dom.js being included directly in BUILD files, bug dbeam to split the
 * bottom part of this file into a separate externs.
 */
@JsName()
library w3c_css;

import 'package:js/js.dart';
import 'package:func/func.dart';

import "es6.dart" as es6;

import "w3c_dom1.dart" as w3c_dom1;

import "w3c_event.dart" as w3c_event;

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-2-Style/stylesheets.html#StyleSheets-StyleSheet
 */
@JsName("StyleSheet")
abstract class StyleSheet {
// @Ignore
  external StyleSheet.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/stylesheets.html#StyleSheets-StyleSheet-type
   */
  String type;

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/stylesheets.html#StyleSheets-StyleSheet-disabled
   */
  bool disabled;

  /**
   * @type {Node}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/stylesheets.html#StyleSheets-StyleSheet-owner
   */
  w3c_dom1.Node ownerNode;

  /**
   * @type {StyleSheet}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/stylesheets.html#StyleSheets-StyleSheet-parentStyleSheet
   */
  StyleSheet parentStyleSheet;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/stylesheets.html#StyleSheets-StyleSheet-href
   */
  String href;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/stylesheets.html#StyleSheets-StyleSheet-title
   */
  String title;

  /**
   * @type {MediaList}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/stylesheets.html#StyleSheets-StyleSheet-media
   */
  MediaList media;

  /** @type {Element} */
  w3c_dom1.Element owningElement;

  /** @type {boolean} */
  bool readOnly;

  /** @type {StyleSheetList} */
  StyleSheetList imports;

  /** @type {string} */
  String id;

  /**
   * @param {string} bstrURL
   * @param {number} lIndex
   * @return {number}
   */
  external num addImport(String bstrURL, num lIndex);
  /**
   * @param {string} bstrSelector
   * @param {string} bstrStyle
   * @param {number=} opt_iIndex
   * @return {number}
   * @see http://msdn.microsoft.com/en-us/library/aa358796%28v=vs.85%29.aspx
   */
  external num addRule(String bstrSelector, String bstrStyle, [num opt_iIndex]);
  /**
   * @param {number} lIndex
   */
  external removeImport(num lIndex);
  /**
   * @param {number} lIndex
   */
  external removeRule(num lIndex);
  /** @type {string} */
  String cssText;

  /** @type {CSSRuleList} */
  CSSRuleList rules;

  /**
   * @param {string} propName
   * @return {string}
   * @see http://msdn.microsoft.com/en-us/library/aa358797(VS.85).aspx
   */
  external String getExpression(String propName);
  /**
   * @param {string} name
   * @param {string} expression
   * @return {undefined}
   * @see http://msdn.microsoft.com/en-us/library/ms531196(VS.85).aspx
   */
  external void setExpression(String name, String expression);
  /**
   * @param {string} expression
   * @return {undefined}
   * @see http://msdn.microsoft.com/en-us/library/aa358798(VS.85).aspx
   */
  external void removeExpression(String expression);
}

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-2-Style/stylesheets.html#StyleSheets-StyleSheetList
 */
@JsName("StyleSheetList")
abstract class StyleSheetList {
// @Ignore
  external StyleSheetList.fakeConstructor$();

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/stylesheets.html#StyleSheets-StyleSheetList-length
   */
  num length;

  /**
   * @param {number} index
   * @return {StyleSheet}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/stylesheets.html#StyleSheets-StyleSheetList-item
   */
  StyleSheet item(num index);
}

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-2-Style/stylesheets.html#StyleSheets-MediaList
 */
@JsName("MediaList")
abstract class MediaList {
// @Ignore
  external MediaList.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/stylesheets.html#StyleSheets-MediaList-mediaText
   */
  String mediaText;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/stylesheets.html#StyleSheets-MediaList-length
   */
  num length;

  /**
   * @param {number} index
   * @return {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/stylesheets.html#StyleSheets-MediaList-item
   */
  String item(num index);
}

/**
 * @interface
 * @see http://www.w3.org/TR/DOM-Level-2-Style/stylesheets.html#StyleSheets-LinkStyle
 */
@JsName("LinkStyle")
abstract class LinkStyle {
// @Ignore
  external LinkStyle.fakeConstructor$();

  /**
   * @type {StyleSheet}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/stylesheets.html#StyleSheets-LinkStyle-sheet
   */
  StyleSheet sheet;
}

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-2-Style/stylesheets.html#StyleSheets-StyleSheet-DocumentStyle
 */
@JsName("DocumentStyle")
abstract class DocumentStyle {
// @Ignore
  external DocumentStyle.fakeConstructor$();

  /**
   * @type {StyleSheetList}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/stylesheets.html#StyleSheets-StyleSheet-DocumentStyle-styleSheets
   */
  StyleSheetList styleSheets;
}

/**
 * @constructor
 * @extends {StyleSheet}
 * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSStyleSheet
 */
@JsName("CSSStyleSheet")
abstract class CSSStyleSheet extends StyleSheet {
// @Ignore
  external CSSStyleSheet.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {CSSRule}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSStyleSheet-ownerRule
   */
  CSSRule ownerRule;

  /**
   * @type {CSSRuleList}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSStyleSheet-cssRules
   */
  CSSRuleList cssRules;

  /**
   * @param {string} rule
   * @param {number} index
   * @return {number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSStyleSheet-insertRule
   */
  num insertRule(String rule, num index);

  /**
   * @param {number} index
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSStyleSheet-deleteRule
   */
  deleteRule(num index);
}

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSRuleList
 */
@JsName("CSSRuleList")
abstract class CSSRuleList {
// @Ignore
  external CSSRuleList.fakeConstructor$();

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSRuleList-length
   */
  num length;

  /**
   * @param {number} index
   * @return {CSSRule}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSRuleList-item
   */
  CSSRule item(num index);
}

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSRule
 */
@JsName("CSSRule")
abstract class CSSRule {
// @Ignore
  external CSSRule.fakeConstructor$();

  external static num get UNKNOWN_RULE;

  external static set UNKNOWN_RULE(num value);

  external static num get STYLE_RULE;

  external static set STYLE_RULE(num value);

  external static num get CHARSET_RULE;

  external static set CHARSET_RULE(num value);

  external static num get IMPORT_RULE;

  external static set IMPORT_RULE(num value);

  external static num get FONT_FACE_RULE;

  external static set FONT_FACE_RULE(num value);

  external static num get MEDIA_RULE;

  external static set MEDIA_RULE(num value);

  external static num get PAGE_RULE;

  external static set PAGE_RULE(num value);

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSRule-ruleType
   */
  num type;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSRule-cssText
   */
  String cssText;

  /**
   * @type {CSSStyleSheet}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSRule-sheet
   */
  CSSStyleSheet parentStyleSheet;

  /**
   * @type {CSSRule}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSRule-parentRule
   */
  CSSRule parentRule;

  /**
   * @type {CSSStyleDeclaration}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSStyleRule
   */
  CSSStyleDeclaration style;
}

/**
 * @constructor
 * @extends {CSSRule}
 * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSStyleRule
 */
@JsName("CSSStyleRule")
abstract class CSSStyleRule extends CSSRule {
// @Ignore
  external CSSStyleRule.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSStyleRule-selectorText
   */
  String selectorText;

  /**
   * @type {CSSStyleDeclaration}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSStyleRule-style
   */
  CSSStyleDeclaration style;
}

/**
 * @constructor
 * @extends {CSSRule}
 * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSMediaRule
 */
@JsName("CSSMediaRule")
abstract class CSSMediaRule extends CSSRule {
// @Ignore
  external CSSMediaRule.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {MediaList}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSMediaRule-mediaTypes
   */
  MediaList media;

  /**
   * @type {CSSRuleList}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSMediaRule-cssRules
   */
  CSSRuleList cssRules;

  /**
   * @param {string} rule
   * @param {number} index
   * @return {number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSMediaRule-insertRule
   */
  num insertRule(String rule, num index);

  /**
   * @param {number} index
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSMediaRule-deleteRule
   */
  deleteRule(num index);
}

/**
 * @constructor
 * @extends {CSSRule}
 * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSFontFaceRule
 */
@JsName("CSSFontFaceRule")
abstract class CSSFontFaceRule extends CSSRule {
// @Ignore
  external CSSFontFaceRule.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {CSSStyleDeclaration}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSFontFaceRule-style
   */
  CSSStyleDeclaration style;
}

/**
 * @constructor
 * @extends {CSSRule}
 * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSPageRule
 */
@JsName("CSSPageRule")
abstract class CSSPageRule extends CSSRule {
// @Ignore
  external CSSPageRule.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSPageRule-name
   */
  String selectorText;

  /**
   * @type {CSSStyleDeclaration}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSPageRule-style
   */
  CSSStyleDeclaration style;
}

/**
 * @constructor
 * @extends {CSSRule}
 * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSImportRule
 */
@JsName("CSSImportRule")
abstract class CSSImportRule extends CSSRule {
// @Ignore
  external CSSImportRule.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSImportRule-href
   */
  String href;

  /**
   * @type {MediaList}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSImportRule-media
   */
  MediaList media;

  /**
   * @type {CSSStyleSheet}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSImportRule-styleSheet
   */
  CSSStyleSheet styleSheet;
}

/**
 * @constructor
 * @extends {CSSRule}
 * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSCharsetRule
 */
@JsName("CSSCharsetRule")
abstract class CSSCharsetRule extends CSSRule {
// @Ignore
  external CSSCharsetRule.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSCharsetRule-encoding
   */
  String encoding;
}

/**
 * @constructor
 * @extends {CSSRule}
 * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSUnknownRule
 */
@JsName("CSSUnknownRule")
abstract class CSSUnknownRule extends CSSRule {
// @Ignore
  external CSSUnknownRule.fakeConstructor$() : super.fakeConstructor$();
}

/**
 * @constructor
 * @extends {CSSProperties}
 * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSStyleDeclaration
 */
@JsName("CSSStyleDeclaration")
abstract class CSSStyleDeclaration extends CSSProperties {
// @Ignore
  external CSSStyleDeclaration.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSStyleDeclaration-cssText
   */
  String cssText;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSStyleDeclaration-length
   */
  num length;

  /**
   * @type {CSSRule}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSStyleDeclaration-parentRule
   */
  CSSRule parentRule;

  /**
   * @param {string} propertyName
   * @return {CSSValue}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSStyleDeclaration-getPropertyCSSValue
   */
  CSSValue getPropertyCSSValue(String propertyName);

  /**
   * @param {string} propertyName
   * @return {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSStyleDeclaration-getPropertyPriority
   */
  String getPropertyPriority(String propertyName);

  /**
   * @param {string} propertyName
   * @return {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSStyleDeclaration-getPropertyValue
   */
  String getPropertyValue(String propertyName);

  /**
   * @param {number} index
   * @return {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSStyleDeclaration-item
   */
  String item(num index);

  /**
   * @param {string} propertyName
   * @return {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSStyleDeclaration-removeProperty
   */
  String removeProperty(String propertyName);

  /**
   * @param {string} propertyName
   * @param {string} value
   * @param {string=} opt_priority
   * @return {undefined}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSStyleDeclaration-setProperty
   */
  void setProperty(String propertyName, String value, [String opt_priority]);

  /**
   * @param {string} name
   * @param {number=} opt_flags
   * @return {string|number|boolean|null}
   * @see http://msdn.microsoft.com/en-us/library/ms536429(VS.85).aspx
   */
/*String|num|bool|null*/ getAttribute(String name, [num opt_flags]);

  /**
   * @param {string} name
   * @return {string|number|boolean|null}
   * @see http://msdn.microsoft.com/en-us/library/aa358797(VS.85).aspx
   */
/*String|num|bool|null*/ getExpression(String name);

  /**
   * @param {string} name
   * @param {number=} opt_flags
   * @return {boolean}
   * @see http://msdn.microsoft.com/en-us/library/ms536696(VS.85).aspx
   */
  bool removeAttribute(String name, [num opt_flags]);

  /**
   * @param {string} name
   * @return {boolean}
   * @see http://msdn.microsoft.com/en-us/library/aa358798(VS.85).aspx
   */
  bool removeExpression(String name);

  /**
   * @param {string} name
   * @param {*} value
   * @param {number=} opt_flags
   * @see http://msdn.microsoft.com/en-us/library/ms536739(VS.85).aspx
   */
  setAttribute(String name, value, [num opt_flags]);

  /**
   * @param {string} name
   * @param {string} expr
   * @param {string=} opt_language
   * @return {undefined}
   * @see http://msdn.microsoft.com/en-us/library/ms531196(VS.85).aspx
   */
  void setExpression(String name, String expr, [String opt_language]);
}

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSValue
 */
@JsName("CSSValue")
abstract class CSSValue {
// @Ignore
  external CSSValue.fakeConstructor$();

  external static num get CSS_CUSTOM;

  external static set CSS_CUSTOM(num value);

  external static num get CSS_VALUE_LIST;

  external static set CSS_VALUE_LIST(num value);

  external static num get CSS_PRIMITIVE_VALUE;

  external static set CSS_PRIMITIVE_VALUE(num value);

  external static num get CSS_INHERIT;

  external static set CSS_INHERIT(num value);

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSValue-cssText
   */
  String cssText;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSValue-cssValueType
   */
  num cssValueType;
}

/**
 * @constructor
 * @extends {CSSValue}
 * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSPrimitiveValue
 */
@JsName("CSSPrimitiveValue")
abstract class CSSPrimitiveValue extends CSSValue {
// @Ignore
  external CSSPrimitiveValue.fakeConstructor$() : super.fakeConstructor$();

  external static num get CSS_IDENT;

  external static set CSS_IDENT(num value);

  external static num get CSS_CM;

  external static set CSS_CM(num value);

  external static num get CSS_DIMENSION;

  external static set CSS_DIMENSION(num value);

  external static num get CSS_UNKNOWN;

  external static set CSS_UNKNOWN(num value);

  external static num get CSS_COUNTER;

  external static set CSS_COUNTER(num value);

  external static num get CSS_RAD;

  external static set CSS_RAD(num value);

  external static num get CSS_S;

  external static set CSS_S(num value);

  external static num get CSS_IN;

  external static set CSS_IN(num value);

  external static num get CSS_PC;

  external static set CSS_PC(num value);

  external static num get CSS_EMS;

  external static set CSS_EMS(num value);

  external static num get CSS_KHZ;

  external static set CSS_KHZ(num value);

  external static num get CSS_MS;

  external static set CSS_MS(num value);

  external static num get CSS_STRING;

  external static set CSS_STRING(num value);

  external static num get CSS_RECT;

  external static set CSS_RECT(num value);

  external static num get CSS_HZ;

  external static set CSS_HZ(num value);

  external static num get CSS_GRAD;

  external static set CSS_GRAD(num value);

  external static num get CSS_MM;

  external static set CSS_MM(num value);

  external static num get CSS_EXS;

  external static set CSS_EXS(num value);

  external static num get CSS_URI;

  external static set CSS_URI(num value);

  external static num get CSS_DEG;

  external static set CSS_DEG(num value);

  external static num get CSS_RGBCOLOR;

  external static set CSS_RGBCOLOR(num value);

  external static num get CSS_NUMBER;

  external static set CSS_NUMBER(num value);

  external static num get CSS_PX;

  external static set CSS_PX(num value);

  external static num get CSS_ATTR;

  external static set CSS_ATTR(num value);

  external static num get CSS_PERCENTAGE;

  external static set CSS_PERCENTAGE(num value);

  external static num get CSS_PT;

  external static set CSS_PT(num value);

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSPrimitiveValue
   */
  num primitiveType;

  /**
   * @return {Counter}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSPrimitiveValue-getCounterValue
   * @throws DOMException {@see DomException.INVALID_ACCESS_ERR}
   */
  Counter getCounterValue();

  /**
   * @param {number} unitType
   * @return {number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSPrimitiveValue-getFloatValue
   * @throws DOMException {@see DomException.INVALID_ACCESS_ERR}
   */
  num getFloatValue(num unitType);

  /**
   * @return {RGBColor}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSPrimitiveValue-getRGBColorValue
   * @throws DOMException {@see DomException.INVALID_ACCESS_ERR}
   */
  RGBColor getRGBColorValue();

  /**
   * @return {Rect}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSPrimitiveValue-getRectValue
   * @throws DOMException {@see DomException.INVALID_ACCESS_ERR}
   */
  Rect getRectValue();

  /**
   * @return {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSPrimitiveValue-getStringValue
   * @throws DOMException {@see DomException.INVALID_ACCESS_ERR}
   */
  String getStringValue();

  /**
   * @param {number} unitType
   * @param {number} floatValue
   * @return {undefined}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSPrimitiveValue-setFloatValue
   * @throws DOMException {@see DomException.INVALID_ACCESS_ERR},
   *                      {@see DomException.NO_MODIFICATION_ALLOWED_ERR}
   */
  void setFloatValue(num unitType, num floatValue);

  /**
   * @param {number} stringType
   * @param {string} stringValue
   * @return {undefined}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSPrimitiveValue-setStringValue
   * @throws DOMException {@see DomException.INVALID_ACCESS_ERR},
   *                      {@see DomException.NO_MODIFICATION_ALLOWED_ERR}
   */
  void setStringValue(num stringType, String stringValue);
}

/**
 * @constructor
 * @extends {CSSValue}
 * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSValueList
 */
@JsName("CSSValueList")
abstract class CSSValueList extends CSSValue {
// @Ignore
  external CSSValueList.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSValueList-length
   */
  num length;

  /**
   * @param {number} index
   * @return {CSSValue}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSValueList-item
   */
  CSSValue item(num index);
}

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-RGBColor
 */
@JsName("RGBColor")
abstract class RGBColor {
// @Ignore
  external RGBColor.fakeConstructor$();

  /**
   * @type {CSSPrimitiveValue}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-RGBColor-red
   */
  CSSPrimitiveValue red;

  /**
   * @type {CSSPrimitiveValue}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-RGBColor-green
   */
  CSSPrimitiveValue green;

  /**
   * @type {CSSPrimitiveValue}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-RGBColor-blue
   */
  CSSPrimitiveValue blue;
}

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-Rect
 */
@JsName("Rect")
abstract class Rect {
// @Ignore
  external Rect.fakeConstructor$();

  /**
   * @type {CSSPrimitiveValue}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-Rect-top
   */
  CSSPrimitiveValue top;

  /**
   * @type {CSSPrimitiveValue}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-Rect-right
   */
  CSSPrimitiveValue right;

  /**
   * @type {CSSPrimitiveValue}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-Rect-bottom
   */
  CSSPrimitiveValue bottom;

  /**
   * @type {CSSPrimitiveValue}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-Rect-left
   */
  CSSPrimitiveValue left;
}

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-Counter
 */
@JsName("Counter")
abstract class Counter {
// @Ignore
  external Counter.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-Counter-identifier
   */
  String identifier;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-Counter-listStyle
   */
  String listStyle;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-Counter-separator
   */
  String separator;
}

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-DocumentCSS
 */
@JsName("DocumentCSS")
abstract class DocumentCSS {
// @Ignore
  external DocumentCSS.fakeConstructor$();

  /**
   * @param {Element} elt
   * @param {string} pseudoElt
   * @return {CSSStyleDeclaration}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-DocumentCSS-getOverrideStyle
   */
  CSSStyleDeclaration getOverrideStyle(w3c_dom1.Element elt, String pseudoElt);
}

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-DOMImplementationCSS
 */
@JsName("DOMImplementationCSS")
abstract class DOMImplementationCSS {
// @Ignore
  external DOMImplementationCSS.fakeConstructor$();

  /**
   * @param {string} title
   * @param {string} media
   * @return {CSSStyleSheet}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-DOMImplementationCSS-createCSSStyleSheet
   * @throws DOMException {@see DomException.SYNTAX_ERR}
   */
  CSSStyleSheet createCSSStyleSheet(String title, String media);
}

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-ElementCSSInlineStyle
 */
@JsName("ElementCSSInlineStyle")
abstract class ElementCSSInlineStyle {
// @Ignore
  external ElementCSSInlineStyle.fakeConstructor$();

  /**
   * @type {CSSStyleDeclaration}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-ElementCSSInlineStyle-style
   */
  CSSStyleDeclaration style;
}

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties
 */
@JsName("CSSProperties")
abstract class CSSProperties {
// @Ignore
  external CSSProperties.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-azimuth
   */
  String azimuth;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-background
   */
  String background;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-backgroundAttachment
   */
  String backgroundAttachment;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-backgroundColor
   */
  String backgroundColor;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-backgroundImage
   */
  String backgroundImage;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-backgroundPosition
   */
  String backgroundPosition;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-backgroundRepeat
   */
  String backgroundRepeat;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/css3-background/#the-background-size
   */
  String backgroundSize;

  /**
   * @implicitCast
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-border
   */
  String border;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-borderCollapse
   */
  String borderCollapse;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-borderColor
   */
  String borderColor;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-borderSpacing
   */
  String borderSpacing;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSPrimitiveValue-borderStyle
   */
  String borderStyle;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-borderTop
   */
  String borderTop;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-borderRight
   */
  String borderRight;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-borderBottom
   */
  String borderBottom;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-borderLeft
   */
  String borderLeft;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-borderTopColor
   */
  String borderTopColor;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-borderRightColor
   */
  String borderRightColor;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-borderBottomColor
   */
  String borderBottomColor;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-borderLeftColor
   */
  String borderLeftColor;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-borderTopStyle
   */
  String borderTopStyle;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-borderRightStyle
   */
  String borderRightStyle;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-borderBottomStyle
   */
  String borderBottomStyle;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-borderLeftStyle
   */
  String borderLeftStyle;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-borderTopWidth
   */
  dynamic /*String|num*/ borderTopWidth;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-borderRightWidth
   */
  dynamic /*String|num*/ borderRightWidth;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-borderBottomWidth
   */
  dynamic /*String|num*/ borderBottomWidth;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-borderLeftWidth
   */
  dynamic /*String|num*/ borderLeftWidth;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-borderWidth
   */
  dynamic /*String|num*/ borderWidth;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/css3-background/#the-border-radius
   */
  dynamic /*String|num*/ borderRadius;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/css3-background/#the-border-radius
   */
  dynamic /*String|num*/ borderBottomLeftRadius;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/css3-background/#the-border-radius
   */
  dynamic /*String|num*/ borderBottomRightRadius;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/css3-background/#the-border-radius
   */
  dynamic /*String|num*/ borderTopLeftRadius;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/css3-background/#the-border-radius
   */
  dynamic /*String|num*/ borderTopRightRadius;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-bottom
   */
  dynamic /*String|num*/ bottom;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-captionSide
   */
  String captionSide;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-clear
   */
  String clear;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-clip
   */
  String clip;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-color
   */
  String color;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-content
   */
  String content;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-counterIncrement
   */
  String counterIncrement;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-counterReset
   */
  String counterReset;

  /**
   * This is not an official part of the W3C spec. In practice, this is a settable
   * property that works cross-browser. It is used in goog.dom.setProperties() and
   * needs to be extern'd so the --disambiguate_properties JS compiler pass works.
   * @type {string}
   */
  String cssText;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-cue
   */
  String cue;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-cueAfter
   */
  String cueAfter;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-cueBefore
   */
  String cueBefore;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-cursor
   */
  String cursor;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-direction
   */
  String direction;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-display
   */
  String display;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-elevation
   */
  String elevation;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-emptyCells
   */
  String emptyCells;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-cssFloat
   */
  String cssFloat;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-font
   */
  String font;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-fontFamily
   */
  String fontFamily;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-fontSize
   */
  dynamic /*String|num*/ fontSize;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-fontSizeAdjust
   */
  String fontSizeAdjust;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-fontStretch
   */
  String fontStretch;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-fontStyle
   */
  String fontStyle;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-fontVariant
   */
  String fontVariant;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-fontWeight
   */
  String fontWeight;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-height
   */
  dynamic /*String|num*/ height;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-left
   */
  dynamic /*String|num*/ left;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-letterSpacing
   */
  String letterSpacing;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-lineHeight
   */
  dynamic /*String|num*/ lineHeight;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-listStyle
   */
  String listStyle;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-listStyleImage
   */
  String listStyleImage;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-listStylePosition
   */
  String listStylePosition;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-listStyleType
   */
  String listStyleType;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-margin
   */
  dynamic /*String|num*/ margin;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-marginTop
   */
  dynamic /*String|num*/ marginTop;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-marginRight
   */
  dynamic /*String|num*/ marginRight;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-marginBottom
   */
  dynamic /*String|num*/ marginBottom;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-marginLeft
   */
  dynamic /*String|num*/ marginLeft;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-markerOffset
   */
  String markerOffset;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-marks
   */
  String marks;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-maxHeight
   */
  dynamic /*String|num*/ maxHeight;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-maxWidth
   */
  dynamic /*String|num*/ maxWidth;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-minHeight
   */
  dynamic /*String|num*/ minHeight;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-minWidth
   */
  dynamic /*String|num*/ minWidth;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-orphans
   */
  String orphans;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-outline
   */
  String outline;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-outlineColor
   */
  String outlineColor;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-outlineStyle
   */
  String outlineStyle;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-outlineWidth
   */
  dynamic /*String|num*/ outlineWidth;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-overflow
   */
  String overflow;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-padding
   */
  dynamic /*String|num*/ padding;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-paddingTop
   */
  dynamic /*String|num*/ paddingTop;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-paddingRight
   */
  dynamic /*String|num*/ paddingRight;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-paddingBottom
   */
  dynamic /*String|num*/ paddingBottom;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-paddingLeft
   */
  dynamic /*String|num*/ paddingLeft;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-page
   */
  String page;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-pageBreakAfter
   */
  String pageBreakAfter;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-pageBreakBefore
   */
  String pageBreakBefore;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-pageBreakInside
   */
  String pageBreakInside;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-pause
   */
  String pause;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-pauseAfter
   */
  String pauseAfter;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-pauseBefore
   */
  String pauseBefore;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-pitch
   */
  String pitch;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-pitchRange
   */
  String pitchRange;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-playDuring
   */
  String playDuring;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-position
   */
  String position;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-quotes
   */
  String quotes;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/css3-ui/#resize
   */
  String resize;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-richness
   */
  String richness;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-right
   */
  dynamic /*String|num*/ right;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-size
   */
  String size;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-speak
   */
  String speak;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-speakHeader
   */
  String speakHeader;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-speakNumeral
   */
  String speakNumeral;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-speakPunctuation
   */
  String speakPunctuation;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-speechRate
   */
  String speechRate;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-stress
   */
  String stress;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-tableLayout
   */
  String tableLayout;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-textAlign
   */
  String textAlign;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-textDecoration
   */
  String textDecoration;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-textIndent
   */
  String textIndent;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-textShadow
   */
  String textShadow;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-textTransform
   */
  String textTransform;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-top
   */
  dynamic /*String|num*/ top;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-unicodeBidi
   */
  String unicodeBidi;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-verticalAlign
   */
  String verticalAlign;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-visibility
   */
  String visibility;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-voiceFamily
   */
  String voiceFamily;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-volume
   */
  String volume;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-whiteSpace
   */
  String whiteSpace;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-widows
   */
  String widows;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-width
   */
  dynamic /*String|num*/ width;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-wordSpacing
   */
  String wordSpacing;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-wordWrap
   */
  String wordWrap;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSProperties-zIndex
   */
  dynamic /*String|num*/ zIndex;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/css3-background/#box-shadow
   */
  String boxShadow;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/css3-ui/#box-sizing
   */
  String boxSizing;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/css3-color/#transparency
   */
  dynamic /*String|num*/ opacity;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/css3-ui/#text-overflow
   */
  String textOverflow;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/css3-2d-transforms/#backface-visibility-property
   */
  String backfaceVisibility;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/css3-2d-transforms/#perspective
   */
  String perspective;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/css3-2d-transforms/#perspective-origin
   */
  dynamic /*String|num*/ perspectiveOrigin;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/css3-2d-transforms/#effects
   */
  String transform;

  /**
   * @type {string|number}
   * @see http://www.w3.org/TR/css3-2d-transforms/#transform-origin
   */
  dynamic /*String|num*/ transformOrigin;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/css3-2d-transforms/#transform-style
   */
  String transformStyle;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/css3-transitions/#transition
   */
  String transition;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/css3-transitions/#transition-delay
   */
  String transitionDelay;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/css3-transitions/#transition-duration
   */
  String transitionDuration;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/css3-transitions/#transition-property-property
   */
  String transitionProperty;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/css3-transitions/#transition-timing-function
   */
  String transitionTimingFunction;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/SVG11/interact.html#PointerEventsProperty
   */
  String pointerEvents;

  /** @type {string} */
  String MozAppearance;

  /** @type {string} */
  String MozBackfaceVisibility;

  /** @type {string} */
  String MozBackgroundClip;

  /** @type {string} */
  String MozBackgroundInlinePolicy;

  /** @type {string} */
  String MozBackgroundOrigin;

  /** @type {string} */
  String MozBinding;

  /** @type {string} */
  String MozBorderBottomColors;

  /** @type {string} */
  String MozBorderEnd;

  /** @type {string} */
  String MozBorderEndColor;

  /** @type {string} */
  String MozBorderEndStyle;

  /** @type {string} */
  String MozBorderEndWidth;

  /** @type {string} */
  String MozBorderImage;

  /** @type {string} */
  String MozBorderLeftColors;

  /** @type {string} */
  String MozBorderRadius;

  /** @type {string} */
  String MozBorderRadiusTopleft;

  /** @type {string} */
  String MozBorderRadiusTopright;

  /** @type {string} */
  String MozBorderRadiusBottomleft;

  /** @type {string} */
  String MozBorderRadiusBottomright;

  /** @type {string} */
  String MozBorderRightColors;

  /** @type {string} */
  String MozBorderStart;

  /** @type {string} */
  String MozBorderStartColor;

  /** @type {string} */
  String MozBorderStartStyle;

  /** @type {string} */
  String MozBorderStartWidth;

  /** @type {string} */
  String MozBorderTopColors;

  /** @type {string} */
  String MozBoxAlign;

  /** @type {string} */
  String MozBoxDirection;

  /** @type {string} */
  String MozBoxFlex;

  /** @type {string} */
  String MozBoxOrdinalGroup;

  /** @type {string} */
  String MozBoxOrient;

  /** @type {string} */
  String MozBoxPack;

  /** @type {string} */
  String MozBoxSizing;

  /** @type {string} */
  String MozBoxShadow;

  /** @type {string} */
  String MozColumnCount;

  /** @type {string} */
  String MozColumnGap;

  /** @type {string} */
  String MozColumnRule;

  /** @type {string} */
  String MozColumnRuleColor;

  /** @type {string} */
  String MozColumnRuleStyle;

  /** @type {string} */
  String MozColumnRuleWidth;

  /** @type {string} */
  String MozColumnWidth;

  /** @type {string} */
  String MozFloatEdge;

  /** @type {string} */
  String MozFontFeatureSettings;

  /** @type {string} */
  String MozFontLanguageOverride;

  /** @type {string} */
  String MozForceBrokenImageIcon;

  /** @type {string} */
  String MozImageRegion;

  /** @type {string} */
  String MozMarginEnd;

  /** @type {string} */
  String MozMarginStart;

  /** @type {number|string} */
  dynamic /*num|String*/ MozOpacity;

  /** @type {string} */
  String MozOutline;

  /** @type {string} */
  String MozOutlineColor;

  /** @type {string} */
  String MozOutlineOffset;

  /** @type {string} */
  String MozOutlineRadius;

  /** @type {string} */
  String MozOutlineRadiusBottomleft;

  /** @type {string} */
  String MozOutlineRadiusBottomright;

  /** @type {string} */
  String MozOutlineRadiusTopleft;

  /** @type {string} */
  String MozOutlineRadiusTopright;

  /** @type {string} */
  String MozOutlineStyle;

  /** @type {string} */
  String MozOutlineWidth;

  /** @type {string} */
  String MozPaddingEnd;

  /** @type {string} */
  String MozPaddingStart;

  /** @type {string} */
  String MozPerspective;

  /** @type {string} */
  String MozStackSizing;

  /** @type {string} */
  String MozTabSize;

  /** @type {string} */
  String MozTransform;

  /** @type {string} */
  String MozTransformOrigin;

  /** @type {string} */
  String MozTransition;

  /** @type {string} */
  String MozTransitionDelay;

  /** @type {string} */
  String MozTransitionDuration;

  /** @type {string} */
  String MozTransitionProperty;

  /** @type {string} */
  String MozTransitionTimingFunction;

  /** @type {string} */
  String MozUserFocus;

  /** @type {string} */
  String MozUserInput;

  /** @type {string} */
  String MozUserModify;

  /** @type {string} */
  String MozUserSelect;

  /** @type {string} */
  String MozWindowShadow;

  /** @type {string} */
  String backgroundPositionX;

  /** @type {string} */
  String backgroundPositionY;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ie/ms531081(v=vs.85).aspx
   * NOTE: Left untyped to avoid conflict with caller.
   */
  dynamic behavior;

  /**
   * @type {string}
   * @see http://msdn.microsoft.com/en-us/library/ms533883.aspx
   */
  String imeMode;

  /**
   * @type {string}
   * @see http://msdn.microsoft.com/en-us/library/ms534176(VS.85).aspx
   */
  String msInterpolationMode;

  /** @type {string} */
  String overflowX;

  /** @type {string} */
  String overflowY;

  /** @type {number} */
  num pixelWidth;

  /** @type {number} */
  num pixelHeight;

  /** @type {number} */
  num pixelLeft;

  /** @type {number} */
  num pixelTop;

  /** @type {string} */
  String styleFloat;

  /**
   * @type {string|number}
   * @see http://msdn.microsoft.com/en-us/library/ms535169(VS.85).aspx
   */
  dynamic /*String|num*/ zoom;

  /**
   * @type {string}
   * @see http://msdn.microsoft.com/en-us/library/ms535153(VS.85).aspx
   */
  String writingMode;

  /** @type {string} */
  String MsAccelerator;

  /** @type {string} */
  String MsBackgroundPositionX;

  /** @type {string} */
  String MsBackgroundPositionY;

  /** @type {string} */
  String MsBehavior;

  /** @type {string} */
  String MsBlockProgression;

  /** @type {string} */
  String MsFilter;

  /** @type {string} */
  String MsImeMode;

  /** @type {string} */
  String MsLayoutGrid;

  /** @type {string} */
  String MsLayoutGridChar;

  /** @type {string} */
  String MsLayoutGridLine;

  /** @type {string} */
  String MsLayoutGridMode;

  /** @type {string} */
  String MsLayoutGridType;

  /** @type {string} */
  String MsLineBreak;

  /** @type {string} */
  String MsLineGridMode;

  /** @type {string} */
  String MsInterpolationMode;

  /** @type {string} */
  String MsOverflowX;

  /** @type {string} */
  String MsOverflowY;

  /** @type {string} */
  String MsScrollbar3dlightColor;

  /** @type {string} */
  String MsScrollbarArrowColor;

  /** @type {string} */
  String MsScrollbarBaseColor;

  /** @type {string} */
  String MsScrollbarDarkshadowColor;

  /** @type {string} */
  String MsScrollbarFaceColor;

  dynamic MsScrollbarHighlightColor;

  /** @type {string} */
  String MsScrollbarShadowColor;

  /** @type {string} */
  String MsScrollbarTrackColor;

  /** @type {string} */
  String MsTextAlignLast;

  /** @type {string} */
  String MsTextAutospace;

  /** @type {string} */
  String MsTextJustify;

  /** @type {string} */
  String MsTextKashidaSpace;

  /** @type {string} */
  String MsTextOverflow;

  /** @type {string} */
  String MsTextUnderlinePosition;

  /** @type {string} */
  String MsWordBreak;

  /** @type {string} */
  String MsWordWrap;

  /** @type {string} */
  String MsWritingMode;

  /** @type {string} */
  String MsZoom;

  /** @type {string} */
  String msContentZooming;

  /** @type {string} */
  String msTouchAction;

  /** @type {string} */
  String msTransform;

  /** @type {string} */
  String msTransition;

  /** @type {string} */
  String WebkitAlignContent;

  /** @type {string} */
  String WebkitAlignItems;

  /** @type {string} */
  String WebkitAlignSelf;

  /** @type {string} */
  String WebkitAnimation;

  /** @type {string} */
  String WebkitAnimationDelay;

  /** @type {string} */
  String WebkitAnimationDirection;

  /** @type {string} */
  String WebkitAnimationDuration;

  /** @type {string} */
  String WebkitAnimationFillMode;

  /** @type {string} */
  String WebkitAnimationIterationCount;

  /** @type {string} */
  String WebkitAnimationName;

  /** @type {string} */
  String WebkitAnimationPlayState;

  /** @type {string} */
  String WebkitAnimationTimingFunction;

  /** @type {string} */
  String WebkitAppearance;

  /** @type {string} */
  String WebkitAppRegion;

  /** @type {string} */
  String WebkitAspectRatio;

  /** @type {string} */
  String WebkitBackfaceVisibility;

  /** @type {string} */
  String WebkitBackgroundClip;

  /** @type {string} */
  String WebkitBackgroundComposite;

  /** @type {string} */
  String WebkitBackgroundOrigin;

  /** @type {string} */
  String WebkitBackgroundSize;

  /** @type {string} */
  String WebkitBinding;

  /** @type {string} */
  String WebkitBlendMode;

  /** @type {string} */
  String WebkitBorderAfter;

  /** @type {string} */
  String WebkitBorderAfterColor;

  /** @type {string} */
  String WebkitBorderAfterStyle;

  /** @type {string} */
  String WebkitBorderAfterWidth;

  /** @type {string} */
  String WebkitBorderBefore;

  /** @type {string} */
  String WebkitBorderBeforeColor;

  /** @type {string} */
  String WebkitBorderBeforeStyle;

  /** @type {string} */
  String WebkitBorderBeforeWidth;

  /** @type {string} */
  String WebkitBorderBottomLeftRadius;

  /** @type {string} */
  String WebkitBorderBottomRightRadius;

  /** @type {string} */
  String WebkitBorderEnd;

  /** @type {string} */
  String WebkitBorderEndColor;

  /** @type {string} */
  String WebkitBorderEndStyle;

  /** @type {string} */
  String WebkitBorderEndWidth;

  /** @type {string} */
  String WebkitBorderFit;

  /** @type {string} */
  String WebkitBorderHorizontalSpacing;

  /** @type {string} */
  String WebkitBorderImage;

  /** @type {string} */
  String WebkitBorderRadius;

  /** @type {string} */
  String WebkitBorderStart;

  /** @type {string} */
  String WebkitBorderStartColor;

  /** @type {string} */
  String WebkitBorderStartStyle;

  /** @type {string} */
  String WebkitBorderStartWidth;

  /** @type {string} */
  String WebkitBorderTopLeftRadius;

  /** @type {string} */
  String WebkitBorderTopRightRadius;

  /** @type {string} */
  String WebkitBorderVerticalSpacing;

  /** @type {string} */
  String WebkitBoxAlign;

  /** @type {string} */
  String WebkitBoxDecorationBreak;

  /** @type {string} */
  String WebkitBoxDirection;

  /** @type {string} */
  String WebkitBoxFlex;

  /** @type {string} */
  String WebkitBoxFlexGroup;

  /** @type {string} */
  String WebkitBoxLines;

  /** @type {string} */
  String WebkitBoxOrdinalGroup;

  /** @type {string} */
  String WebkitBoxOrient;

  /** @type {string} */
  String WebkitBoxPack;

  /** @type {string} */
  String WebkitBoxReflect;

  /** @type {string} */
  String WebkitBoxShadow;

  /** @type {string} */
  String WebkitBoxSizing;

  /** @type {string} */
  String WebkitColorCorrection;

  /** @type {string} */
  String WebkitColumnAxis;

  /** @type {string} */
  String WebkitColumnBreakAfter;

  /** @type {string} */
  String WebkitColumnBreakBefore;

  /** @type {string} */
  String WebkitColumnBreakInside;

  /** @type {string} */
  String WebkitColumnCount;

  /** @type {string} */
  String WebkitColumnGap;

  /** @type {string} */
  String WebkitColumnProgression;

  /** @type {string} */
  String WebkitColumnRule;

  /** @type {string} */
  String WebkitColumnRuleColor;

  /** @type {string} */
  String WebkitColumnRuleStyle;

  /** @type {string} */
  String WebkitColumnRuleWidth;

  /** @type {string} */
  String WebkitColumns;

  /** @type {string} */
  String WebkitColumnSpan;

  /** @type {string} */
  String WebkitColumnWidth;

  /** @type {string} */
  String WebkitDashboardRegion;

  /** @type {string} */
  String WebkitFilter;

  /** @type {string} */
  String WebkitFlex;

  /** @type {string} */
  String WebkitFlexBasis;

  /** @type {string} */
  String WebkitFlexDirection;

  /** @type {string} */
  String WebkitFlexFlow;

  /** @type {string} */
  String WebkitFlexGrow;

  /** @type {string} */
  String WebkitFlexShrink;

  /** @type {string} */
  String WebkitFlexWrap;

  /** @type {string} */
  String WebkitFlowFrom;

  /** @type {string} */
  String WebkitFlowInto;

  /** @type {string} */
  String WebkitFontSizeDelta;

  /** @type {string} */
  String WebkitFontSmoothing;

  /** @type {string} */
  String WebkitGridColumn;

  /** @type {string} */
  String WebkitGridColumns;

  /** @type {string} */
  String WebkitGridRow;

  /** @type {string} */
  String WebkitGridRows;

  /** @type {string} */
  String WebkitHighlight;

  /** @type {string} */
  String WebkitHyphenateCharacter;

  /** @type {string} */
  String WebkitHyphenateLimitAfter;

  /** @type {string} */
  String WebkitHyphenateLimitBefore;

  /** @type {string} */
  String WebkitHyphenateLimitLines;

  /** @type {string} */
  String WebkitHyphens;

  /** @type {string} */
  String WebkitJustifyContent;

  /** @type {string} */
  String WebkitLineAlign;

  /** @type {string} */
  String WebkitLineBoxContain;

  /** @type {string} */
  String WebkitLineBreak;

  /** @type {string} */
  String WebkitLineClamp;

  /** @type {string} */
  String WebkitLineGrid;

  /** @type {string} */
  String WebkitLineSnap;

  /** @type {string} */
  String WebkitLocale;

  /** @type {string} */
  String WebkitLogicalHeight;

  /** @type {string} */
  String WebkitLogicalWidth;

  /** @type {string} */
  String WebkitMarginAfter;

  /** @type {string} */
  String WebkitMarginAfterCollapse;

  /** @type {string} */
  String WebkitMarginBefore;

  /** @type {string} */
  String WebkitMarginBeforeCollapse;

  /** @type {string} */
  String WebkitMarginBottomCollapse;

  /** @type {string} */
  String WebkitMarginCollapse;

  /** @type {string} */
  String WebkitMarginEnd;

  /** @type {string} */
  String WebkitMarginStart;

  /** @type {string} */
  String WebkitMarginTopCollapse;

  /** @type {string} */
  String WebkitMarquee;

  /** @type {string} */
  String WebkitMarqueeDirection;

  /** @type {string} */
  String WebkitMarqueeIncrement;

  /** @type {string} */
  String WebkitMarqueeRepetition;

  /** @type {string} */
  String WebkitMarqueeSpeed;

  /** @type {string} */
  String WebkitMarqueeStyle;

  /** @type {string} */
  String WebkitMask;

  /** @type {string} */
  String WebkitMaskAttachment;

  /** @type {string} */
  String WebkitMaskBoxImage;

  /** @type {string} */
  String WebkitMaskBoxImageOutset;

  /** @type {string} */
  String WebkitMaskBoxImageRepeat;

  /** @type {string} */
  String WebkitMaskBoxImageSlice;

  /** @type {string} */
  String WebkitMaskBoxImageSource;

  /** @type {string} */
  String WebkitMaskBoxImageWidth;

  /** @type {string} */
  String WebkitMaskClip;

  /** @type {string} */
  String WebkitMaskComposite;

  /** @type {string} */
  String WebkitMaskImage;

  /** @type {string} */
  String WebkitMaskOrigin;

  /** @type {string} */
  String WebkitMaskPosition;

  /** @type {string} */
  String WebkitMaskPositionX;

  /** @type {string} */
  String WebkitMaskPositionY;

  /** @type {string} */
  String WebkitMaskRepeat;

  /** @type {string} */
  String WebkitMaskRepeatX;

  /** @type {string} */
  String WebkitMaskRepeatY;

  /** @type {string} */
  String WebkitMaskSize;

  /** @type {string} */
  String WebkitMatchNearestMailBlockquoteColor;

  /** @type {string} */
  String WebkitMaxLogicalHeight;

  /** @type {string} */
  String WebkitMaxLogicalWidth;

  /** @type {string} */
  String WebkitMinLogicalHeight;

  /** @type {string} */
  String WebkitMinLogicalWidth;

  /** @type {string} */
  String WebkitNbspMode;

  /** @type {string} */
  String WebkitOrder;

  /** @type {string} */
  String WebkitOverflowScrolling;

  /** @type {string} */
  String WebkitPaddingAfter;

  /** @type {string} */
  String WebkitPaddingBefore;

  /** @type {string} */
  String WebkitPaddingEnd;

  /** @type {string} */
  String WebkitPaddingStart;

  /** @type {string} */
  String WebkitPerspective;

  /** @type {string} */
  String WebkitPerspectiveOrigin;

  /** @type {string} */
  String WebkitPerspectiveOriginX;

  /** @type {string} */
  String WebkitPerspectiveOriginY;

  /** @type {string} */
  String WebkitPrintColorAdjust;

  /** @type {string} */
  String WebkitRegionBreakAfter;

  /** @type {string} */
  String WebkitRegionBreakBefore;

  /** @type {string} */
  String WebkitRegionBreakInside;

  /** @type {string} */
  String WebkitRegionOverflow;

  /** @type {string} */
  String WebkitRtlOrdering;

  /** @type {string} */
  String WebkitRubyPosition;

  /** @type {string} */
  String WebkitShapeInside;

  /** @type {string} */
  String WebkitShapeMargin;

  /** @type {string} */
  String WebkitShapeOutside;

  /** @type {string} */
  String WebkitShapePadding;

  /** @type {string} */
  String WebkitTapHighlightColor;

  /** @type {string} */
  String WebkitTextAlignLast;

  /** @type {string} */
  String WebkitTextCombine;

  /** @type {string} */
  String WebkitTextDecorationLine;

  /** @type {string} */
  String WebkitTextDecorationsInEffect;

  /** @type {string} */
  String WebkitTextDecorationStyle;

  /** @type {string} */
  String WebkitTextEmphasis;

  /** @type {string} */
  String WebkitTextEmphasisColor;

  /** @type {string} */
  String WebkitTextEmphasisPosition;

  /** @type {string} */
  String WebkitTextEmphasisStyle;

  /** @type {string} */
  String WebkitTextFillColor;

  /** @type {string} */
  String WebkitTextOrientation;

  /** @type {string} */
  String WebkitTextSecurity;

  /** @type {string} */
  String WebkitTextSizeAdjust;

  /** @type {string} */
  String WebkitTextStroke;

  /** @type {string} */
  String WebkitTextStrokeColor;

  /** @type {string} */
  String WebkitTextStrokeWidth;

  /** @type {string} */
  String WebkitTransform;

  /** @type {string} */
  String WebkitTransformOrigin;

  /** @type {string} */
  String WebkitTransformOriginX;

  /** @type {string} */
  String WebkitTransformOriginY;

  /** @type {string} */
  String WebkitTransformOriginZ;

  /** @type {string} */
  String WebkitTransformStyle;

  /** @type {string} */
  String WebkitTransition;

  /** @type {string} */
  String WebkitTransitionDelay;

  /** @type {string} */
  String WebkitTransitionDuration;

  /** @type {string} */
  String WebkitTransitionProperty;

  /** @type {string} */
  String WebkitTransitionRepeatCount;

  /** @type {string} */
  String WebkitTransitionTimingFunction;

  /** @type {string} */
  String WebkitUserDrag;

  /** @type {string} */
  String WebkitUserModify;

  /** @type {string} */
  String WebkitUserSelect;

  /** @type {string} */
  String WebkitWrap;

  /** @type {string} */
  String WebkitWrapFlow;

  /** @type {string} */
  String WebkitWrapThrough;

  /** @type {string} */
  String WebkitWritingMode;

  /** @type {string} */
  String webkitAlignContent;

  /** @type {string} */
  String webkitAlignItems;

  /** @type {string} */
  String webkitAlignSelf;

  /** @type {string} */
  String webkitAnimation;

  /** @type {string} */
  String webkitAnimationDelay;

  /** @type {string} */
  String webkitAnimationDirection;

  /** @type {string} */
  String webkitAnimationDuration;

  /** @type {string} */
  String webkitAnimationFillMode;

  /** @type {string} */
  String webkitAnimationIterationCount;

  /** @type {string} */
  String webkitAnimationName;

  /** @type {string} */
  String webkitAnimationPlayState;

  /** @type {string} */
  String webkitAnimationTimingFunction;

  /** @type {string} */
  String webkitAppearance;

  /** @type {string} */
  String webkitAppRegion;

  /** @type {string} */
  String webkitAspectRatio;

  /** @type {string} */
  String webkitBackfaceVisibility;

  /** @type {string} */
  String webkitBackgroundClip;

  /** @type {string} */
  String webkitBackgroundComposite;

  /** @type {string} */
  String webkitBackgroundOrigin;

  /** @type {string} */
  String webkitBackgroundSize;

  /** @type {string} */
  String webkitBinding;

  /** @type {string} */
  String webkitBlendMode;

  /** @type {string} */
  String webkitBorderAfter;

  /** @type {string} */
  String webkitBorderAfterColor;

  /** @type {string} */
  String webkitBorderAfterStyle;

  /** @type {string} */
  String webkitBorderAfterWidth;

  /** @type {string} */
  String webkitBorderBefore;

  /** @type {string} */
  String webkitBorderBeforeColor;

  /** @type {string} */
  String webkitBorderBeforeStyle;

  /** @type {string} */
  String webkitBorderBeforeWidth;

  /** @type {string} */
  String webkitBorderBottomLeftRadius;

  /** @type {string} */
  String webkitBorderBottomRightRadius;

  /** @type {string} */
  String webkitBorderEnd;

  /** @type {string} */
  String webkitBorderEndColor;

  /** @type {string} */
  String webkitBorderEndStyle;

  /** @type {string} */
  String webkitBorderEndWidth;

  /** @type {string} */
  String webkitBorderFit;

  /** @type {string} */
  String webkitBorderHorizontalSpacing;

  /** @type {string} */
  String webkitBorderImage;

  /** @type {string} */
  String webkitBorderRadius;

  /** @type {string} */
  String webkitBorderStart;

  /** @type {string} */
  String webkitBorderStartColor;

  /** @type {string} */
  String webkitBorderStartStyle;

  /** @type {string} */
  String webkitBorderStartWidth;

  /** @type {string} */
  String webkitBorderTopLeftRadius;

  /** @type {string} */
  String webkitBorderTopRightRadius;

  /** @type {string} */
  String webkitBorderVerticalSpacing;

  /** @type {string} */
  String webkitBoxAlign;

  /** @type {string} */
  String webkitBoxDecorationBreak;

  /** @type {string} */
  String webkitBoxDirection;

  /** @type {string} */
  String webkitBoxFlex;

  /** @type {string} */
  String webkitBoxFlexGroup;

  /** @type {string} */
  String webkitBoxLines;

  /** @type {string} */
  String webkitBoxOrdinalGroup;

  /** @type {string} */
  String webkitBoxOrient;

  /** @type {string} */
  String webkitBoxPack;

  /** @type {string} */
  String webkitBoxReflect;

  /** @type {string} */
  String webkitBoxShadow;

  /** @type {string} */
  String webkitBoxSizing;

  /** @type {string} */
  String webkitColorCorrection;

  /** @type {string} */
  String webkitColumnAxis;

  /** @type {string} */
  String webkitColumnBreakAfter;

  /** @type {string} */
  String webkitColumnBreakBefore;

  /** @type {string} */
  String webkitColumnBreakInside;

  /** @type {string} */
  String webkitColumnCount;

  /** @type {string} */
  String webkitColumnGap;

  /** @type {string} */
  String webkitColumnProgression;

  /** @type {string} */
  String webkitColumnRule;

  /** @type {string} */
  String webkitColumnRuleColor;

  /** @type {string} */
  String webkitColumnRuleStyle;

  /** @type {string} */
  String webkitColumnRuleWidth;

  /** @type {string} */
  String webkitColumns;

  /** @type {string} */
  String webkitColumnSpan;

  /** @type {string} */
  String webkitColumnWidth;

  /** @type {string} */
  String webkitDashboardRegion;

  /** @type {string} */
  String webkitFilter;

  /** @type {string} */
  String webkitFlex;

  /** @type {string} */
  String webkitFlexBasis;

  /** @type {string} */
  String webkitFlexDirection;

  /** @type {string} */
  String webkitFlexFlow;

  /** @type {string} */
  String webkitFlexGrow;

  /** @type {string} */
  String webkitFlexShrink;

  /** @type {string} */
  String webkitFlexWrap;

  /** @type {string} */
  String webkitFlowFrom;

  /** @type {string} */
  String webkitFlowInto;

  /** @type {string} */
  String webkitFontSizeDelta;

  /** @type {string} */
  String webkitFontSmoothing;

  /** @type {string} */
  String webkitGridColumn;

  /** @type {string} */
  String webkitGridColumns;

  /** @type {string} */
  String webkitGridRow;

  /** @type {string} */
  String webkitGridRows;

  /** @type {string} */
  String webkitHighlight;

  /** @type {string} */
  String webkitHyphenateCharacter;

  /** @type {string} */
  String webkitHyphenateLimitAfter;

  /** @type {string} */
  String webkitHyphenateLimitBefore;

  /** @type {string} */
  String webkitHyphenateLimitLines;

  /** @type {string} */
  String webkitHyphens;

  /** @type {string} */
  String webkitJustifyContent;

  /** @type {string} */
  String webkitLineAlign;

  /** @type {string} */
  String webkitLineBoxContain;

  /** @type {string} */
  String webkitLineBreak;

  /** @type {string} */
  String webkitLineClamp;

  /** @type {string} */
  String webkitLineGrid;

  /** @type {string} */
  String webkitLineSnap;

  /** @type {string} */
  String webkitLocale;

  /** @type {string} */
  String webkitLogicalHeight;

  /** @type {string} */
  String webkitLogicalWidth;

  /** @type {string} */
  String webkitMarginAfter;

  /** @type {string} */
  String webkitMarginAfterCollapse;

  /** @type {string} */
  String webkitMarginBefore;

  /** @type {string} */
  String webkitMarginBeforeCollapse;

  /** @type {string} */
  String webkitMarginBottomCollapse;

  /** @type {string} */
  String webkitMarginCollapse;

  /** @type {string} */
  String webkitMarginEnd;

  /** @type {string} */
  String webkitMarginStart;

  /** @type {string} */
  String webkitMarginTopCollapse;

  /** @type {string} */
  String webkitMarquee;

  /** @type {string} */
  String webkitMarqueeDirection;

  /** @type {string} */
  String webkitMarqueeIncrement;

  /** @type {string} */
  String webkitMarqueeRepetition;

  /** @type {string} */
  String webkitMarqueeSpeed;

  /** @type {string} */
  String webkitMarqueeStyle;

  /** @type {string} */
  String webkitMask;

  /** @type {string} */
  String webkitMaskAttachment;

  /** @type {string} */
  String webkitMaskBoxImage;

  /** @type {string} */
  String webkitMaskBoxImageOutset;

  /** @type {string} */
  String webkitMaskBoxImageRepeat;

  /** @type {string} */
  String webkitMaskBoxImageSlice;

  /** @type {string} */
  String webkitMaskBoxImageSource;

  /** @type {string} */
  String webkitMaskBoxImageWidth;

  /** @type {string} */
  String webkitMaskClip;

  /** @type {string} */
  String webkitMaskComposite;

  /** @type {string} */
  String webkitMaskImage;

  /** @type {string} */
  String webkitMaskOrigin;

  /** @type {string} */
  String webkitMaskPosition;

  /** @type {string} */
  String webkitMaskPositionX;

  /** @type {string} */
  String webkitMaskPositionY;

  /** @type {string} */
  String webkitMaskRepeat;

  /** @type {string} */
  String webkitMaskRepeatX;

  /** @type {string} */
  String webkitMaskRepeatY;

  /** @type {string} */
  String webkitMaskSize;

  /** @type {string} */
  String webkitMatchNearestMailBlockquoteColor;

  /** @type {string} */
  String webkitMaxLogicalHeight;

  /** @type {string} */
  String webkitMaxLogicalWidth;

  /** @type {string} */
  String webkitMinLogicalHeight;

  /** @type {string} */
  String webkitMinLogicalWidth;

  /** @type {string} */
  String webkitNbspMode;

  /** @type {string} */
  String webkitOrder;

  /** @type {string} */
  String webkitOverflowScrolling;

  /** @type {string} */
  String webkitPaddingAfter;

  /** @type {string} */
  String webkitPaddingBefore;

  /** @type {string} */
  String webkitPaddingEnd;

  /** @type {string} */
  String webkitPaddingStart;

  /** @type {string} */
  String webkitPerspective;

  /** @type {string} */
  String webkitPerspectiveOrigin;

  /** @type {string} */
  String webkitPerspectiveOriginX;

  /** @type {string} */
  String webkitPerspectiveOriginY;

  /** @type {string} */
  String webkitPrintColorAdjust;

  /** @type {string} */
  String webkitRegionBreakAfter;

  /** @type {string} */
  String webkitRegionBreakBefore;

  /** @type {string} */
  String webkitRegionBreakInside;

  /** @type {string} */
  String webkitRegionOverflow;

  /** @type {string} */
  String webkitRtlOrdering;

  /** @type {string} */
  String webkitRubyPosition;

  /** @type {string} */
  String webkitShapeInside;

  /** @type {string} */
  String webkitShapeMargin;

  /** @type {string} */
  String webkitShapeOutside;

  /** @type {string} */
  String webkitShapePadding;

  /** @type {string} */
  String webkitTapHighlightColor;

  /** @type {string} */
  String webkitTextAlignLast;

  /** @type {string} */
  String webkitTextCombine;

  /** @type {string} */
  String webkitTextDecorationLine;

  /** @type {string} */
  String webkitTextDecorationsInEffect;

  /** @type {string} */
  String webkitTextDecorationStyle;

  /** @type {string} */
  String webkitTextEmphasis;

  /** @type {string} */
  String webkitTextEmphasisColor;

  /** @type {string} */
  String webkitTextEmphasisPosition;

  /** @type {string} */
  String webkitTextEmphasisStyle;

  /** @type {string} */
  String webkitTextFillColor;

  /** @type {string} */
  String webkitTextOrientation;

  /** @type {string} */
  String webkitTextSecurity;

  /** @type {string} */
  String webkitTextSizeAdjust;

  /** @type {string} */
  String webkitTextStroke;

  /** @type {string} */
  String webkitTextStrokeColor;

  /** @type {string} */
  String webkitTextStrokeWidth;

  /** @type {string} */
  String webkitTransform;

  /** @type {string} */
  String webkitTransformOrigin;

  /** @type {string} */
  String webkitTransformOriginX;

  /** @type {string} */
  String webkitTransformOriginY;

  /** @type {string} */
  String webkitTransformOriginZ;

  /** @type {string} */
  String webkitTransformStyle;

  /** @type {string} */
  String webkitTransition;

  /** @type {string} */
  String webkitTransitionDelay;

  /** @type {string} */
  String webkitTransitionDuration;

  /** @type {string} */
  String webkitTransitionProperty;

  /** @type {string} */
  String webkitTransitionRepeatCount;

  /** @type {string} */
  String webkitTransitionTimingFunction;

  /** @type {string} */
  String webkitUserDrag;

  /** @type {string} */
  String webkitUserModify;

  /** @type {string} */
  String webkitUserSelect;

  /** @type {string} */
  String webkitWrap;

  /** @type {string} */
  String webkitWrapFlow;

  /** @type {string} */
  String webkitWrapThrough;

  /** @type {string} */
  String webkitWritingMode;
}

/**
 * @constructor
 * @see http://www.w3.org/TR/cssom-view/#mediaquerylist
 */
@JsName("MediaQueryList")
abstract class MediaQueryList {
// @Ignore
  external MediaQueryList.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/cssom-view/#dom-mediaquerylist-media
   */
  String media;

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/cssom-view/#dom-mediaquerylist-matches
   */
  bool matches;

  /**
   * @param {MediaQueryListListener} listener
   * @see http://www.w3.org/TR/cssom-view/#dom-mediaquerylist-addlistener
   */
  addListener(listener(MediaQueryList arg1));

  /**
   * @param {MediaQueryListListener} listener
   * @see http://www.w3.org/TR/cssom-view/#dom-mediaquerylist-removelistener
   */
  removeListener(listener(MediaQueryList arg1));
}
/**
 * @typedef {(function(!MediaQueryList) : void)}
 * @see http://www.w3.org/TR/cssom-view/#mediaquerylistlistener
 */
typedef MediaQueryListListener(MediaQueryList arg1);

/**
 * @constructor
 * @see http://www.w3.org/TR/cssom-view/#screen
 */
@JsName("Screen")
abstract class Screen {
// @Ignore
  external Screen.fakeConstructor$();

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-screen-availwidth
   */
  num availWidth;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-screen-availheight
   */
  num availHeight;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-screen-width
   */
  num width;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-screen-height
   */
  num height;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-screen-colordepth
   */
  num colorDepth;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-screen-pixeldepth
   */
  num pixelDepth;

  /**
   * @type {number}
   * @see https://developer.mozilla.org/En/DOM/window.screen.availTop
   */
  num availTop;

  /**
   * @type {number}
   * @see https://developer.mozilla.org/En/DOM/window.screen.availLeft
   */
  num availLeft;

  /**
   * @type {number}
   * @see https://developer.mozilla.org/En/DOM/window.screen.left
   */
  num left;

  /**
   * @type {number}
   * @see https://developer.mozilla.org/En/DOM/window.screen.top
   */
  num top;
}

/**
 * @constructor
 * @see http://www.w3.org/TR/cssom-view/#caretposition
 */
@JsName("CaretPosition")
abstract class CaretPosition {
// @Ignore
  external CaretPosition.fakeConstructor$();

  /**
   * @type {Node}
   * @see http://www.w3.org/TR/cssom-view/#dom-caretposition-offsetnode
   */
  w3c_dom1.Node offsetNode;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-caretposition-offset
   */
  num offset;
}

/**
 * @constructor
 * @see http://www.w3.org/TR/cssom-view/#the-clientrectlist-interface
 */
@JsName("ClientRectList")
abstract class ClientRectList {
// @Ignore
  external ClientRectList.fakeConstructor$();

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-clientrectlist-length
   */
  num length;

  /**
   * @param {number} index
   * @return {ClientRect}
   * @see http://www.w3.org/TR/cssom-view/#dom-clientrectlist-item
   */
  ClientRect item(num index);
}

/**
 * @constructor
 * @see http://www.w3.org/TR/cssom-view/#the-clientrect-interface
 */
@JsName("ClientRect")
abstract class ClientRect {
// @Ignore
  external ClientRect.fakeConstructor$();

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-clientrect-top
   */
  num top;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-clientrect-right
   */
  num right;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-clientrect-bottom
   */
  num bottom;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-clientrect-left
   */
  num left;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-clientrect-width
   */
  num width;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-clientrect-height
   */
  num height;
}

/**
 * @constructor
 * http://www.w3.org/TR/css3-conditional/#CSS-interface
 */
@JsName("CSSInterface")
abstract class CSSInterface {
// @Ignore
  external CSSInterface.fakeConstructor$();

  /**
   * @param {string} property
   * @param {string=} opt_value
   * @return {boolean}
   */
  bool supports(String property, [String opt_value]);
}
/**
 * TODO(nicksantos): This suppress tag probably isn't needed, and
 * should be removed.
 * @suppress {duplicate}
 * @type {CSSInterface}
 */
@JsName("CSS")
external CSSInterface get CSS;
@JsName("CSS")
external set CSS(CSSInterface value);
/**
 * @enum {string}
 * @see http://dev.w3.org/csswg/css-font-loading/#enumdef-fontfaceloadstatus
 */
@JsName("FontFaceLoadStatus")
abstract class FontFaceLoadStatus {
  external static String get LOADING;
  external static String get LOADED;
  external static String get ERROR;
  external static String get UNLOADED;
}

/**
 * @typedef {{
 *   style: (string|undefined),
 *   weight: (string|undefined),
 *   stretch: (string|undefined),
 *   unicodeRange: (string|undefined),
 *   variant: (string|undefined),
 *   featureSettings: (string|undefined)
 * }}
 * @see http://dev.w3.org/csswg/css-font-loading/#dictdef-fontfacedescriptors
 */
@JsName()
abstract class FontFaceDescriptors {
  String featureSettings;
  String stretch;
  String style;
  String unicodeRange;
  String variant;
  String weight;
}

/**
 * @constructor
 * @param {string} fontFamily
 * @param {(string|ArrayBuffer|ArrayBufferView)} source
 * @param {!FontFaceDescriptors} descriptors
 * @see http://dev.w3.org/csswg/css-font-loading/#font-face-constructor
 */
@JsName("FontFace")
abstract class FontFace {
// @Ignore
  external FontFace.fakeConstructor$();

  /**
   * @constructor
   * @param {string} fontFamily
   * @param {(string|ArrayBuffer|ArrayBufferView)} source
   * @param {!FontFaceDescriptors} descriptors
   * @see http://dev.w3.org/csswg/css-font-loading/#font-face-constructor
   */
  external factory FontFace(String fontFamily,
      source /*String|es6.ArrayBuffer|null|es6.ArrayBufferView*/,
      FontFaceDescriptors descriptors);

  /**
   * @type {string}
   * @see http://dev.w3.org/csswg/css-font-loading/#dom-fontface-family
   */
  String family;

  /**
   * @type {string}
   * @see http://dev.w3.org/csswg/css-font-loading/#dom-fontface-style
   */
  String style;

  /**
   * @type {string}
   * @see http://dev.w3.org/csswg/css-font-loading/#dom-fontface-weight
   */
  String weight;

  /**
   * @type {string}
   * @see http://dev.w3.org/csswg/css-font-loading/#dom-fontface-stretch
   */
  String stretch;

  /**
   * @type {string}
   * @see http://dev.w3.org/csswg/css-font-loading/#dom-fontface-unicoderange
   */
  String unicodeRange;

  /**
   * @type {string}
   * @see http://dev.w3.org/csswg/css-font-loading/#dom-fontface-variant
   */
  String variant;

  /**
   * @type {string}
   * @see http://dev.w3.org/csswg/css-font-loading/#dom-fontface-featuresettings
   */
  String featureSettings;

  /**
   * @type {FontFaceLoadStatus}
   * @see http://dev.w3.org/csswg/css-font-loading/#dom-fontface-status
   */
  String status;

  /**
   * @return {!Promise.<!FontFace>}
   * @see http://dev.w3.org/csswg/css-font-loading/#font-face-load
   */
  es6.Promise<FontFace> load();
}
/**
 * @enum
 * @see http://dev.w3.org/csswg/css-font-loading/#enumdef-fontfacesetloadstatus
 */
@JsName("FontFaceSetLoadStatus")
abstract class FontFaceSetLoadStatus {
  external static num get LOADING;
  external static num get LOADED;
}

/**
 * @interface
 * @see http://dev.w3.org/csswg/css-font-loading/#FontFaceSet-interface
 */
@JsName("FontFaceSet")
abstract class FontFaceSet {
// @Ignore
  external FontFaceSet.fakeConstructor$();

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onloading;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onloadingdone;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onloadingerror;

  /**
   * @param {!FontFace} value
   * @see http://dev.w3.org/csswg/css-font-loading/#dom-fontfaceset-add
   */
  add(FontFace value);

  /**
   * @see http://dev.w3.org/csswg/css-font-loading/#dom-fontfaceset-clear
   */
  clear();

  /**
   * @param {!FontFace} value
   * @see http://dev.w3.org/csswg/css-font-loading/#dom-fontfaceset-delete
   */
  delete(FontFace value);

  /**
   * @param {!FontFace} font
   * @return {boolean}
   * @see http://dev.w3.org/csswg/css-font-loading/#dom-fontfaceset-has
   */
  bool has(FontFace font);

  /**
   * @param {function(!FontFace, number, !FontFaceSet)} cb
   * @param {Object|undefined=} opt_selfObj
   * see http://dev.w3.org/csswg/css-font-loading/#dom-fontfaceset-foreach
   */
  forEach(cb(FontFace arg1, num arg2, FontFaceSet arg3), [Object opt_selfObj]);

  /**
   * @param {string} font
   * @param {string=} opt_text
   * @return DOH_THIS_RETURN_TYPE_CRASHES
   * @see http://dev.w3.org/csswg/css-font-loading/#dom-fontfaceset-load
   */
  load(String font, [String opt_text]);

  /**
   * @param {string} font
   * @param {string=} opt_text
   * @return {boolean}
   * @see http://dev.w3.org/csswg/css-font-loading/#dom-fontfaceset-check
   */
  bool check(String font, [String opt_text]);

  /**
   * @type {!Promise.<!FontFaceSet>}
   * @see http://dev.w3.org/csswg/css-font-loading/#dom-fontfaceset-ready
   */
  es6.Promise<FontFaceSet> ready;

  /**
   * @type {FontFaceSetLoadStatus}
   * @see http://dev.w3.org/csswg/css-font-loading/#dom-fontfaceset-status
   */
  num status;
}
