/*
 * Copyright 2010 The Closure Compiler Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// This file has been automatically generated.  Please do not edit it manually.
/**
 * @fileoverview Definitions for W3C's CSS 3D Transforms specification.
 *  The whole file has been fully type annotated. Created from
 *  https://dvcs.w3.org/hg/FXTF/raw-file/tip/matrix/index.html
 *
 * FROM_EXTERN
 */
@JsName()
library w3c_css3d;

import 'package:js/js.dart';
import 'package:func/func.dart';

/**
 * @constructor
 * @param {string=} opt_matrix
 * @see https://dvcs.w3.org/hg/FXTF/raw-file/tip/matrix/index.html#the-cssmatrix-interface
 */
@JsName("CSSMatrix")
abstract class CSSMatrix {
// @Ignore
  external CSSMatrix.fakeConstructor$();

  /**
   * @constructor
   * @param {string=} opt_matrix
   * @see https://dvcs.w3.org/hg/FXTF/raw-file/tip/matrix/index.html#the-cssmatrix-interface
   */
  external factory CSSMatrix([String opt_matrix]);

  /**
   * @type {number}
   * @see https://dvcs.w3.org/hg/FXTF/raw-file/tip/matrix/index.html#three-dimensional-attributes
   */
  num m11;

  /**
   * @type {number}
   * @see https://dvcs.w3.org/hg/FXTF/raw-file/tip/matrix/index.html#three-dimensional-attributes
   */
  num m12;

  /**
   * @type {number}
   * @see https://dvcs.w3.org/hg/FXTF/raw-file/tip/matrix/index.html#three-dimensional-attributes
   */
  num m13;

  /**
   * @type {number}
   * @see https://dvcs.w3.org/hg/FXTF/raw-file/tip/matrix/index.html#three-dimensional-attributes
   */
  num m14;

  /**
   * @type {number}
   * @see https://dvcs.w3.org/hg/FXTF/raw-file/tip/matrix/index.html#three-dimensional-attributes
   */
  num m21;

  /**
   * @type {number}
   * @see https://dvcs.w3.org/hg/FXTF/raw-file/tip/matrix/index.html#three-dimensional-attributes
   */
  num m22;

  /**
   * @type {number}
   * @see https://dvcs.w3.org/hg/FXTF/raw-file/tip/matrix/index.html#three-dimensional-attributes
   */
  num m23;

  /**
   * @type {number}
   * @see https://dvcs.w3.org/hg/FXTF/raw-file/tip/matrix/index.html#three-dimensional-attributes
   */
  num m24;

  /**
   * @type {number}
   * @see https://dvcs.w3.org/hg/FXTF/raw-file/tip/matrix/index.html#three-dimensional-attributes
   */
  num m31;

  /**
   * @type {number}
   * @see https://dvcs.w3.org/hg/FXTF/raw-file/tip/matrix/index.html#three-dimensional-attributes
   */
  num m32;

  /**
   * @type {number}
   * @see https://dvcs.w3.org/hg/FXTF/raw-file/tip/matrix/index.html#three-dimensional-attributes
   */
  num m33;

  /**
   * @type {number}
   * @see https://dvcs.w3.org/hg/FXTF/raw-file/tip/matrix/index.html#three-dimensional-attributes
   */
  num m34;

  /**
   * @type {number}
   * @see https://dvcs.w3.org/hg/FXTF/raw-file/tip/matrix/index.html#three-dimensional-attributes
   */
  num m41;

  /**
   * @type {number}
   * @see https://dvcs.w3.org/hg/FXTF/raw-file/tip/matrix/index.html#three-dimensional-attributes
   */
  num m42;

  /**
   * @type {number}
   * @see https://dvcs.w3.org/hg/FXTF/raw-file/tip/matrix/index.html#three-dimensional-attributes
   */
  num m43;

  /**
   * @type {number}
   * @see https://dvcs.w3.org/hg/FXTF/raw-file/tip/matrix/index.html#three-dimensional-attributes
   */
  num m44;

  /**
   * @param {string} string
   * @return {void}
   */
  void setMatrixValue(String string);

  /**
   * @param {!CSSMatrix} secondMatrix
   * @return {!CSSMatrix}
   * @see https://dvcs.w3.org/hg/FXTF/raw-file/tip/matrix/index.html#widl-CSSMatrix-multiply-CSSMatrix-CSSMatrix-other
   */
  CSSMatrix multiply(CSSMatrix secondMatrix);

  /**
   * @return {CSSMatrix} Returns void if the matrix is non-invertable.
   * @see https://dvcs.w3.org/hg/FXTF/raw-file/tip/matrix/index.html#widl-CSSMatrix-inverse-CSSMatrix
   */
  CSSMatrix inverse();

  /**
   * @param {number=} opt_x Defaults to 0.
   * @param {number=} opt_y Defaults to 0.
   * @param {number=} opt_z Defaults to 0.
   * @return {!CSSMatrix}
   * @see https://dvcs.w3.org/hg/FXTF/raw-file/tip/matrix/index.html#widl-CSSMatrix-translate-CSSMatrix-unrestricted-double-tx-unrestricted-double-ty-unrestricted-double-tz
   */
  CSSMatrix translate([num opt_x, num opt_y, num opt_z]);

  /**
   * @param {number=} opt_scaleX Defaults to 1.
   * @param {number=} opt_scaleY Defaults to scaleX.
   * @param {number=} opt_scaleZ Defaults to 1.
   * @return {!CSSMatrix}
   * @see https://dvcs.w3.org/hg/FXTF/raw-file/tip/matrix/index.html#widl-CSSMatrix-scale-CSSMatrix-unrestricted-double-scale-unrestricted-double-originX-unrestricted-double-originY
   */
  CSSMatrix scale([num opt_scaleX, num opt_scaleY, num opt_scaleZ]);

  /**
   * @param {number=} opt_rotX Defaults to 0.
   * @param {number=} opt_rotY Defaults to 0.
   * @param {number=} opt_rotZ Defaults to rotX if rotY is not defined, else 0.
   * @return {!CSSMatrix}
   * @see https://dvcs.w3.org/hg/FXTF/raw-file/tip/matrix/index.html#widl-CSSMatrix-rotate-CSSMatrix-unrestricted-double-angle-unrestricted-double-originX-unrestricted-double-originY
   */
  CSSMatrix rotate([num opt_rotX, num opt_rotY, num opt_rotZ]);

  /**
   * @param {number=} opt_x Defaults to 0.
   * @param {number=} opt_y Defaults to 0.
   * @param {number=} opt_z Defaults to 0.
   * @param {number=} opt_angle Defaults to 0.
   * @return {!CSSMatrix}
   * @see https://dvcs.w3.org/hg/FXTF/raw-file/tip/matrix/index.html#widl-CSSMatrix-rotateAxisAngle-CSSMatrix-unrestricted-double-x-unrestricted-double-y-unrestricted-double-z-unrestricted-double-angle
   */
  CSSMatrix rotateAxisAngle([num opt_x, num opt_y, num opt_z, num opt_angle]);
}

/**
 * @constructor
 * @param {string=} opt_matrix
 * @extends {CSSMatrix}
 * @see http://developer.apple.com/safari/library/documentation/AudioVideo/Reference/WebKitCSSMatrixClassReference/WebKitCSSMatrix/WebKitCSSMatrix.html#//apple_ref/javascript/instm/WebKitCSSMatrix/setMatrixValue
 */
@JsName("WebKitCSSMatrix")
abstract class WebKitCSSMatrix extends CSSMatrix {
// @Ignore
  external WebKitCSSMatrix.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @constructor
   * @param {string=} opt_matrix
   * @extends {CSSMatrix}
   * @see http://developer.apple.com/safari/library/documentation/AudioVideo/Reference/WebKitCSSMatrixClassReference/WebKitCSSMatrix/WebKitCSSMatrix.html#//apple_ref/javascript/instm/WebKitCSSMatrix/setMatrixValue
   */
  external factory WebKitCSSMatrix([String opt_matrix]);
}

/**
 * @constructor
 * @param {string=} opt_matrix
 * @extends {CSSMatrix}
 * @see http://msdn.microsoft.com/en-us/library/windows/apps/hh453593.aspx
 */
@JsName("MSCSSMatrix")
abstract class MSCSSMatrix extends CSSMatrix {
// @Ignore
  external MSCSSMatrix.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @constructor
   * @param {string=} opt_matrix
   * @extends {CSSMatrix}
   * @see http://msdn.microsoft.com/en-us/library/windows/apps/hh453593.aspx
   */
  external factory MSCSSMatrix([String opt_matrix]);
}
