/*
 * Copyright 2008 The Closure Compiler Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// This file has been automatically generated.  Please do not edit it manually.
/**
 * @fileoverview Definitions for W3C's DOM Level 1 specification.
 *  The whole file has been fully type annotated. Created from
 *  http://www.w3.org/TR/REC-DOM-Level-1/ecma-script-language-binding.html
 *
 * FROM_EXTERN
 * @author stevey@google.com (Steve Yegge)
 */
@JsName()
library w3c_dom1;

import 'package:js/js.dart';
import 'package:func/func.dart';

import "es6.dart" as es6;

import "fetchapi.dart" as fetchapi;

import "fileapi.dart" as fileapi;

import "gecko_dom.dart" as gecko_dom;

import "html5.dart" as html5;

import "ie_dom.dart" as ie_dom;

import "iphone.dart" as iphone;

import "w3c_css.dart" as w3c_css;

import "w3c_dom2.dart" as w3c_dom2;

import "w3c_dom3.dart" as w3c_dom3;

import "w3c_event.dart" as w3c_event;

import "w3c_indexeddb.dart" as w3c_indexeddb;

import "w3c_navigation_timing.dart" as w3c_navigation_timing;

import "w3c_range.dart" as w3c_range;

import "webkit_dom.dart" as webkit_dom;

import "webkit_notifications.dart" as webkit_notifications;

import "webstorage.dart" as webstorage;

/**
 * @constructor
 * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-17189187
 */
@JsName("DOMException")
abstract class DOMException {
// @Ignore
  external DOMException.fakeConstructor$();

  external static num get NOT_SUPPORTED_ERR;

  external static set NOT_SUPPORTED_ERR(num value);

  external static num get INVALID_CHARACTER_ERR;

  external static set INVALID_CHARACTER_ERR(num value);

  external static num get NAMESPACE_ERR;

  external static set NAMESPACE_ERR(num value);

  external static num get INVALID_MODIFICATION_ERR;

  external static set INVALID_MODIFICATION_ERR(num value);

  external static num get INVALID_ACCESS_ERR;

  external static set INVALID_ACCESS_ERR(num value);

  external static num get INVALID_STATE_ERR;

  external static set INVALID_STATE_ERR(num value);

  external static num get INUSE_ATTRIBUTE_ERR;

  external static set INUSE_ATTRIBUTE_ERR(num value);

  external static num get SYNTAX_ERR;

  external static set SYNTAX_ERR(num value);

  external static num get TYPE_MISMATCH_ERR;

  external static set TYPE_MISMATCH_ERR(num value);

  external static num get NOT_FOUND_ERR;

  external static set NOT_FOUND_ERR(num value);

  external static num get HIERARCHY_REQUEST_ERR;

  external static set HIERARCHY_REQUEST_ERR(num value);

  external static num get WRONG_DOCUMENT_ERR;

  external static set WRONG_DOCUMENT_ERR(num value);

  external static num get NO_MODIFICATION_ALLOWED_ERR;

  external static set NO_MODIFICATION_ALLOWED_ERR(num value);

  external static num get INDEX_SIZE_ERR;

  external static set INDEX_SIZE_ERR(num value);

  external static num get NO_DATA_ALLOWED_ERR;

  external static set NO_DATA_ALLOWED_ERR(num value);

  external static num get VALIDATION_ERR;

  external static set VALIDATION_ERR(num value);

  external static num get DOMSTRING_SIZE_ERR;

  external static set DOMSTRING_SIZE_ERR(num value);

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-258A00AF
   */
  num code;
}

/**
 * @constructor
 * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-258A00AF
 */
@JsName("ExceptionCode")
abstract class ExceptionCode {
// @Ignore
  external ExceptionCode.fakeConstructor$();
}

/**
 * @constructor
 * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-102161490
 */
@JsName("DOMImplementation")
abstract class DOMImplementation {
// @Ignore
  external DOMImplementation.fakeConstructor$();

  /**
   * @param {string} feature
   * @param {string} version
   * @return {boolean}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-5CED94D7
   * @nosideeffects
   */
  bool hasFeature(String feature, String version);

  /**
   * @param {string} namespaceURI
   * @param {string} publicId
   * @param {DocumentType} doctype
   * @return {Document}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Level-2-Core-DOM-createDocument
   * @nosideeffects
   */
  Document createDocument(
      String namespaceURI, String publicId, DocumentType doctype);

  /**
   * @param {string} qualifiedName
   * @param {string} publicId
   * @param {string} systemId
   * @return {DocumentType}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Level-2-Core-DOM-createDocType
   * @nosideeffects
   */
  DocumentType createDocumentType(
      String qualifiedName, String publicId, String systemId);

  /**
   * @param {string} feature
   * @param {string} version
   * @return {Object}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#DOMImplementation3-getFeature
   * @nosideeffects
   */
  Object getFeature(String feature, String version);

  /**
   * @see http://dom.spec.whatwg.org/#dom-domimplementation-createhtmldocument
   * @param {string=} opt_title A title to give the new HTML document
   * @return {!HTMLDocument}
   */
  w3c_dom2.HTMLDocument createHTMLDocument([String opt_title]);
}

/**
 * @constructor
 * @implements {EventTarget}
 * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-1950641247
 */
@JsName("Node")
abstract class Node implements w3c_event.EventTarget {
// @Ignore
  external Node.fakeConstructor$();

  /**
   * @type {number}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-1950641247
   */
  external static num get TEXT_NODE;

  external static set TEXT_NODE(num value);

  /**
   * @type {number}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-1950641247
   */
  external static num get ENTITY_NODE;

  external static set ENTITY_NODE(num value);

  /**
   * @type {number}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-1950641247
   */
  external static num get DOCUMENT_TYPE_NODE;

  external static set DOCUMENT_TYPE_NODE(num value);

  /**
   * @type {number}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-1950641247
   */
  external static num get NOTATION_NODE;

  external static set NOTATION_NODE(num value);

  /**
   * @type {number}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-1950641247
   */
  external static num get CDATA_SECTION_NODE;

  external static set CDATA_SECTION_NODE(num value);

  external static num get DOCUMENT_POSITION_DISCONNECTED;

  external static set DOCUMENT_POSITION_DISCONNECTED(num value);

  /**
   * @type {number}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-1950641247
   */
  external static num get DOCUMENT_FRAGMENT_NODE;

  external static set DOCUMENT_FRAGMENT_NODE(num value);

  /**
   * @type {number}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-1950641247
   */
  external static num get COMMENT_NODE;

  external static set COMMENT_NODE(num value);

  /**
   * @type {number}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-1950641247
   */
  external static num get PROCESSING_INSTRUCTION_NODE;

  external static set PROCESSING_INSTRUCTION_NODE(num value);

  external static num get DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC;

  external static set DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC(num value);

  /**
   * @type {number}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-1950641247
   */
  external static num get XPATH_NAMESPACE_NODE;

  external static set XPATH_NAMESPACE_NODE(num value);

  /**
   * @type {number}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-1950641247
   */
  external static num get ELEMENT_NODE;

  external static set ELEMENT_NODE(num value);

  external static num get DOCUMENT_POSITION_FOLLOWING;

  external static set DOCUMENT_POSITION_FOLLOWING(num value);

  /**
   * @type {number}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-1950641247
   */
  external static num get ENTITY_REFERENCE_NODE;

  external static set ENTITY_REFERENCE_NODE(num value);

  /**
   * @type {number}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-1950641247
   */
  external static num get ATTRIBUTE_NODE;

  external static set ATTRIBUTE_NODE(num value);

  /**
   * @type {number}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-1950641247
   */
  external static num get DOCUMENT_NODE;

  external static set DOCUMENT_NODE(num value);

  external static num get DOCUMENT_POSITION_CONTAINED_BY;

  external static set DOCUMENT_POSITION_CONTAINED_BY(num value);

  external static num get DOCUMENT_POSITION_PRECEDING;

  external static set DOCUMENT_POSITION_PRECEDING(num value);

  external static num get DOCUMENT_POSITION_CONTAINS;

  external static set DOCUMENT_POSITION_CONTAINS(num value);

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  addEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  removeEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /** @override */
  bool dispatchEvent(w3c_event.Event evt);

  /**
   * @type {NamedNodeMap}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#attribute-attributes
   */
  NamedNodeMap attributes;

  /**
   * @type {!NodeList}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#attribute-childNodes
   */
  NodeList childNodes;

  /**
   * @type {Node}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#attribute-firstChild
   */
  Node firstChild;

  /**
   * @type {Node}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#attribute-lastChild
   */
  Node lastChild;

  /**
   * @type {Node}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#attribute-nextSibling
   */
  Node nextSibling;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#attribute-nodeName
   */
  String nodeName;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#attribute-nodeValue
   */
  String nodeValue;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#attribute-nodeType
   */
  num nodeType;

  /**
   * @type {Document}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#attribute-ownerDocument
   */
  Document ownerDocument;

  /**
   * @type {Node}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#attribute-parentNode
   */
  Node parentNode;

  /**
   * @type {Node}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#attribute-previousSibling
   */
  Node previousSibling;

  /**
   * @param {Node} newChild
   * @return {Node}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#method-appendChild
   */
  Node appendChild(Node newChild);

  /**
   * @param {boolean} deep
   * @return {!Node}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#method-cloneNode
   * @nosideeffects
   */
  Node cloneNode(bool deep);

  /**
   * @return {boolean}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#method-hasChildNodes
   * @nosideeffects
   */
  bool hasChildNodes();

  /**
   * @param {Node} newChild
   * @param {Node} refChild
   * @return {!Node}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#method-insertBefore
   */
  Node insertBefore(Node newChild, Node refChild);

  /**
   * @param {Node} oldChild
   * @return {!Node}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#method-removeChild
   */
  Node removeChild(Node oldChild);

  /**
   * @param {Node} newChild
   * @param {Node} oldChild
   * @return {!Node}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#method-replaceChild
   */
  Node replaceChild(Node newChild, Node oldChild);

  /**
   * @type {?string}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Node3-baseURI
   */
  String baseURI;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-NodeNSLocalN
   */
  String localName;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-NodeNSname
   */
  String namespaceURI;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-NodeNSPrefix
   */
  String prefix;

  /**
   * @type {string}
   * @implicitCast
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Node3-textContent
   */
  String textContent;

  /**
   * @param {Node} other
   * @return {number}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Node3-compareDocumentPosition
   * @nosideeffects
   */
  num compareDocumentPosition(Node other);

  /**
   * @param {string} feature
   * @param {string} version
   * @return {Object}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Node3-getFeature
   * @nosideeffects
   */
  Object getFeature(String feature, String version);

  /**
   * @param {string} key
   * @return {Object}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Node3-getUserData
   * @nosideeffects
   */
  Object getUserData(String key);

  /**
   * @return {boolean}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-NodeHasAttrs
   * @nosideeffects
   */
  bool hasAttributes();

  /**
   * @param {string} namespaceURI
   * @return {boolean}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Node3-isDefaultNamespace
   * @nosideeffects
   */
  bool isDefaultNamespace(String namespaceURI);

  /**
   * @param {Node} arg
   * @return {boolean}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Node3-isEqualNode
   * @nosideeffects
   */
  bool isEqualNode(Node arg);

  /**
   * @param {Node} other
   * @return {boolean}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Node3-isSameNode
   * @nosideeffects
   */
  bool isSameNode(Node other);

  /**
   * @param {string} feature
   * @param {string} version
   * @return {boolean}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Level-2-Core-Node-supports
   * @nosideeffects
   */
  bool isSupported(String feature, String version);

  /**
   * @param {string} prefix
   * @return {string}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Node3-lookupNamespaceURI
   * @nosideeffects
   */
  String lookupNamespaceURI(String prefix);

  /**
   * @param {string} namespaceURI
   * @return {string}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Node3-lookupNamespacePrefix
   * @nosideeffects
   */
  String lookupPrefix(String namespaceURI);

  /**
   * @return undefined
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-normalize
   */
  normalize();

  /**
   * @param {Object} key
   * @param {Object} data
   * @param {UserDataHandler} handler
   * @return {Object}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Node3-setUserData'
   */
  Object setUserData(Object key, Object data, w3c_dom3.UserDataHandler handler);

  /**
   * @param {string} query
   * @return {Node}
   * @see http://www.w3.org/TR/selectors-api/#queryselector
   * @nosideeffects
   */
  Node querySelector(String query);

  /**
   * @param {string} query
   * @return {!NodeList}
   * @see http://www.w3.org/TR/selectors-api/#queryselectorall
   * @nosideeffects
   */
  NodeList querySelectorAll(String query);

  /**
   * @type {string}
   * @see http://msdn.microsoft.com/en-us/library/ms767570(VS.85).aspx
   */
  String baseName;

  /**
   * @type {?string}
   * @see http://msdn.microsoft.com/en-us/library/ms762763(VS.85).aspx
   */
  String dataType;

  /**
   * @type {Node}
   * @see http://msdn.microsoft.com/en-us/library/ms764733(VS.85).aspx
   */
  Node definition;

  /**
   * IE5 used document instead of ownerDocument.
   * Old versions of WebKit used document instead of contentDocument.
   * @type {Document}
   */
  Document document;

  /**
   * Inserts the given HTML text into the element at the location.
   * @param {string} sWhere Where to insert the HTML text, one of 'beforeBegin',
   *     'afterBegin', 'beforeEnd', 'afterEnd'.
   * @param {string} sText HTML text to insert.
   * @see http://msdn.microsoft.com/en-us/library/ms536452(VS.85).aspx
   */
  insertAdjacentHTML(String sWhere, String sText);

  /**
   * @type {*}
   * @see http://msdn.microsoft.com/en-us/library/ms762308(VS.85).aspx
   */
  dynamic nodeTypedValue;

  /**
   * @type {string}
   * @see http://msdn.microsoft.com/en-us/library/ms757895(VS.85).aspx
   */
  String nodeTypeString;

  /**
   * @type {boolean}
   * @see http://msdn.microsoft.com/en-us/library/ms762237(VS.85).aspx
   */
  bool parsed;

  /**
   * @type {Element}
   * @see http://msdn.microsoft.com/en-us/library/ms534327(VS.85).aspx
   */
  Element parentElement;

  /**
   * @type {boolean}
   * @see http://msdn.microsoft.com/en-us/library/ms753816(VS.85).aspx
   */
  bool specified;

  /**
   * @type {string}
   * @see http://msdn.microsoft.com/en-us/library/ms762687(VS.85).aspx
   */
  String text;

  /**
   * @type {string}
   * @see http://msdn.microsoft.com/en-us/library/ms755989(VS.85).aspx
   */
  String xml;

  /**
   * @param {string} expression An XPath expression.
   * @return {NodeList}
   * @see http://msdn.microsoft.com/en-us/library/ms754523(VS.85).aspx
   * @nosideeffects
   */
  NodeList selectNodes(String expression);

  /**
   * @param {string} expression An XPath expression.
   * @return {Node}
   * @see http://msdn.microsoft.com/en-us/library/ms757846(VS.85).aspx
   * @nosideeffects
   */
  Node selectSingleNode(String expression);

  /**
   * @param {Node} stylesheet XSLT stylesheet.
   * @return {string}
   * @see http://msdn.microsoft.com/en-us/library/ms761399(VS.85).aspx
   * @nosideeffects
   */
  String transformNode(Node stylesheet);

  /**
   * @param {Node} stylesheet XSLT stylesheet.
   * @param {Object} outputObject
   * @see http://msdn.microsoft.com/en-us/library/ms766561(VS.85).aspx
   */
  transformNodeToObject(Node stylesheet, Object outputObject);

  /**
   * @param {boolean=} opt_bRemoveChildren Whether to remove the entire sub-tree.
   *    Defaults to false.
   * @return {Node} The object that was removed.
   * @see http://msdn.microsoft.com/en-us/library/ms536708(VS.85).aspx
   */
  Node removeNode([bool opt_bRemoveChildren]);

  /**
   * Note: In IE, the contains() method only exists on Elements, not Nodes.
   * Therefore, it is recommended that you use the Conformance framework to
   * prevent calling this on Nodes which are not Elements.
   * @see https://connect.microsoft.com/IE/feedback/details/780874/node-contains-is-incorrect
   *
   * @param {Node} n The node to check
   * @return {boolean} If 'n' is this Node, or is contained within this Node.
   * @see https://developer.mozilla.org/en-US/docs/Web/API/Node.contains
   * @nosideeffects
   */
  bool contains(Node n);
}

/**
 * @constructor
 * @extends {Node}
 * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-B63ED1A3
 */
@JsName("DocumentFragment")
abstract class DocumentFragment extends Node {
// @Ignore
  external DocumentFragment.fakeConstructor$() : super.fakeConstructor$();
}

/**
 * @constructor
 * @extends {Node}
 * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#i-Document
 */
@JsName("Document")
abstract class Document extends Node {
// @Ignore
  external Document.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {DocumentType}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#attribute-doctype
   */
  DocumentType doctype;

  /**
   * @type {!Element}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#attribute-documentElement
   */
  Element documentElement;

  /**
   * @type {DOMImplementation}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#attribute-implementation
   */
  DOMImplementation implementation;

  /**
   * @param {string} name
   * @return {!Attr}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#method-createAttribute
   * @nosideeffects
   */
  Attr createAttribute(String name);

  /**
   * @param {string} data
   * @return {!Comment}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#method-createComment
   * @nosideeffects
   */
  Comment createComment(String data);

  /**
   * @param {string} data
   * @return {!CDATASection}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#method-createCDATASection
   * @nosideeffects
   */
  CDATASection createCDATASection(String data);

  /**
   * @return {!DocumentFragment}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#method-createDocumentFragment
   * @nosideeffects
   */
  DocumentFragment createDocumentFragment();

  /**
   * Create a DOM element.
   *
   * Web components introduced the second parameter as a way of extending existing
   * tags (e.g. document.createElement('button', 'fancy-button')).
   *
   * @param {string} tagName
   * @param {string=} opt_typeExtension
   * @return {!Element}
   * @nosideeffects
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#method-createElement
   * @see http://w3c.github.io/webcomponents/spec/custom/#extensions-to-document-interface-to-instantiate
   */
  Element createElement(String tagName, [String opt_typeExtension]);

  /**
   * @param {string} name
   * @return {!EntityReference}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#method-createEntityReference
   * @nosideeffects
   */
  EntityReference createEntityReference(String name);

  /**
   * @param {string} target
   * @param {string} data
   * @return {!ProcessingInstruction}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#method-createProcessingInstruction
   * @nosideeffects
   */
  ProcessingInstruction createProcessingInstruction(String target, String data);

  /**
   * @param {number|string} data
   * @return {!Text}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#method-createTextNode
   * @nosideeffects
   */
  Text createTextNode(data /*num|String*/);

  /**
   * @param {string} tagname
   * @return {!NodeList}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-A6C9094
   * @nosideeffects
   */
  NodeList getElementsByTagName(String tagname);

  /**
   * @param {Node} externalNode
   * @return {Node}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Document3-adoptNode
   */
  Node adoptNode(Node externalNode);

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Document3-documentURI
   */
  String documentURI;

  /**
   * @type {DOMConfiguration}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Document3-domConfig
   */
  w3c_dom3.DOMConfiguration domConfig;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Document3-inputEncoding
   */
  String inputEncoding;

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Document3-strictErrorChecking
   */
  bool strictErrorChecking;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Document3-encoding
   */
  String xmlEncoding;

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Document3-standalone
   */
  bool xmlStandalone;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Document3-version
   */
  String xmlVersion;

  /**
   * @return {undefined}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Document3-normalizeDocument
   */
  void normalizeDocument();

  /**
   * @param {Node} n
   * @param {string} namespaceURI
   * @param {string} qualifiedName
   * @return {Node}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Document3-renameNode
   */
  Node renameNode(Node n, String namespaceURI, String qualifiedName);

  /**
   * @see https://developer.mozilla.org/en/DOM/document.alinkColor
   * @type {string}
   */
  String alinkColor;

  /**
   * @see https://developer.mozilla.org/en/DOM/document.anchors
   * @type {HTMLCollection}
   */
  w3c_dom2.HTMLCollection anchors;

  /**
   * @see https://developer.mozilla.org/en/DOM/document.applets
   * @type {HTMLCollection}
   */
  w3c_dom2.HTMLCollection applets;

  /** @type {boolean} */
  bool async;

  /** @type {string?} */
  String baseURI;

  dynamic baseURIObject;

  /**
   * @see https://developer.mozilla.org/en/DOM/document.bgColor
   * @type {string}
   */
  String bgColor;

  /** @type {HTMLBodyElement} */
  w3c_dom2.HTMLBodyElement body;

  dynamic characterSet;

  /**
   * @see https://developer.mozilla.org/en/DOM/document.compatMode
   * @type {string}
   */
  String compatMode;

  dynamic contentType;

  /** @type {string} */
  String cookie;

  dynamic defaultView;

  /**
   * @see https://developer.mozilla.org/en/DOM/document.designMode
   * @type {string}
   */
  String designMode;

  dynamic documentURIObject;

  /**
   * @see https://developer.mozilla.org/en/DOM/document.domain
   * @type {string}
   */
  String domain;

  /**
   * @see https://developer.mozilla.org/en/DOM/document.embeds
   * @type {HTMLCollection}
   */
  w3c_dom2.HTMLCollection embeds;

  /**
   * @see https://developer.mozilla.org/en/DOM/document.fgColor
   * @type {string}
   */
  String fgColor;

  /** @type {Element} */
  Element firstChild;

  /**
   * @see https://developer.mozilla.org/en/DOM/document.forms
   * @type {HTMLCollection}
   */
  w3c_dom2.HTMLCollection forms;

  /** @type {number} */
  num height;

  /** @type {HTMLCollection} */
  w3c_dom2.HTMLCollection images;

  /**
   * @type {string}
   * @see https://developer.mozilla.org/en/DOM/document.lastModified
   */
  String lastModified;

  /**
   * @type {string}
   * @see https://developer.mozilla.org/en/DOM/document.linkColor
   */
  String linkColor;

  /**
   * @see https://developer.mozilla.org/en/DOM/document.links
   * @type {HTMLCollection}
   */
  w3c_dom2.HTMLCollection links;

  /**
   * @type {!Location}
   * @implicitCast
   */
  ie_dom.Location location;

  String namespaceURI;

  dynamic nodePrincipal;

  dynamic plugins;

  dynamic popupNode;

  /**
   * @type {string}
   * @see https://developer.mozilla.org/en/DOM/document.referrer
   */
  String referrer;

  /**
   * @type {StyleSheetList}
   * @see https://developer.mozilla.org/en/DOM/document.styleSheets
   */
  w3c_css.StyleSheetList styleSheets;

  /** @type {?string} */
  String title;

  dynamic tooltipNode;

  /** @type {string} */
  String URL;

  /**
   * @type {string}
   * @see https://developer.mozilla.org/en/DOM/document.vlinkColor
   */
  String vlinkColor;

  /** @type {number} */
  num width;

  /**
   * @see https://developer.mozilla.org/en/DOM/document.clear
   */
  clear();

  /**
   * @see https://developer.mozilla.org/en-US/docs/Web/API/document.createElementNS
   * @see http://w3c.github.io/webcomponents/spec/custom/#extensions-to-document-interface-to-instantiate
   * @param {?string} namespaceURI
   * @param {string} qualifiedName
   * @param {string=} opt_typeExtension
   * @return {!Element}
   */
  Element createElementNS(String namespaceURI, String qualifiedName,
      [String opt_typeExtension]);

  /**
   * @param {string} type
   * @return {Event}
   */
  w3c_event.Event createEvent(String type);

  dynamic createNSResolver;

  /** @return {Range} */
  w3c_range.Range createRange();

  dynamic evaluate;

  /**
   * @param {string} commandName
   * @param {?boolean=} opt_showUi
   * @param {*=} opt_value
   * @see https://developer.mozilla.org/en/Rich-Text_Editing_in_Mozilla#Executing_Commands
   */
  external execCommand(String commandName, [bool opt_showUi, opt_value]);
  /**
   * @param {string} s id.
   * @return {HTMLElement}
   * @nosideeffects
   * @see https://developer.mozilla.org/en/DOM/document.getElementById
   */
  w3c_dom2.HTMLElement getElementById(String s);

  /**
   * @param {string} name
   * @return {!NodeList}
   * @nosideeffects
   * @see https://developer.mozilla.org/en/DOM/document.getElementsByClassName
   */
  NodeList getElementsByClassName(String name);

  /**
   * @param {string} name
   * @return {!NodeList}
   * @nosideeffects
   * @see https://developer.mozilla.org/en/DOM/document.getElementsByName
   */
  NodeList getElementsByName(String name);

  /**
   * @param {string} namespace
   * @param {string} name
   * @return {!NodeList}
   * @nosideeffects
   * @see https://developer.mozilla.org/en/DOM/document.getElementsByTagNameNS
   */
  NodeList getElementsByTagNameNS(String namespace, String name);

  /**
   * @param {Node} externalNode
   * @param {boolean} deep
   * @return {Node}
   */
  Node importNode(Node externalNode, bool deep);

  /** @param {string} uri */
  load(String uri);

  dynamic loadOverlay;

  /**
   * @see https://developer.mozilla.org/en/Midas
   * @see http://msdn.microsoft.com/en-us/library/ms536676(VS.85).aspx
   */
  dynamic queryCommandEnabled;

  /**
   * @see https://developer.mozilla.org/en/Midas
   * @see http://msdn.microsoft.com/en-us/library/ms536678(VS.85).aspx
   */
  dynamic queryCommandIndeterm;

  /**
   * @see https://developer.mozilla.org/en/Midas
   * @see http://msdn.microsoft.com/en-us/library/ms536679(VS.85).aspx
   */
  dynamic queryCommandState;

  /**
   * @see https://developer.mozilla.org/en/DOM/document.queryCommandSupported
   * @see http://msdn.microsoft.com/en-us/library/ms536681(VS.85).aspx
   * @param {string} command
   * @return {?} Implementation-specific.
   */
  external queryCommandSupported(String command);
  /**
   * @see https://developer.mozilla.org/en/Midas
   * @see http://msdn.microsoft.com/en-us/library/ms536683(VS.85).aspx
   */
  dynamic queryCommandValue;

  /**
   * @see https://developer.mozilla.org/en/DOM/document.write
   * @param {string} text
   */
  write(String text);

  /**
   * @see https://developer.mozilla.org/en/DOM/document.writeln
   * @param {string} text
   */
  writeln(String text);

  dynamic ononline;

  dynamic onoffline;

  /**
   * @see http://developer.mozilla.org/en/DOM/document.getBoxObjectFor
   * @return {BoxObject}
   * @nosideeffects
   */
  gecko_dom.BoxObject getBoxObjectFor(element);

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms533065(VS.85).aspx
   */
  dynamic activeElement;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms533553(VS.85).aspx
   */
  dynamic charset;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms533693(VS.85).aspx
   */
  /**
   * @see http://msdn.microsoft.com/en-us/library/ms533714(VS.85).aspx
   */
  dynamic defaultCharset;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms533731(VS.85).aspx
   */
  dynamic dir;

  /**
   * @see http://msdn.microsoft.com/en-us/library/cc196988(VS.85).aspx
   */
  dynamic documentMode;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms533747(VS.85).aspx
   */
  dynamic expando;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms533750(VS.85).aspx
   */
  dynamic fileCreatedDate;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms533751(VS.85).aspx
   */
  dynamic fileModifiedDate;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms533752(VS.85).aspx
   */
  dynamic fileSize;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms534331(VS.85).aspx
   */
  dynamic parentWindow;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms534353(VS.85).aspx
   */
  dynamic protocol;

  /**
   * @type {Selection}
   * @see http://msdn.microsoft.com/en-us/library/ms535869(VS.85).aspx
   */
  gecko_dom.Selection selection;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms534704(VS.85).aspx
   */
  dynamic uniqueID;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms534709(VS.85).aspx
   */
  dynamic URLUnencoded;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms535155(VS.85).aspx
   */
  dynamic XMLDocument;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms535163(VS.85).aspx
   */
  dynamic XSLDocument;

  /**
   * @param {string} event
   * @param {Function} handler
   * @see http://msdn.microsoft.com/en-us/library/ms536343(VS.85).aspx
   */
  external attachEvent(String event, Function handler);
  /**
   * @see http://msdn.microsoft.com/en-us/library/ms536390(VS.85).aspx
   */
  dynamic createEventObject;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms531194(VS.85).aspx
   */
  dynamic createStyleSheet;

  /**
   * @param {string} event
   * @param {Function} handler
   * @see http://msdn.microsoft.com/en-us/library/ms536411(VS.85).aspx
   */
  external detachEvent(String event, Function handler);
  /**
   * @see http://msdn.microsoft.com/en-us/library/ms536425(VS.85).aspx
   */
  dynamic focus;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms536447(VS.85).aspx
   * @return {boolean}
   */
  bool hasFocus();

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms536614(VS.85).aspx
   */
  dynamic mergeAttributes;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms536685(VS.85).aspx
   */
  dynamic recalc;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms536689(VS.85).aspx
   */
  dynamic releaseCapture;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms536738(VS.85).aspx
   */
  dynamic setActive;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms537434(VS.85).aspx
   */
  dynamic all;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms537445(VS.85).aspx
   */
  NodeList childNodes;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms537459(VS.85).aspx
   */
  dynamic frames;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms537470(VS.85).aspx
   */
  dynamic namespaces;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms537487(VS.85).aspx
   */
  dynamic scripts;

  /**
   * @param {string} contextId
   * @param {string} name
   * @param {number} width
   * @param {number} height
   * @nosideeffects
   */
  getCSSCanvasContext(String contextId, String name, num width, num height);

  /**
   * @param {number} x
   * @param {number} y
   * @return {?Element}
   * @see http://www.w3.org/TR/cssom-view/#dom-document-elementfrompoint
   */
  Element elementFromPoint(num x, num y);

  /**
   * @param {number} x
   * @param {number} y
   * @return {CaretPosition}
   * @see http://www.w3.org/TR/cssom-view/#dom-document-caretpositionfrompoint
   */
  w3c_css.CaretPosition caretPositionFromPoint(num x, num y);

  /**
   * The postMessage method (as implemented in Opera).
   * @param {string} message
   */
  postMessage(String message);

  /**
   * Document head accessor.
   * @see http://www.whatwg.org/specs/web-apps/current-work/multipage/dom.html#the-head-element-0
   * @type {HTMLHeadElement}
   */
  w3c_dom2.HTMLHeadElement head;

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/2012/WD-fullscreen-20120703/#dom-document-fullscreenenabled
   */
  bool fullscreenEnabled;

  /**
   * @type {Element}
   * @see http://www.w3.org/TR/2012/WD-fullscreen-20120703/#dom-document-fullscreenelement
   */
  Element fullscreenElement;

  /**
   * @see http://www.w3.org/TR/2012/WD-fullscreen-20120703/#dom-document-exitfullscreen
   */
  exitFullscreen();

  /** @type {boolean} */
  bool mozFullScreen;

  mozCancelFullScreen();

  /** @type {Element} */
  Element mozFullScreenElement;

  /** @type {boolean} */
  bool mozFullScreenEnabled;

  /** @type {boolean} */
  bool webkitIsFullScreen;

  webkitCancelFullScreen();

  /** @type {Element} */
  Element webkitCurrentFullScreenElement;

  /** @type {Element} */
  Element webkitFullscreenElement;

  /** @type {boolean} */
  bool webkitFullScreenKeyboardInputAllowed;

  /** @type {boolean} */
  bool msFullscreenEnabled;

  /** @type {Element} */
  Element msFullscreenElement;

  /**
   * @see http://www.w3.org/TR/page-visibility/
   * @type {VisibilityState}
   */
  String visibilityState;

  /**
   * @type {string}
   */
  String mozVisibilityState;

  /**
   * @type {string}
   */
  String webkitVisibilityState;

  /**
   * @type {string}
   */
  String msVisibilityState;

  /**
   * @see http://www.w3.org/TR/page-visibility/
   * @type {boolean}
   */
  bool hidden;

  /**
   * @type {boolean}
   */
  bool mozHidden;

  /**
   * @type {boolean}
   */
  bool webkitHidden;

  /**
   * @type {boolean}
   */
  bool msHidden;

  /**
   * @see http://www.w3.org/TR/components-intro/
   * @see http://w3c.github.io/webcomponents/spec/custom/#extensions-to-document-interface-to-register
   * @param {string} type
   * @param {{extends: (string|undefined), prototype: (Object|undefined)}} options
   */
  external registerElement(String type, OptionsRecord options);
  /**
   * This method is deprecated and should be removed by the end of 2014.
   * @see http://www.w3.org/TR/components-intro/
   * @see http://w3c.github.io/webcomponents/spec/custom/#extensions-to-document-interface-to-register
   * @param {string} type
   * @param {{extends: (string|undefined), prototype: (Object|undefined)}} options
   */
  external register(String type, OptionsRecord options);
  /**
   * @type {!FontFaceSet}
   * @see http://dev.w3.org/csswg/css-font-loading/#dom-fontfacesource-fonts
   */
  w3c_css.FontFaceSet fonts;

  /**
   * @see https://developer.mozilla.org/en-US/docs/Web/API/Document/currentScript
   */
  dynamic currentScript;

  /**
   * Creates a new Touch object.
   * @see http://developer.apple.com/library/safari/#documentation/UserExperience/Reference/DocumentAdditionsReference/DocumentAdditions/DocumentAdditions.html#//apple_ref/javascript/instm/Document/createTouch
   * @param {Window} view
   * @param {EventTarget} target
   * @param {number} identifier
   * @param {number} pageX
   * @param {number} pageY
   * @param {number} screenX
   * @param {number} screenY
   * @return {Touch}
   */
  iphone.Touch createTouch(Window view, w3c_event.EventTarget target,
      num identifier, num pageX, num pageY, num screenX, num screenY);

  /**
   * Creates a new TouchList object.
   * @see http://developer.apple.com/library/safari/#documentation/UserExperience/Reference/DocumentAdditionsReference/DocumentAdditions/DocumentAdditions.html#//apple_ref/javascript/instm/Document/createTouchList
   * @param {Array.<Touch>} touches
   * @return {TouchList}
   */
  iphone.TouchList createTouchList(List<iphone.Touch> touches);

  /**
   * @param {string} selectors
   * @return {Element}
   * @override
   */
  Element querySelector(String selectors);

  /**
   * @param {string} selectors
   * @return {!NodeList}
   * @override
   */
  NodeList querySelectorAll(String selectors);
}

/**
 * @constructor
 * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-536297177
 */
@JsName("NodeList")
abstract class NodeList {
// @Ignore
  external NodeList.fakeConstructor$();

  /**
   * @type {number}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-203510337
   */
  num length;

  /**
   * @param {number} index
   * @return {Node}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-844377136
   */
  Node item(num index);
}

/**
 * @constructor
 * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-1780488922
 */
@JsName("NamedNodeMap")
abstract class NamedNodeMap {
// @Ignore
  external NamedNodeMap.fakeConstructor$();

  /**
   * @type {number}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-6D0FB19E
   */
  num length;

  /**
   * @param {string} name
   * @return {Node}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-1074577549
   * @nosideeffects
   */
  Node getNamedItem(String name);

  /**
   * @param {number} index
   * @return {Node}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-349467F9
   * @nosideeffects
   */
  Node item(num index);

  /**
   * @param {string} name
   * @return {Node}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-D58B193
   */
  Node removeNamedItem(String name);

  /**
   * @param {Node} arg
   * @return {Node}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-1025163788
   */
  Node setNamedItem(Node arg);
}

/**
 * @constructor
 * @extends {Node}
 * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-FF21A306
 */
@JsName("CharacterData")
abstract class CharacterData extends Node {
// @Ignore
  external CharacterData.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-72AB8359
   */
  String data;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-7D61178C
   */
  num length;

  /**
   * @param {string} arg
   * @return {undefined}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-32791A2F
   */
  void appendData(String arg);

  /**
   * @param {number} offset
   * @param {number} count
   * @return {undefined}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-7C603781
   */
  void deleteData(num offset, num count);

  /**
   * @param {number} offset
   * @param {string} arg
   * @return {undefined}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-3EDB695F
   */
  void insertData(num offset, String arg);

  /**
   * @param {number} offset
   * @param {number} count
   * @param {string} arg
   * @return {undefined}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-E5CBA7FB
   */
  void replaceData(num offset, num count, String arg);

  /**
   * @param {number} offset
   * @param {number} count
   * @return {string}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-6531BCCF
   * @nosideeffects
   */
  String substringData(num offset, num count);
}

/**
 * @constructor
 * @extends {Node}
 * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-637646024
 */
@JsName("Attr")
abstract class Attr extends Node {
// @Ignore
  external Attr.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-1112119403
   */
  String name;

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-862529273
   */
  bool specified;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-221662474
   */
  String value;

  /**
   * @type {Element}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Attr-ownerElement
   */
  Element ownerElement;

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Attr-isId
   */
  bool isId;

  /**
   * @type {TypeInfo}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Attr-schemaTypeInfo
   */
  w3c_dom3.TypeInfo schemaTypeInfo;
}

/**
 * @constructor
 * @extends {Node}
 * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-745549614
 */
@JsName("Element")
abstract class Element extends Node {
// @Ignore
  external Element.fakeConstructor$() : super.fakeConstructor$();

  @JsName(r"ALLOW_KEYBOARD_INPUT")
  external static num get ALLOW_KEYBOARD_INPUT$static;

  @JsName(r"ALLOW_KEYBOARD_INPUT")
  external static set ALLOW_KEYBOARD_INPUT$static(num value);

  /**
   * @type {string}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#attribute-tagName
   */
  String tagName;

  /**
   * @param {string} name
   * @param {number?=} opt_flags
   * @return {string}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#method-getAttribute
   * @see http://msdn.microsoft.com/en-us/library/ms536429(VS.85).aspx
   * @nosideeffects
   */
  String getAttribute(String name, [num opt_flags]);

  /**
   * @param {string} name
   * @return {Attr}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#method-getAttributeNode
   * @nosideeffects
   */
  Attr getAttributeNode(String name);

  /**
   * @param {string} tagname
   * @return {!NodeList}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-1938918D
   * @nosideeffects
   */
  NodeList getElementsByTagName(String tagname);

  /**
   * @param {string} name
   * @return {undefined}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#method-removeAttribute
   */
  void removeAttribute(String name);

  /**
   * @param {Attr} oldAttr
   * @return {?Attr}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#method-removeAttributeNode
   */
  Attr removeAttributeNode(Attr oldAttr);

  /**
   * @param {string} name
   * @param {string|number|boolean} value Values are converted to strings with
   *     ToString, so we accept number and boolean since both convert easily to
   *     strings.
   * @return {undefined}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#method-setAttribute
   */
  void setAttribute(String name, value /*String|num|bool*/);

  /**
   * @param {Attr} newAttr
   * @return {?Attr}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#method-setAttributeNode
   */
  Attr setAttributeNode(Attr newAttr);

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onabort;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onbeforeinput;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onbeforeunload;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onblur;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onchange;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onclick;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> oncompositionstart;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> oncompositionupdate;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> oncompositionend;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> oncontextmenu;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> oncopy;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> oncut;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> ondblclick;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onerror;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onfocus;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onfocusin;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onfocusout;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> oninput;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onkeydown;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onkeypress;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onkeyup;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onload;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onunload;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onmousedown;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onmousemove;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onmouseout;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onmouseover;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onmouseup;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onmousewheel;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onpaste;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onreset;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onresize;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onscroll;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onselect;

  /** @type {?function (Event=)} */
  Func1Opt1<w3c_event.Event, dynamic> onsubmit;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> ontextinput;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onwheel;

  /**
   * @type {TypeInfo}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Element-schemaTypeInfo
   */
  w3c_dom3.TypeInfo schemaTypeInfo;

  /**
   * @param {string} namespaceURI
   * @param {string} localName
   * @return {Attr}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-ElGetAtNodeNS
   * @nosideeffects
   */
  Attr getAttributeNodeNS(String namespaceURI, String localName);

  /**
   * @param {string} namespaceURI
   * @param {string} localName
   * @return {string}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-ElGetAttrNS
   * @nosideeffects
   */
  String getAttributeNS(String namespaceURI, String localName);

  /**
   * @param {string} namespaceURI
   * @param {string} localName
   * @return {!NodeList}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-A6C90942
   * @nosideeffects
   */
  NodeList getElementsByTagNameNS(String namespaceURI, String localName);

  /**
   * @param {string} name
   * @return {boolean}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-ElHasAttr
   * @nosideeffects
   */
  bool hasAttribute(String name);

  /**
   * @param {string} namespaceURI
   * @param {string} localName
   * @return {boolean}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-ElHasAttrNS
   * @nosideeffects
   */
  bool hasAttributeNS(String namespaceURI, String localName);

  /**
   * @param {string} namespaceURI
   * @param {string} localName
   * @return {undefined}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-ElRemAtNS
   */
  void removeAttributeNS(String namespaceURI, String localName);

  /**
   * @param {Attr} newAttr
   * @return {Attr}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-ElSetAtNodeNS
   */
  Attr setAttributeNodeNS(Attr newAttr);

  /**
   * @param {string} namespaceURI
   * @param {string} qualifiedName
   * @param {string|number|boolean} value Values are converted to strings with
   *     ToString, so we accept number and boolean since both convert easily to
   *     strings.
   * @return {undefined}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-ElSetAttrNS
   */
  void setAttributeNS(
      String namespaceURI, String qualifiedName, value /*String|num|bool*/);

  /**
   * @param {string} name
   * @param {boolean} isId
   * @return {undefined}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-ElSetIdAttr
   */
  void setIdAttribute(String name, bool isId);

  /**
   * @param {Attr} idAttr
   * @param {boolean} isId
   * @return {undefined}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-ElSetIdAttrNode
   */
  void setIdAttributeNode(Attr idAttr, bool isId);

  /**
   * @param {string} namespaceURI
   * @param {string} localName
   * @param {boolean} isId
   * @return {undefined}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-ElSetIdAttrNS
   */
  void setIdAttributeNS(String namespaceURI, String localName, bool isId);

  /** @type {NamedNodeMap} */
  NamedNodeMap attributes;

  dynamic baseURIObject;

  /** @type {!NodeList} */
  NodeList childNodes;

  /**
   * @type {!NodeList}
   * @see https://developer.mozilla.org/en/DOM/element.children
   */
  NodeList children;

  /**
   * @type {string}
   * @implicitCast
   */
  String className;

  /** @type {string} */
  String dir;

  /**
   * Firebug sets this property on elements it is inserting into the DOM.
   * @type {boolean}
   */
  bool firebugIgnore;

  /** @type {Node} */
  Node firstChild;

  /**
   * @type {string}
   * @implicitCast
   */
  String id;

  /**
   * @type {string}
   * @implicitCast
   */
  String innerHTML;

  /** @type {string} */
  String lang;

  /** @type {Node} */
  Node lastChild;

  String localName;

  dynamic name;

  String namespaceURI;

  /** @type {Node} */
  Node nextSibling;

  String nodeName;

  dynamic nodePrincipal;

  /** @type {number} */
  num nodeType;

  String nodeValue;

  /** @type {Document} */
  Document ownerDocument;

  /** @type {Node} */
  Node parentNode;

  String prefix;

  /** @type {Node} */
  Node previousSibling;

  /** @type {!CSSStyleDeclaration} */
  w3c_css.CSSStyleDeclaration style;

  /**
   * @type {number}
   * @implicitCast
   */
  num tabIndex;

  /**
   * @type {string}
   * @implicitCast
   */
  String textContent;

  /** @type {string} */
  String title;

  /**
   * @param {Node} child
   * @return {Node} appendedElement.
   * @override
   */
  Node appendChild(Node child);

  /**
   * @override
   * @return {!Element}
   */
  Element cloneNode(bool deep);

  /** @override */
  bool dispatchEvent(w3c_event.Event event);

  /** @return {undefined} */
  void blur();

  /** @return {undefined} */
  void click();

  /** @return {undefined} */
  void focus();

  /**
   * @return {boolean}
   * @override
   * @nosideeffects
   */
  bool hasAttributes();

  /**
   * @return {boolean}
   * @override
   * @nosideeffects
   */
  bool hasChildNodes();

  /** @override */
  Node insertBefore(Node insertedNode, Node adjacentNode);

  /**
   * @return {undefined}
   * @override
   */
  void normalize();

  /**
   * @param {Node} removedNode
   * @return {!Node}
   * @override
   */
  Node removeChild(Node removedNode);

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  removeEventListener(String type, bool handler(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /** @override */
  Node replaceChild(Node insertedNode, Node replacedNode);

  /**
   * @param {string} sUrl
   * @return {number}
   * @see http://msdn.microsoft.com/en-us/library/ms535922(VS.85).aspx
   */
  num addBehavior(String sUrl);

  /**
   * @param {string} event
   * @param {Function} handler
   * @see http://msdn.microsoft.com/en-us/library/mm536343(v=vs.85).aspx
   */
  external attachEvent(String event, Function handler);
  /**
   * @type {boolean}
   * @see http://msdn.microsoft.com/en-us/library/ms533546(VS.85).aspx
   */
  bool canHaveChildren;

  /**
   * @type {string}
   * @see http://msdn.microsoft.com/en-us/library/ms533559(v=vs.85).aspx
   */
  String classid;

  /**
   * @param {number} iCoordX Integer that specifies the client window coordinate
   *     of x.
   * @param {number} iCoordY Integer that specifies the client window coordinate
   *     of y.
   * @return {string} The component of an element located at the specified
   *     coordinates.
   * @see http://msdn.microsoft.com/en-us/library/ms536375(VS.85).aspx
   * @nosideeffects
   */
  String componentFromPoint(num iCoordX, num iCoordY);

  /**
   * @type {boolean}
   * @see http://msdn.microsoft.com/en-us/library/ms533690(VS.85).aspx
   */
  bool contentEditable;

  /**
   * @return {TextRange}
   * @see http://msdn.microsoft.com/en-us/library/ms536401(VS.85).aspx
   */
  external ie_dom.TextRange createTextRange();
  /**
   * @see http://msdn.microsoft.com/en-us/library/ms535231(VS.85).aspx
   */
  dynamic currentStyle;

  /**
   * @param {string} event
   * @param {Function} handler
   * @see http://msdn.microsoft.com/en-us/library/ie/ms536411(v=vs.85).aspx
   */
  external detachEvent(String event, Function handler);
  /**
   * @param {string=} opt_action
   * @see http://msdn.microsoft.com/en-us/library/ms536414%28VS.85%29.aspx
   */
  doScroll([String opt_action]);

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms536423(VS.85).aspx
   */
  dynamic fireEvent;

  /**
   * @type {boolean}
   * @see http://msdn.microsoft.com/en-us/library/ms533783(VS.85).aspx
   */
  bool hideFocus;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms533899.aspx
   */
  dynamic innerText;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms537838(VS.85).aspx
   */
  dynamic isContentEditable;

  /**
   * @param {number} pointerId Id of the pointer that is assign to the element.
   * @see http://msdn.microsoft.com/en-us/library/ie/hh771882(v=vs.85).aspx
   */
  msSetPointerCapture(num pointerId);

  /**
   * @param {number} pointerId
   * @see http://msdn.microsoft.com/en-us/library/ie/hh771880.aspx
   */
  msReleasePointerCapture(num pointerId);

  /**
   * @type {?function(Event)}
   * @see http://msdn.microsoft.com/en-us/library/ms536903(v=vs.85).aspx
   */
  Func1<w3c_event.Event, dynamic> onbeforedeactivate;

  /**
   * @type {?function(Event)}
   * @see http://msdn.microsoft.com/en-us/library/ms536945(VS.85).aspx
   */
  Func1<w3c_event.Event, dynamic> onmouseenter;

  /**
   * @type {?function(Event)}
   * @see http://msdn.microsoft.com/en-us/library/ms536946(VS.85).aspx
   */
  Func1<w3c_event.Event, dynamic> onmouseleave;

  /**
   * @type {?function(Event)}
   * @see http://msdn.microsoft.com/en-us/library/ms536969(VS.85).aspx
   */
  Func1<w3c_event.Event, dynamic> onselectstart;

  /**
   * @type {string}
   * @see http://msdn.microsoft.com/en-us/library/aa752326(VS.85).aspx
   */
  String outerHTML;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms536689(VS.85).aspx
   */
  releaseCapture();

  /**
   * @param {number} iID
   * @return {boolean}
   * @see http://msdn.microsoft.com/en-us/library/ms536700(VS.85).aspx
   */
  bool removeBehavior(num iID);

  /**
   * @see http://msdn.microsoft.com/en-us/library/aa703996(VS.85).aspx
   */
  dynamic runtimeStyle;

  /**
   * @param {string} sStoreName The arbitrary name assigned to a persistent object
   *     in a UserData store.
   * @see http://msdn.microsoft.com/en-us/library/ms531403(v=vs.85).aspx
   */
  save(String sStoreName);

  /**
   * @param {boolean=} opt_bContainerCapture Events originating in a container are
   *     captured by the container. Defaults to true.
   * @see http://msdn.microsoft.com/en-us/library/ms536742(VS.85).aspx
   */
  setCapture([bool opt_bContainerCapture]);

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms534635(VS.85).aspx
   */
  dynamic sourceIndex;

  /**
   * @type {string}
   * @see http://msdn.microsoft.com/en-us/library/ms537840.aspx
   */
  String unselectable;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms537452(v=vs.85).aspx
   * @type {HTMLFiltersCollection}
   */
  ie_dom.HTMLFiltersCollection filters;

  /**
   * @param {boolean=} opt_center
   * @see https://bugzilla.mozilla.org/show_bug.cgi?id=403510
   */
  scrollIntoViewIfNeeded([bool opt_center]);

  /**
   * @return {!ClientRectList}
   * @see http://www.w3.org/TR/cssom-view/#dom-element-getclientrects
   */
  w3c_css.ClientRectList getClientRects();

  /**
   * @return {!ClientRect}
   * @see http://www.w3.org/TR/cssom-view/#dom-element-getboundingclientrect
   */
  w3c_css.ClientRect getBoundingClientRect();

  /**
   * @param {boolean=} opt_top
   * @see http://www.w3.org/TR/cssom-view/#dom-element-scrollintoview
   */
  scrollIntoView([bool opt_top]);

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-element-scrolltop
   */
  num scrollTop;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-element-scrollleft
   */
  num scrollLeft;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-element-scrollwidth
   */
  num scrollWidth;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-element-scrollheight
   */
  num scrollHeight;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-element-clienttop
   */
  num clientTop;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-element-clientleft
   */
  num clientLeft;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-element-clientwidth
   */
  num clientWidth;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-element-clientheight
   */
  num clientHeight;

  /**
   * @see http://www.w3.org/TR/2012/WD-fullscreen-20120703/#dom-element-requestfullscreen
   */
  requestFullscreen();

  mozRequestFullScreen();

  mozRequestFullScreenWithKeys();

  /**
   * The current fullscreen element for the document is set to this element.
   * Valid only for Webkit browsers.
   * @param {number=} opt_allowKeyboardInput Whether keyboard input is desired.
   *     Should use ALLOW_KEYBOARD_INPUT constant.
   */
  webkitRequestFullScreen([num opt_allowKeyboardInput]);

  /**
   * The current fullscreen element for the document is set to this element.
   * Valid only for Webkit browsers.
   * @param {number=} opt_allowKeyboardInput Whether keyboard input is desired.
   *     Should use ALLOW_KEYBOARD_INPUT constant.
   */
  webkitRequestFullscreen([num opt_allowKeyboardInput]);

  /** @return {void} */
  void msRequestFullscreen();

  /** @return {void} */
  void msExitFullscreen();

  /**
   * @type {Object|string}
   * @see http://msdn.microsoft.com/en-us/library/bb263836(VS.85).aspx
   */
  dynamic /*Object|null|String*/ coordorigin;

  /**
   * @type {Object|string}
   * @see http://msdn.microsoft.com/en-us/library/bb263837(VS.85).aspx
   */
  dynamic /*Object|null|String*/ coordsize;

  /**
   * @type {string}
   * @see http://msdn.microsoft.com/en-us/library/bb263839(VS.85).aspx
   */
  String fillcolor;

  /**
   * @type {boolean}
   * @see http://msdn.microsoft.com/en-us/library/bb263840(VS.85).aspx
   */
  bool filled;

  /**
   * @type {string}
   * @see http://msdn.microsoft.com/en-us/library/bb263871(VS.85).aspx
   */
  String path;

  /**
   * @type {number|string}
   * @see http://msdn.microsoft.com/en-us/library/bb263877(VS.85).aspx
   */
  dynamic /*num|String*/ rotation;

  /**
   * @type {string}
   * @see http://msdn.microsoft.com/en-us/library/bb263881(VS.85).aspx
   */
  String strokecolor;

  /**
   * @type {boolean}
   * @see http://msdn.microsoft.com/en-us/library/bb263882(VS.85).aspx
   */
  bool stroked;

  /**
   * @type {number|string}
   * @see http://msdn.microsoft.com/en-us/library/bb263883(VS.85).aspx
   */
  dynamic /*num|String*/ strokeweight;

  /**
   * Specifies the JavaScript method to invoke when the system cancels tracking
   * for the touch.
   * @type {?function(!TouchEvent)}
   */
  Func1<iphone.TouchEvent, dynamic> ontouchcancel;

  /**
   * Specifies the JavaScript method to invoke when a given event lifts from the
   * surface.
   * @type {?function(!TouchEvent)}
   */
  Func1<iphone.TouchEvent, dynamic> ontouchend;

  /**
   * Specifies the JavaScript method to invoke when a finger for a given event
   * moves on the surface.
   * @type {?function(!TouchEvent)}
   */
  Func1<iphone.TouchEvent, dynamic> ontouchmove;

  /**
   * Specifies the JavaScript method to invoke when a finger for a given event
   * touches the surface.
   * @type {?function(!TouchEvent)}
   */
  Func1<iphone.TouchEvent, dynamic> ontouchstart;

  /**
   * Specifies the JavaScript method to invoke when a gesture is started by
   * two or more fingers touching the surface.
   * @type {?function(!GestureEvent)}
   */
  Func1<iphone.GestureEvent, dynamic> ongesturestart;

  /**
   * Specifies the JavaScript method to invoke when fingers are moved during a
   * gesture.
   * @type {?function(!GestureEvent)}
   */
  Func1<iphone.GestureEvent, dynamic> ongesturechange;

  /**
   * Specifies the JavaScript method to invoke when a gesture ends (when there are
   * 0 or 1 fingers touching the surface).
   * @type {?function(!GestureEvent)}
   */
  Func1<iphone.GestureEvent, dynamic> ongestureend;

  /**
   * @type {Element}
   * @see https://developer.mozilla.org/En/DOM/Element.firstElementChild
   */
  Element firstElementChild;

  /**
   * @type {Element}
   * @see https://developer.mozilla.org/En/DOM/Element.lastElementChild
   */
  Element lastElementChild;

  /**
   * @type {Element}
   * @see https://developer.mozilla.org/En/DOM/Element.previousElementSibling
   */
  Element previousElementSibling;

  /**
   * @type {Element}
   * @see https://developer.mozilla.org/En/DOM/Element.nextElementSibling
   */
  Element nextElementSibling;

  /**
   * @type {number}
   * @see https://developer.mozilla.org/En/DOM/Element.childElementCount
   */
  num childElementCount;

  /**
   * @param {string} selectors
   * @return {Element}
   * @override
   */
  Element querySelector(String selectors);

  /**
   * @param {string} selectors
   * @return {!NodeList}
   * @override
   */
  NodeList querySelectorAll(String selectors);

  /**
   * https://dom.spec.whatwg.org/#dom-element-matches
   * https://developer.mozilla.org/en-US/docs/Web/API/Element.matches
   * @param {string} selectors
   * @return {boolean}
   */
  bool matches(String selectors);

  /**
   * @param {string} selectors
   * @param {(Node|NodeList)=} refNodes
   * @return {boolean}
   */
  bool matchesSelector(String selectors,
      [refNodes /*Node|null|NodeList|undefined*/]);

  /**
   * @see https://developer.mozilla.org/en/DOM/Node.mozMatchesSelector
   * @param {string} selectors
   * @return {boolean}
   */
  bool mozMatchesSelector(String selectors);

  /**
   * @see http://developer.apple.com/library/safari/documentation/WebKit/Reference/ElementClassRef/Element/Element.html
   * @param {string} selectors
   * @return {boolean}
   */
  bool webkitMatchesSelector(String selectors);

  /**
   * @see http://msdn.microsoft.com/en-us/library/ff975201.aspx
   * @param {string} selectors
   * @return {boolean}
   */
  bool msMatchesSelector(String selectors);

  /**
   * @see http://www.opera.com/docs/changelogs/windows/1150/
   * @param {string} selectors
   * @return {boolean}
   */
  bool oMatchesSelector(String selectors);
}

/**
 * @constructor
 * @extends {CharacterData}
 * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-1312295772
 */
@JsName("Text")
abstract class Text extends CharacterData {
// @Ignore
  external Text.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @param {number} offset
   * @return {Text}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-38853C1D
   */
  Text splitText(num offset);

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Text3-wholeText
   */
  String wholeText;

  /**
   * @param {string} newText
   * @return {Text}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Text3-replaceWholeText
   */
  Text replaceWholeText(String newText);

  /**
   * @see http://www.w3.org/TR/shadow-dom/
   * @return {!NodeList}
   */
  NodeList getDestinationInsertionPoints();
}

/**
 * @constructor
 * @extends {CharacterData}
 * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-1728279322
 */
@JsName("Comment")
abstract class Comment extends CharacterData {
// @Ignore
  external Comment.fakeConstructor$() : super.fakeConstructor$();
}

/**
 * @constructor
 * @extends {Text}
 * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-667469212
 */
@JsName("CDATASection")
abstract class CDATASection extends Text {
// @Ignore
  external CDATASection.fakeConstructor$() : super.fakeConstructor$();
}

/**
 * @constructor
 * @extends {Node}
 * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-412266927
 */
@JsName("DocumentType")
abstract class DocumentType extends Node {
// @Ignore
  external DocumentType.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {NamedNodeMap}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-1788794630
   */
  NamedNodeMap entities;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-1844763134
   */
  String name;

  /**
   * @type {NamedNodeMap}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-D46829EF
   */
  NamedNodeMap notations;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-Core-DocType-internalSubset
   */
  String internalSubset;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-Core-DocType-publicId
   */
  String publicId;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-Core-DocType-systemId
   */
  String systemId;
}

/**
 * @constructor
 * @extends {Node}
 * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-5431D1B9
 */
@JsName("Notation")
abstract class Notation extends Node {
// @Ignore
  external Notation.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-54F2B4D0
   */
  String publicId;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-E8AAB1D0
   */
  String systemId;
}

/**
 * @constructor
 * @extends {Node}
 * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-527DCFF2
 */
@JsName("Entity")
abstract class Entity extends Node {
// @Ignore
  external Entity.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-D7303025
   */
  String publicId;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-D7C29F3E
   */
  String systemId;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-6ABAEB38
   */
  String notationName;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Entity3-inputEncoding
   */
  String inputEncoding;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Entity3-encoding
   */
  String xmlEncoding;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Entity3-version
   */
  String xmlVersion;
}

/**
 * @constructor
 * @extends {Node}
 * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-11C98490
 */
@JsName("EntityReference")
abstract class EntityReference extends Node {
// @Ignore
  external EntityReference.fakeConstructor$() : super.fakeConstructor$();
}

/**
 * @constructor
 * @extends {Node}
 * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-1004215813
 */
@JsName("ProcessingInstruction")
abstract class ProcessingInstruction extends Node {
// @Ignore
  external ProcessingInstruction.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-837822393
   */
  String data;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/1998/REC-DOM-Level-1-19981001/level-one-core.html#ID-1478689192
   */
  String target;
}

/**
 * @constructor
 * @implements {EventTarget}
 */
@JsName("Window")
abstract class Window implements w3c_event.EventTarget {
// @Ignore
  external Window.fakeConstructor$();

// Skipping as not allowed in dart. dynamic Window;

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  addEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  removeEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /** @override */
  bool dispatchEvent(w3c_event.Event evt);

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onabort;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onbeforeunload;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onblur;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onchange;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onclick;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onclose;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> oncontextmenu;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> ondblclick;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> ondragdrop;

  /** @type {?function (string, string, number)} */
  Func3<String, String, num, dynamic> onerror;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onfocus;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onhashchange;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onkeydown;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onkeypress;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onkeyup;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onload;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onmousedown;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onmousemove;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onmouseout;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onmouseover;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onmouseup;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onmousewheel;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onpaint;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onpopstate;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onreset;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onresize;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onscroll;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onselect;

  /** @type {?function (Event=)} */
  Func1Opt1<w3c_event.Event, dynamic> onsubmit;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onunload;

  /** @type {?function (Event)} */
  Func1<w3c_event.Event, dynamic> onwheel;

  /**
   * @see https://developer.mozilla.org/en/Components_object
   */
  dynamic Components;

  /**
   * @type Window
   * @see https://developer.mozilla.org/en/DOM/window.content
   */
  Window content;

  /**
   * @type {boolean}
   * @see https://developer.mozilla.org/en/DOM/window.closed
   */
  bool closed;

  /** @see https://developer.mozilla.org/en/DOM/window.controllers */
  dynamic controllers;

  /** @see https://developer.mozilla.org/en/DOM/window.crypto */
  dynamic crypto;

  /**
   * Gets/sets the status bar text for the given window.
   * @type {string}
   * @see https://developer.mozilla.org/en/DOM/window.defaultStatus
   */
  String defaultStatus;

  /** @see https://developer.mozilla.org/en/DOM/window.dialogArguments */
  dynamic dialogArguments;

  /** @see https://developer.mozilla.org/en/DOM/window.directories */
  dynamic directories;

  /**
   * @type {HTMLObjectElement|HTMLIFrameElement|null}
   * @see https://developer.mozilla.org/en/DOM/window.frameElement
   */
  dynamic /*w3c_dom2.HTMLObjectElement|null|w3c_dom2.HTMLIFrameElement*/ frameElement;

  /**
   * Allows lookup of frames by index or by name.
   * @type {?Object}
   * @see https://developer.mozilla.org/en/DOM/window.frames
   */
  Object frames;

  /**
   * @type {boolean}
   * @see https://developer.mozilla.org/en/DOM/window.fullScreen
   */
  bool fullScreen;

  /**
   * @see https://developer.mozilla.org/en/DOM/Storage#globalStorage
   */
  dynamic globalStorage;

  /**
   * @type {!History}
   * @see https://developer.mozilla.org/en/DOM/window.history
   */
  ie_dom.History history;

  /**
   * Returns the number of frames (either frame or iframe elements) in the
   * window.
   *
   * @type {number}
   * @see https://developer.mozilla.org/en/DOM/window.length
   */
  num length;

  /**
   * @type {!Location}
   * @implicitCast
   * @see https://developer.mozilla.org/en/DOM/window.location
   */
  ie_dom.Location location;

  /**
   * @see https://developer.mozilla.org/en/DOM/window.locationbar
   */
  dynamic locationbar;

  /**
   * @see https://developer.mozilla.org/en/DOM/window.menubar
   */
  dynamic menubar;

  /**
   * @type {string}
   * @see https://developer.mozilla.org/en/DOM/window.name
   */
  String name;

  /**
   * @type {Navigator}
   * @see https://developer.mozilla.org/en/DOM/window.navigator
   */
  gecko_dom.Navigator navigator;

  /**
   * @type {?Window}
   * @see https://developer.mozilla.org/en/DOM/window.opener
   */
  Window opener;

  /**
   * @type {!Window}
   * @see https://developer.mozilla.org/en/DOM/window.parent
   */
  Window parent;

  /** @see https://developer.mozilla.org/en/DOM/window.personalbar */
  dynamic personalbar;

  /** @see https://developer.mozilla.org/en/DOM/window.pkcs11 */
  dynamic pkcs11;

  /** @see https://developer.mozilla.org/en/DOM/window */
  dynamic returnValue;

  /** @see https://developer.mozilla.org/en/DOM/window.scrollbars */
  dynamic scrollbars;

  /**
   * @type {number}
   * @see https://developer.mozilla.org/En/DOM/window.scrollMaxX
   */
  num scrollMaxX;

  /**
   * @type {number}
   * @see https://developer.mozilla.org/En/DOM/window.scrollMaxY
   */
  num scrollMaxY;

  /**
   * @type {!Window}
   * @see https://developer.mozilla.org/en/DOM/window.self
   */
  Window self;

  /** @see https://developer.mozilla.org/en/DOM/Storage#sessionStorage */
  webstorage.Storage sessionStorage;

  /** @see https://developer.mozilla.org/en/DOM/window.sidebar */
  dynamic sidebar;

  /**
   * @type {?string}
   * @see https://developer.mozilla.org/en/DOM/window.status
   */
  String status;

  /** @see https://developer.mozilla.org/en/DOM/window.statusbar */
  dynamic statusbar;

  /** @see https://developer.mozilla.org/en/DOM/window.toolbar */
  dynamic toolbar;

  /**
   * @type {!Window}
   * @see https://developer.mozilla.org/en/DOM/window.self
   */
  Window top;

  /**
   * @type {!Window}
   * @see https://developer.mozilla.org/en/DOM/window.self
   */
  Window window;

  /**
   * @param {*} message
   * @see https://developer.mozilla.org/en/DOM/window.alert
   */
  alert(message);

  /** @see https://developer.mozilla.org/en/DOM/window.back */
  back();

  /** @see https://developer.mozilla.org/en/DOM/window.blur */
  blur();

  /** @deprecated */
  dynamic captureEvents;

  /** @see https://developer.mozilla.org/en/DOM/window.close */
  close();

  /** @see https://developer.mozilla.org/en/DOM/window.find */
  dynamic find;

  /** @see https://developer.mozilla.org/en/DOM/window.focus */
  focus();

  /** @see https://developer.mozilla.org/en/DOM/window.forward */
  forward();

  /** @see https://developer.mozilla.org/en/DOM/window.getAttention */
  getAttention();

  /**
   * @return {Selection}
   * @see https://developer.mozilla.org/en/DOM/window.getSelection
   * @nosideeffects
   */
  gecko_dom.Selection getSelection();

  /** @see https://developer.mozilla.org/en/DOM/window.home */
  home();

  dynamic openDialog;

  dynamic releaseEvents;

  dynamic scrollByLines;

  dynamic scrollByPages;

  /**
   * @param {string} uri
   * @param {?=} opt_arguments
   * @param {string=} opt_options
   * @see https://developer.mozilla.org/en/DOM/window.showModalDialog
   */
  external showModalDialog(String uri, [opt_arguments, String opt_options]);
  dynamic sizeToContent;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms536769(VS.85).aspx
   */
  stop();

  dynamic updateCommands;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms535220(VS.85).aspx
   * @type ClipboardData
   */
  ie_dom.ClipboardData clipboardData;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms533724(VS.85).aspx
   */
  dynamic dialogHeight;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms533725(VS.85).aspx
   */
  dynamic dialogLeft;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms533726(VS.85).aspx
   */
  dynamic dialogTop;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms533727(VS.85).aspx
   */
  dynamic dialogWidth;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms535863(VS.85).aspx
   */
  dynamic event;

  /**
   * @see http://msdn.microsoft.com/en-us/library/cc197012(VS.85).aspx
   */
  dynamic maxConnectionsPer1_0Server;

  /**
   * @see http://msdn.microsoft.com/en-us/library/cc197013(VS.85).aspx
   */
  dynamic maxConnectionsPerServer;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms534198(VS.85).aspx
   */
  dynamic offscreenBuffering;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms534389(VS.85).aspx
   */
  dynamic screenLeft;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms534389(VS.85).aspx
   */
  dynamic screenTop;

  /**
   * @param {string} event
   * @param {Function} handler
   * @see http://msdn.microsoft.com/en-us/library/ms536343(VS.85).aspx
   */
  external attachEvent(String event, Function handler);
  /**
   * @see http://msdn.microsoft.com/en-us/library/ms536392(VS.85).aspx
   */
  dynamic createPopup;

  /**
   * @param {string} event
   * @param {Function} handler
   * @see http://msdn.microsoft.com/en-us/library/ms536411(VS.85).aspx
   */
  external detachEvent(String event, Function handler);
  /**
   * @see http://msdn.microsoft.com/en-us/library/ms536420(VS.85).aspx
   */
  dynamic execScript;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms536425(VS.85).aspx
   */
  /**
   * @param {number} x
   * @param {number} y
   * @see http://msdn.microsoft.com/en-us/library/ms536618(VS.85).aspx
   */
  moveBy(num x, num y);

  /**
   * @param {number} x
   * @param {number} y
   * @see http://msdn.microsoft.com/en-us/library/ms536626(VS.85).aspx
   */
  moveTo(num x, num y);

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms536638(VS.85).aspx
   */
  dynamic navigate;

  /**
   * @param {*=} opt_url
   * @param {string=} opt_windowName
   * @param {string=} opt_windowFeatures
   * @param {boolean=} opt_replace
   * @return {Window}
   * @see http://msdn.microsoft.com/en-us/library/ms536651(VS.85).aspx
   */
  Window open([opt_url, String opt_windowName, String opt_windowFeatures,
      bool opt_replace]);

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms536672(VS.85).aspx
   */
  print();

  /**
   * @param {number} width
   * @param {number} height
   * @see http://msdn.microsoft.com/en-us/library/ms536722(VS.85).aspx
   */
  resizeBy(num width, num height);

  /**
   * @param {number} width
   * @param {number} height
   * @see http://msdn.microsoft.com/en-us/library/ms536723(VS.85).aspx
   */
  resizeTo(num width, num height);

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms536738(VS.85).aspx
   */
  dynamic setActive;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms536758(VS.85).aspx
   */
  dynamic showHelp;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms536761(VS.85).aspx
   */
  dynamic showModelessDialog;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms535246%28v=vs.85%29.aspx
   * @const {!Object}
   */
  Object external;

  /** @type {!Console} */
  webkit_dom.Console console;

  /**
   * @type {number}
   * @see http://developer.android.com/reference/android/webkit/WebView.html
   */
  num devicePixelRatio;

  /**
   * @param {string} media_query_list
   * @return {MediaQueryList}
   * @see http://www.w3.org/TR/cssom-view/#dom-window-matchmedia
   */
  w3c_css.MediaQueryList matchMedia(String media_query_list);

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-window-innerwidth
   */
  num innerWidth;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-window-innerheight
   */
  num innerHeight;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-window-scrollx
   */
  num scrollX;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-window-pagexoffset
   */
  num pageXOffset;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-window-scrolly
   */
  num scrollY;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-window-pageyoffset
   */
  num pageYOffset;

  /**
   * @param {number} x
   * @param {number} y
   * @see http://www.w3.org/TR/cssom-view/#dom-window-scroll
   */
  scroll(num x, num y);

  /**
   * @param {number} x
   * @param {number} y
   * @see http://www.w3.org/TR/cssom-view/#dom-window-scrollto
   */
  scrollTo(num x, num y);

  /**
   * @param {number} x
   * @param {number} y
   * @see http://www.w3.org/TR/cssom-view/#dom-window-scrollby
   */
  scrollBy(num x, num y);

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-window-screenx
   */
  num screenX;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-window-screeny
   */
  num screenY;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-window-outerwidth
   */
  num outerWidth;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-window-outerheight
   */
  num outerHeight;

  /** @type {CSSInterface} */
  w3c_css.CSSInterface CSS;

  /**
   * @param {string} name
   * @param {string} version
   * @param {string} description
   * @param {number} size
   * @param {(DatabaseCallback|function(Database))=} opt_callback
   * @return {Database}
   */
  html5.Database openDatabase(
      String name, String version, String description, num size,
      [opt_callback /*html5.DatabaseCallback|null|Func1<html5.Database,dynamic>|undefined*/]);

  /** @type {DOMApplicationCache} */
  html5.DOMApplicationCache applicationCache;

  /**
   * @see https://developer.mozilla.org/En/DOM/Worker/Functions_available_to_workers
   * @param {...string} var_args
   */
  importScripts(
      [String var_args1, String var_args2, String var_args3, String var_args4]);

  /**
   * @type {function(new:MutationObserver, function(Array.<MutationRecord>))}
   */
  external WebKitMutationObserver(arg1(List<html5.MutationRecord> arg1));
  /**
   * @type {function(new:MutationObserver, function(Array.<MutationRecord>))}
   */
  external MozMutationObserver(arg1(List<html5.MutationRecord> arg1));
  dynamic opera;

  dynamic postError;

  /**
   * @see http://www.w3.org/TR/file-system-api/#widl-LocalFileSystem-requestFileSystem
   * @param {number} type
   * @param {number} size
   * @param {function(!FileSystem)} successCallback
   * @param {function(!FileError)=} errorCallback
   */
  requestFileSystem(
      num type, num size, successCallback(fileapi.FileSystem arg1),
      [errorCallback(fileapi.FileError arg1)]);

  /**
   * @see http://www.w3.org/TR/file-system-api/#widl-LocalFileSystem-resolveLocalFileSystemURI
   * @param {string} uri
   * @param {function(!Entry)} successCallback
   * @param {function(!FileError)=} errorCallback
   */
  resolveLocalFileSystemURI(String uri, successCallback(fileapi.Entry arg1),
      [errorCallback(fileapi.FileError arg1)]);

  /**
   * This has replaced requestFileSystem in Chrome since WebKit revision 84224.
   * @see http://www.w3.org/TR/file-system-api/#widl-LocalFileSystem-requestFileSystem
   * @param {number} type
   * @param {number} size
   * @param {function(!FileSystem)} successCallback
   * @param {function(!FileError)=} errorCallback
   */
  webkitRequestFileSystem(
      num type, num size, successCallback(fileapi.FileSystem arg1),
      [errorCallback(fileapi.FileError arg1)]);

  /**
   * This has replaced resolveLocalFileSystemURI in Chrome since WebKit revision
   * 84224.
   * @see http://www.w3.org/TR/file-system-api/#widl-LocalFileSystem-resolveLocalFileSystemURI
   * @param {string} uri
   * @param {function(!Entry)} successCallback
   * @param {function(!FileError)=} errorCallback
   */
  webkitResolveLocalFileSystemURI(
      String uri, successCallback(fileapi.Entry arg1),
      [errorCallback(fileapi.FileError arg1)]);

  /**
   * @see http://www.w3.org/TR/FileAPI/#dfn-createObjectURL
   * @param {!Object} obj
   * @return {string}
   */
  String createObjectURL(Object obj);

  /**
   * @see http://www.w3.org/TR/FileAPI/#dfn-revokeObjectURL
   * @param {string} url
   */
  revokeObjectURL(String url);

  /**
   * @see https://developers.google.com/chrome/whitepapers/storage
   * @type {!StorageInfo}
   */
  fileapi.StorageInfo webkitStorageInfo;

  /**
   * Specifies the JavaScript method to invoke when the browser device's
   * orientation changes, i.e.the device is rotated.
   * @type {?function(!Event)}
   * @see http://developer.apple.com/library/IOS/#documentation/AppleApplications/Reference/SafariWebContent/HandlingEvents/HandlingEvents.html
   */
  Func1<w3c_event.Event, dynamic> onorientationchange;

  /**
   * Returns the orientation of the browser's device, one of [-90, 0, 90, 180].
   * @type {number}
   * @see http://developer.apple.com/library/IOS/#documentation/AppleApplications/Reference/SafariWebContent/HandlingEvents/HandlingEvents.html
   */
  num orientation;

  /**
   * Window implements WindowSessionStorage
   *
   * @type {Storage}
   */
  /**
   * Window implements WindowLocalStorage
   *
   * @type {Storage}
   */
  webstorage.Storage localStorage;

  /**
   * @param {function(number)} callback
   * @param {Element=} opt_element
   * @return {number}
   */
  num requestAnimationFrame(callback(num arg1), [Element opt_element]);

  /**
   * @param {number} handle
   */
  cancelRequestAnimationFrame(num handle);

  /**
   * @param {number} handle
   */
  cancelAnimationFrame(num handle);

  /**
   * @param {function(number)} callback
   * @param {Element=} opt_element
   * @return {number}
   */
  num webkitRequestAnimationFrame(callback(num arg1), [Element opt_element]);

  /**
   * @param {number} handle
   */
  webkitCancelRequestAnimationFrame(num handle);

  /**
   * @param {number} handle
   */
  webkitCancelAnimationFrame(num handle);

  /**
   * @param {?function(number)} callback It's legitimate to pass a null
   *     callback and listen on the MozBeforePaint event instead.
   * @param {Element=} opt_element
   * @return {number}
   */
  num mozRequestAnimationFrame(callback(num arg1), [Element opt_element]);

  /**
   * @param {number} handle
   */
  mozCancelRequestAnimationFrame(num handle);

  /**
   * @param {number} handle
   */
  mozCancelAnimationFrame(num handle);

  /**
   * @param {function(number)} callback
   * @param {Element=} opt_element
   * @return {number}
   */
  num msRequestAnimationFrame(callback(num arg1), [Element opt_element]);

  /**
   * @param {number} handle
   */
  msCancelRequestAnimationFrame(num handle);

  /**
   * @param {number} handle
   */
  msCancelAnimationFrame(num handle);

  /**
   * @param {function(number)} callback
   * @param {Element=} opt_element
   * @return {number}
   */
  num oRequestAnimationFrame(callback(num arg1), [Element opt_element]);

  /**
   * @param {number} handle
   */
  oCancelRequestAnimationFrame(num handle);

  /**
   * @param {number} handle
   */
  oCancelAnimationFrame(num handle);

  /** @type {IDBFactory} */
  w3c_indexeddb.IDBFactory moz_indexedDB;

  /** @type {IDBFactory} */
  w3c_indexeddb.IDBFactory mozIndexedDB;

  /** @type {IDBFactory} */
  w3c_indexeddb.IDBFactory webkitIndexedDB;

  /** @type {IDBFactory} */
  w3c_indexeddb.IDBFactory msIndexedDB;

  /** @type {IDBFactory} */
  w3c_indexeddb.IDBFactory indexedDB;

  /** @type {Performance} */
  w3c_navigation_timing.Performance performance;

  /**
   * WebKit browsers expose the NotificationCenter API through
   * window.webkitNotifications.
   * @type {NotificationCenter}
   */
  webkit_notifications.NotificationCenter webkitNotifications;

  /**
   * @see https://fetch.spec.whatwg.org/#dom-global-fetch
   * @param {!RequestInfo} input
   * @param {RequestInit=} opt_init
   * @return {!Promise.<!Response>}
   */
  es6.Promise<fetchapi.Response> fetch(input /*fetchapi.Request|String*/,
      [fetchapi.RequestInit opt_init]);
}
/// This record class is not given a name in the Closure JavaScript code base.
/// The name we have chosen may not make sense.
@JsName()
abstract class OptionsRecord {
  String extends$;
  Object prototype;
}
