/*
 * Copyright 2008 The Closure Compiler Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// This file has been automatically generated.  Please do not edit it manually.
/**
 * @fileoverview Definitions for W3C's DOM Level 2 specification.
 *  This file depends on w3c_dom1.js.
 *  The whole file has been fully type annotated.
 *  Created from
 *  http://www.w3.org/TR/REC-DOM-Level-1/ecma-script-language-binding.html
 *
 * FROM_EXTERN
 * @author stevey@google.com (Steve Yegge)
 */
@JsName()
library w3c_dom2;

import 'package:js/js.dart';
import 'package:func/func.dart';

import "html5.dart" as html5;

import "w3c_css.dart" as w3c_css;

import "w3c_dom1.dart" as w3c_dom1;

/**
 * @constructor
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-75708506
 */
@JsName("HTMLCollection")
abstract class HTMLCollection {
// @Ignore
  external HTMLCollection.fakeConstructor$();

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-40057551
   */
  num length;

  /**
   * @param {number} index
   * @return {Node}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-33262535
   * @nosideeffects
   */
  w3c_dom1.Node item(num index);

  /**
   * @param {string} name
   * @return {?Node}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-21069976
   * @nosideeffects
   */
  w3c_dom1.Node namedItem(String name);
}

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-2-HTML/html.html#HTMLOptionsCollection
 */
@JsName("HTMLOptionsCollection")
abstract class HTMLOptionsCollection {
// @Ignore
  external HTMLOptionsCollection.fakeConstructor$();

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-2-HTML/html.html#HTMLOptionsCollection-length
   */
  num length;

  /**
   * @param {number} index
   * @return {Node}
   * @see http://www.w3.org/TR/DOM-Level-2-HTML/html.html#HTMLOptionsCollection-item
   * @nosideeffects
   */
  w3c_dom1.Node item(num index);
}

/**
 * @constructor
 * @extends {Document}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-26809268
 */
@JsName("HTMLDocument")
abstract class HTMLDocument extends w3c_dom1.Document {
// @Ignore
  external HTMLDocument.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-18446827
   */
  String title;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-95229140
   */
  String referrer;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-2250147
   */
  String domain;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-46183437
   */
  String URL;

  /**
   * @type {HTMLBodyElement}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-56360201
   */
  HTMLBodyElement body;

  /**
   * @type {HTMLCollection}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-90379117
   */
  HTMLCollection images;

  /**
   * @type {HTMLCollection}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-85113862
   */
  HTMLCollection applets;

  /**
   * @type {HTMLCollection}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-7068919
   */
  HTMLCollection links;

  /**
   * @type {HTMLCollection}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-1689064
   */
  HTMLCollection forms;

  /**
   * @type {HTMLCollection}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-7577272
   */
  HTMLCollection anchors;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-8747038
   */
  String cookie;

  /**
   * @return {undefined}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-72161170
   * @override
   */
  void open();

  /**
   * @return {undefined}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-98948567
   * @override
   */
  void close();

  /**
   * @param {string} text
   * @return {undefined}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-75233634
   * @override
   */
  void write(String text);

  /**
   * @param {string} text
   * @return {undefined}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-35318390
   * @override
   */
  void writeln(String text);

  /**
   * @param {string} elementName
   * @return {!NodeList}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-71555259
   * @override
   * @nosideeffects
   */
  w3c_dom1.NodeList getElementsByName(String elementName);

  /**
   * @param {Node} root
   * @param {number=} whatToShow
   * @param {NodeFilter=} filter
   * @param {boolean=} entityReferenceExpansion
   * @return {!NodeIterator}
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/traversal.html#Traversal-Document
   * @nosideeffects
   */
  NodeIterator createNodeIterator(w3c_dom1.Node root,
      [num whatToShow, NodeFilter filter, bool entityReferenceExpansion]);

  /**
   * @param {Node} root
   * @param {number=} whatToShow
   * @param {NodeFilter=} filter
   * @param {boolean=} entityReferenceExpansion
   * @return {!TreeWalker}
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/traversal.html#Traversal-Document
   * @nosideeffects
   */
  TreeWalker createTreeWalker(w3c_dom1.Node root,
      [num whatToShow, NodeFilter filter, bool entityReferenceExpansion]);

  /**
   * @type {string}
   * @see http://msdn.microsoft.com/en-us/library/ms534359(VS.85).aspx
   */
  String readyState;
}
/** @typedef {{
  createNodeIterator: function(Node, number=, NodeFilter=, boolean=) : NodeIterator,
  createTreeWalker: function(Node, number=, NodeFilter=, boolean=) : TreeWalker
}} */
@JsName()
abstract class TraversalDocument {
  NodeIterator createNodeIterator(w3c_dom1.Node arg1,
      [num arg2, NodeFilter arg3, bool arg4]);
  TreeWalker createTreeWalker(w3c_dom1.Node arg1,
      [num arg2, NodeFilter arg3, bool arg4]);
}

/**
 * @interface
 * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/traversal.html#Traversal-NodeFilter
*/
@JsName("NodeFilter")
abstract class NodeFilter {
// @Ignore
  external NodeFilter.fakeConstructor$();

  /** @const {number} */
  external static num get SHOW_PROCESSING_INSTRUCTION;

  external static set SHOW_PROCESSING_INSTRUCTION(num value);

  /** @const {number} */
  external static num get FILTER_REJECT;

  external static set FILTER_REJECT(num value);

  /** @const {number} */
  external static num get FILTER_SKIP;

  external static set FILTER_SKIP(num value);

  /** @const {number} */
  external static num get SHOW_ALL;

  external static set SHOW_ALL(num value);

  /** @const {number} */
  external static num get SHOW_DOCUMENT_TYPE;

  external static set SHOW_DOCUMENT_TYPE(num value);

  /** @const {number} */
  external static num get SHOW_ELEMENT;

  external static set SHOW_ELEMENT(num value);

  /** @const {number} */
  external static num get SHOW_ENTITY_REFERENCE;

  external static set SHOW_ENTITY_REFERENCE(num value);

  /** @const {number} */
  external static num get SHOW_NOTATION;

  external static set SHOW_NOTATION(num value);

  /** @const {number} */
  external static num get SHOW_COMMENT;

  external static set SHOW_COMMENT(num value);

  /** @const {number} */
  external static num get SHOW_ENTITY;

  external static set SHOW_ENTITY(num value);

  /** @const {number} */
  external static num get SHOW_CDATA_SECTION;

  external static set SHOW_CDATA_SECTION(num value);

  /** @const {number} */
  external static num get SHOW_ATTRIBUTE;

  external static set SHOW_ATTRIBUTE(num value);

  /** @const {number} */
  external static num get SHOW_DOCUMENT;

  external static set SHOW_DOCUMENT(num value);

  /** @const {number} */
  external static num get SHOW_DOCUMENT_FRAGMENT;

  external static set SHOW_DOCUMENT_FRAGMENT(num value);

  /** @const {number} */
  external static num get SHOW_TEXT;

  external static set SHOW_TEXT(num value);

  /** @const {number} */
  external static num get FILTER_ACCEPT;

  external static set FILTER_ACCEPT(num value);

  /**
   * @param {Node} n
   * @return {number} Any of NodeFilter.FILTER_* constants.
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/traversal.html#Traversal-NodeFilter-acceptNode
  */
  num acceptNode(w3c_dom1.Node n);
}

/**
 * @interface
 * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/traversal.html#Traversal-NodeIterator
*/
@JsName("NodeIterator")
abstract class NodeIterator {
// @Ignore
  external NodeIterator.fakeConstructor$();

  /**
   * Detach and invalidate the NodeIterator.
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/traversal.html#Traversal-NodeIterator-detach
   */
  detach();

  /**
   * @return {Node} Next node in the set.
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/traversal.html#Traversal-NodeIterator-nextNode
   */
  w3c_dom1.Node nextNode();

  /**
   * @return {Node} Previous node in the set.
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/traversal.html#Traversal-NodeIterator-previousNode
   */
  w3c_dom1.Node previousNode();
}

/**
 * @interface
 * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/traversal.html#Traversal-TreeWalker
*/
@JsName("TreeWalker")
abstract class TreeWalker {
// @Ignore
  external TreeWalker.fakeConstructor$();

  /**
   * @return {?Node} The new Node or null.
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/traversal.html#Traversal-TreeWalker-firstChild
   */
  w3c_dom1.Node firstChild();

  /**
   * @return {?Node} The new Node or null..
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/traversal.html#Traversal-TreeWalker-lastChild
   */
  w3c_dom1.Node lastChild();

  /**
   * @return {?Node} The new Node or null.
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/traversal.html#Traversal-TreeWalker-nextNode
   */
  w3c_dom1.Node nextNode();

  /**
   * @return {?Node} The new Node or null.
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/traversal.html#Traversal-TreeWalker-nextSibling
   */
  w3c_dom1.Node nextSibling();

  /**
   * @return {?Node} The new Node or null.
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/traversal.html#Traversal-TreeWalker-parentNode
   */
  w3c_dom1.Node parentNode();

  /**
   * @return {?Node} The new Node or null.
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/traversal.html#Traversal-TreeWalker-previousNode
   */
  w3c_dom1.Node previousNode();

  /**
   * @return {?Node} The new Node or null.
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/traversal.html#Traversal-TreeWalker-previousSibling
   */
  w3c_dom1.Node previousSibling();

  /**
   * @type {Node}
   */
  w3c_dom1.Node root;

  /**
   * @type {number}
   */
  num whatToShow;

  /**
   * @type {NodeFilter}
   */
  NodeFilter filter;

  /**
   * @type {boolean}
   */
  bool expandEntityReference;

  /**
   * @type {Node}
   */
  w3c_dom1.Node currentNode;
}

/**
 * @constructor
 * @extends {Element}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-58190037
 */
@JsName("HTMLElement")
abstract class HTMLElement extends w3c_dom1.Element {
// @Ignore
  external HTMLElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @implicitCast
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-63534901
   */
  String id;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-78276800
   */
  String title;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-59132807
   */
  String lang;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-52460740
   */
  String dir;

  /**
   * @implicitCast
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-95362176
   */
  String className;

  /**
   * @type {Element}
   * @see http://www.w3.org/TR/cssom-view/#dom-htmlelement-offsetparent
   */
  w3c_dom1.Element offsetParent;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-htmlelement-offsettop
   */
  num offsetTop;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-htmlelement-offsetleft
   */
  num offsetLeft;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-htmlelement-offsetwidth
   */
  num offsetWidth;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-htmlelement-offsetheight
   */
  num offsetHeight;

  /** @type {Element} */
  w3c_dom1.Element contextMenu;

  /** @type {boolean} */
  bool draggable;

  /**
   * This is actually a DOMSettableTokenList property. However since that
   * interface isn't currently defined and no known browsers implement this
   * feature, just define the property for now.
   *
   * @const
   * @type {Object}
   */
  Object dropzone;

  /**
   * @see http://www.w3.org/TR/html5/dom.html#dom-getelementsbyclassname
   * @param {string} classNames
   * @return {!NodeList}
   * @nosideeffects
   */
  w3c_dom1.NodeList getElementsByClassName(String classNames);

  /** @type {boolean} */
  bool hidden;

  /** @type {boolean} */
  bool spellcheck;

  /**
   * @see http://www.w3.org/TR/components-intro/
   * @return {!ShadowRoot}
   */
  external html5.ShadowRoot createShadowRoot();
  /**
   * @see http://www.w3.org/TR/components-intro/
   * @return {!ShadowRoot}
   */
  external html5.ShadowRoot webkitCreateShadowRoot();
  /**
   * @see http://www.w3.org/TR/shadow-dom/
   * @type {ShadowRoot}
   */
  html5.ShadowRoot shadowRoot;

  /**
   * @see http://www.w3.org/TR/shadow-dom/
   * @return {!NodeList}
   */
  w3c_dom1.NodeList getDestinationInsertionPoints();

  /**
   * @see http://www.w3.org/TR/components-intro/
   * @type {function()}
   */
  external createdCallback();
  /**
   * @see http://w3c.github.io/webcomponents/explainer/#lifecycle-callbacks
   * @type {function()}
   */
  external attachedCallback();
  /**
   * @see http://w3c.github.io/webcomponents/explainer/#lifecycle-callbacks
   * @type {function()}
   */
  external detachedCallback();
  /**
   * Dataset collection.
   * This is really a DOMStringMap but it behaves close enough to an object to
   * pass as an object.
   * @type {Object}
   * @const
   */
  Object dataset;

  /**
   * A better interface to CSS classes than className.
   * @type {DOMTokenList}
   * @see http://www.w3.org/TR/html5/elements.html#dom-classlist
   * @const
   */
  html5.DOMTokenList classList;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-33759296
 */
@JsName("HTMLHtmlElement")
abstract class HTMLHtmlElement extends HTMLElement {
// @Ignore
  external HTMLHtmlElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-9383775
   */
  String version;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-77253168
 */
@JsName("HTMLHeadElement")
abstract class HTMLHeadElement extends HTMLElement {
// @Ignore
  external HTMLHeadElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-96921909
   */
  String profile;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @implements {LinkStyle}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-35143001
 */
@JsName("HTMLLinkElement")
abstract class HTMLLinkElement extends HTMLElement
    implements w3c_css.LinkStyle {
// @Ignore
  external HTMLLinkElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-87355129
   */
  bool disabled;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-63954491
   */
  String charset;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-33532588
   */
  String href;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-85145682
   */
  String hreflang;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-75813125
   */
  String media;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-41369587
   */
  String rel;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-40715461
   */
  String rev;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-84183095
   */
  String target;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-32498296
   */
  String type;

  /** @type {StyleSheet} */
  w3c_css.StyleSheet sheet;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-79243169
 */
@JsName("HTMLTitleElement")
abstract class HTMLTitleElement extends HTMLElement {
// @Ignore
  external HTMLTitleElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-77500413
   */
  String text;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-37041454
 */
@JsName("HTMLMetaElement")
abstract class HTMLMetaElement extends HTMLElement {
// @Ignore
  external HTMLMetaElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-87670826
   */
  String content;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-77289449
   */
  String httpEquiv;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-31037081
   */
  String name;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-35993789
   */
  String scheme;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-73629039
 */
@JsName("HTMLBaseElement")
abstract class HTMLBaseElement extends HTMLElement {
// @Ignore
  external HTMLBaseElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-65382887
   */
  String href;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-73844298
   */
  String target;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-85283003
 */
@JsName("HTMLIsIndexElement")
abstract class HTMLIsIndexElement extends HTMLElement {
// @Ignore
  external HTMLIsIndexElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {HTMLFormElement}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-87069980
   */
  HTMLFormElement form;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-33589862
   */
  String prompt;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @implements {LinkStyle}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-16428977
 */
@JsName("HTMLStyleElement")
abstract class HTMLStyleElement extends HTMLElement
    implements w3c_css.LinkStyle {
// @Ignore
  external HTMLStyleElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-51162010
   */
  bool disabled;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-76412738
   */
  String media;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-22472002
   */
  String type;

  /** @type {StyleSheet} */
  w3c_css.StyleSheet sheet;

  /**
   * @type {StyleSheet}
   * @see http://msdn.microsoft.com/en-us/library/dd347030(VS.85).aspx
   */
  w3c_css.StyleSheet styleSheet;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-62018039
 */
@JsName("HTMLBodyElement")
abstract class HTMLBodyElement extends HTMLElement {
// @Ignore
  external HTMLBodyElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-59424581
   */
  String aLink;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-37574810
   */
  String background;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-24940084
   */
  String bgColor;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-7662206
   */
  String link;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-73714763
   */
  String text;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-83224305
   */
  String vLink;

  /**
   * @see http://msdn.microsoft.com/en-us/library/ms536385(VS.85).aspx
   */
  dynamic createControlRange;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-40002357
 */
@JsName("HTMLFormElement")
abstract class HTMLFormElement extends HTMLElement {
// @Ignore
  external HTMLFormElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {HTMLCollection}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-76728479
   */
  HTMLCollection elements;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#HTML-HTMLFormElement-length
   */
  num length;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-22051454
   */
  String name;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-19661795
   */
  String acceptCharset;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-74049184
   */
  String action;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-84227810
   */
  String enctype;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-82545539
   */
  String method;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-6512890
   */
  String target;

  /**
   * @return {undefined}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-76767676
   */
  void submit();

  /**
   * @return {undefined}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-76767677
   */
  void reset();

  /** @return {boolean} */
  bool checkValidity();

  /** @type {boolean} */
  bool noValidate;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-94282980
 */
@JsName("HTMLSelectElement")
abstract class HTMLSelectElement extends HTMLElement {
// @Ignore
  external HTMLSelectElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-58783172
   */
  String type;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-85676760
   */
  num selectedIndex;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-59351919
   */
  String value;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-5933486
   */
  num length;

  /**
   * @type {HTMLFormElement}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-20489458
   */
  HTMLFormElement form;

  /**
   * @type {!HTMLOptionsCollection}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-30606413
   */
  HTMLOptionsCollection options;

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-79102918
   */
  bool disabled;

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-13246613
   */
  bool multiple;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-41636323
   */
  String name;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-18293826
   */
  num size;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-40676705
   */
  num tabIndex;

  /**
   * @param {HTMLElement} element
   * @param {HTMLElement} before
   * @return {undefined}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-14493106
   */
  void add(HTMLElement element, HTMLElement before);

  /**
   * @return {undefined}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-28216144
   * @override
   */
  void blur();

  /**
   * @return {undefined}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-32130014
   * @override
   */
  void focus();

  /**
   * @param {number} index
   * @return {undefined}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-33404570
   */
  void remove(num index);

  /** @type {boolean} */
  bool autofocus;

  /**
   * @const
   * @type {NodeList}
   */
  w3c_dom1.NodeList labels;

  /** @type {HTMLCollection} */
  HTMLCollection selectedOptions;

  /** @type {string} */
  String validationMessage;

  /**
   * @const
   * @type {ValidityState}
   */
  html5.ValidityState validity;

  /** @type {boolean} */
  bool willValidate;

  /** @return {boolean} */
  bool checkValidity();

  /** @param {string} message */
  setCustomValidity(String message);
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-38450247
 */
@JsName("HTMLOptGroupElement")
abstract class HTMLOptGroupElement extends HTMLElement {
// @Ignore
  external HTMLOptGroupElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-15518803
   */
  bool disabled;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-95806054
   */
  String label;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-70901257
 */
@JsName("HTMLOptionElement")
abstract class HTMLOptionElement extends HTMLElement {
// @Ignore
  external HTMLOptionElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-37770574
   */
  bool defaultSelected;

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-23482473
   */
  bool disabled;

  /**
   * @type {HTMLFormElement}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-17116503
   */
  HTMLFormElement form;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-14038413
   */
  num index;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-40736115
   */
  String label;

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-70874476
   */
  bool selected;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-48154426
   */
  String text;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-6185554
   */
  String value;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-6043025
 */
@JsName("HTMLInputElement")
abstract class HTMLInputElement extends HTMLElement {
// @Ignore
  external HTMLInputElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-15328520
   */
  String accept;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-59914154
   */
  String accessKey;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-96991182
   */
  String align;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-92701314
   */
  String alt;

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-30233917
   */
  bool checked;

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-20509171
   */
  bool defaultChecked;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-26091157
   */
  String defaultValue;

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-50886781
   */
  bool disabled;

  /**
   * @type {HTMLFormElement}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-63239895
   */
  HTMLFormElement form;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-54719353
   */
  num maxLength;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-89658498
   */
  String name;

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-88461592
   */
  bool readOnly;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-79659438
   */
  num size;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-97320704
   */
  String src;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-62176355
   */
  num tabIndex;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-62883744
   */
  String type;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-32463706
   */
  String useMap;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-49531485
   */
  String value;

  /**
   * @return {undefined}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-26838235
   * @override
   */
  void blur();

  /**
   * @return {undefined}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-2651361
   * @override
   */
  void click();

  /**
   * @return {undefined}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-65996295
   * @override
   */
  void focus();

  /**
   * @return {undefined}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-34677168
   */
  void select();

  /** @type {number} */
  num selectionStart;

  /** @type {number} */
  num selectionEnd;

  /**
   * @param {number} selectionStart
   * @param {number} selectionEnd
   * @see http://www.whatwg.org/specs/web-apps/current-work/multipage/editing.html#dom-textarea/input-setselectionrange
   */
  setSelectionRange(num selectionStart, num selectionEnd);

  /** @type {string} */
  String autocomplete;

  /** @type {string} */
  String dirname;

  /** @type {FileList} */
  html5.FileList files;

  /** @type {string} */
  String list;

  /** @type {string} */
  String max;

  /** @type {string} */
  String min;

  /** @type {string} */
  String pattern;

  /** @type {boolean} */
  bool multiple;

  /** @type {string} */
  String placeholder;

  /** @type {boolean} */
  bool required;

  /** @type {string} */
  String step;

  /** @type {Date} */
  DateTime valueAsDate;

  /** @type {number} */
  num valueAsNumber;

  /**
   * Changes the form control's value by the value given in the step attribute
   * multiplied by opt_n.
   * @param {number=} opt_n step multiplier.  Defaults to 1.
   */
  stepDown([num opt_n]);

  /**
   * Changes the form control's value by the value given in the step attribute
   * multiplied by opt_n.
   * @param {number=} opt_n step multiplier.  Defaults to 1.
   */
  stepUp([num opt_n]);

  /** @type {boolean} */
  bool autofocus;

  /** @type {boolean} */
  bool formNoValidate;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/html5/forms.html#attr-fs-formaction
   */
  String formAction;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/html5/forms.html#attr-fs-formenctype
   */
  String formEnctype;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/html5/forms.html#attr-fs-formmethod
   */
  String formMethod;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/html5/forms.html#attr-fs-formtarget
   */
  String formTarget;

  /**
   * @const
   * @type {NodeList}
   */
  w3c_dom1.NodeList labels;

  /** @type {string} */
  String validationMessage;

  /**
   * @const
   * @type {ValidityState}
   */
  html5.ValidityState validity;

  /** @type {boolean} */
  bool willValidate;

  /** @return {boolean} */
  bool checkValidity();

  /** @param {string} message */
  setCustomValidity(String message);

  /**
   * @implicitCast
   * @type {boolean}
   */
  bool autocorrect;

  /**
   * @implicitCast
   * @type {boolean}
   */
  bool autocapitalize;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-24874179
 */
@JsName("HTMLTextAreaElement")
abstract class HTMLTextAreaElement extends HTMLElement {
// @Ignore
  external HTMLTextAreaElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-93102991
   */
  String accessKey;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-51387225
   */
  num cols;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-36152213
   */
  String defaultValue;

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-98725443
   */
  bool disabled;

  /**
   * @type {HTMLFormElement}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-18911464
   */
  HTMLFormElement form;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-70715578
   */
  String name;

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-39131423
   */
  bool readOnly;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-46975887
   */
  num rows;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-60363303
   */
  num tabIndex;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#HTML-HTMLTextAreaElement-type
   */
  String type;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-70715579
   */
  String value;

  /**
   * @return {undefined}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-6750689
   * @override
   */
  void blur();

  /**
   * @return {undefined}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-39055426
   * @override
   */
  void focus();

  /**
   * @return {undefined}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-48880622
   */
  void select();

  /** @type {number} */
  num selectionStart;

  /** @type {number} */
  num selectionEnd;

  /**
   * @param {number} selectionStart
   * @param {number} selectionEnd
   * @see http://www.whatwg.org/specs/web-apps/current-work/multipage/editing.html#dom-textarea/input-setselectionrange
   */
  setSelectionRange(num selectionStart, num selectionEnd);

  /** @type {boolean} */
  bool autofocus;

  /**
   * @const
   * @type {NodeList}
   */
  w3c_dom1.NodeList labels;

  /** @type {string} */
  String validationMessage;

  /**
   * @const
   * @type {ValidityState}
   */
  html5.ValidityState validity;

  /** @type {boolean} */
  bool willValidate;

  /** @return {boolean} */
  bool checkValidity();

  /** @param {string} message */
  setCustomValidity(String message);

  /**
   * @implicitCast
   * @type {boolean}
   */
  bool autocorrect;

  /**
   * @implicitCast
   * @type {boolean}
   */
  bool autocapitalize;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-34812697
 */
@JsName("HTMLButtonElement")
abstract class HTMLButtonElement extends HTMLElement {
// @Ignore
  external HTMLButtonElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-73169431
   */
  String accessKey;

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-92757155
   */
  bool disabled;

  /**
   * @type {HTMLFormElement}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-71254493
   */
  HTMLFormElement form;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-11029910
   */
  String name;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-39190908
   */
  num tabIndex;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-27430092
   */
  String type;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-72856782
   */
  String value;

  /** @type {boolean} */
  bool autofocus;

  /**
   * @const
   * @type {NodeList}
   */
  w3c_dom1.NodeList labels;

  /** @type {string} */
  String validationMessage;

  /**
   * @const
   * @type {ValidityState}
   */
  html5.ValidityState validity;

  /** @type {boolean} */
  bool willValidate;

  /** @return {boolean} */
  bool checkValidity();

  /** @param {string} message */
  setCustomValidity(String message);

  /**
   * @type {string}
   * @see http://www.w3.org/TR/html5/forms.html#attr-fs-formaction
   */
  String formAction;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/html5/forms.html#attr-fs-formenctype
   */
  String formEnctype;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/html5/forms.html#attr-fs-formmethod
   */
  String formMethod;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/html5/forms.html#attr-fs-formtarget
   */
  String formTarget;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-13691394
 */
@JsName("HTMLLabelElement")
abstract class HTMLLabelElement extends HTMLElement {
// @Ignore
  external HTMLLabelElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-43589892
   */
  String accessKey;

  /**
   * @type {HTMLFormElement}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-32480901
   */
  HTMLFormElement form;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-96509813
   */
  String htmlFor;

  /** @type {Element} */
  w3c_dom1.Element control;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-7365882
 */
@JsName("HTMLFieldSetElement")
abstract class HTMLFieldSetElement extends HTMLElement {
// @Ignore
  external HTMLFieldSetElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {HTMLFormElement}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-75392630
   */
  HTMLFormElement form;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-21482039
 */
@JsName("HTMLLegendElement")
abstract class HTMLLegendElement extends HTMLElement {
// @Ignore
  external HTMLLegendElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-11297832
   */
  String accessKey;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-79538067
   */
  String align;

  /**
   * @type {HTMLFormElement}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-29594519
   */
  HTMLFormElement form;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-86834457
 */
@JsName("HTMLUListElement")
abstract class HTMLUListElement extends HTMLElement {
// @Ignore
  external HTMLUListElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-39864178
   */
  bool compact;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-96874670
   */
  String type;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-58056027
 */
@JsName("HTMLOListElement")
abstract class HTMLOListElement extends HTMLElement {
// @Ignore
  external HTMLOListElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-76448506
   */
  bool compact;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-14793325
   */
  num start;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-40971103
   */
  String type;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-52368974
 */
@JsName("HTMLDListElement")
abstract class HTMLDListElement extends HTMLElement {
// @Ignore
  external HTMLDListElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-21738539
   */
  bool compact;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-71600284
 */
@JsName("HTMLDirectoryElement")
abstract class HTMLDirectoryElement extends HTMLElement {
// @Ignore
  external HTMLDirectoryElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-75317739
   */
  bool compact;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-72509186
 * @see http://www.w3.org/html/wg/drafts/html/master/interactive-elements.html#the-menu-element
 */
@JsName("HTMLMenuElement")
abstract class HTMLMenuElement extends HTMLElement {
// @Ignore
  external HTMLMenuElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-68436464
   */
  bool compact;

  /**
   * @see http://www.w3.org/html/wg/drafts/html/master/interactive-elements.html#dom-menu-type
   * @type {string}
   */
  String type;

  /**
   * @see http://www.w3.org/html/wg/drafts/html/master/interactive-elements.html#dom-menu-label
   * @type {string}
   */
  String label;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-74680021
 */
@JsName("HTMLLIElement")
abstract class HTMLLIElement extends HTMLElement {
// @Ignore
  external HTMLLIElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-52387668
   */
  String type;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-45496263
   */
  num value;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-22445964
 */
@JsName("HTMLDivElement")
abstract class HTMLDivElement extends HTMLElement {
// @Ignore
  external HTMLDivElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-70908791
   */
  String align;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-84675076
 */
@JsName("HTMLParagraphElement")
abstract class HTMLParagraphElement extends HTMLElement {
// @Ignore
  external HTMLParagraphElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-53465507
   */
  String align;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-43345119
 */
@JsName("HTMLHeadingElement")
abstract class HTMLHeadingElement extends HTMLElement {
// @Ignore
  external HTMLHeadingElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-6796462
   */
  String align;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-70319763
 */
@JsName("HTMLQuoteElement")
abstract class HTMLQuoteElement extends HTMLElement {
// @Ignore
  external HTMLQuoteElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-53895598
   */
  String cite;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-11383425
 */
@JsName("HTMLPreElement")
abstract class HTMLPreElement extends HTMLElement {
// @Ignore
  external HTMLPreElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-13894083
   */
  num width;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-56836063
 */
@JsName("HTMLBRElement")
abstract class HTMLBRElement extends HTMLElement {
// @Ignore
  external HTMLBRElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-82703081
   */
  String clear;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-32774408
 */
@JsName("HTMLBaseFontElement")
abstract class HTMLBaseFontElement extends HTMLElement {
// @Ignore
  external HTMLBaseFontElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-87502302
   */
  String color;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-88128969
   */
  String face;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-38930424
   */
  num size;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-43943847
 */
@JsName("HTMLFontElement")
abstract class HTMLFontElement extends HTMLElement {
// @Ignore
  external HTMLFontElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-53532975
   */
  String color;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-55715655
   */
  String face;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-90127284
   */
  String size;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-68228811
 */
@JsName("HTMLHRElement")
abstract class HTMLHRElement extends HTMLElement {
// @Ignore
  external HTMLHRElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-15235012
   */
  String align;

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-79813978
   */
  bool noShade;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-77612587
   */
  String size;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-87744198
   */
  String width;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-79359609
 */
@JsName("HTMLModElement")
abstract class HTMLModElement extends HTMLElement {
// @Ignore
  external HTMLModElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-75101708
   */
  String cite;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-88432678
   */
  String dateTime;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-48250443
 */
@JsName("HTMLAnchorElement")
abstract class HTMLAnchorElement extends HTMLElement {
// @Ignore
  external HTMLAnchorElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-89647724
   */
  String accessKey;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-67619266
   */
  String charset;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-92079539
   */
  String coords;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-88517319
   */
  String href;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-87358513
   */
  String hreflang;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-32783304
   */
  String name;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-3815891
   */
  String rel;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-58259771
   */
  String rev;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-49899808
   */
  String shape;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-41586466
   */
  num tabIndex;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-6414197
   */
  String target;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-63938221
   */
  String type;

  /**
   * @return {undefined}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-65068939
   * @override
   */
  void blur();

  /**
   * @return {undefined}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-47150313
   * @override
   */
  void focus();

  /** @type {string} */
  String hash;

  /** @type {string} */
  String host;

  /** @type {string} */
  String hostname;

  /** @type {string} */
  String pathname;

  /**
   * The 'ping' attribute is known to be supported in recent versions (as of
   * mid-2014) of Chrome, Safari, and Firefox, and is not supported in any
   * current version of Internet Explorer.
   *
   * @type {string}
   * @see http://www.whatwg.org/specs/web-apps/current-work/multipage/semantics.html#hyperlink-auditing
   */
  String ping;

  /** @type {string} */
  String port;

  /** @type {string} */
  String protocol;

  /** @type {string} */
  String search;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-17701901
 */
@JsName("HTMLImageElement")
abstract class HTMLImageElement extends HTMLElement {
// @Ignore
  external HTMLImageElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-3211094
   */
  String align;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-95636861
   */
  String alt;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-136671
   */
  String border;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-91561496
   */
  num height;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-53675471
   */
  num hspace;

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-58983880
   */
  bool isMap;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-77376969
   */
  String longDesc;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-91256910
   */
  String lowSrc;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-47534097
   */
  String name;

  /**
   * @type {string}
   * @implicitCast
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-87762984
   */
  String src;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-35981181
   */
  String useMap;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-85374897
   */
  num vspace;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-13839076
   */
  num width;

  /**
   * @type {number}
   * @see http://www.google.com/codesearch/p?hl=en#eksvcKKj5Ng/mozilla/dom/public/idl/html/nsIDOMNSHTMLImageElement.idl&q=naturalWidth
   */
  num naturalWidth;

  /**
   * @type {number}
   * @see http://www.google.com/codesearch/p?hl=en#eksvcKKj5Ng/mozilla/dom/public/idl/html/nsIDOMNSHTMLImageElement.idl&q=naturalHeight
   */
  num naturalHeight;

  /**
   * @type {string}
   * @see http://msdn.microsoft.com/en-us/library/ms534359(VS.85).aspx
   */
  String readyState;

  /**
   * @type {boolean}
   */
  bool complete;

  /**
   * @type {string}
   * @see http://www.whatwg.org/specs/web-apps/current-work/multipage/embedded-content-1.html#attr-img-crossorigin
   */
  String crossOrigin;

  /** @type {string} */
  String sizes;

  /** @type {string} */
  String srcset;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-9893177
 */
@JsName("HTMLObjectElement")
abstract class HTMLObjectElement extends HTMLElement {
// @Ignore
  external HTMLObjectElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-16962097
   */
  String align;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-47783837
   */
  String archive;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-82818419
   */
  String border;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-75241146
   */
  String code;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-25709136
   */
  String codeBase;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-19945008
   */
  String codeType;

  /**
   * @type {Document}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-38538621
   */
  w3c_dom1.Document contentDocument;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-81766986
   */
  String data;

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-942770
   */
  bool declare;

  /**
   * @type {HTMLFormElement}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-46094773
   */
  HTMLFormElement form;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-88925838
   */
  String height;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-17085376
   */
  num hspace;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-20110362
   */
  String name;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-25039673
   */
  String standby;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-27083787
   */
  num tabIndex;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-91665621
   */
  String type;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-6649772
   */
  String useMap;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-8682483
   */
  num vspace;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-38538620
   */
  String width;

  /**
   * @type {string}
   * @see http://msdn.microsoft.com/en-us/library/ms534359(VS.85).aspx
   */
  String readyState;

  /**
   * Call a Flash function exported by ExternalInterface.
   * @param {string} xmlString The XML string passed to Flash. The outer element
   *     should be {@code <invoke>}. A sample invocation string:
   *     {@code <invoke name="function_name" returntype="javascript">
   *     <string>test</string></invoke>}
   * @return {string} The serialized return value from Flash that you can eval.
   */
  String CallFunction(String xmlString);

  /**
   * Returns the value of the Flash variable specified by varName or null if the
   * variable does not exist.
   * @param {string} varName The variable name.
   * @return {string?} The variable value.
   */
  String GetVariable(String varName);

  /**
   * Activates the frame number specified by {@code frameNumber} in the current
   * movie.
   * @param {number} frameNumber A non-negative integer frame number.
   */
  GotoFrame(num frameNumber);

  /**
   * @return {boolean} Whether the movie is currently playing.
   */
  bool IsPlaying();

  /**
   * Loads the movie identified by {@code url} to the layer specified by {@code
   * layerNumber}.
   * @param {number} layerNumber The layer number.
   * @param {string} url The movie URL.
   */
  LoadMovie(num layerNumber, String url);

  /**
   * Pans a zoomed-in movie to the coordinates specified by x and y. Use mode to
   * specify whether the values for x and y are pixels or a percent of the window.
   * When mode is 0, the coordinates are pixels; when mode is 1, the coordinates
   * are percent of the window.
   * @param {number} x The x-coordinate.
   * @param {number} y The y-coordinate.
   * @param {number} mode The mode.
   */
  Pan(num x, num y, num mode);

  /**
   * @return {number} The percent of the Flash Player movie that has streamed
   *     into the browser so far; Possible values are from 0 to 100.
   */
  num PercentLoaded();

  /**
   * Starts playing the movie.
   */
  Play();

  /**
   * Goes to the first frame.
   */
  Rewind();

  /**
   * Sets the value of the flash variable.
   * @param {string} variableName The variable name.
   * @param {string} value The value.
   */
  SetVariable(String variableName, String value);

  /**
   * Zooms in on a rectangular area of the movie. The units of the coordinates
   * are in twips (1440 units per inch).
   * @param {number} left The left coordinate.
   * @param {number} top The top coordinate.
   * @param {number} right The right coordinate.
   * @param {number} bottom The bottom coordinate.
   */
  SetZoomRect(num left, num top, num right, num bottom);

  /**
   * Stops playing the movie.
   */
  StopPlay();

  /**
   * @return {number} The total number of frames in the movie.
   */
  num TotalFrames();

  /**
   * Zooms the view by a relative scale factor.
   * @param {number} percent The percentage scale factor, should be an integer.
   */
  Zoom(num percent);

  /**
   * Executes the action in the timeline specified by {@code target} in the
   * specified frame.
   * @param {string} target The timeline.
   * @param {number} frameNumber The frame number.
   */
  TCallFrame(String target, num frameNumber);

  /**
   * Executes the action in the timeline specified by {@code target} in the
   * specified frame.
   * @param {string} target The timeline.
   * @param {string} label The frame label.
   */
  TCallLabel(String target, String label);

  /**
   * Returns the number of the current frame for the specified timeline.
   * @param {string} target The timeline.
   * @return {number} The number of the current frame.
   */
  num TCurentFrame(String target);

  /**
   * Returns the label of the current frame for the specified timeline.
   * @param {string} target The timeline.
   * @return {string} The label of the current frame, empty string if no
   *     current frame.
   */
  String TCurrentLabel(String target);

  /**
   * Returns a string indicating the value of the property in the
   * specified timeline.
   * @param {string} target The timeline.
   * @param {number} property The integer corresponding to the desired property.
   * @return {string} The value of the property.
   */
  String TGetProperty(String target, num property);

  /**
   * Returns a number indicating the value of the property in the specified
   * timeline.
   * @param {string} target The timeline.
   * @param {number} property The integer corresponding to the desired property.
   * @return {number} A number indicating the value of the property.
   */
  num TGetPropertyAsNumber(String target, num property);

  /**
   * Goes to the specified frame number in the specified timeline.
   * @param {string} target The timeline.
   * @param {number} frameNumber The frame number.
   */
  TGotoFrame(String target, num frameNumber);

  /**
   * Goes to the specified frame label in the specified timeline.
   * @param {string} target The timeline.
   * @param {string} label The framelabel.
   */
  TGotoLabel(String target, String label);

  /**
   * Plays the specified timeline.
   * @param {number} target The timeline.
   */
  TPlay(num target);

  /**
   * Sets the value of the property in the specified timeline.
   * @param {number} target The timeline.
   * @param {number} property The integer corresponding to the desired property.
   * @param {string|number} value The value.
   */
  TSetProperty(num target, num property, value /*String|num*/);

  /**
   * Stops the specified timeline.
   * @param {number} target The timeline.
   */
  TStopPlay(num target);
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-64077273
 */
@JsName("HTMLParamElement")
abstract class HTMLParamElement extends HTMLElement {
// @Ignore
  external HTMLParamElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-59871447
   */
  String name;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-18179888
   */
  String type;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-77971357
   */
  String value;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-23931872
   */
  String valueType;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-31006348
 */
@JsName("HTMLAppletElement")
abstract class HTMLAppletElement extends HTMLElement {
// @Ignore
  external HTMLAppletElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-8049912
   */
  String align;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-58610064
   */
  String alt;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-14476360
   */
  String archive;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-61509645
   */
  String code;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-6581160
   */
  String codeBase;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-90184867
   */
  String height;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-1567197
   */
  num hspace;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-39843695
   */
  String name;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-93681523
   */
  String object;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-22637173
   */
  num vspace;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-16526327
   */
  String width;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-94109203
 */
@JsName("HTMLMapElement")
abstract class HTMLMapElement extends HTMLElement {
// @Ignore
  external HTMLMapElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {HTMLCollection}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-71838730
   */
  HTMLCollection areas;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-52696514
   */
  String name;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-26019118
 */
@JsName("HTMLAreaElement")
abstract class HTMLAreaElement extends HTMLElement {
// @Ignore
  external HTMLAreaElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-57944457
   */
  String accessKey;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-39775416
   */
  String alt;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-66021476
   */
  String coords;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-34672936
   */
  String href;

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-61826871
   */
  bool noHref;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-85683271
   */
  String shape;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-8722121
   */
  num tabIndex;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-46054682
   */
  String target;

  /**
   * @type {string}
   * @see http://www.whatwg.org/specs/web-apps/current-work/multipage/semantics.html#hyperlink-auditing
   */
  String ping;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-81598695
 */
@JsName("HTMLScriptElement")
abstract class HTMLScriptElement extends HTMLElement {
// @Ignore
  external HTMLScriptElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-35305677
   */
  String charset;

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-93788534
   */
  bool defer;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-56700403
   */
  String event;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-66979266
   */
  String htmlFor;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-75147231
   */
  String src;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-46872999
   */
  String text;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-30534818
   */
  String type;

  /**
   * @type {string}
   * @see http://msdn.microsoft.com/en-us/library/ms534359(VS.85).aspx
   */
  String readyState;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-64060425
 */
@JsName("HTMLTableElement")
abstract class HTMLTableElement extends HTMLElement {
// @Ignore
  external HTMLTableElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-23180977
   */
  String align;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-83532985
   */
  String bgColor;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-50969400
   */
  String border;

  /**
   * @type {HTMLTableCaptionElement}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-14594520
   */
  HTMLTableCaptionElement caption;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-59162158
   */
  String cellPadding;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-68907883
   */
  String cellSpacing;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-64808476
   */
  String frame;

  /**
   * @type {HTMLCollection}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-6156016
   */
  HTMLCollection rows;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-26347553
   */
  String rules;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-44998528
   */
  String summary;

  /**
   * @type {HTMLCollection}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-63206416
   */
  HTMLCollection tBodies;

  /**
   * @type {HTMLTableSectionElement}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-64197097
   */
  HTMLTableSectionElement tFoot;

  /**
   * @type {HTMLTableSectionElement}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-9530944
   */
  HTMLTableSectionElement tHead;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-77447361
   */
  String width;

  /**
   * @return {HTMLElement}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-96920263
   */
  HTMLElement createCaption();

  /**
   * @return {HTMLElement}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-8453710
   */
  HTMLElement createTFoot();

  /**
   * @return {HTMLElement}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-70313345
   */
  HTMLElement createTHead();

  /**
   * @return {undefined}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-22930071
   */
  void deleteCaption();

  /**
   * @param {number} index
   * @return {HTMLElement}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-13114938
   */
  HTMLElement deleteRow(num index);

  /**
   * @return {undefined}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-78363258
   */
  void deleteTFoot();

  /**
   * @return {undefined}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-38310198
   */
  void deleteTHead();

  /**
   * @param {number} index
   * @return {HTMLElement}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-39872903
   */
  HTMLElement insertRow(num index);
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-12035137
 */
@JsName("HTMLTableCaptionElement")
abstract class HTMLTableCaptionElement extends HTMLElement {
// @Ignore
  external HTMLTableCaptionElement.fakeConstructor$()
      : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-79875068
   */
  String align;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-84150186
 */
@JsName("HTMLTableColElement")
abstract class HTMLTableColElement extends HTMLElement {
// @Ignore
  external HTMLTableColElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-31128447
   */
  String align;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-9447412
   */
  String ch;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-57779225
   */
  String chOff;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-96511335
   */
  num span;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-83291710
   */
  String vAlign;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-25196799
   */
  String width;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-67417573
 */
@JsName("HTMLTableSectionElement")
abstract class HTMLTableSectionElement extends HTMLElement {
// @Ignore
  external HTMLTableSectionElement.fakeConstructor$()
      : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-40530119
   */
  String align;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-83470012
   */
  String ch;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-53459732
   */
  String chOff;

  /**
   * @type {HTMLCollection}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-52092650
   */
  HTMLCollection rows;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-4379116
   */
  String vAlign;

  /**
   * @param {number} index
   * @return {HTMLElement}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-5625626
   */
  HTMLElement deleteRow(num index);

  /**
   * @param {number} index
   * @return {HTMLElement}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-93995626
   */
  HTMLElement insertRow(num index);
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-6986576
 */
@JsName("HTMLTableRowElement")
abstract class HTMLTableRowElement extends HTMLElement {
// @Ignore
  external HTMLTableRowElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-74098257
   */
  String align;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-18161327
   */
  String bgColor;

  /**
   * @type {HTMLCollection}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-67349879
   */
  HTMLCollection cells;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-16230502
   */
  String ch;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-68207461
   */
  String chOff;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-67347567
   */
  num rowIndex;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-79105901
   */
  num sectionRowIndex;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-90000058
   */
  String vAlign;

  /**
   * @param {number} index
   * @return {undefined}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-11738598
   */
  void deleteCell(num index);

  /**
   * @param {number} index
   * @return {HTMLElement}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-68927016
   */
  HTMLElement insertCell(num index);
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-82915075
 */
@JsName("HTMLTableCellElement")
abstract class HTMLTableCellElement extends HTMLElement {
// @Ignore
  external HTMLTableCellElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-74444037
   */
  String abbr;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-98433879
   */
  String align;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-76554418
   */
  String axis;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-88135431
   */
  String bgColor;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-80748363
   */
  num cellIndex;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-30914780
   */
  String ch;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-20144310
   */
  String chOff;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-84645244
   */
  num colSpan;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-89104817
   */
  String headers;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-83679212
   */
  String height;

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-62922045
   */
  bool noWrap;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-48237625
   */
  num rowSpan;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-36139952
   */
  String scope;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-58284221
   */
  String vAlign;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-27480795
   */
  String width;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-43829095
 */
@JsName("HTMLFrameSetElement")
abstract class HTMLFrameSetElement extends HTMLElement {
// @Ignore
  external HTMLFrameSetElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-98869594
   */
  String cols;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-19739247
   */
  String rows;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-97790553
 */
@JsName("HTMLFrameElement")
abstract class HTMLFrameElement extends HTMLElement {
// @Ignore
  external HTMLFrameElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {Document}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-78799536
   */
  w3c_dom1.Document contentDocument;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-11858633
   */
  String frameBorder;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-7836998
   */
  String longDesc;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-55569778
   */
  String marginHeight;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-8369969
   */
  String marginWidth;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-91128709
   */
  String name;

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-80766578
   */
  bool noResize;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-45411424
   */
  String scrolling;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-78799535
   */
  String src;

  /**
   * @type {boolean}
   * @implicitCast
   * @see http://msdn.microsoft.com/en-us/library/ie/ms533072(v=vs.85).aspx
   */
  bool allowTransparency;

  /**
   * @type {Window}
   * @see http://msdn.microsoft.com/en-us/library/ms533692(VS.85).aspx
   */
  w3c_dom1.Window contentWindow;
}

/**
 * @constructor
 * @extends {HTMLElement}
 * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-50708718
 */
@JsName("HTMLIFrameElement")
abstract class HTMLIFrameElement extends HTMLElement {
// @Ignore
  external HTMLIFrameElement.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-11309947
   */
  String align;

  /**
   * @type {Document}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-67133006
   */
  w3c_dom1.Document contentDocument;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-22463410
   */
  String frameBorder;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-1678118
   */
  String height;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-70472105
   */
  String longDesc;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-91371294
   */
  String marginHeight;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-66486595
   */
  String marginWidth;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-96819659
   */
  String name;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-36369822
   */
  String scrolling;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-43933957
   */
  String src;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/2000/CR-DOM-Level-2-20000510/html.html#ID-67133005
   */
  String width;

  /**
   * @type {boolean}
   * @implicitCast
   * @see http://msdn.microsoft.com/en-us/library/ie/ms533072(v=vs.85).aspx
   */
  bool allowTransparency;

  /**
   * @type {Window}
   * @see http://msdn.microsoft.com/en-us/library/ms533692(VS.85).aspx
   */
  w3c_dom1.Window contentWindow;

  /**
   * @type {string}
   * @see http://msdn.microsoft.com/en-us/library/ms534359(VS.85).aspx
   */
  String readyState;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/html-markup/iframe.html#iframe.attrs.srcdoc
   */
  String srcdoc;

  /**
   * @type {?string}
   * @see http://www.w3.org/TR/2012/WD-html5-20121025/the-iframe-element.html#attr-iframe-sandbox
   */
  String sandbox;
}
