/*
 * Copyright 2008 The Closure Compiler Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// This file has been automatically generated.  Please do not edit it manually.
/**
 * @fileoverview Definitions for W3C's DOM Level 3 specification.
 *  This file depends on w3c_dom2.js.
 *  The whole file has been fully type annotated.
 *  Created from
 *   http://www.w3.org/TR/DOM-Level-3-Core/ecma-script-binding.html
 *
 * FROM_EXTERN
 * @author stevey@google.com (Steve Yegge)
 */
@JsName()
library w3c_dom3;

import 'package:js/js.dart';
import 'package:func/func.dart';

import "w3c_dom1.dart" as w3c_dom1;

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#DOMStringList
 */
@JsName("DOMStringList")
abstract class DOMStringList {
// @Ignore
  external DOMStringList.fakeConstructor$();

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#DOMStringList-length
   */
  num length;

  /**
   * @param {string} str
   * @return {boolean}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#DOMStringList-contains
   */
  bool contains(String str);

  /**
   * @param {number} index
   * @return {string}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#DOMStringList-item
   */
  String item(num index);
}

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#NameList
 */
@JsName("NameList")
abstract class NameList {
// @Ignore
  external NameList.fakeConstructor$();

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#NameList-length
   */
  num length;

  /**
   * @param {string} str
   * @return {boolean}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#NameList-contains
   * @nosideeffects
   */
  bool contains(String str);

  /**
   * @param {string} namespaceURI
   * @param {string} name
   * @return {boolean}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#NameList-containsNS
   * @nosideeffects
   */
  bool containsNS(String namespaceURI, String name);

  /**
   * @param {number} index
   * @return {string}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#NameList-getName
   * @nosideeffects
   */
  String getName(num index);

  /**
   * @param {number} index
   * @return {string}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#NameList-getNamespaceURI
   * @nosideeffects
   */
  String getNamespaceURI(num index);
}

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#DOMImplementationList
 */
@JsName("DOMImplementationList")
abstract class DOMImplementationList {
// @Ignore
  external DOMImplementationList.fakeConstructor$();

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#DOMImplementationList-length
   */
  num length;

  /**
   * @param {number} index
   * @return {DOMImplementation}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#DOMImplementationList-item
   * @nosideeffects
   */
  w3c_dom1.DOMImplementation item(num index);
}

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#DOMImplementationSource
 */
@JsName("DOMImplementationSource")
abstract class DOMImplementationSource {
// @Ignore
  external DOMImplementationSource.fakeConstructor$();

  /**
   * @param {string} features
   * @return {DOMImplementation}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-getDOMImpl
   * @nosideeffects
   */
  w3c_dom1.DOMImplementation getDOMImplementation(String features);

  /**
   * @param {string} features
   * @return {DOMImplementationList}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-getDOMImpls
   * @nosideeffects
   */
  DOMImplementationList getDOMImplementationList(String features);
}

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#TypeInfo
 */
@JsName("TypeInfo")
abstract class TypeInfo {
// @Ignore
  external TypeInfo.fakeConstructor$();

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#TypeInfo-DERIVATION_EXTENSION
   */
  num DERIVATION_EXTENSION;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#TypeInfo-DERIVATION_LIST
   */
  num DERIVATION_LIST;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#TypeInfo-DERIVATION_RESTRICTION
   */
  num DERIVATION_RESTRICTION;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#TypeInfo-DERIVATION_UNION
   */
  num DERIVATION_UNION;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#TypeInfo-typeName
   */
  String typeName;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#TypeInfo-typeNamespace
   */
  String typeNamespace;

  /**
   * @param {string} typeNamespaceArg
   * @param {string} typeNameArg
   * @param {number} derivationMethod
   * @return {boolean}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#TypeInfo-isDerivedFrom
   * @nosideeffects
   */
  bool isDerivedFrom(
      String typeNamespaceArg, String typeNameArg, num derivationMethod);
}

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#UserDataHandler
 */
@JsName("UserDataHandler")
abstract class UserDataHandler {
// @Ignore
  external UserDataHandler.fakeConstructor$();

  /**
   * @param {number} operation
   * @param {string} key
   * @param {*=} opt_data
   * @param {?Node=} opt_src
   * @param {?Node=} opt_dst
   * @return {undefined}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-handleUserDataEvent
   */
  void handle(num operation, String key,
      [opt_data, w3c_dom1.Node opt_src, w3c_dom1.Node opt_dst]);
}

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ERROR-Interfaces-DOMError
 */
@JsName("DOMError")
abstract class DOMError {
// @Ignore
  external DOMError.fakeConstructor$();

  external static num get SEVERITY_WARNING;

  external static set SEVERITY_WARNING(num value);

  external static num get SEVERITY_ERROR;

  external static set SEVERITY_ERROR(num value);

  external static num get SEVERITY_FATAL_ERROR;

  external static set SEVERITY_FATAL_ERROR(num value);

  /**
   * @type {DOMLocator}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ERROR-DOMError-location
   */
  DOMLocator location;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ERROR-DOMError-message
   */
  String message;

  /**
   * @type {Object}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ERROR-DOMError-relatedData
   */
  Object relatedData;

  /**
   * @type {Object}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ERROR-DOMError-relatedException
   */
  Object relatedException;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ERROR-DOMError-severity
   */
  num severity;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ERROR-DOMError-type
   */
  String type;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/dom/#domerror
   */
  String name;
}

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ERROR-Interfaces-DOMErrorHandler
 */
@JsName("DOMErrorHandler")
abstract class DOMErrorHandler {
// @Ignore
  external DOMErrorHandler.fakeConstructor$();

  /**
   * @param {DOMError} error
   * @return {boolean}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#ID-ERRORS-DOMErrorHandler-handleError
   */
  bool handleError(DOMError error);
}

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#Interfaces-DOMLocator
 */
@JsName("DOMLocator")
abstract class DOMLocator {
// @Ignore
  external DOMLocator.fakeConstructor$();

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#DOMLocator-byteOffset
   */
  num byteOffset;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#DOMLocator-column-number
   */
  num columnNumber;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#DOMLocator-line-number
   */
  num lineNumber;

  /**
   * @type {Node}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#DOMLocator-node
   */
  w3c_dom1.Node relatedNode;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#DOMLocator-uri
   */
  String uri;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#DOMLocator-utf16Offset
   */
  num utf16Offset;
}

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#DOMConfiguration
 */
@JsName("DOMConfiguration")
abstract class DOMConfiguration {
// @Ignore
  external DOMConfiguration.fakeConstructor$();

  /**
   * @type {DOMStringList}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#DOMConfiguration-parameterNames
   */
  DOMStringList parameterNames;

  /**
   * @param {string} name
   * @return {boolean}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#DOMConfiguration-canSetParameter
   * @nosideeffects
   */
  bool canSetParameter(String name);

  /**
   * @param {string} name
   * @return {*}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#DOMConfiguration-getParameter
   * @nosideeffects
   */
  getParameter(String name);

  /**
   * @param {string} name
   * @param {*} value
   * @return {*}
   * @see http://www.w3.org/TR/DOM-Level-3-Core/core.html#DOMConfiguration-property
   */
  setParameter(String name, value);
}
