/*
 * Copyright 2008 The Closure Compiler Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// This file has been automatically generated.  Please do not edit it manually.
/**
 * @fileoverview Definitions for W3C's event specification.
 *  The whole file has been fully type annotated.
 *  Created from
 *   http://www.w3.org/TR/DOM-Level-2-Events/ecma-script-binding.html
 *
 * FROM_EXTERN
 */
@JsName()
library w3c_event;

import 'package:js/js.dart';
import 'package:func/func.dart';

import "html5.dart" as html5;

import "ie_dom.dart" as ie_dom;

import "w3c_dom1.dart" as w3c_dom1;

/**
 * @interface
 */
@JsName("EventTarget")
abstract class EventTarget {
// @Ignore
  external EventTarget.fakeConstructor$();

  /**
   * @param {string} type
   * @param {EventListener|function(!Event):(boolean|undefined)} listener
   * @param {boolean} useCapture
   * @return {undefined}
   */
  void addEventListener(
      String type, bool listener(Event arg1), bool useCapture);

  /**
   * @param {string} type
   * @param {EventListener|function(!Event):(boolean|undefined)} listener
   * @param {boolean} useCapture
   * @return {undefined}
   */
  void removeEventListener(
      String type, bool listener(Event arg1), bool useCapture);

  /**
   * @param {!Event} evt
   * @return {boolean}
   */
  bool dispatchEvent(Event evt);
}

/**
 * @interface
 */
@JsName("EventListener")
abstract class EventListener {
// @Ignore
  external EventListener.fakeConstructor$();

  /**
   * @param {!Event} evt
   * @return {undefined}
   */
  void handleEvent(Event evt);
}
/**
 * @typedef {{
 *   bubbles: (boolean|undefined),
 *   cancelable: (boolean|undefined)
 * }}
 */
@JsName()
abstract class EventInit {
  bool bubbles;
  bool cancelable;
}

/**
 * @constructor
 * @param {string} type
 * @param {EventInit=} opt_eventInitDict
 */
@JsName("Event")
abstract class Event {
// @Ignore
  external Event.fakeConstructor$();

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-2-Events/ecma-script-binding.html
   */
  external static num get CAPTURING_PHASE;

  external static set CAPTURING_PHASE(num value);

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-2-Events/ecma-script-binding.html
   */
  external static num get BUBBLING_PHASE;

  external static set BUBBLING_PHASE(num value);

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-2-Events/ecma-script-binding.html
   */
  external static num get AT_TARGET;

  external static set AT_TARGET(num value);

  /**
   * @constructor
   * @param {string} type
   * @param {EventInit=} opt_eventInitDict
   */
  external factory Event(String type, [EventInit opt_eventInitDict]);

  /** @type {string} */
  String type;

  /** @type {EventTarget} */
  EventTarget target;

  /** @type {EventTarget} */
  EventTarget currentTarget;

  /** @type {number} */
  num eventPhase;

  /** @type {boolean} */
  bool bubbles;

  /** @type {boolean} */
  bool cancelable;

  /** @type {number} */
  num timeStamp;

  /**
   * Present for events spawned in browsers that support shadow dom.
   * @type {Array.<!Element>|undefined}
   */
  List<w3c_dom1.Element> path;

  /**
   * @return {undefined}
   */
  void stopPropagation();

  /**
   * @return {undefined}
   */
  void preventDefault();

  /**
   * @param {string} eventTypeArg
   * @param {boolean} canBubbleArg
   * @param {boolean} cancelableArg
   * @return {undefined}
   */
  void initEvent(String eventTypeArg, bool canBubbleArg, bool cancelableArg);

  /** @type {boolean} */
  bool defaultPrevented;

  /** @type {string} */
  String namespaceURI;

  /** @return {undefined} */
  void stopImmediatePropagation();

  /** @type {number} */
  num HORIZONTAL_AXIS;

  /** @type {number} */
  num VERTICAL_AXIS;

  /** @type {boolean} */
  bool altKey;

  /** @type {number} */
  num axis;

  /** @type {number} */
  num button;

  /** @type {boolean} */
  bool cancelBubble;

  /** @type {number} */
  num charCode;

  /** @type {number} */
  num clientX;

  /** @type {number} */
  num clientY;

  /** @type {boolean} */
  bool ctrlKey;

  /** @type {EventTarget} */
  EventTarget explicitOriginalTarget;

  /** @type {boolean} */
  bool isChar;

  /** @type {number} */
  num keyCode;

  /** @type {number} */
  num layerX;

  /** @type {number} */
  num layerY;

  /** @type {boolean} */
  bool metaKey;

  /** @type {EventTarget} */
  EventTarget originalTarget;

  /** @type {number} */
  num pageX;

  /** @type {number} */
  num pageY;

  /** @type {EventTarget} */
  EventTarget relatedTarget;

  /** @type {number} */
  num screenX;

  /** @type {number} */
  num screenY;

  /** @type {boolean} */
  bool shiftKey;

  /** @type {Window} */
  w3c_dom1.Window view;

  /** @type {number} */
  num which;

  dynamic preventBubble;

  dynamic preventCapture;

  /** @type {string} */
  String Abstract;

  /** @type {boolean} */
  bool altLeft;

  /** @type {string} */
  String Banner;

  /**
   * A ClipboardData on IE, but a DataTransfer on WebKit.
   * @see http://msdn.microsoft.com/en-us/library/ms535220.aspx
   * @type {(ClipboardData|undefined)}
   */
  ie_dom.ClipboardData clipboardData;

  /** @type {boolean} */
  bool contentOverflow;

  /** @type {boolean} */
  bool ctrlLeft;

  /** @type {string} */
  String dataFld;

  dynamic domain;

  /** @type {Element} */
  w3c_dom1.Element fromElement;

  /** @type {string} */
  String MoreInfo;

  /** @type {string} */
  String nextPage;

  /** @type {number} */
  num offsetX;

  /** @type {number} */
  num offsetY;

  /** @type {string} */
  String propertyName;

  /** @type {string} */
  String qualifier;

  /** @type {number} */
  num reason;

  /** @type {Object.<*,*>} */
  Object /*<dynamic, dynamic>*/ recordset;

  /** @type {boolean} */
  bool repeat;

  /** @type {(boolean|string|undefined)} */
  dynamic /*bool|String|undefined*/ returnValue;

  /** @type {string} */
  String saveType;

  dynamic scheme;

  /** @type {boolean} */
  bool shiftLeft;

  /** @type {Window} */
  w3c_dom1.Window source;

  /** @type {Element} */
  w3c_dom1.Element srcElement;

  dynamic srcFilter;

  /** @type {string} */
  String srcUrn;

  /** @type {Element} */
  w3c_dom1.Element toElement;

  dynamic userName;

  /** @type {number} */
  num wheelDelta;

  /** @type {number} */
  num x;

  /** @type {number} */
  num y;

  /** @type {number} */
  num wheelDeltaX;

  /** @type {number} */
  num wheelDeltaY;
}
/**
 * @typedef {{
 *   bubbles: (boolean|undefined),
 *   cancelable: (boolean|undefined),
 *   detail: *
 * }}
 */
@JsName()
abstract class CustomEventInit {
  bool bubbles;
  bool cancelable;
  dynamic detail;
}

/**
 * @constructor
 * @extends {Event}
 * @param {string} type
 * @param {CustomEventInit=} opt_eventInitDict
 * @see http://www.w3.org/TR/DOM-Level-3-Events/#interface-CustomEvent
 */
@JsName("CustomEvent")
abstract class CustomEvent extends Event {
// @Ignore
  external CustomEvent.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @constructor
   * @extends {Event}
   * @param {string} type
   * @param {CustomEventInit=} opt_eventInitDict
   * @see http://www.w3.org/TR/DOM-Level-3-Events/#interface-CustomEvent
   */
  external factory CustomEvent(String type,
      [CustomEventInit opt_eventInitDict]);

  /**
   * @param {string} eventType
   * @param {boolean} bubbles
   * @param {boolean} cancelable
   * @param {*} detail
   */
  initCustomEvent(String eventType, bool bubbles, bool cancelable, detail);

  /**
   * @type {*}
   */
  dynamic detail;
}

/**
 * @interface
 */
@JsName("DocumentEvent")
abstract class DocumentEvent {
// @Ignore
  external DocumentEvent.fakeConstructor$();

  /**
   * @param {string} eventType
   * @return {!Event}
   */
  Event createEvent(String eventType);
}
/**
 * @typedef {{
 *   bubbles: (boolean|undefined),
 *   cancelable: (boolean|undefined),
 *   view: (Window|undefined),
 *   detail: (number|undefined)
 * }}
 */
@JsName()
abstract class UIEventInit {
  bool bubbles;
  bool cancelable;
  num detail;
  w3c_dom1.Window view;
}

/**
 * @constructor
 * @extends {Event}
 * @param {string} type
 * @param {UIEventInit=} opt_eventInitDict
 */
@JsName("UIEvent")
abstract class UIEvent extends Event {
// @Ignore
  external UIEvent.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @constructor
   * @extends {Event}
   * @param {string} type
   * @param {UIEventInit=} opt_eventInitDict
   */
  external factory UIEvent(String type, [UIEventInit opt_eventInitDict]);

  /** @type {number} */
  num detail;

  /**
   * @param {string} typeArg
   * @param {boolean} canBubbleArg
   * @param {boolean} cancelableArg
   * @param {Window} viewArg
   * @param {number} detailArg
   * @return {undefined}
   */
  void initUIEvent(String typeArg, bool canBubbleArg, bool cancelableArg,
      w3c_dom1.Window viewArg, num detailArg);
}
/**
 * @typedef {{
 *   bubbles: (boolean|undefined),
 *   cancelable: (boolean|undefined),
 *   view: (Window|undefined),
 *   detail: (number|undefined),
 *   screenX: (number|undefined),
 *   screenY: (number|undefined),
 *   clientX: (number|undefined),
 *   clientY: (number|undefined),
 *   ctrlKey: (boolean|undefined),
 *   shiftKey: (boolean|undefined),
 *   altKey: (boolean|undefined),
 *   metaKey: (boolean|undefined),
 *   button: (number|undefined),
 *   buttons: (number|undefined),
 *   relatedTarget: (EventTarget|undefined)
 * }}
 */
@JsName()
abstract class MouseEventInit {
  bool altKey;
  bool bubbles;
  num button;
  num buttons;
  bool cancelable;
  num clientX;
  num clientY;
  bool ctrlKey;
  num detail;
  bool metaKey;
  EventTarget relatedTarget;
  num screenX;
  num screenY;
  bool shiftKey;
  w3c_dom1.Window view;
}

/**
 * @constructor
 * @extends {UIEvent}
 * @param {string} type
 * @param {MouseEventInit=} opt_eventInitDict
 */
@JsName("MouseEvent")
abstract class MouseEvent extends UIEvent {
// @Ignore
  external MouseEvent.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @constructor
   * @extends {UIEvent}
   * @param {string} type
   * @param {MouseEventInit=} opt_eventInitDict
   */
  external factory MouseEvent(String type, [MouseEventInit opt_eventInitDict]);

  /** @type {number} */
  num screenX;

  /** @type {number} */
  num screenY;

  /** @type {number} */
  num clientX;

  /** @type {number} */
  num clientY;

  /** @type {boolean} */
  bool ctrlKey;

  /** @type {boolean} */
  bool shiftKey;

  /** @type {boolean} */
  bool altKey;

  /** @type {boolean} */
  bool metaKey;

  /** @type {number} */
  num button;

  /** @type {EventTarget} */
  EventTarget relatedTarget;

  /** @type {number} */
  num buttons;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-mouseevent-pagex
   */
  num pageX;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-mouseevent-pagey
   */
  num pageY;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-mouseevent-x
   */
  num x;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-mouseevent-y
   */
  num y;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-mouseevent-offsetx
   */
  num offsetX;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/cssom-view/#dom-mouseevent-offsety
   */
  num offsetY;

  /**
   * Addition for accessing clipboard file data that are part of the proposed
   * HTML5 spec.
   * @type {DataTransfer}
   */
  html5.DataTransfer dataTransfer;
}

/**
 * @constructor
 * @extends {Event}
 */
@JsName("MutationEvent")
abstract class MutationEvent extends Event {
// @Ignore
  external MutationEvent.fakeConstructor$() : super.fakeConstructor$();

  /** @type {Node} */
  w3c_dom1.Node relatedNode;

  /** @type {string} */
  String prevValue;

  /** @type {string} */
  String newValue;

  /** @type {string} */
  String attrName;

  /** @type {number} */
  num attrChange;

  /**
   * @param {string} typeArg
   * @param {boolean} canBubbleArg
   * @param {boolean} cancelableArg
   * @param {Node} relatedNodeArg
   * @param {string} prevValueArg
   * @param {string} newValueArg
   * @param {string} attrNameArg
   * @param {number} attrChangeArg
   * @return {undefined}
   */
  void initMutationEvent(String typeArg, bool canBubbleArg, bool cancelableArg,
      w3c_dom1.Node relatedNodeArg, String prevValueArg, String newValueArg,
      String attrNameArg, num attrChangeArg);
}
/**
 * @typedef {{
 *   bubbles: (boolean|undefined),
 *   cancelable: (boolean|undefined),
 *   view: (Window|undefined),
 *   detail: (number|undefined),
 *   char: (string|undefined),
 *   key: (string|undefined),
 *   code: (string|undefined),
 *   location: (number|undefined),
 *   ctrlKey: (boolean|undefined),
 *   shiftKey: (boolean|undefined),
 *   altKey: (boolean|undefined),
 *   metaKey: (boolean|undefined),
 *   repeat: (boolean|undefined),
 *   locale: (string|undefined)
 * }}
 */
@JsName()
abstract class KeyboardEventInit {
  bool altKey;
  bool bubbles;
  bool cancelable;
  String char;
  String code;
  bool ctrlKey;
  num detail;
  String key;
  String locale;
  num location;
  bool metaKey;
  bool repeat;
  bool shiftKey;
  w3c_dom1.Window view;
}

/**
 * @constructor
 * @extends {UIEvent}
 * @param {string} type
 * @param {KeyboardEventInit=} opt_eventInitDict
 */
@JsName("KeyboardEvent")
abstract class KeyboardEvent extends UIEvent {
// @Ignore
  external KeyboardEvent.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @constructor
   * @extends {UIEvent}
   * @param {string} type
   * @param {KeyboardEventInit=} opt_eventInitDict
   */
  external factory KeyboardEvent(String type,
      [KeyboardEventInit opt_eventInitDict]);

  /** @type {string} */
  String keyIdentifier;

  /** @type {boolean} */
  bool ctrlKey;

  /** @type {boolean} */
  bool shiftKey;

  /** @type {boolean} */
  bool altKey;

  /** @type {boolean} */
  bool metaKey;

  /**
   * @param {string} keyIdentifierArg
   * @return {boolean}
   */
  bool getModifierState(String keyIdentifierArg);

  /**
   * @param {string} typeArg
   * @param {boolean} canBubbleArg
   * @param {boolean} cancelableArg
   * @param {Window} viewArg
   * @param {string} keyIdentifierArg
   * @param {number} keyLocationArg
   * @param {string} modifiersList
   */
  initKeyboardEvent(String typeArg, bool canBubbleArg, bool cancelableArg,
      w3c_dom1.Window viewArg, String keyIdentifierArg, num keyLocationArg,
      String modifiersList);

  /** @type {string} */
  String char;

  /** @type {string} */
  String key;

  /** @type {number} */
  num location;

  /** @type {boolean} */
  bool repeat;

  /** @type {string} */
  String locale;
}
