/*
 * Copyright 2011 The Closure Compiler Authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// This file has been automatically generated.  Please do not edit it manually.
/**
 * @fileoverview Definitions for W3C's IndexedDB API. In Chrome all the
 * IndexedDB classes are prefixed with 'webkit'. In order to access constants
 * and static methods of these classes they must be duplicated with the
 * prefix here.
 * @see http://www.w3.org/TR/IndexedDB/
 *
 * FROM_EXTERN
 * @author guido.tapia@picnet.com.au (Guido Tapia)
 */
@JsName()
library w3c_indexeddb;

import 'package:js/js.dart';
import 'package:func/func.dart';

import "w3c_dom3.dart" as w3c_dom3;

import "w3c_event.dart" as w3c_event;

/**
 * @constructor
 * @see http://www.w3.org/TR/IndexedDB/#idl-def-IDBFactory
 */
@JsName("IDBFactory")
abstract class IDBFactory {
// @Ignore
  external IDBFactory.fakeConstructor$();

  /**
   * @param {string} name The name of the database to open.
   * @param {number=} opt_version The version at which to open the database.
   * @return {!IDBOpenDBRequest} The IDBRequest object.
   */
  IDBOpenDBRequest open(String name, [num opt_version]);

  /**
   * @param {string} name The name of the database to delete.
   * @return {!IDBOpenDBRequest} The IDBRequest object.
   */
  IDBOpenDBRequest deleteDatabase(String name);
}

/**
 * @constructor
 * @see http://www.w3.org/TR/IndexedDB/#idl-def-IDBDatabaseException
 */
@JsName("IDBDatabaseException")
abstract class IDBDatabaseException {
// @Ignore
  external IDBDatabaseException.fakeConstructor$();

  /**
   * @const
   * @type {number}
   */
  external static num get CONSTRAINT_ERR;

  external static set CONSTRAINT_ERR(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get TIMEOUT_ERR;

  external static set TIMEOUT_ERR(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get QUOTA_ERR;

  external static set QUOTA_ERR(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get TRANSACTION_INACTIVE_ERR;

  external static set TRANSACTION_INACTIVE_ERR(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get DATA_ERR;

  external static set DATA_ERR(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get ABORT_ERR;

  external static set ABORT_ERR(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get NOT_ALLOWED_ERR;

  external static set NOT_ALLOWED_ERR(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get NON_TRANSIENT_ERR;

  external static set NON_TRANSIENT_ERR(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get UNKNOWN_ERR;

  external static set UNKNOWN_ERR(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get READ_ONLY_ERR;

  external static set READ_ONLY_ERR(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get NOT_FOUND_ERR;

  external static set NOT_FOUND_ERR(num value);

  /**
   * @const
   * @type {number}
   */
  num code;

  /**
   * @const
   * @type {string}
   */
  String message;
}

/**
 * @constructor
 * @extends {IDBDatabaseException}
 * @see http://www.w3.org/TR/IndexedDB/#idl-def-IDBDatabaseException
 */
@JsName("webkitIDBDatabaseException")
abstract class webkitIDBDatabaseException extends IDBDatabaseException {
// @Ignore
  external webkitIDBDatabaseException.fakeConstructor$()
      : super.fakeConstructor$();

  /**
   * @const
   * @type {number}
   */
  external static num get CONSTRAINT_ERR;

  external static set CONSTRAINT_ERR(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get TIMEOUT_ERR;

  external static set TIMEOUT_ERR(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get QUOTA_ERR;

  external static set QUOTA_ERR(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get TRANSACTION_INACTIVE_ERR;

  external static set TRANSACTION_INACTIVE_ERR(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get DATA_ERR;

  external static set DATA_ERR(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get ABORT_ERR;

  external static set ABORT_ERR(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get NOT_ALLOWED_ERR;

  external static set NOT_ALLOWED_ERR(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get NON_TRANSIENT_ERR;

  external static set NON_TRANSIENT_ERR(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get UNKNOWN_ERR;

  external static set UNKNOWN_ERR(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get READ_ONLY_ERR;

  external static set READ_ONLY_ERR(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get NOT_FOUND_ERR;

  external static set NOT_FOUND_ERR(num value);

  /**
   * @const
   * @type {number}
   */
  num code;

  /**
   * @const
   * @type {string}
   */
  String message;
}

/**
 * @constructor
 * @implements {EventTarget}
 * @see http://www.w3.org/TR/IndexedDB/#idl-def-IDBRequest
 */
@JsName("IDBRequest")
abstract class IDBRequest implements w3c_event.EventTarget {
// @Ignore
  external IDBRequest.fakeConstructor$();

  /**
   * @type {number}
   * @const
   */
  external static num get LOADING;

  external static set LOADING(num value);

  /**
   * @type {number}
   * @const
   */
  external static num get DONE;

  external static set DONE(num value);

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  addEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  removeEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /** @override */
  bool dispatchEvent(w3c_event.Event evt);

  /** @type {number} */
  num readyState;

  /** @type {function(!Event)} */
  onsuccess(w3c_event.Event e);

  /** @type {function(!Event)} */
  onerror(w3c_event.Event e);

  /** @type {*} */
  dynamic result;

  /**
   * @type {number}
   * @deprecated Use "error"
   */
  num errorCode;

  /** @type {!DOMError} */
  w3c_dom3.DOMError error;

  /** @type {Object} */
  Object source;

  /** @type {IDBTransaction} */
  IDBTransaction transaction;
}

/**
 * @constructor
 * @extends {IDBRequest}
 * @see http://www.w3.org/TR/IndexedDB/#idl-def-IDBRequest
 */
@JsName("webkitIDBRequest")
abstract class webkitIDBRequest extends IDBRequest {
// @Ignore
  external webkitIDBRequest.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {number}
   * @const
   */
  external static num get LOADING;

  external static set LOADING(num value);

  /**
   * @type {number}
   * @const
   */
  external static num get DONE;

  external static set DONE(num value);
}

/**
 * @constructor
 * @extends {IDBRequest}
 * @see http://www.w3.org/TR/IndexedDB/#idl-def-IDBOpenDBRequest
 */
@JsName("IDBOpenDBRequest")
abstract class IDBOpenDBRequest extends IDBRequest {
// @Ignore
  external IDBOpenDBRequest.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {function(!IDBVersionChangeEvent)}
   */
  onblocked(IDBVersionChangeEvent e);

  /**
   * @type {function(!IDBVersionChangeEvent)}
   */
  onupgradeneeded(IDBVersionChangeEvent e);
}

/**
 * @constructor
 * @implements {EventTarget}
 * @see http://www.w3.org/TR/IndexedDB/#idl-def-IDBDatabase
 */
@JsName("IDBDatabase")
abstract class IDBDatabase implements w3c_event.EventTarget {
// @Ignore
  external IDBDatabase.fakeConstructor$();

  /**
   * @type {string}
   * @const
   */
  String name;

  /**
   * @type {string}
   * @const
   */
  String description;

  /**
   * @type {string}
   * @const
   */
  String version;

  /**
   * @type {DOMStringList}
   * @const
   */
  w3c_dom3.DOMStringList objectStoreNames;

  /**
   * @param {string} name The name of the object store.
   * @param {Object=} opt_parameters Parameters to be passed
   *     creating the object store.
   * @return {!IDBObjectStore} The created/open object store.
   */
  IDBObjectStore createObjectStore(String name, [Object opt_parameters]);

  /**
   * @param {string} name The name of the object store to remove.
   */
  deleteObjectStore(String name);

  /**
   * @param {string} version The new version of the database.
   * @return {!IDBRequest} The IDBRequest object.
   */
  IDBRequest setVersion(String version);

  /**
   * @param {Array.<string>} storeNames The stores to open in this transaction.
   * @param {(number|string)=} mode The mode for opening the object stores.
   * @return {!IDBTransaction} The IDBRequest object.
   */
  IDBTransaction transaction(List<String> storeNames,
      [mode /*num|String|undefined*/]);

  /**
   * Closes the database connection.
   */
  close();

  /**
   * @type {Function}
   */
  onabort([opt_varargs1, opt_varargs2, opt_varargs3, opt_varargs4]);

  /**
   * @type {Function}
   */
  onerror([opt_varargs1, opt_varargs2, opt_varargs3, opt_varargs4]);

  /**
   * @type {Function}
   */
  onversionchange([opt_varargs1, opt_varargs2, opt_varargs3, opt_varargs4]);

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  addEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  removeEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /** @override */
  bool dispatchEvent(w3c_event.Event evt);
}
/**
 * Typedef for valid key types according to the w3 specification. Note that this
 * is slightly wider than what is actually allowed, as all Array elements must
 * have a valid key type.
 * @see http://www.w3.org/TR/IndexedDB/#key-construct
 * @typedef {number|string|!Date|!Array.<?>}
 */
// Dart does not support union types. Skipping: IDBKeyType

/**
 * @constructor
 * @see http://www.w3.org/TR/IndexedDB/#idl-def-IDBObjectStore
 */
@JsName("IDBObjectStore")
abstract class IDBObjectStore {
// @Ignore
  external IDBObjectStore.fakeConstructor$();

  /**
   * @type {string}
   */
  String name;

  /**
   * @type {string}
   */
  String keyPath;

  /**
   * @type {DOMStringList}
   */
  w3c_dom3.DOMStringList indexNames;

  /** @type {IDBTransaction} */
  IDBTransaction transaction;

  /** @type {boolean} */
  bool autoIncrement;

  /**
   * @param {*} value The value to put into the object store.
   * @param {IDBKeyType=} key The key of this value.
   * @return {!IDBRequest} The IDBRequest object.
   */
  IDBRequest put(value, [key /*num|String|DateTime|List<dynamic>|undefined*/]);

  /**
   * @param {*} value The value to add into the object store.
   * @param {IDBKeyType=} key The key of this value.
   * @return {!IDBRequest} The IDBRequest object.
   */
  IDBRequest add(value, [key /*num|String|DateTime|List<dynamic>|undefined*/]);

  /**
   * @param {IDBKeyType} key The key of this value.
   * @return {!IDBRequest} The IDBRequest object.
   */
  IDBRequest delete(key /*num|String|DateTime|List<dynamic>*/);

  /**
   * @param {IDBKeyType|!IDBKeyRange} key The key of the document to retrieve.
   * @return {!IDBRequest} The IDBRequest object.
   */
  IDBRequest get(key /*num|String|DateTime|List<dynamic>|IDBKeyRange*/);

  /**
   * @return {!IDBRequest} The IDBRequest object.
   */
  IDBRequest clear();

  /**
   * @param {IDBKeyRange=} range The range of the cursor.
   * @param {(number|string)=} direction The direction of cursor enumeration.
   * @return {!IDBRequest} The IDBRequest object.
   */
  IDBRequest openCursor(
      [IDBKeyRange range, direction /*num|String|undefined*/]);

  /**
   * @param {string} name The name of the index.
   * @param {string|!Array.<string>} keyPath The path to the index key.
   * @param {Object=} opt_paramters Optional parameters
   *     for the created index.
   * @return {!IDBIndex} The IDBIndex object.
   */
  IDBIndex createIndex(String name, keyPath /*String|List<String>*/,
      [Object opt_paramters]);

  /**
   * @param {string} name The name of the index to retrieve.
   * @return {!IDBIndex} The IDBIndex object.
   */
  IDBIndex index(String name);

  /**
   * @param {string} indexName The name of the index to remove.
   */
  deleteIndex(String indexName);

  /**
   * @param {(IDBKeyType|IDBKeyRange)=} key The key of this value.
   * @return {!IDBRequest} The IDBRequest object.
   * @see http://www.w3.org/TR/IndexedDB/#widl-IDBObjectStore-count
   */
  IDBRequest count(
      [key /*num|String|DateTime|List<dynamic>|IDBKeyRange|null|undefined*/]);
}

/**
 * @constructor
 * @see http://www.w3.org/TR/IndexedDB/#idl-def-IDBIndex
 */
@JsName("IDBIndex")
abstract class IDBIndex {
// @Ignore
  external IDBIndex.fakeConstructor$();

  /**
   * @type {string}
   * @const
   */
  String name;

  /**
   * @type {!IDBObjectStore}
   * @const
   */
  IDBObjectStore objectStore;

  /**
   * @type {string}
   * @const
   */
  String keyPath;

  /**
   * @type {boolean}
   * @const
   */
  bool unique;

  /**
   * @param {IDBKeyRange=} range The range of the cursor.
   * @param {(number|string)=} direction The direction of cursor enumeration.
   * @return {!IDBRequest} The IDBRequest object.
   */
  IDBRequest openCursor(
      [IDBKeyRange range, direction /*num|String|undefined*/]);

  /**
   * @param {IDBKeyRange=} range The range of the cursor.
   * @param {(number|string)=} direction The direction of cursor enumeration.
   * @return {!IDBRequest} The IDBRequest object.
   */
  IDBRequest openKeyCursor(
      [IDBKeyRange range, direction /*num|String|undefined*/]);

  /**
   * @param {IDBKeyType|!IDBKeyRange} key The id of the object to retrieve.
   * @return {!IDBRequest} The IDBRequest object.
   */
  IDBRequest get(key /*num|String|DateTime|List<dynamic>|IDBKeyRange*/);

  /**
   * @param {IDBKeyType|!IDBKeyRange} key The id of the object to retrieve.
   * @return {!IDBRequest} The IDBRequest object.
   */
  IDBRequest getKey(key /*num|String|DateTime|List<dynamic>|IDBKeyRange*/);
}

/**
 * @constructor
 * @see http://www.w3.org/TR/IndexedDB/#idl-def-IDBCursor
 */
@JsName("IDBCursor")
abstract class IDBCursor {
// @Ignore
  external IDBCursor.fakeConstructor$();

  /**
   * @const
   * @type {number}
   */
  external static num get NEXT;

  external static set NEXT(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get PREV_NO_DUPLICATE;

  external static set PREV_NO_DUPLICATE(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get PREV;

  external static set PREV(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get NEXT_NO_DUPLICATE;

  external static set NEXT_NO_DUPLICATE(num value);

  /**
   * @type {*}
   * @const
   */
  dynamic source;

  /**
   * @type {number}
   * @const
   */
  num direction;

  /**
   * @type {IDBKeyType}
   * @const
   */
  dynamic /*num|String|DateTime|List<dynamic>*/ key;

  /**
   * @type {number}
   * @const
   */
  num primaryKey;

  /**
   * @param {*} value The new value for the current object in the cursor.
   * @return {!IDBRequest} The IDBRequest object.
   */
  IDBRequest update(value);

  /**
   * Note: Must be quoted to avoid parse error.
   * @param {IDBKeyType=} key Continue enumerating the cursor from the specified
   *     key (or next).
   */
  continue$([key /*num|String|DateTime|List<dynamic>|undefined*/]);

  /**
   * @param {number} count Number of times to iterate the cursor.
   */
  advance(num count);

  /**
   * Note: Must be quoted to avoid parse error.
   * @return {!IDBRequest} The IDBRequest object.
   */
  IDBRequest delete();
}

/**
 * @constructor
 * @extends {IDBCursor}
 * @see http://www.w3.org/TR/IndexedDB/#idl-def-IDBCursor
 */
@JsName("webkitIDBCursor")
abstract class webkitIDBCursor extends IDBCursor {
// @Ignore
  external webkitIDBCursor.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @const
   * @type {number}
   */
  external static num get NEXT;

  external static set NEXT(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get PREV_NO_DUPLICATE;

  external static set PREV_NO_DUPLICATE(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get PREV;

  external static set PREV(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get NEXT_NO_DUPLICATE;

  external static set NEXT_NO_DUPLICATE(num value);
}

/**
 * @constructor
 * @extends {IDBCursor}
 * @see http://www.w3.org/TR/IndexedDB/#idl-def-IDBCursorWithValue
 */
@JsName("IDBCursorWithValue")
abstract class IDBCursorWithValue extends IDBCursor {
// @Ignore
  external IDBCursorWithValue.fakeConstructor$() : super.fakeConstructor$();

  /** @type {*} */
  dynamic value;
}

/**
 * @constructor
 * @see http://www.w3.org/TR/IndexedDB/#idl-def-IDBTransaction
 */
@JsName("IDBTransaction")
abstract class IDBTransaction {
// @Ignore
  external IDBTransaction.fakeConstructor$();

  /**
   * @const
   * @type {number}
   */
  external static num get VERSION_CHANGE;

  external static set VERSION_CHANGE(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get READ_WRITE;

  external static set READ_WRITE(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get READ_ONLY;

  external static set READ_ONLY(num value);

  /**
   * @type {number|string}
   * @const
   */
  dynamic /*num|String*/ mode;

  /**
   * @type {IDBDatabase}
   * @const
   */
  IDBDatabase db;

  /**
   * @param {string} name The name of the object store to retrieve.
   * @return {!IDBObjectStore} The object store.
   */
  IDBObjectStore objectStore(String name);

  /**
   * Aborts the transaction.
   */
  abort();

  /**
   * @type {Function}
   */
  onabort([opt_varargs1, opt_varargs2, opt_varargs3, opt_varargs4]);

  /**
   * @type {Function}
   */
  oncomplete([opt_varargs1, opt_varargs2, opt_varargs3, opt_varargs4]);

  /**
   * @type {Function}
   */
  onerror([opt_varargs1, opt_varargs2, opt_varargs3, opt_varargs4]);
}

/**
 * @constructor
 * @extends {IDBTransaction}
 * @see http://www.w3.org/TR/IndexedDB/#idl-def-IDBTransaction
 */
@JsName("webkitIDBTransaction")
abstract class webkitIDBTransaction extends IDBTransaction {
// @Ignore
  external webkitIDBTransaction.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @const
   * @type {number}
   */
  external static num get VERSION_CHANGE;

  external static set VERSION_CHANGE(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get READ_WRITE;

  external static set READ_WRITE(num value);

  /**
   * @const
   * @type {number}
   */
  external static num get READ_ONLY;

  external static set READ_ONLY(num value);
}

/**
 * @constructor
 * @see http://www.w3.org/TR/IndexedDB/#idl-def-IDBKeyRange
 */
@JsName("IDBKeyRange")
abstract class IDBKeyRange {
// @Ignore
  external IDBKeyRange.fakeConstructor$();

  /**
   * @param {IDBKeyType} bound Creates a lower bound key range.
   * @param {boolean=} open Open the key range.
   * @return {!IDBKeyRange} The key range.
   */
  external static IDBKeyRange lowerBound(
      bound /*num|String|DateTime|List<dynamic>*/, [bool open]);

  /**
   * @param {IDBKeyType} value The single key value of this range.
   * @return {!IDBKeyRange} The key range.
   */
  external static IDBKeyRange only(value /*num|String|DateTime|List<dynamic>*/);

  /**
   * @param {IDBKeyType} bound Creates an upper bound key range.
   * @param {boolean=} open Open the key range.
   * @return {!IDBKeyRange} The key range.
   */
  external static IDBKeyRange upperBound(
      bound /*num|String|DateTime|List<dynamic>*/, [bool open]);

  /**
   * @param {IDBKeyType} left The left bound value.
   * @param {IDBKeyType} right The right bound value.
   * @param {boolean=} openLeft Whether the left bound value should be excluded.
   * @param {boolean=} openRight Whether the right bound value should be excluded.
   * @return {!IDBKeyRange} The key range.
   */
  external static IDBKeyRange bound(left /*num|String|DateTime|List<dynamic>*/,
      right /*num|String|DateTime|List<dynamic>*/,
      [bool openLeft, bool openRight]);

  /**
   * @type {*}
   * @const
   */
  dynamic lower;

  /**
   * @type {*}
   * @const
   */
  dynamic upper;

  /**
   * @type {*}
   * @const
   */
  dynamic lowerOpen;

  /**
   * @type {*}
   * @const
   */
  dynamic upperOpen;
}

/**
 * @constructor
 * @extends {IDBKeyRange}
 * @see http://www.w3.org/TR/IndexedDB/#idl-def-IDBKeyRange
 */
@JsName("webkitIDBKeyRange")
abstract class webkitIDBKeyRange extends IDBKeyRange {
// @Ignore
  external webkitIDBKeyRange.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @param {IDBKeyType} bound Creates a lower bound key range.
   * @param {boolean=} open Open the key range.
   * @return {!IDBKeyRange} The key range.
   */
  external static IDBKeyRange lowerBound(
      bound /*num|String|DateTime|List<dynamic>*/, [bool open]);

  /**
   * @param {IDBKeyType} value The single key value of this range.
   * @return {!IDBKeyRange} The key range.
   */
  external static IDBKeyRange only(value /*num|String|DateTime|List<dynamic>*/);

  /**
   * @param {IDBKeyType} bound Creates an upper bound key range.
   * @param {boolean=} open Open the key range.
   * @return {!IDBKeyRange} The key range.
   */
  external static IDBKeyRange upperBound(
      bound /*num|String|DateTime|List<dynamic>*/, [bool open]);

  /**
   * @param {IDBKeyType} left The left bound value.
   * @param {IDBKeyType} right The right bound value.
   * @param {boolean=} openLeft Whether the left bound value should be excluded.
   * @param {boolean=} openRight Whether the right bound value should be excluded.
   * @return {!IDBKeyRange} The key range.
   */
  external static IDBKeyRange bound(left /*num|String|DateTime|List<dynamic>*/,
      right /*num|String|DateTime|List<dynamic>*/,
      [bool openLeft, bool openRight]);
}

/**
 * @constructor
 * @extends {Event}
 * @see http://www.w3.org/TR/IndexedDB/#idl-def-IDBVersionChangeEvent
 */
@JsName("IDBVersionChangeEvent")
abstract class IDBVersionChangeEvent extends w3c_event.Event {
// @Ignore
  external IDBVersionChangeEvent.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {number}
   * @const
   */
  num oldVersion;

  /**
   * @type {?number}
   * @const
   */
  num newVersion;
}

/**
 * @constructor
 * @extends {IDBVersionChangeEvent}
 * @see http://www.w3.org/TR/IndexedDB/#idl-def-IDBVersionChangeEvent
 */
@JsName("webkitIDBVersionChangeEvent")
abstract class webkitIDBVersionChangeEvent extends IDBVersionChangeEvent {
// @Ignore
  external webkitIDBVersionChangeEvent.fakeConstructor$()
      : super.fakeConstructor$();

  /**
   * @type {string}
   * @const
   */
  String version;
}
