/*
 * Copyright 2011 The Closure Compiler Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// This file has been automatically generated.  Please do not edit it manually.
/**
 * @fileoverview Definitions for W3C's Navigation Timing specification.
 *
 * Created from
 * @see http://dvcs.w3.org/hg/webperf/raw-file/tip/specs/NavigationTiming/Overview.html
 * @see http://w3c-test.org/webperf/specs/ResourceTiming
 * @see http://www.w3.org/TR/performance-timeline
 * @see http://www.w3.org/TR/user-timing/
 *
 * FROM_EXTERN
 */
@JsName()
library w3c_navigation_timing;

import 'package:js/js.dart';
import 'package:func/func.dart';

/** @constructor */
@JsName("PerformanceTiming")
abstract class PerformanceTiming {
// @Ignore
  external PerformanceTiming.fakeConstructor$();

  /** @type {number} */
  num navigationStart;

  /** @type {number} */
  num unloadEventStart;

  /** @type {number} */
  num unloadEventEnd;

  /** @type {number} */
  num redirectStart;

  /** @type {number} */
  num redirectEnd;

  /** @type {number} */
  num fetchStart;

  /** @type {number} */
  num domainLookupStart;

  /** @type {number} */
  num domainLookupEnd;

  /** @type {number} */
  num connectStart;

  /** @type {number} */
  num connectEnd;

  /** @type {number} */
  num secureConnectionStart;

  /** @type {number} */
  num requestStart;

  /** @type {number} */
  num responseStart;

  /** @type {number} */
  num responseEnd;

  /** @type {number} */
  num domLoading;

  /** @type {number} */
  num domInteractive;

  /** @type {number} */
  num domContentLoadedEventStart;

  /** @type {number} */
  num domContentLoadedEventEnd;

  /** @type {number} */
  num domComplete;

  /** @type {number} */
  num loadEventStart;

  /** @type {number} */
  num loadEventEnd;
}

/** @constructor */
@JsName("PerformanceEntry")
abstract class PerformanceEntry {
// @Ignore
  external PerformanceEntry.fakeConstructor$();

  /** @type {string} */
  String name;

  /** @type {string} */
  String entryType;

  /** @type {number} */
  num startTime;

  /** @type {number} */
  num duration;
}

/**
 * @constructor
 * @extends {PerformanceEntry}
 */
@JsName("PerformanceResourceTiming")
abstract class PerformanceResourceTiming extends PerformanceEntry {
// @Ignore
  external PerformanceResourceTiming.fakeConstructor$()
      : super.fakeConstructor$();

  /** @type {number} */
  num redirectStart;

  /** @type {number} */
  num redirectEnd;

  /** @type {number} */
  num fetchStart;

  /** @type {number} */
  num domainLookupStart;

  /** @type {number} */
  num domainLookupEnd;

  /** @type {number} */
  num connectStart;

  /** @type {number} */
  num connectEnd;

  /** @type {number} */
  num secureConnectionStart;

  /** @type {number} */
  num requestStart;

  /** @type {number} */
  num responseStart;

  /** @type {number} */
  num responseEnd;

  /** @type {string} */
  String initiatorType;
}

/** @constructor */
@JsName("PerformanceNavigation")
abstract class PerformanceNavigation {
// @Ignore
  external PerformanceNavigation.fakeConstructor$();

  /** @type {number} */
  num type;

  /** @type {number} */
  num redirectCount;
}

/** @constructor */
@JsName("PerformanceMemory")
abstract class PerformanceMemory {
// @Ignore
  external PerformanceMemory.fakeConstructor$();

  /** @type {number} */
  num jsHeapSizeLimit;

  /** @type {number} */
  num totalJSHeapSize;

  /** @type {number} */
  num usedJSHeapSize;
}

/** @constructor */
@JsName("Performance")
abstract class Performance {
// @Ignore
  external Performance.fakeConstructor$();

  /** @type {PerformanceTiming} */
  PerformanceTiming timing;

  /** @type {PerformanceNavigation} */
  PerformanceNavigation navigation;

  /**
   * Clears the buffer used to store the current list of
   * PerformanceResourceTiming resources.
   * @return {undefined}
   */
  void clearResourceTimings();

  /**
   * Clear out the buffer of performance timing events for webkit browsers.
   * @return {undefined}
   */
  void webkitClearResourceTimings();

  /**
   * Set the maximum number of PerformanceResourceTiming resources that may be
   * stored in the buffer.
   * @param {number} maxSize
   */
  setResourceTimingBufferSize(num maxSize);

  /**
   * @return {Array.<PerformanceEntry>} A copy of the PerformanceEntry list,
   *     in chronological order with respect to startTime.
   * @nosideeffects
   */
  List<PerformanceEntry> getEntries();

  /**
   * @param {string} entryType Only return {@code PerformanceEntry}s with this
   *     entryType.
   * @return {Array.<PerformanceEntry>} A copy of the PerformanceEntry list,
   *     in chronological order with respect to startTime.
   * @nosideeffects
   */
  List<PerformanceEntry> getEntriesByType(String entryType);

  /**
   * @param {string} name Only return {@code PerformanceEntry}s with this name.
   * @param {string=} opt_entryType Only return {@code PerformanceEntry}s with
   *     this entryType.
   * @return {Array.<PerformanceEntry>} PerformanceEntry list in chronological
   *     order with respect to startTime.
   * @nosideeffects
   */
  List<PerformanceEntry> getEntriesByName(String name, [String opt_entryType]);

  /** @type {PerformanceMemory} */
  PerformanceMemory memory;

  /**
   * @return {number}
   * @nosideeffects
   */
  num now();

  /**
   * @return {number}
   * @nosideeffects
   */
  num webkitNow();

  /**
   * @param {string} markName
   */
  mark(String markName);

  /**
   * @param {string=} opt_markName
   */
  clearMarks([String opt_markName]);

  /**
   * @param {string} measureName
   * @param {string=} opt_startMark
   * @param {string=} opt_endMark
   */
  measure(String measureName, [String opt_startMark, String opt_endMark]);

  /**
   * @param {string=} opt_measureName
   */
  clearMeasures([String opt_measureName]);
}
