/*
 * Copyright 2008 The Closure Compiler Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// This file has been automatically generated.  Please do not edit it manually.
/**
 * @fileoverview Definitions for W3C's range specification.
 *  This file depends on w3c_dom2.js.
 *  The whole file has been fully type annotated.
 *  Created from
 *   http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html
 *
 * FROM_EXTERN
 * @author stevey@google.com (Steve Yegge)
 */
@JsName()
library w3c_range;

import 'package:js/js.dart';
import 'package:func/func.dart';

import "w3c_css.dart" as w3c_css;

import "w3c_dom1.dart" as w3c_dom1;

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html#Level-2-Range-Interface
 */
@JsName("Range")
abstract class Range {
// @Ignore
  external Range.fakeConstructor$();

  /**
   * @type {Node}
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html#Level-2-Range-attr-startParent
   */
  w3c_dom1.Node startContainer;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html#Level-2-Range-attr-startOffset
   */
  num startOffset;

  /**
   * @type {Node}
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html#Level-2-Range-attr-endParent
   */
  w3c_dom1.Node endContainer;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html#Level-2-Range-attr-endOffset
   */
  num endOffset;

  /**
   * @type {boolean}
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html#Level-2-Range-attr-collapsed
   */
  bool collapsed;

  /**
   * @type {Node}
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html#Level-2-Range-attr-commonParent
   */
  w3c_dom1.Node commonAncestorContainer;

  /**
   * @param {Node} refNode
   * @param {number} offset
   * @return {undefined}
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html#Level2-Range-method-setStart
   */
  void setStart(w3c_dom1.Node refNode, num offset);

  /**
   * @param {Node} refNode
   * @param {number} offset
   * @return {undefined}
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html#Level2-Range-method-setEnd
   */
  void setEnd(w3c_dom1.Node refNode, num offset);

  /**
   * @param {Node} refNode
   * @return {undefined}
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html#Level2-Range-setStartBefore
   */
  void setStartBefore(w3c_dom1.Node refNode);

  /**
   * @param {Node} refNode
   * @return {undefined}
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html#Level2-Range-method-setStartAfter
   */
  void setStartAfter(w3c_dom1.Node refNode);

  /**
   * @param {Node} refNode
   * @return {undefined}
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html#Level2-Range-method-setEndBefore
   */
  void setEndBefore(w3c_dom1.Node refNode);

  /**
   * @param {Node} refNode
   * @return {undefined}
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html#Level2-Range-method-setEndAfter
   */
  void setEndAfter(w3c_dom1.Node refNode);

  /**
   * @param {boolean} toStart
   * @return {undefined}
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html#Level2-Range-method-collapse
   */
  void collapse(bool toStart);

  /**
   * @param {Node} refNode
   * @return {undefined}
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html#Level2-Range-method-selectNode
   */
  void selectNode(w3c_dom1.Node refNode);

  /**
   * @param {Node} refNode
   * @return {undefined}
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html#Level2-Range-method-selectNodeContents
   */
  void selectNodeContents(w3c_dom1.Node refNode);

  /**
   * @param {number} how
   * @param {Range} sourceRange
   * @return {number}
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html#Level2-Range-method-compareBoundaryPoints
   */
  num compareBoundaryPoints(num how, Range sourceRange);

  /**
   * @return {number}
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html#Level2-Range-method-deleteContents
   */
  num deleteContents();

  /**
   * @return {DocumentFragment}
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html#Level2-Range-method-extractContents
   */
  w3c_dom1.DocumentFragment extractContents();

  /**
   * @return {DocumentFragment}
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html#Level2-Range-method-cloneContents
   */
  w3c_dom1.DocumentFragment cloneContents();

  /**
   * @param {Node} newNode
   * @return {DocumentFragment}
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html#Level2-Range-method-insertNode
   */
  w3c_dom1.DocumentFragment insertNode(w3c_dom1.Node newNode);

  /**
   * @param {Node} newParent
   * @return {undefined}
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html#Level2-Range-method-surroundContents
   */
  void surroundContents(w3c_dom1.Node newParent);

  /**
   * @return {Range}
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html#Level2-Range-method-clone
   */
  Range cloneRange();

  /**
   * @return {undefined}
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html#Level2-Range-method-detach
   */
  void detach();

  /**
   * @return {!ClientRectList}
   * @see http://www.w3.org/TR/cssom-view/#dom-range-getclientrects
   */
  w3c_css.ClientRectList getClientRects();

  /**
   * @return {!ClientRect}
   * @see http://www.w3.org/TR/cssom-view/#dom-range-getboundingclientrect
   */
  w3c_css.ClientRect getBoundingClientRect();
}

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html#Level-2-DocumentRange-idl
 */
@JsName("DocumentRange")
abstract class DocumentRange {
// @Ignore
  external DocumentRange.fakeConstructor$();

  /**
   * @return {Range}
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html#Level2-DocumentRange-method-createRange
   */
  Range createRange();
}

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html#RangeException
 */
@JsName("RangeException")
abstract class RangeException {
// @Ignore
  external RangeException.fakeConstructor$();

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/ranges.html#RangeExceptionCode
   */
  num code;
}
