/*
 * Copyright 2012 The Closure Compiler Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// This file has been automatically generated.  Please do not edit it manually.
/**
 * @fileoverview Definitions for components of the WebRTC browser API.
 * @see http://dev.w3.org/2011/webrtc/editor/webrtc.html
 * @see http://tools.ietf.org/html/draft-ietf-rtcweb-jsep-01
 * @see http://www.w3.org/TR/mediacapture-streams/
 *
 * FROM_EXTERN
 * @author bemasc@google.com (Benjamin M. Schwartz)
 */
@JsName()
library w3c_rtc;

import 'package:js/js.dart';
import 'package:func/func.dart';

import "es6.dart" as es6;

import "fileapi.dart" as fileapi;

import "html5.dart" as html5;

import "w3c_event.dart" as w3c_event;

/**
 * @typedef {string}
 * @see {http://dev.w3.org/2011/webrtc/editor/getusermedia.html
 *     #idl-def-MediaStreamTrackState}
 * In WebIDL this is an enum with values 'live', 'mute', and 'ended',
 * but there is no mechanism in Closure for describing a specialization of
 * the string type.
 */
// Dart does not support aliases for primitive types. Skipping: MediaStreamTrackState

/**
 * @interface
 */
@JsName("SourceInfo")
abstract class SourceInfo {
// @Ignore
  external SourceInfo.fakeConstructor$();

  /** @const {string} */
  String kind;

  /** @const {string} */
  String id;

  /** @const {?string} */
  String label;

  /** @const {boolean} */
  bool facing;
}

/**
 * @interface
 * @see http://www.w3.org/TR/mediacapture-streams/#mediastreamtrack
 */
@JsName("MediaStreamTrack")
abstract class MediaStreamTrack {
// @Ignore
  external MediaStreamTrack.fakeConstructor$();

  /**
   * @param {!function(!Array.<!SourceInfo>)} callback
   */
  external static getSources(callback(List<SourceInfo> arg1));

  /**
   * @type {string}
   * @const
   */
  String kind;

  /**
   * @type {string}
   * @const
   */
  String id;

  /**
   * @type {string}
   * @const
   */
  String label;

  /**
   * @type {boolean}
   */
  bool enabled;

  /**
   * @type {MediaStreamTrackState}
   * Read only.
   */
  String readyState;

  /**
   * @type {?function(!Event)}
   */
  Func1<w3c_event.Event, dynamic> onmute;

  /**
   * @type {?function(!Event)}
   */
  Func1<w3c_event.Event, dynamic> onunmute;

  /**
   * @type {?function(!Event)}
   */
  Func1<w3c_event.Event, dynamic> onended;

  /**
   * @return {!MediaStreamTrack}
   */
  MediaStreamTrack clone();

  /** @return {void} */
  void stop();
}

/**
 * @constructor
 * @extends {Event}
 * @private
 * @see http://dev.w3.org/2011/webrtc/editor/
 * webrtc-20120720.html#mediastreamtrackevent
 * TODO(bemasc): Update this link to the final definition once one exists
 * (https://www.w3.org/Bugs/Public/show_bug.cgi?id=19568)
 */
@JsName("MediaStreamTrackEvent")
abstract class MediaStreamTrackEvent extends w3c_event.Event {
// @Ignore
  external MediaStreamTrackEvent.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {!MediaStreamTrack}
   * @const
   */
  MediaStreamTrack track;
}

/**
 * @param {!MediaStream|!Array.<!MediaStreamTrack>=} streamOrTracks
 * @constructor
 * @implements {EventTarget}
 * @see http://www.w3.org/TR/mediacapture-streams/#mediastream
 */
@JsName("MediaStream")
abstract class MediaStream implements w3c_event.EventTarget {
// @Ignore
  external MediaStream.fakeConstructor$();

  /**
   * @param {!MediaStream|!Array.<!MediaStreamTrack>=} streamOrTracks
   * @constructor
   * @implements {EventTarget}
   * @see http://www.w3.org/TR/mediacapture-streams/#mediastream
   */
  external factory MediaStream(
      [streamOrTracks /*MediaStream|List<MediaStreamTrack>|undefined*/]);

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  addEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  removeEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /** @override */
  bool dispatchEvent(w3c_event.Event evt);

  /**
   * TODO(bemasc): Remove this property.
   * @deprecated
   * @type {string}
   * @const
   */
  String label;

  /**
   * @type {string}
   * @const
   */
  String id;

  /**
   * @return {!Array.<!MediaStreamTrack>}
   */
  List<MediaStreamTrack> getAudioTracks();

  /**
   * @return {!Array.<!MediaStreamTrack>}
   */
  List<MediaStreamTrack> getVideoTracks();

  /**
   * @param {string} trackId
   * @return {MediaStreamTrack}
   */
  MediaStreamTrack getTrackById(String trackId);

  /**
   * @param {!MediaStreamTrack} track
   */
  addTrack(MediaStreamTrack track);

  /**
   * @param {!MediaStreamTrack} track
   */
  removeTrack(MediaStreamTrack track);

  /**
   * @type {boolean}
   */
  bool ended;

  /**
   * @type {?function(!Event)}
   */
  Func1<w3c_event.Event, dynamic> onended;

  /**
   * @type {?function(!MediaStreamTrackEvent)}
   */
  Func1<MediaStreamTrackEvent, dynamic> onaddtrack;

  /**
   * @type {?function(!MediaStreamTrackEvent)}
   */
  Func1<MediaStreamTrackEvent, dynamic> onremovetrack;

  /**
   * @deprecated
   * TODO(bemasc): Remove this method once browsers have updated to
   * MediaStreamTrack.stop().
   */
  stop();
}
/**
 * @type {function(new: MediaStream,
 *                 (!MediaStream|!Array.<!MediaStreamTrack>)=)}
 */
external webkitMediaStream(
    [arg1 /*MediaStream|List<MediaStreamTrack>|undefined*/]);

/**
 * This interface defines the available constraint attributes.  These are the
 * attributes defined in
 * {@see http://tools.ietf.org/html/draft-alvestrand-constraints-resolution-01}.
 * Note that although that draft refers to "Media Constraints", the W3C uses
 * the terms "Media[Stream|Track]Constraints" for this type, and
 * defines a different type (for RTCPeerConnection) called "MediaConstraints".
 *
 * This interface type is not part of any standard, so it is marked as private.
 * It is defined here in order to reserve the property names, which would
 * otherwise be rewritten when the compiler processes an object literal.
 * Several subsequent interfaces are defined in the same pattern.
 *
 * Note that although this list includes all the properties supported by
 * libjingle (and hence by Chromium), browsers are permitted to offer other
 * properties as well ({
 * @see http://tools.ietf.org/html/draft-burnett-rtcweb-constraints-registry-02
 * }), and browsers are expected to silently ignore unknown properties.  This
 * creates the potential for a very confusing situation in which properties
 * not listed here are renamed by the compiler and then ignored by the browser.
 *
 * @interface
 * @private
 */
@JsName("MediaTrackConstraintSetInterface_")
abstract class MediaTrackConstraintSetInterface_ {
// @Ignore
  external MediaTrackConstraintSetInterface_.fakeConstructor$();

  /**
   * @type {?number}
   */
  num minWidth;

  /**
   * @type {?number}
   */
  num maxWidth;

  /**
   * @type {?number}
   */
  num minHeight;

  /**
   * @type {?number}
   */
  num maxHeight;

  /**
   * @type {?number}
   */
  num minAspectRatio;

  /**
   * @type {?number}
   */
  num maxAspectRatio;

  /**
   * Due to a typo, this is called "minFramerate" in the -01 draft.
   * @type {?number}
   */
  num minFrameRate;

  /**
   * @type {?number}
   */
  num maxFrameRate;
}
/**
 * This type and two more below are defined as unions with Object because they
 * are normally used as record types by constructing an Object literal, but all
 * of their properties are optional.
 * @typedef {Object|MediaTrackConstraintSetInterface_}
 */
// Dart does not support union types. Skipping: MediaTrackConstraintSet

/**
 * @interface
 * @private
 */
@JsName("MediaTrackConstraintsInterface_")
abstract class MediaTrackConstraintsInterface_ {
// @Ignore
  external MediaTrackConstraintsInterface_.fakeConstructor$();

  /**
   * @type {?MediaTrackConstraintSet}
   */
  Object mandatory;

  /**
   * @type {?Array.<!MediaTrackConstraintSet>}
   */
  List<Object> optional;
}
/**
 * @typedef {Object|MediaTrackConstraintsInterface_}
 */
// Dart does not support union types. Skipping: MediaTrackConstraints

/**
 * @interface
 * @private
 */
@JsName("MediaStreamConstraintsInterface_")
abstract class MediaStreamConstraintsInterface_ {
// @Ignore
  external MediaStreamConstraintsInterface_.fakeConstructor$();

  /**
   * @type {boolean|MediaTrackConstraints}
   */
  dynamic /*bool|Object|null*/ audio;

  /**
   * @type {boolean|MediaTrackConstraints}
   */
  dynamic /*bool|Object|null*/ video;
}
/**
 * @typedef {Object|MediaStreamConstraintsInterface_}
 */
// Dart does not support union types. Skipping: MediaStreamConstraints

/**
 * @see {http://dev.w3.org/2011/webrtc/editor/getusermedia.html#
 *     navigatorusermediaerror-and-navigatorusermediaerrorcallback}
 * @interface
 */
@JsName("NavigatorUserMediaError")
abstract class NavigatorUserMediaError {
// @Ignore
  external NavigatorUserMediaError.fakeConstructor$();

  /**
   * @type {number}
   * @deprecated Removed from the standard and some browsers.
   * @const
   */
  num PERMISSION_DENIED;

  /**
   * @type {number}
   * @deprecated Removed from the standard and some browsers.
   * Read only.
   */
  num code;

  /**
   * @type {string}
   * Read only.
   */
  String name;

  /**
   * @type {?string}
   * Read only.
   */
  String message;

  /**
   * @type {?string}
   * Read only.
   */
  String constraintName;
}

/**
 * @param {string} type
 * @param {!Object} eventInitDict
 * @constructor
 */
@JsName("MediaStreamEvent")
abstract class MediaStreamEvent {
// @Ignore
  external MediaStreamEvent.fakeConstructor$();

  /**
   * @param {string} type
   * @param {!Object} eventInitDict
   * @constructor
   */
  external factory MediaStreamEvent(String type, Object eventInitDict);

  /**
   * @type {?MediaStream}
   * @const
   */
  MediaStream stream;
}
/**
 * @typedef {string}
 * @see http://www.w3.org/TR/webrtc/#rtcsdptype
 * In WebIDL this is an enum with values 'offer', 'pranswer', and 'answer',
 * but there is no mechanism in Closure for describing a specialization of
 * the string type.
 */
// Dart does not support aliases for primitive types. Skipping: RTCSdpType

/**
 * @param {!Object=} descriptionInitDict The RTCSessionDescriptionInit
 * dictionary.  This optional argument may have type
 * {type:RTCSdpType, sdp:string}, but neither of these keys are required to be
 * present, and other keys are ignored, so the closest Closure type is Object.
 * @constructor
 * @see http://dev.w3.org/2011/webrtc/editor/webrtc.html#rtcsessiondescription-class
 */
@JsName("RTCSessionDescription")
abstract class RTCSessionDescription {
// @Ignore
  external RTCSessionDescription.fakeConstructor$();

  /**
   * @param {!Object=} descriptionInitDict The RTCSessionDescriptionInit
   * dictionary.  This optional argument may have type
   * {type:RTCSdpType, sdp:string}, but neither of these keys are required to be
   * present, and other keys are ignored, so the closest Closure type is Object.
   * @constructor
   * @see http://dev.w3.org/2011/webrtc/editor/webrtc.html#rtcsessiondescription-class
   */
  external factory RTCSessionDescription([Object descriptionInitDict]);

  /**
   * @type {?RTCSdpType}
   * @see http://www.w3.org/TR/webrtc/#widl-RTCSessionDescription-type
   */
  String type;

  /**
   * @type {?string}
   * @see http://www.w3.org/TR/webrtc/#widl-RTCSessionDescription-sdp
   */
  String sdp;
}

/**
 * TODO(bemasc): Remove this definition once it is removed from the browser.
 * @param {string} label The label index (audio/video/data -> 0,1,2)
 * @param {string} sdp The ICE candidate in SDP text form
 * @constructor
 */
@JsName("IceCandidate")
abstract class IceCandidate {
// @Ignore
  external IceCandidate.fakeConstructor$();

  /**
   * TODO(bemasc): Remove this definition once it is removed from the browser.
   * @param {string} label The label index (audio/video/data -> 0,1,2)
   * @param {string} sdp The ICE candidate in SDP text form
   * @constructor
   */
  external factory IceCandidate(String label, String sdp);

  /**
   * @return {string}
   */
  String toSdp();

  /**
   * @type {?string}
   */
  String label;
}

/**
 * @param {!Object=} candidateInitDict  The RTCIceCandidateInit dictionary.
 * This optional argument may have type
 * {candidate: string, sdpMid: string, sdpMLineIndex:number}, but none of
 * these keys are required to be present, and other keys are ignored, so the
 * closest Closure type is Object.
 * @constructor
 * @see http://www.w3.org/TR/webrtc/#rtcicecandidate-type
 */
@JsName("RTCIceCandidate")
abstract class RTCIceCandidate {
// @Ignore
  external RTCIceCandidate.fakeConstructor$();

  /**
   * @param {!Object=} candidateInitDict  The RTCIceCandidateInit dictionary.
   * This optional argument may have type
   * {candidate: string, sdpMid: string, sdpMLineIndex:number}, but none of
   * these keys are required to be present, and other keys are ignored, so the
   * closest Closure type is Object.
   * @constructor
   * @see http://www.w3.org/TR/webrtc/#rtcicecandidate-type
   */
  external factory RTCIceCandidate([Object candidateInitDict]);

  /**
   * @type {?string}
   */
  String candidate;

  /**
   * @type {?string}
   */
  String sdpMid;

  /**
   * @type {?number}
   */
  num sdpMLineIndex;
}
/**
 * @typedef {{url: string}}
 * @private
 * @see http://www.w3.org/TR/webrtc/#rtciceserver-type
 * This dictionary type also has an optional key {credential: ?string}.
 */
@JsName()
abstract class RTCIceServerRecord_ {
  String url;
}

/**
 * @interface
 * @private
 */
@JsName("RTCIceServerInterface_")
abstract class RTCIceServerInterface_ {
// @Ignore
  external RTCIceServerInterface_.fakeConstructor$();

  /**
   * @type {string}
   */
  String url;

  /**
   * @type {?string}
   */
  String credential;
}
/**
 * This type, and several below it, are constructed as unions between records
 *
 * @typedef {RTCIceServerRecord_|RTCIceServerInterface_}
 * @private
 */
// Dart does not support union types. Skipping: RTCIceServer
/**
 * @typedef {{iceServers: !Array.<!RTCIceServer>}}
 * @private
 */
@JsName()
abstract class RTCConfigurationRecord_ {
  List<RTCIceServerRecord_> iceServers;
}

/**
 * @interface
 * @private
 */
@JsName("RTCConfigurationInterface_")
abstract class RTCConfigurationInterface_ {
// @Ignore
  external RTCConfigurationInterface_.fakeConstructor$();

  /**
   * @type {!Array.<!RTCIceServer>}
   */
  List<RTCIceServerRecord_> iceServers;
}
/**
 * @typedef {RTCConfigurationRecord_|RTCConfigurationInterface_}
 */
// Dart does not support union types. Skipping: RTCConfiguration
/**
 * @typedef {function(!RTCSessionDescription)}
 */
typedef RTCSessionDescriptionCallback(RTCSessionDescription arg1);
/**
 * @typedef {function(string)}
 */
typedef RTCPeerConnectionErrorCallback(String arg1);
/**
 * @typedef {function()}
 */
typedef RTCVoidCallback();
/**
 * @typedef {string}
 */
// Dart does not support aliases for primitive types. Skipping: RTCSignalingState
/**
 * @typedef {string}
 */
// Dart does not support aliases for primitive types. Skipping: RTCIceConnectionState
/**
 * @typedef {string}
 */
// Dart does not support aliases for primitive types. Skipping: RTCIceGatheringState

/**
 * @param {string} type
 * @param {!Object} eventInitDict
 * @constructor
 */
@JsName("RTCPeerConnectionIceEvent")
abstract class RTCPeerConnectionIceEvent {
// @Ignore
  external RTCPeerConnectionIceEvent.fakeConstructor$();

  /**
   * @param {string} type
   * @param {!Object} eventInitDict
   * @constructor
   */
  external factory RTCPeerConnectionIceEvent(String type, Object eventInitDict);

  /**
   * @type {RTCIceCandidate}
   * @const
   */
  RTCIceCandidate candidate;
}

/**
 * @interface
 */
@JsName("RTCStatsReport")
abstract class RTCStatsReport {
// @Ignore
  external RTCStatsReport.fakeConstructor$();

  /**
   * @type {Date}
   * @const
   */
  DateTime timestamp;

  /**
   * @return {!Array.<!string>}
   */
  List<String> names();

  /**
   * @param {string} name
   * @return {string}
   */
  String stat(String name);

  /**
   * @deprecated
   * @type {RTCStatsReport}
   * @const
   */
  RTCStatsReport local;

  /**
   * @deprecated
   * @type {RTCStatsReport}
   * @const
   */
  RTCStatsReport remote;

  /**
   * @type {string}
   * @const
   */
  String type;

  /**
   * @type {string}
   * @const
   */
  String id;
}
/**
 * TODO(bemasc): Remove this type once it is no longer in use.  It has already
 * been removed from the specification.
 * @typedef {RTCStatsReport}
 * @deprecated
 */
// Dart does not support union types. Skipping: RTCStatsElement

/**
 * @interface
 */
@JsName("RTCStatsResponse")
abstract class RTCStatsResponse {
// @Ignore
  external RTCStatsResponse.fakeConstructor$();

  /**
   * @return {!Array.<!RTCStatsReport>}
   */
  List<RTCStatsReport> result();
}
/**
 * @typedef {function(!RTCStatsResponse, MediaStreamTrack=)}
 */
typedef RTCStatsCallback(RTCStatsResponse arg1, [MediaStreamTrack arg2]);

/**
 * This type is not yet standardized, so the properties here only represent
 * the current capabilities of libjingle (and hence Chromium).
 * TODO(bemasc): Add a link to the relevant standard once MediaConstraint has a
 * standard definition.
 *
 * @interface
 * @private
 */
@JsName("MediaConstraintSetInterface_")
abstract class MediaConstraintSetInterface_ {
// @Ignore
  external MediaConstraintSetInterface_.fakeConstructor$();

  /**
   * @type {?boolean}
   */
  bool OfferToReceiveAudio;

  /**
   * @type {?boolean}
   */
  bool OfferToReceiveVideo;

  /**
   * @type {?boolean}
   */
  bool DtlsSrtpKeyAgreement;

  /**
   * @type {?boolean}
   */
  bool RtpDataChannels;
}
/**
 * TODO(bemasc): Make this type public once it is defined in a standard.
 *
 * @typedef {Object|MediaConstraintSetInterface_}
 * @private
 */
// Dart does not support union types. Skipping: MediaConstraintSet_

/**
 * @interface
 * @private
 */
@JsName("MediaConstraintsInterface_")
abstract class MediaConstraintsInterface_ {
// @Ignore
  external MediaConstraintsInterface_.fakeConstructor$();

  /**
   * @type {?MediaConstraintSet_}
   */
  Object mandatory;

  /**
   * @type {?Array.<!MediaConstraintSet_>}
   */
  List<Object> optional;
}
/**
 * This type is used extensively in
 * {@see http://dev.w3.org/2011/webrtc/editor/webrtc.html} but is not yet
 * defined.
 *
 * @typedef {Object|MediaConstraintsInterface_}
 */
// Dart does not support union types. Skipping: MediaConstraints

/**
 * @interface
 */
@JsName("RTCDataChannel")
abstract class RTCDataChannel {
// @Ignore
  external RTCDataChannel.fakeConstructor$();

  /**
   * @type {string}
   * @const
   */
  String label;

  /**
   * @type {boolean}
   * @const
   */
  bool reliable;

  /**
   * An enumerated string type (RTCDataChannelState) with values:
   * "connecting", "open", "closing", and "closed".
   * @type {string}
   * Read only.
   */
  String readyState;

  /**
   * @type {number}
   * Read only.
   */
  num bufferedAmount;

  /**
   * @type {?function(!Event)}
   */
  Func1<w3c_event.Event, dynamic> onopen;

  /**
   * @type {?function(!Event)}
   */
  Func1<w3c_event.Event, dynamic> onerror;

  /**
   * @type {?function(!Event)}
   */
  Func1<w3c_event.Event, dynamic> onclose;

  close();

  /**
   * @type {?function(!MessageEvent.<*>)}
   */
  Func1<html5.MessageEvent<dynamic>, dynamic> onmessage;

  /**
   * @type {string}
   */
  String binaryType;

  /**
   * @param {string|!Blob|!ArrayBuffer|!ArrayBufferView} data
   */
  send(data /*String|fileapi.Blob|es6.ArrayBuffer|es6.ArrayBufferView*/);
}

/**
 * @constructor
 * @extends {Event}
 * @private
 */
@JsName("RTCDataChannelEvent")
abstract class RTCDataChannelEvent extends w3c_event.Event {
// @Ignore
  external RTCDataChannelEvent.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @type {!RTCDataChannel}
   * Read only.
   */
  RTCDataChannel channel;
}
/**
 * @typedef {{reliable: boolean}}
 */
@JsName()
abstract class RTCDataChannelInitRecord_ {
  bool reliable;
}

/**
 * @interface
 * @private
 */
@JsName("RTCDataChannelInitInterface_")
abstract class RTCDataChannelInitInterface_ {
// @Ignore
  external RTCDataChannelInitInterface_.fakeConstructor$();

  /**
   * @type {boolean}
   */
  bool reliable;
}
/**
 * @typedef {RTCDataChannelInitInterface_|RTCDataChannelInitRecord_}
 */
// Dart does not support union types. Skipping: RTCDataChannelInit

/**
 * @param {RTCConfiguration} configuration
 * @param {!MediaConstraints=} constraints
 * @constructor
 * @implements {EventTarget}
 */
@JsName("RTCPeerConnection")
abstract class RTCPeerConnection implements w3c_event.EventTarget {
// @Ignore
  external RTCPeerConnection.fakeConstructor$();

  /**
   * @param {RTCConfiguration} configuration
   * @param {!MediaConstraints=} constraints
   * @constructor
   * @implements {EventTarget}
   */
  external factory RTCPeerConnection(RTCConfigurationRecord_ configuration,
      [Object constraints]);

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  addEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  removeEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /** @override */
  bool dispatchEvent(w3c_event.Event evt);

  /**
   * @param {!RTCSessionDescriptionCallback} successCallback
   * @param {!RTCPeerConnectionErrorCallback=} failureCallback
   * @param {!MediaConstraints=} constraints
   */
  createOffer(successCallback(RTCSessionDescription arg1),
      [failureCallback(String arg1), Object constraints]);

  /**
   * @param {RTCSessionDescriptionCallback} successCallback
   * @param {?RTCPeerConnectionErrorCallback=} failureCallback
   * @param {!MediaConstraints=} constraints
   */
  createAnswer(successCallback(RTCSessionDescription arg1),
      [failureCallback(String arg1), Object constraints]);

  /**
   * @param {!RTCSessionDescription} description
   * @param {!RTCVoidCallback=} successCallback
   * @param {!RTCPeerConnectionErrorCallback=} failureCallback
   */
  setLocalDescription(RTCSessionDescription description,
      [successCallback(), failureCallback(String arg1)]);

  /**
   * @param {!RTCSessionDescription} description
   * @param {!RTCVoidCallback=} successCallback
   * @param {!RTCPeerConnectionErrorCallback=} failureCallback
   */
  setRemoteDescription(RTCSessionDescription description,
      [successCallback(), failureCallback(String arg1)]);

  /**
   * @type {?RTCSessionDescription}
   * Read only.
   */
  RTCSessionDescription localDescription;

  /**
   * @type {?RTCSessionDescription}
   * Read only.
   */
  RTCSessionDescription remoteDescription;

  /**
   * @type {RTCSignalingState}
   * Read only.
   */
  String signalingState;

  /**
   * @param {?RTCConfiguration=} configuration
   * @param {?MediaConstraints=} constraints
   */
  updateIce([RTCConfigurationRecord_ configuration, Object constraints]);

  /**
   * @param {!RTCIceCandidate} candidate
   */
  addIceCandidate(RTCIceCandidate candidate);

  /**
   * @type {!RTCIceGatheringState}
   * Read only.
   */
  String iceGatheringState;

  /**
   * @type {!RTCIceConnectionState}
   * Read only.
   */
  String iceConnectionState;

  /**
   * @return {!Array.<!MediaStream>}
   */
  List<MediaStream> getLocalStreams();

  /**
   * @return {!Array.<!MediaStream>}
   */
  List<MediaStream> getRemoteStreams();

  /**
   * @param {string} streamId
   * @return {MediaStream}
   */
  MediaStream getStreamById(String streamId);

  /**
   * @param {?string} label
   * @param {RTCDataChannelInit=} dataChannelDict
   * @return {!RTCDataChannel}
   */
  RTCDataChannel createDataChannel(String label,
      [RTCDataChannelInitRecord_ dataChannelDict]);

  /**
   * @param {!MediaStream} stream
   * @param {!MediaConstraints=} constraints
   */
  addStream(MediaStream stream, [Object constraints]);

  /**
   * @param {!MediaStream} stream
   */
  removeStream(MediaStream stream);

  /**
   * @param {!RTCStatsCallback} successCallback
   * @param {MediaStreamTrack=} selector
   */
  getStats(successCallback(RTCStatsResponse arg1, [MediaStreamTrack arg2]),
      [MediaStreamTrack selector]);

  close();

  /**
   * @type {?function(!Event)}
   */
  Func1<w3c_event.Event, dynamic> onnegotiationneeded;

  /**
   * @type {?function(!RTCPeerConnectionIceEvent)}
   */
  Func1<RTCPeerConnectionIceEvent, dynamic> onicecandidate;

  /**
   * @type {?function(!Event)}
   */
  Func1<w3c_event.Event, dynamic> onsignalingstatechange;

  /**
   * @type {?function(!MediaStreamEvent)}
   */
  Func1<MediaStreamEvent, dynamic> onaddstream;

  /**
   * @type {?function(!MediaStreamEvent)}
   */
  Func1<MediaStreamEvent, dynamic> onremovestream;

  /**
   * @type {?function(!Event)}
   */
  Func1<w3c_event.Event, dynamic> oniceconnectionstatechange;

  /**
   * @type {?function(!RTCDataChannelEvent)}
   */
  Func1<RTCDataChannelEvent, dynamic> ondatachannel;
}
/**
 * @type {function(new: RTCPeerConnection, RTCConfiguration,
 *     !MediaConstraints=)}
 */
external webkitRTCPeerConnection(RTCConfigurationRecord_ arg1, [Object arg2]);
