/*
 * Copyright 2014 The Closure Compiler Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
// This file has been automatically generated.  Please do not edit it manually.
/**
 * @fileoverview Externs for service worker.
 *
 * @see http://www.w3.org/TR/service-workers/
 * FROM_EXTERN
 */
@JsName()
library w3c_serviceworker;

import 'package:js/js.dart';
import 'package:func/func.dart';

import "es6.dart" as es6;

import "fetchapi.dart" as fetchapi;

import "fileapi.dart" as fileapi;

import "html5.dart" as html5;

import "w3c_event.dart" as w3c_event;

import "webkit_dom.dart" as webkit_dom;

import "webkit_notifications.dart" as webkit_notifications;

/**
 * @see http://www.w3.org/TR/service-workers/#service-worker-interface
 * @constructor
 * @extends {Worker}
 */
@JsName("ServiceWorker")
abstract class ServiceWorker extends html5.Worker {
// @Ignore
  external ServiceWorker.fakeConstructor$() : super.fakeConstructor$();

  /** @type {string} */
  String scriptURL;

  /** @type {ServiceWorkerState} */
  String state;

  /** @type {?function(!Event)} */
  Func1<w3c_event.Event, dynamic> onstatechange;
}
/** @enum {string} */
@JsName("ServiceWorkerState")
abstract class ServiceWorkerState {
  external static String get INSTALLED;
  external static String get INSTALLING;
  external static String get ACTIVATED;
  external static String get ACTIVATING;
  external static String get REDUNDANT;
}

/**
 * @see https://w3c.github.io/push-api/
 * @constructor
 */
@JsName("PushSubscription")
abstract class PushSubscription {
// @Ignore
  external PushSubscription.fakeConstructor$();

  /** @type {string} */
  String endpoint;

  /**
   * Please note there is an intent to deprecate this field in Chrome 43 or 44.
   * See https://www.chromestatus.com/feature/5283829761703936.
   * @type {string}
   */
  String subscriptionId;

  /** @return {!Promise.<boolean>} */
  es6.Promise<bool> unsubscribe();
}

/**
 * @see https://w3c.github.io/push-api/
 * @constructor
 */
@JsName("PushManager")
abstract class PushManager {
// @Ignore
  external PushManager.fakeConstructor$();

  /** @return {!Promise<PushSubscription>} */
  es6.Promise<PushSubscription> subscribe();

  /** @return {!Promise<PushSubscription>} */
  es6.Promise<PushSubscription> getSubscription();
}

/**
 * @see http://www.w3.org/TR/push-api/#idl-def-PushMessageData
 * @constructor
 */
@JsName("PushMessageData")
abstract class PushMessageData {
// @Ignore
  external PushMessageData.fakeConstructor$();

  /** @return {!ArrayBuffer} */
  es6.ArrayBuffer arrayBuffer();

  /** @return {!Blob} */
  fileapi.Blob blob();

  /** @return {*} */
  json();

  /** @return {string} */
  String text();
}

/**
 * @see http://www.w3.org/TR/push-api/#idl-def-PushEvent
 * @constructor
 * @param {string} type
 * @param {!ExtendableEventInit=} opt_eventInitDict
 * @extends {ExtendableEvent}
 */
@JsName("PushEvent")
abstract class PushEvent extends ExtendableEvent {
// @Ignore
  external PushEvent.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @see http://www.w3.org/TR/push-api/#idl-def-PushEvent
   * @constructor
   * @param {string} type
   * @param {!ExtendableEventInit=} opt_eventInitDict
   * @extends {ExtendableEvent}
   */
  external factory PushEvent(String type,
      [w3c_event.EventInit opt_eventInitDict]);

  /** @type {?PushMessageData} */
  PushMessageData data;
}

/**
 * @see http://www.w3.org/TR/service-workers/#service-worker-registration-interface
 * @interface
 * @extends {EventTarget}
 */
@JsName("ServiceWorkerRegistration")
abstract class ServiceWorkerRegistration {
// @Ignore
  external ServiceWorkerRegistration.fakeConstructor$();

  /** @type {ServiceWorker} */
  ServiceWorker installing;

  /** @type {ServiceWorker} */
  ServiceWorker waiting;

  /** @type {ServiceWorker} */
  ServiceWorker active;

  /** @type {string} */
  String scope;

  /** @return {!Promise<boolean>} */
  es6.Promise<bool> unregister();

  /** @type {?function(!Event)} */
  Func1<w3c_event.Event, dynamic> onupdatefound;

  /** @type {function()} */
  update();

  /**
   * @see https://w3c.github.io/push-api/
   * @type {!PushManager}
   */
  PushManager pushManager;

  /**
   * @see https://notifications.spec.whatwg.org/#service-worker-api
   * @param {string} title
   * @param {NotificationOptions=} opt_options
   * @return {!Promise<void>}
   */
  es6.Promise<dynamic> showNotification(String title,
      [webkit_notifications.NotificationOptions opt_options]);
}

/**
 * @see http://www.w3.org/TR/service-workers/#service-worker-container-interface
 * @interface
 * @extends {EventTarget}
 */
@JsName("ServiceWorkerContainer")
abstract class ServiceWorkerContainer {
// @Ignore
  external ServiceWorkerContainer.fakeConstructor$();

  /** @type {?ServiceWorker} */
  ServiceWorker controller;

  /** @type {!Promise<!ServiceWorkerRegistration>} */
  es6.Promise<ServiceWorkerRegistration> ready;

  /**
   * @param {string} scriptURL
   * @param {RegistrationOptions=} opt_options
   * @return {!Promise<!ServiceWorkerRegistration>}
   */
  es6.Promise<ServiceWorkerRegistration> register(String scriptURL,
      [RegistrationOptions opt_options]);

  /**
   * @param {string=} opt_documentURL
   * @return {!Promise<!ServiceWorkerRegistration|undefined>}
   */
  es6.Promise<ServiceWorkerRegistration> getRegistration(
      [String opt_documentURL]);

  /**
   * @return {!Promise<Array<!ServiceWorkerRegistration>>}
   */
  es6.Promise<List<ServiceWorkerRegistration>> getRegistrations();

  /** @type {?function(!Event)} */
  Func1<w3c_event.Event, dynamic> oncontrollerchange;

  /** @type {?function(!ErrorEvent)} */
  Func1<html5.ErrorEvent, dynamic> onerror;
}
/**
 * @typedef {{scope: string}}
 */
@JsName()
abstract class RegistrationOptions {
  String scope;
}

/**
 * @see http://www.w3.org/TR/service-workers/#service-worker-global-scope-interface
 * @interface
 * @extends {WorkerGlobalScope}
 */
@JsName("ServiceWorkerGlobalScope")
abstract class ServiceWorkerGlobalScope {
// @Ignore
  external ServiceWorkerGlobalScope.fakeConstructor$();

  /** @type {!Cache} */
  Cache scriptCache;

  /** @type {!CacheStorage} */
  CacheStorage caches;

  /** @type {!ServiceWorkerClients} */
  ServiceWorkerClients clients;

  /** @type {string} */
  String scope;

  /** @type {!ServiceWorkerRegistration} */
  ServiceWorkerRegistration registration;

  /** @return {!Promise<void>} */
  es6.Promise<dynamic> skipWaiting();

  /** @type {!Console} */
  webkit_dom.Console console;

  /** @type {?function(!InstallEvent)} */
  Func1<InstallEvent, dynamic> oninstall;

  /** @type {?function(!ExtendableEvent)} */
  Func1<ExtendableEvent, dynamic> onactivate;

  /** @type {?function(!FetchEvent)} */
  Func1<FetchEvent, dynamic> onfetch;

  /**
   * TODO(user): This handler should get a custom event in the future.
   * @type {?function(!Event)}
   */
  Func1<w3c_event.Event, dynamic> onbeforeevicted;

  /**
   * TODO(user): This handler should get a custom event in the future.
   * @type {?function(!Event)}
   */
  Func1<w3c_event.Event, dynamic> onevicted;

  /** @type {?function(!MessageEvent)} */
  Func1<html5.MessageEvent, dynamic> onmessage;
}

/**
 * @see http://www.w3.org/TR/service-workers/#service-worker-client-interface
 * @constructor
 */
@JsName("ServiceWorkerClient")
abstract class ServiceWorkerClient {
// @Ignore
  external ServiceWorkerClient.fakeConstructor$();

  /** @type {!Promise<void>} */
  es6.Promise<dynamic> ready;

  /** @type {boolean} */
  bool hidden;

  /** @type {boolean} */
  bool focused;

  /** @type {VisibilityState} */
  String visibilityState;

  /** @type {string} */
  String url;

  /**
   * // TODO(user): Possibly replace the type with enum ContextFrameType once
   * the enum is defined.
   * @type {string}
   */
  String frameType;

  /**
   * @param {*} message
   * @param {(!Array<!Transferable>|undefined)=} opt_transfer
   */
  postMessage(message, [List<es6.Transferable> opt_transfer]);

  /** @return {!Promise} */
  es6.Promise focus();
}

/**
 * @see http://www.w3.org/TR/service-workers/#service-worker-clients-interface
 * @interface
 */
@JsName("ServiceWorkerClients")
abstract class ServiceWorkerClients {
// @Ignore
  external ServiceWorkerClients.fakeConstructor$();

  /**
   * Deprecated in Chrome M43+, use matchAll instead. Reference:
   * https://github.com/slightlyoff/ServiceWorker/issues/610.
   * TODO(user): Remove when getAll is fully deprecated.
   * @param {ServiceWorkerClientQueryOptions=} opt_options
   * @return {!Promise<!Array<!ServiceWorkerClient>>}
   */
  es6.Promise<List<ServiceWorkerClient>> getAll(
      [ServiceWorkerClientQueryOptions opt_options]);

  /**
   * @param {ServiceWorkerClientQueryOptions=} opt_options
   * @return {!Promise<!Array<!ServiceWorkerClient>>}
   */
  es6.Promise<List<ServiceWorkerClient>> matchAll(
      [ServiceWorkerClientQueryOptions opt_options]);

  /**
   * @return {!Promise<void>}
   */
  es6.Promise<dynamic> claim();

  /**
   * @param {string} url
   * @return {!Promise<!ServiceWorkerClient>}
   */
  es6.Promise<ServiceWorkerClient> openWindow(String url);
}
/** @typedef {{includeUncontrolled: (boolean|undefined)}} */
@JsName()
abstract class ServiceWorkerClientQueryOptions {
  bool includeUncontrolled;
}

/**
 * @see http://www.w3.org/TR/service-workers/#cache-interface
 * @interface
 */
@JsName("Cache")
abstract class Cache {
// @Ignore
  external Cache.fakeConstructor$();

  /**
   * @param {!RequestInfo} request
   * @param {CacheQueryOptions=} opt_options
   * @return {!Promise<!Response|undefined>}
   */
  es6.Promise<fetchapi.Response> match(request /*fetchapi.Request|String*/,
      [CacheQueryOptions opt_options]);

  /**
   * @param {RequestInfo=} opt_request
   * @param {CacheQueryOptions=} opt_options
   * @return {!Promise<!Array<!Response>>}
   */
  es6.Promise<List<fetchapi.Response>> matchAll(
      [opt_request /*fetchapi.Request|String|undefined*/,
      CacheQueryOptions opt_options]);

  /**
   * @param {!RequestInfo} request
   * @return {!Promise<void>}
   */
  es6.Promise<dynamic> add(request /*fetchapi.Request|String*/);

  /**
   * @param {!Array<!RequestInfo>} requests
   * @return {!Promise<void>}
   */
  es6.Promise<dynamic> addAll(
      List<dynamic /*fetchapi.Request|String*/ > requests);

  /**
   * @param {!RequestInfo} request
   * @param {!Response} response
   * @return {!Promise<void>}
   */
  es6.Promise<dynamic> put(
      request /*fetchapi.Request|String*/, fetchapi.Response response);

  /**
   * @param {!RequestInfo} request
   * @param {CacheQueryOptions=} opt_options
   * @return {!Promise<boolean>}
   */
  es6.Promise<bool> delete(request /*fetchapi.Request|String*/,
      [CacheQueryOptions opt_options]);

  /**
   * @param {RequestInfo=} opt_request
   * @param {CacheQueryOptions=} opt_options
   * @return {!Promise<!Array<!Response>>}
   */
  es6.Promise<List<fetchapi.Response>> keys(
      [opt_request /*fetchapi.Request|String|undefined*/,
      CacheQueryOptions opt_options]);
}
/**
 * @typedef {{
 *   ignoreSearch: (boolean|undefined),
 *   ignoreMethod: (boolean|undefined),
 *   ignoreVary: (boolean|undefined),
 *   prefixMatch: (boolean|undefined),
 *   cacheName: (string|undefined)
 * }}
 */
@JsName()
abstract class CacheQueryOptions {
  String cacheName;
  bool ignoreMethod;
  bool ignoreSearch;
  bool ignoreVary;
  bool prefixMatch;
}

/**
 * @see http://www.w3.org/TR/service-workers/#cache-storage-interface
 * @interface
 */
@JsName("CacheStorage")
abstract class CacheStorage {
// @Ignore
  external CacheStorage.fakeConstructor$();

  /**
   * @param {!RequestInfo} request
   * @param {CacheQueryOptions=} opt_options
   * @return {!Promise<!Response|undefined>}
   */
  es6.Promise<fetchapi.Response> match(request /*fetchapi.Request|String*/,
      [CacheQueryOptions opt_options]);

  /**
   * @param {string} cacheName
   * @return {!Promise<boolean>}
   */
  es6.Promise<bool> has(String cacheName);

  /**
   * @param {string} cacheName
   * @return {!Promise<!Cache>}
   */
  es6.Promise<Cache> open(String cacheName);

  /**
   * @param {string} cacheName
   * @return {!Promise<boolean>}
   */
  es6.Promise<bool> delete(String cacheName);

  /** @return {!Promise<!Array<string>>} */
  es6.Promise<List<String>> keys();
}

/**
 * @see http://www.w3.org/TR/service-workers/#extendable-event-interface
 * @constructor
 * @param {string} type
 * @param {ExtendableEventInit=} opt_eventInitDict
 * @extends {Event}
 */
@JsName("ExtendableEvent")
abstract class ExtendableEvent extends w3c_event.Event {
// @Ignore
  external ExtendableEvent.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @see http://www.w3.org/TR/service-workers/#extendable-event-interface
   * @constructor
   * @param {string} type
   * @param {ExtendableEventInit=} opt_eventInitDict
   * @extends {Event}
   */
  external factory ExtendableEvent(String type,
      [w3c_event.EventInit opt_eventInitDict]);

  /** @param {Promise} f */
  waitUntil(es6.Promise f);

  /** @type {ServiceWorker} */
  ServiceWorker activeWorker;
}
/**
 * @typedef {{
 *   bubbles: (boolean|undefined),
 *   cancelable: (boolean|undefined)
 * }}
 */
@JsName()
abstract class ExtendableEventInit {
  bool bubbles;
  bool cancelable;
}

/**
 * @see http://www.w3.org/TR/service-workers/#install-event-interface
 * @constructor
 * @param {string} type
 * @param {InstallEventInit=} opt_eventInitDict
 * @extends {ExtendableEvent}
 */
@JsName("InstallEvent")
abstract class InstallEvent extends ExtendableEvent {
// @Ignore
  external InstallEvent.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @see http://www.w3.org/TR/service-workers/#install-event-interface
   * @constructor
   * @param {string} type
   * @param {InstallEventInit=} opt_eventInitDict
   * @extends {ExtendableEvent}
   */
  external factory InstallEvent(String type,
      [InstallEventInit opt_eventInitDict]);
}
/**
 * @typedef {{
 *   bubbles: (boolean|undefined),
 *   cancelable: (boolean|undefined),
 *   activeWorker: (!ServiceWorker|undefined)
 * }}
 */
@JsName()
abstract class InstallEventInit {
  ServiceWorker activeWorker;
  bool bubbles;
  bool cancelable;
}

/**
 * @see http://www.w3.org/TR/service-workers/#fetch-event-interface
 * @constructor
 * @param {string} type
 * @param {FetchEventInit=} opt_eventInitDict
 * @extends {Event}
 */
@JsName("FetchEvent")
abstract class FetchEvent extends w3c_event.Event {
// @Ignore
  external FetchEvent.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @see http://www.w3.org/TR/service-workers/#fetch-event-interface
   * @constructor
   * @param {string} type
   * @param {FetchEventInit=} opt_eventInitDict
   * @extends {Event}
   */
  external factory FetchEvent(String type, [FetchEventInit opt_eventInitDict]);

  /** @type {!Request} */
  fetchapi.Request request;

  /** @type {!ServiceWorkerClient} */
  ServiceWorkerClient client;

  /** @type {!boolean} */
  bool isReload;

  /**
   * @param {(Response|Promise<Response>)} r
   */
  respondWith(r /*fetchapi.Response|null|es6.Promise<fetchapi.Response>*/);

  /**
   * @param {string} url
   * @return {!Promise<!Response>}
   */
  es6.Promise<fetchapi.Response> forwardTo(String url);

  /**
   * @return {!Promise<!Response>}
   */
  es6.Promise<fetchapi.Response> default$();
}
/**
 * @typedef {{
 *   bubbles: (boolean|undefined),
 *   cancelable: (boolean|undefined),
 *   request: (!Request|undefined),
 *   client: (!ServiceWorkerClient|undefined),
 *   isReload: (!boolean|undefined)
 * }}
 */
@JsName()
abstract class FetchEventInit {
  bool bubbles;
  bool cancelable;
  ServiceWorkerClient client;
  bool isReload;
  fetchapi.Request request;
}
