/*
 * Copyright 2015 The Closure Compiler Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// This file has been automatically generated.  Please do not edit it manually.
/**
 * @fileoverview Definitions for W3C's Web Cryptography specification
 * http://www.w3.org/TR/webCryptoAPI
 * FROM_EXTERN
 * This file was created using the best practices as described in:
 *     chrome_extensions.js
 */
@JsName()
library w3c_webcrypto;

import 'package:js/js.dart';
import 'package:func/func.dart';

/**
 * @const
 * @see http://www.w3.org/TR/webCryptoAPI
 */

@JsName("webCrypto.CryptoKey")
abstract class CryptoKey {
// @Ignore
  external CryptoKey.fakeConstructor$();

  /**
   * @type {string} An enumerated value representing the type of the key, a secret
   *    key (for symmetric algorithm), a public or a private key
   *    (for an asymmetric algorithm).
   */
  String type;

  /**
   * @type {boolean} Determines whether or not the raw keying material may be
   *     exported by the application.
   */
  bool extractable;

  /**
   * @type {!Object} An opaque object representing a particular cipher the key
   *     has to be used with.
   */
  Object algorithm;

  /**
   * @type {!Object} Returns the cached ECMAScript object associated with the
   *   usages internal slot, which indicates which cryptographic operations are
   *   permissible to be used with this key.
   */
  Object usages;
}

@JsName("webCrypto.JsonWebKey")
abstract class JsonWebKey {
// @Ignore
  external JsonWebKey.fakeConstructor$();

  /**
   * @type {string} Identifies the cryptographic algorithm family used with
   *     the key, such as "RSA" or "EC".
   */
  String kty;

  /**
   * @type {string} Identifies the intended use of the public key.
   */
  String use;

  /**
   * @type {!Array.<string>} Identifies the operation(s) that the key is
   *     intended to be used for.
   */
  List<String> key_ops;

  /**
   * @type {string} Identifies the algorithm intended for use with the key.
   */
  String alg;

  /**
   * @type {boolean} Boolean to be used with kty values.
   */
  bool ext;

  /**
   * @type {string} Identifies the cryptographic curve used with the key.
   */
  String crv;

  /**
   * @type {string} Contains the x coordinate for the elliptic curve point.
   */
  String x;

  /**
   * @type {string} Contains the y coordinate for the elliptic curve point.
   */
  String y;

  /**
   * @type {string} Contains the Elliptic Curve private key value.
   */
  String d;

  /**
   * @type {string} Contains the modulus value for the RSA public key.
   */
  String n;

  /**
   * @type {string} Contains the exponent value for the RSA public key.
   */
  String e;

  /**
   * @type {string} Contains the first prime factor.
   */
  String p;

  /**
   * @type {string} Contains the second prime factor.
   */
  String q;

  /**
   * @type {string} Contains the Chinese Remainder Theorem (CRT) exponent of
   *     the first factor.
   */
  String dp;

  /**
   * @type {string} Contains the Chinese Remainder Theorem (CRT) exponent of
   *     the second factor.
   */
  String dq;

  /**
   * @type {string} Contains the Chinese Remainder Theorem (CRT) coefficient
   *     of the second factor.
   */
  String qi;

  /**
   * @type {!Array.<!webCrypto.RsaOtherPrimesInfo>} Contains an array of
   *     information about any third and subsequent primes, should they exist.
   */
  List<RsaOtherPrimesInfo> oth;

  /**
   * @type {string} Contains the value of the symmetric (or other
   *     single-valued) key.
   */
  String k;
}

@JsName("webCrypto.RsaOtherPrimesInfo")
abstract class RsaOtherPrimesInfo {
// @Ignore
  external RsaOtherPrimesInfo.fakeConstructor$();

  /**
   * @type {string} Parameter within an "oth" array member represents the value
   *     of a subsequent prime factor.
   */
  String r;

  /**
   * @type {string} Parameter within an "oth" array member represents the CRT
   *     exponent of the corresponding prime factor.
   */
  String d;

  /**
   * @type {string} Parameter within an "oth" array member represents the CRT
   *     coefficient of the corresponding prime factor.
   */
  String t;
}
