/*
 * Copyright 2008 The Closure Compiler Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// This file has been automatically generated.  Please do not edit it manually.
/**
 * @fileoverview Definitions for W3C's XML related specifications.
 *  This file depends on w3c_dom2.js.
 *  The whole file has been fully type annotated.
 *
 *  Provides the XML standards from W3C.
 *   Includes:
 *    XPath          - Fully type annotated
 *    XMLHttpRequest - Fully type annotated
 *
 * @see http://www.w3.org/TR/DOM-Level-3-XPath/xpath.html
 * @see http://www.w3.org/TR/XMLHttpRequest/
 * @see http://www.w3.org/TR/XMLHttpRequest2/
 *
 * FROM_EXTERN
 * @author stevey@google.com (Steve Yegge)
 */
@JsName()
library w3c_xml;

import 'package:js/js.dart';
import 'package:func/func.dart';

import "es6.dart" as es6;

import "fileapi.dart" as fileapi;

import "html5.dart" as html5;

import "w3c_dom1.dart" as w3c_dom1;

import "w3c_event.dart" as w3c_event;

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-3-XPath/xpath.html#XPathException
 */
@JsName("XPathException")
abstract class XPathException {
// @Ignore
  external XPathException.fakeConstructor$();

  external static num get INVALID_EXPRESSION_ERR;

  external static set INVALID_EXPRESSION_ERR(num value);

  external static num get TYPE_ERR;

  external static set TYPE_ERR(num value);

  /**
   * @type {number}
   * @see http://www.w3.org/TR/DOM-Level-3-XPath/xpath.html#
   */
  num code;
}

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-3-XPath/xpath.html#XPathEvaluator
 */
@JsName("XPathEvaluator")
abstract class XPathEvaluator {
// @Ignore
  external XPathEvaluator.fakeConstructor$();

  /**
   * @param {string} expr
   * @param {?XPathNSResolver=} opt_resolver
   * @see http://www.w3.org/TR/DOM-Level-3-XPath/xpath.html#XPathEvaluator-createExpression
   * @throws XPathException
   * @throws DOMException
   */
  createExpression(String expr, [XPathNSResolver opt_resolver]);

  /**
   * @param {Node} nodeResolver
   * @see http://www.w3.org/TR/DOM-Level-3-XPath/xpath.html#XPathEvaluator-createNSResolver
   */
  createNSResolver(w3c_dom1.Node nodeResolver);

  /**
   * @param {string} expr
   * @param {Node} contextNode
   * @param {?XPathNSResolver=} opt_resolver
   * @param {?number=} opt_type
   * @param {*=} opt_result
   * @return {XPathResult}
   * @throws XPathException
   * @throws DOMException
   * @see http://www.w3.org/TR/DOM-Level-3-XPath/xpath.html#XPathEvaluator-evaluate
   */
  XPathResult evaluate(String expr, w3c_dom1.Node contextNode,
      [XPathNSResolver opt_resolver, num opt_type, opt_result]);
}

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-3-XPath/xpath.html#XPathExpression
 */
@JsName("XPathExpression")
abstract class XPathExpression {
// @Ignore
  external XPathExpression.fakeConstructor$();

  /**
   * @param {Node} contextNode
   * @param {number=} opt_type
   * @param {*=} opt_result
   * @return {*}
   * @throws XPathException
   * @throws DOMException
   * @see http://www.w3.org/TR/DOM-Level-3-XPath/xpath.html#XPathExpression-evaluate
   */
  evaluate(w3c_dom1.Node contextNode, [num opt_type, opt_result]);
}

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-3-XPath/xpath.html#XPathNSResolver
 */
@JsName("XPathNSResolver")
abstract class XPathNSResolver {
// @Ignore
  external XPathNSResolver.fakeConstructor$();

  /**
   * @param {string} prefix
   * @return {?string}
   * @see http://www.w3.org/TR/DOM-Level-3-XPath/xpath.html#XPathNSResolver-lookupNamespaceURI
   */
  String lookupNamespaceURI(String prefix);
}

/**
 * From http://www.w3.org/TR/xpath
 *
 * XPath is a language for addressing parts of an XML document, designed to be
 * used by both XSLT and XPointer.
 *
 * @noalias
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-3-XPath/xpath.html#XPathResult
 */
@JsName("XPathResult")
abstract class XPathResult {
// @Ignore
  external XPathResult.fakeConstructor$();

  external static num get UNORDERED_NODE_SNAPSHOT_TYPE;

  external static set UNORDERED_NODE_SNAPSHOT_TYPE(num value);

  external static num get ANY_UNORDERED_NODE_TYPE;

  external static set ANY_UNORDERED_NODE_TYPE(num value);

  external static num get FIRST_ORDERED_NODE_TYPE;

  external static set FIRST_ORDERED_NODE_TYPE(num value);

  external static num get BOOLEAN_TYPE;

  external static set BOOLEAN_TYPE(num value);

  external static num get ANY_TYPE;

  external static set ANY_TYPE(num value);

  external static num get UNORDERED_NODE_ITERATOR_TYPE;

  external static set UNORDERED_NODE_ITERATOR_TYPE(num value);

  external static num get ORDERED_NODE_SNAPSHOT_TYPE;

  external static set ORDERED_NODE_SNAPSHOT_TYPE(num value);

  external static num get NUMBER_TYPE;

  external static set NUMBER_TYPE(num value);

  external static num get ORDERED_NODE_ITERATOR_TYPE;

  external static set ORDERED_NODE_ITERATOR_TYPE(num value);

  external static num get STRING_TYPE;

  external static set STRING_TYPE(num value);

  /**
   * @type {boolean} {@see XPathException.TYPE_ERR}
   * @see http://www.w3.org/TR/DOM-Level-3-XPath/xpath.html#XPathResult-booleanValue
   */
  bool booleanValue;

  /**
   * @type {boolean} {@see XPathException.TYPE_ERR}
   * @see http://www.w3.org/TR/DOM-Level-3-XPath/xpath.html#XPathResult-invalid-iterator-state
   */
  bool invalidInteratorState;

  /**
   * @type {number}
   * @throws XPathException {@see XPathException.TYPE_ERR}
   * @see http://www.w3.org/TR/DOM-Level-3-XPath/xpath.html#XPathResult-numberValue
   */
  num numberValue;

  /**
   * @type {number}
   * @throws XPathException {@see XPathException.TYPE_ERR}
   * @see http://www.w3.org/TR/DOM-Level-3-XPath/xpath.html#XPathResult-resultType
   */
  num resultType;

  /**
   * @type {Node}
   * @throws XPathException {@see XPathException.TYPE_ERR}
   * @see http://www.w3.org/TR/DOM-Level-3-XPath/xpath.html#XPathResult-singleNodeValue
   */
  w3c_dom1.Node singleNodeValue;

  /**
   * @type {number}
   * @throws XPathException {@see XPathException.TYPE_ERR}
   * @see http://www.w3.org/TR/DOM-Level-3-XPath/xpath.html#XPathResult-snapshot-length
   */
  num snapshotLength;

  /**
   * @type {string}
   * @throws XPathException {@see XPathException.TYPE_ERR}
   * @see http://www.w3.org/TR/DOM-Level-3-XPath/xpath.html#XPathResult-stringValue
   */
  String stringValue;

  /**
   * @return {Node}
   * @throws XPathException {@see XPathException.TYPE_ERR}
   * @throws DOMException {@see DOMException.INVALID_STATE_ERR}
   * @see http://www.w3.org/TR/DOM-Level-3-XPath/xpath.html#XPathResult-iterateNext
   */
  w3c_dom1.Node iterateNext();

  /**
   * @param {number} index
   * @return {Node}
   * @throws XPathException
   * @see http://www.w3.org/TR/DOM-Level-3-XPath/xpath.html#XPathResult-snapshotItem
   */
  w3c_dom1.Node snapshotItem(num index);
}

/**
 * @constructor
 * @extends {Node}
 * @see http://www.w3.org/TR/DOM-Level-3-XPath/xpath.html#XPathNamespace
 */
@JsName("XPathNamespace")
abstract class XPathNamespace extends w3c_dom1.Node {
// @Ignore
  external XPathNamespace.fakeConstructor$() : super.fakeConstructor$();

  external static num get XPATH_NAMESPACE_NODE;

  external static set XPATH_NAMESPACE_NODE(num value);

  /**
   * @type {Element}
   * @see http://www.w3.org/TR/DOM-Level-3-XPath/xpath.html#XPathNamespace-ownerElement
   */
  w3c_dom1.Element ownerElement;
}

/**
 * From http://www.w3.org/TR/XMLHttpRequest/
 *
 * (Draft)
 *
 * The XMLHttpRequest Object specification defines an API that provides
 * scripted client functionality for transferring data between a client and a
 * server.
 *
 * @constructor
 * @implements {EventTarget}
 * @see http://www.w3.org/TR/XMLHttpRequest/#xmlhttprequest-object
 */
@JsName("XMLHttpRequest")
abstract class XMLHttpRequest implements w3c_event.EventTarget {
// @Ignore
  external XMLHttpRequest.fakeConstructor$();

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  addEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  removeEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /** @override */
  bool dispatchEvent(w3c_event.Event evt);

  /**
   * @param {string} method
   * @param {string} url
   * @param {?boolean=} opt_async
   * @param {?string=} opt_user
   * @param {?string=} opt_password
   * @return {undefined}
   * @see http://www.w3.org/TR/XMLHttpRequest/#the-open()-method
   */
  void open(String method, String url,
      [bool opt_async, String opt_user, String opt_password]);

  /**
   * @param {string} header
   * @param {string} value
   * @return {undefined}
   * @see http://www.w3.org/TR/XMLHttpRequest/#the-setrequestheader()-method
   */
  void setRequestHeader(String header, String value);

  /**
   * @param {ArrayBuffer|ArrayBufferView|Blob|Document|FormData|string=} opt_data
   * @return {undefined}
   * @see http://www.w3.org/TR/XMLHttpRequest/#the-send()-method
   */
  void send(
      [opt_data /*es6.ArrayBuffer|null|es6.ArrayBufferView|fileapi.Blob|w3c_dom1.Document|FormData|String|undefined*/]);

  /**
   * @return {undefined}
   * @see http://www.w3.org/TR/XMLHttpRequest/#the-abort()-method
   */
  void abort();

  /**
   * @return {string}
   * @see http://www.w3.org/TR/XMLHttpRequest/#the-getallresponseheaders()-method
   */
  String getAllResponseHeaders();

  /**
   * @param {string} header
   * @return {string}
   * @see http://www.w3.org/TR/XMLHttpRequest/#the-getresponseheader()-method
   */
  String getResponseHeader(String header);

  /**
   * @type {string}
   * @see http://www.w3.org/TR/XMLHttpRequest/#the-responsetext-attribute
   */
  String responseText;

  /**
   * @type {Document}
   * @see http://www.w3.org/TR/XMLHttpRequest/#the-responsexml-attribute
   */
  w3c_dom1.Document responseXML;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/XMLHttpRequest/#the-readystate-attribute
   */
  num readyState;

  /**
   * @type {number}
   * @see http://www.w3.org/TR/XMLHttpRequest/#the-status-attribute
   */
  num status;

  /**
   * @type {string}
   * @see http://www.w3.org/TR/XMLHttpRequest/#the-statustext-attribute
   */
  String statusText;

  /**
   * @type {Function}
   * @see http://www.w3.org/TR/XMLHttpRequest/#handler-xhr-onreadystatechange
   */
  Function onreadystatechange;

  /**
   * @type {Function}
   * @see http://www.w3.org/TR/XMLHttpRequest/#handler-xhr-onerror
   */
  Function onerror;

  /**
   * @type {boolean}
   * @see http://dev.w3.org/2006/webapi/XMLHttpRequest-2/#withcredentials
   */
  bool withCredentials;

  /**
   * @type {XMLHttpRequestUpload}
   * @see http://dev.w3.org/2006/webapi/XMLHttpRequest-2/#the-upload-attribute
   */
  html5.XMLHttpRequestUpload upload;

  /**
   * @param {string} mimeType The mime type to override with.
   */
  overrideMimeType(String mimeType);

  /**
   * @type {string}
   * @see http://dev.w3.org/2006/webapi/XMLHttpRequest-2/#the-responsetype-attribute
   */
  String responseType;

  /**
   * @type {*}
   * @see http://dev.w3.org/2006/webapi/XMLHttpRequest-2/#the-responsetype-attribute
   */
  dynamic response;

  /**
   * @type {ArrayBuffer}
   * Implemented as a draft spec in Firefox 4 as the way to get a requested array
   * buffer from an XMLHttpRequest.
   * @see https://developer.mozilla.org/En/Using_XMLHttpRequest#Receiving_binary_data_using_JavaScript_typed_arrays
   */
  es6.ArrayBuffer mozResponseArrayBuffer;
}

/**
 * The FormData object represents an ordered collection of entries. Each entry
 * has a name and value.
 *
 * @param {?Element=} opt_form An optional form to use for constructing the form
 *     data set.
 * @constructor
 * @see http://www.w3.org/TR/XMLHttpRequest2/#the-formdata-interface
 */
@JsName("FormData")
abstract class FormData {
// @Ignore
  external FormData.fakeConstructor$();

  /**
   * The FormData object represents an ordered collection of entries. Each entry
   * has a name and value.
   *
   * @param {?Element=} opt_form An optional form to use for constructing the form
   *     data set.
   * @constructor
   * @see http://www.w3.org/TR/XMLHttpRequest2/#the-formdata-interface
   */
  external factory FormData([w3c_dom1.Element opt_form]);

  /**
   * @param {string} name
   * @param {Blob|string} value
   * @param {string=} opt_filename
   */
  append(String name, value /*fileapi.Blob|null|String*/,
      [String opt_filename]);
}
