/*
 * Copyright 2008 The Closure Compiler Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// This file has been automatically generated.  Please do not edit it manually.
/**
 * @fileoverview Definitions for all the extensions over W3C's DOM
 *  specification by WebKit. This file depends on w3c_dom2.js.
 *  All the provided definitions has been type annotated
 *
 * FROM_EXTERN
 */
@JsName()
library webkit_dom;

import 'package:js/js.dart';
import 'package:func/func.dart';

import "w3c_css.dart" as w3c_css;

import "w3c_dom1.dart" as w3c_dom1;

/**
 * @constructor
 * @see http://trac.webkit.org/browser/trunk/Source/WebCore/page/MemoryInfo.idl
 * @see http://trac.webkit.org/browser/trunk/Source/WebCore/page/MemoryInfo.cpp
 */
@JsName("MemoryInfo")
abstract class MemoryInfo {
// @Ignore
  external MemoryInfo.fakeConstructor$();

  /** @type {number} */
  num totalJSHeapSize;

  /** @type {number} */
  num usedJSHeapSize;

  /** @type {number} */
  num jsHeapSizeLimit;
}

/**
 * @constructor
 * @see http://trac.webkit.org/browser/trunk/Source/WebCore/inspector/ScriptProfileNode.idl
 */
@JsName("ScriptProfileNode")
abstract class ScriptProfileNode {
// @Ignore
  external ScriptProfileNode.fakeConstructor$();

  /** @type {string} */
  String functionName;

  /** @type {string} */
  String url;

  /** @type {number} */
  num lineNumber;

  /** @type {number} */
  num totalTime;

  /** @type {number} */
  num selfTime;

  /** @type {number} */
  num numberOfCalls;

  /** @type {Array.<ScriptProfileNode>} */
  List<ScriptProfileNode> children;

  /** @type {boolean} */
  bool visible;

  /** @type {number} */
  num callUID;
}

/**
 * @constructor
 * @see http://trac.webkit.org/browser/trunk/Source/WebCore/inspector/ScriptProfile.idl
 */
@JsName("ScriptProfile")
abstract class ScriptProfile {
// @Ignore
  external ScriptProfile.fakeConstructor$();

  /** @type {string} */
  String title;

  /** @type {number} */
  num uid;

  /** @type {ScriptProfileNode} */
  ScriptProfileNode head;
}

/**
 * @constructor
 * @see http://trac.webkit.org/browser/trunk/Source/WebCore/page/Console.idl
 * @see http://trac.webkit.org/browser/trunk/Source/WebCore/page/Console.cpp
 */
@JsName("Console")
abstract class Console {
// @Ignore
  external Console.fakeConstructor$();

  /**
   * @param {*} condition
   * @param {...*} var_args
   */
  assert$(condition, [var_args1, var_args2, var_args3, var_args4]);

  /**
   * @param {...*} var_args
   */
  error([var_args1, var_args2, var_args3, var_args4]);

  /**
   * @param {...*} var_args
   */
  info([var_args1, var_args2, var_args3, var_args4]);

  /**
   * @param {...*} var_args
   */
  log([var_args1, var_args2, var_args3, var_args4]);

  /**
   * @param {...*} var_args
   */
  warn([var_args1, var_args2, var_args3, var_args4]);

  /**
   * @param {...*} var_args
   */
  debug([var_args1, var_args2, var_args3, var_args4]);

  /**
   * @param {*} value
   */
  dir(value);

  /**
   * @param {...*} var_args
   */
  dirxml([var_args1, var_args2, var_args3, var_args4]);

  /**
   * @param {!Object} data
   * @param {*=} opt_columns
   */
  table(Object data, [opt_columns]);

  /**
   * @return {undefined}
   */
  void trace();

  /**
   * @param {*} value
   */
  count(value);

  /**
   * @param {*} value
   */
  markTimeline(value);

  /**
   * @param {string=} opt_title
   */
  profile([String opt_title]);

  /** @type {Array.<ScriptProfile>} */
  List<ScriptProfile> profiles;

  /**
   * @param {string=} opt_title
   */
  profileEnd([String opt_title]);

  /**
   * @param {string} name
   */
  time(String name);

  /**
   * @param {string} name
   */
  timeEnd(String name);

  /**
   * @param {*} value
   */
  timeStamp(value);

  /**
   * @param {...*} var_args
   */
  group([var_args1, var_args2, var_args3, var_args4]);

  /**
   * @param {...*} var_args
   */
  groupCollapsed([var_args1, var_args2, var_args3, var_args4]);

  groupEnd();

  clear();

  /** @type {MemoryInfo} */
  MemoryInfo memory;
}
/**
 * @type {!Console}
 * @suppress {duplicate}
 */
@JsName("console")
external Console get console;
@JsName("console")
external set console(Console value);

/**
 * @constructor
 * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-ViewCSS
 */
@JsName("ViewCSS")
abstract class ViewCSS {
// @Ignore
  external ViewCSS.fakeConstructor$();

  /**
   * @param {Element} element
   * @param {string} pseudoElement
   * @param {boolean=} opt_authorOnly
   * @return {CSSRuleList}
   * @nosideeffects
   */
  w3c_css.CSSRuleList getMatchedCSSRules(
      w3c_dom1.Element element, String pseudoElement, [bool opt_authorOnly]);

  /**
   * @param {Element} elt
   * @param {?string=} opt_pseudoElt This argument is required according to the
   *     CSS2 specification, but optional in all major browsers. See the note at
   *     https://developer.mozilla.org/en-US/docs/Web/API/Window.getComputedStyle
   * @return {CSSStyleDeclaration}
   * @see http://www.w3.org/TR/DOM-Level-2-Style/css.html#CSS-CSSview-getComputedStyle
   */
  w3c_css.CSSStyleDeclaration getComputedStyle(w3c_dom1.Element elt,
      [String opt_pseudoElt]);
}
