/*
 * Copyright 2010 The Closure Compiler Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
// This file has been automatically generated.  Please do not edit it manually.
/**
 * @fileoverview Definitions for W3C's Notifications specification.
 * FROM_EXTERN
 */
@JsName()
library webkit_notifications;

import 'package:js/js.dart';
import 'package:func/func.dart';

import "w3c_event.dart" as w3c_event;

import "w3c_serviceworker.dart" as w3c_serviceworker;

/**
 * @typedef {{dir: (string|undefined), lang: (string|undefined),
 *            body: (string|undefined), tag: (string|undefined),
 *            icon: (string|undefined)}}
 * @see http://notifications.spec.whatwg.org/#notificationoptions
 */
@JsName()
abstract class NotificationOptions {
  String body;
  String dir;
  String icon;
  String lang;
  String tag;
}
/** @interface */
external NotificationOptionsInterface_();

/**
 * @param {string} title
 * @param {NotificationOptions=} opt_options
 * @constructor
 * @implements {EventTarget}
 * @see http://notifications.spec.whatwg.org/#notification
 */
@JsName("Notification")
abstract class Notification implements w3c_event.EventTarget {
// @Ignore
  external Notification.fakeConstructor$();

  /**
   * @type {string}
   */
  external static String get permission;

  external static set permission(String value);

  /**
   * @param {NotificationPermissionCallback=} opt_callback
   */
  external static requestPermission([opt_callback(String arg1)]);

  /**
   * @param {string} title
   * @param {NotificationOptions=} opt_options
   * @constructor
   * @implements {EventTarget}
   * @see http://notifications.spec.whatwg.org/#notification
   */
  external factory Notification(String title,
      [NotificationOptions opt_options]);

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  addEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /**
   * @param {boolean=} opt_useCapture
   * @override
   */
  removeEventListener(String type, bool listener(w3c_event.Event arg1),
      [bool opt_useCapture]);

  /** @override */
  bool dispatchEvent(w3c_event.Event evt);

  /**
   * The ID used by clients to uniquely identify notifications to eliminate
   * duplicate notifications.
   * @type {string}
   * @deprecated Use NotificationOptions.tag instead.
   */
  String replaceId;

  /**
   * The string used by clients to specify the directionality (rtl/ltr) of the
   * notification.
   * @type {string}
   * @deprecated Use NotificationOptions.titleDir and bodyDir instead.
   */
  String dir;

  /**
   * Displays the notification.
   */
  show();

  /**
   * Prevents the notification from being displayed, or closes it if it is already
   * displayed.
   */
  cancel();

  /**
   * Prevents the notification from being displayed, or closes it if it is already
   * displayed.
   */
  close();

  /**
   * An event handler called when notification is closed.
   * @type {?function(Event)}
   */
  Func1<w3c_event.Event, dynamic> onclose;

  /**
   * An event handler called if the notification could not be displayed due to
   * an error (i.e. resource could not be loaded).
   * @type {?function(Event)}
   */
  Func1<w3c_event.Event, dynamic> onerror;

  /**
   * An event handler called when the notification has become visible.
   * @type {?function(Event)}
   * @deprecated Use onshow instead.
   */
  Func1<w3c_event.Event, dynamic> ondisplay;

  /**
   * An event handler called when the notification has become visible.
   * @type {?function(Event)}
   */
  Func1<w3c_event.Event, dynamic> onshow;

  /**
   * An event handler called when the notification has been clicked on.
   * @type {?function(Event)}
   */
  Func1<w3c_event.Event, dynamic> onclick;
}
/**
 * @typedef {function(string)}
 * @see http://notifications.spec.whatwg.org/#notificationpermissioncallback
 */
typedef NotificationPermissionCallback(String arg1);

/**
 * @constructor
 * @see http://dev.w3.org/2006/webapi/WebNotifications/publish/#dialog-if
 * @deprecated Use Notification instead.
 */
@JsName("NotificationCenter")
abstract class NotificationCenter {
// @Ignore
  external NotificationCenter.fakeConstructor$();

  /**
   * Creates a text+icon notification and displays it to the user.
   * @param {string} iconUrl
   * @param {string} title
   * @param {string} body
   * @return {Notification}
   */
  Notification createNotification(String iconUrl, String title, String body);

  /**
   * Creates an HTML notification and displays it to the user.
   * @param {string} url
   * @return {Notification}
   */
  Notification createHTMLNotification(String url);

  /**
   * Checks if the user has permission to display notifications.
   * @return {number}
   */
  num checkPermission();

  /**
   * Requests permission from the user to display notifications.
   * @param {Function=} opt_callback
   * @return {void}
   */
  void requestPermission([Function opt_callback]);
}

/**
 * @see https://notifications.spec.whatwg.org/#notificationevent
 * @constructor
 * @param {string} type
 * @param {!ExtendableEventInit=} opt_eventInitDict
 * @extends {ExtendableEvent}
 */
@JsName("NotificationEvent")
abstract class NotificationEvent extends w3c_serviceworker.ExtendableEvent {
// @Ignore
  external NotificationEvent.fakeConstructor$() : super.fakeConstructor$();

  /**
   * @see https://notifications.spec.whatwg.org/#notificationevent
   * @constructor
   * @param {string} type
   * @param {!ExtendableEventInit=} opt_eventInitDict
   * @extends {ExtendableEvent}
   */
  external factory NotificationEvent(String type,
      [w3c_event.EventInit opt_eventInitDict]);

  /** @type {?Notification} */
  Notification notification;
}
